<?php
require_once("../../prepend.inc");
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Admin
 * @author Phil Calder
 */
 
/**
 * Application backup utility.
 * 
 * @package Admin
 * @author Phil Calder
 */ 
class AdminBackupUtility {
	
	/**
	 * @access private
	 */
	var $ua = false;
	
	/**
	 * @access private
	 */
	var $modules = false;
	
	/**
	 * @access private
	 */
	var $config = false;
	
	/**
	 * @access private
	 */
	var $mod = false;
	
	/**
	 * @access private
	 */
	var $db = false;
	
	/**
	 * @access private
	 */
	var $dd = false;
	
	/**
	 * @access private
	 */
	var $filesystem = false;
	
	/**
	 * @access private
	 */
	var $backup_path = false;
	
	/**
	 * @access private
	 */
	var $export_path = false;
	
	/**
	 * @access private
	 */
	var $errors = false;
	
	/**
	 * @access private
	 */
	var $messages = array();
	
	/**
	 * Calls {@link init}
	 */
	function AdminBackupUtility() {
		$this->init();
	}
	
	/**
	 * Initialise objects and variables used by this class
	 *
	 * Restricted to Administrators of the Admin module
	 */
	function init() {
		global $SERVICES;
		
		$this->form_values = $this->arrayMergeReplace($_GET, $_POST);
		
		$this->errors = array();
		
		$this->ua =& $SERVICES->getService('Authentication');
		$this->ua->adminRestrict('Admin');
		
		$this->modules    =& $SERVICES->getService('Modules');
		$this->config     =& $SERVICES->getService('Config');
		$this->mod        =& $SERVICES->modules->getModule('Admin');
		$this->db         =& $SERVICES->getService('Database');
		$this->dd         =& $SERVICES->getService('DataDefinition');
		$this->filesystem =& $SERVICES->getService('FileSystem');
		$this->output     =& $SERVICES->getService('Output');
		
		// $this->setExportPath(); // creates a directory
		
	}
	
	/**
	 * Merges two arrays.
	 *
	 * Values in the new array overwrite those in the old array.
	 * 
	 * @param array $old_array The existing array
	 * @param array $new_array The new array
	 * @return array The new merged array
	 * @author Phil Calder
	 */
	function arrayMergeReplace($old_array, $new_array) {
		// see also the Config service arrayMergeReplace
		if (!is_array($new_array)) {
			$new_array = array($new_array);	
		}
		
		foreach($new_array as $k => $v) {
		   if (is_array($v)) {
			   if (!isset($old_array[$k])) {
				   $old_array[$k] = array();
			   }
			   $old_array[$k] = $this->arrayMergeReplace($old_array[$k], $v);
		   } else {
			   $old_array[$k] = $v;
		   }
		}
		return $old_array;
	}
	
	/**
	 * @return boolean whether the Export path exists and is write-able
	 */
	function setExportPath($export_subdir = '') {
		if ($this->export_path === false) {
			$parent_dir = $this->getBackupPath();
			if (!$export_subdir) {
				$new_subdir = date("YmdHis", time());
			} else {
				$new_subdir = $export_subdir;	
			}
			if(!$export_subdir && !$this->filesystem->createDir($new_subdir, $parent_dir, true)) {
				// print '<div>cannot create ' . $new_subdir . ' ' . $parent_dir . '</div>';
				return false;
			} else {
				$this->export_path = $parent_dir . '/' . $new_subdir;
			}
		}
		
		if ($this->export_path) {
			if ( @is_dir($this->export_path) && @is_writeable($this->export_dir) ) {
				return true;
			} 
		}
		
		return false;
	}
	
	/** 
	 * @return boolean|string false, or the export path
	 */
	function getExportPath() {
		if ($this->export_path === false) {
			$this->setExportPath();
		}
		return $this->export_path;
	}
	
	function getBackupPath() {
		return $this->config->lookup('modules.Admin.backup.dir');
	}
	
	function storeError($error = false) {
		$this->errors[] = $error;
	}
	
	function hasErrors() {
		return (count((array)$this->errors) ? true : false);
	}
	
	function getErrors() {
		return (array)$this->errors;
	}
	
	function printErrors() {
		$content = '';
		if ($this->hasErrors()) {
			foreach ($this->getErrors() as $k => $v) {
				if (is_string($v)) {
					$content .= $v . '<br />';
				}
			}
		}
		return $content;
	}
	
	function clearErrors() {
		$this->errors = array();
	}
	
	function addMessage($str = false) {
		if (strlen($str)) {
			$this->messages[] = $str;
		}
	}
	
	function hasMessages() {
		if (is_array($this->messages) && count($this->messages)) {
			return true;
		}
		return false;
	}
	
	function getMessages() {
		return (array)$this->messages;
	}
	
	function getDescriptionContent() {
		$content = '';
		if ($this->hasMessages()) {
			$padding = true;
			foreach ($this->getMessages() as $k => $v) {
				$content .= $this->output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 5px"' : '') . '>';
				$content .= $v;
				$content .= '</td>' . $this->output->getEndRow(true);
			}
		}
		return $content;
	}
	
	function backupFiles($type = 0, $backup_path_prepend = false) {
		// $this->filesystem->printDebug('AdminBackupUtility::backupFiles(' . $type . ', [' . $backup_path_prepend . '])');
		$path_to_files = false;
		$secondary_path = false; // incase we are backing up site root and files dir is not in site root
		
		$exclusions = array();
		$exclusions[] = $this->getBackupPath();
		
		switch ((int)$type) {
			case 1: // mail dir only
				$path_to_files = $this->config->lookup('mail.dir');
			break;
			case 2: // files dir
				$path_to_files = $this->config->lookup('files.dir');
			break;
			case 3: // site root (whether files dir is a sub directory or not)
				$path_to_files = $this->config->lookup('site.root');
			break;
			case 4: // site root, no files dir
				$path_to_files = $this->config->lookup('site.root');
				$exclusions[] = $this->config->lookup('files.dir');
			break;
			case 5: // site root, no mail dir
				$path_to_files = $this->config->lookup('site.root');
				$exclusions[] = $this->config->lookup('mail.dir');
			break;
			
			default: // entire application directory (site root and files dir)
				$path_to_files = $this->config->lookup('site.root');
				
				$files_dir = $this->config->lookup('files.dir');
				if ($files_dir && ($this->filesystem->checkDirExists(basename($files_dir), dirname($files_dir))) && substr($files_dir, 0, strlen($path_to_files)) != $path_to_files) {
					// $this->filesystem->printDebug('site root: ' . $path_to_files);
					// $this->filesystem->printDebug('files dir: ' . $files_dir);
					// $this->filesystem->printDebug('Compare files_dir and site root dir: (not the same) ' . substr($files_dir, 0, strlen($path_to_files)));
					$secondary_path = $files_dir;
				}
				
			break;
		}
		
		if ($export_path = $this->getExportPath()) {
			if ($backup_path_prepend) {
				// $this->filesystem->printDebug('backup_path_prepend set: ' . $backup_path_prepend);
				$export_path = $export_path . '/' . $backup_path_prepend;
				// $this->filesystem->printDebug('new export_path set: ' . $export_path);
			}
			$this->filesystem->setDirectory($export_path, true);
			$result = $this->filesystem->recursiveCopy($path_to_files, false, $exclusions);
			if (!$result) {
				// $this->filesystem->printDebug('AdminBackupUtility called filesystem recursiveCopy - returned false, aborting backup');
				return false;	
			}
			
			if ($secondary_path) {
				// $this->filesystem->printDebug('secondary backup required for: ' . $secondary_path);
				// $this->filesystem->printDebug('current export path: ' . $export_path);
				// $this->filesystem->printDebug('calling backupFiles with path prepend:: ' . basename($secondary_path));
				
				$secondary_result = $this->backupFiles(2, basename($secondary_path));
				if (!$secondary_result) {
					// $this->filesystem->printDebug('AdminBackupUtility called filesystem recursiveCopy for secondary backup - returned false, aborting backup');
					return false;	
				}
			}
			
			
			return $result;
		} else {
			global $L;
			$this->storeError($L['adminBackupNoExportPath']);
		}
		return false;
	}
	
	function backupDatabase() {
		$result = $this->mod->exportDatabase($this->getExportPath());
		if ($result === true) {
			return $result;
		}
		
		$this->storeError($result);
		return false;
	}
	
	function compressBackup() {
		$result = $this->mod->compressExport($this->getExportPath());
		if ($result === true) {
			return $result;
		}
		
		$this->storeError($result);
		return false;
	}
	
	function deleteBackup($name, $add_messages = true) {
		// For security, we require that the file is in the backup directory
		$path = $this->getBackupPath();
		$full_path = $path . '/' . $name;
		// $this->filesystem->printDebug('Delete backup ' . $name . ' [' . $full_path . ']');
		if (file_exists($full_path)) {
			global $L;
			if (@is_dir($full_path)) {
				
				// $this->filesystem->printDebug('Delete backup calling filesystem recursiveDelete: ' . $full_path);
				// return $this->filesystem->recursiveDelete($full_path);	
				
				// --
				if ($delete_result = $this->filesystem->recursiveDelete($full_path)) {
					if ($add_messages) {
						$this->addMessage($L['adminBackupDeleteComplete'] . ': ' . basename($full_path));
					}
				} else {
					if ($add_messages) {
						$this->addMessage($L['adminBackupDeleteProblem'] . ': ' . basename($full_path));
						$this->addMessage($this->printErrors());
						$this->clearErrors();
					}
				}
				return $delete_result;
				
				// --
			} else {
				// $this->filesystem->printDebug('Delete backup calling filesystem deleteFile: ' . basename($full_path) . ', ' . dirname($full_path));
				if ($delete_result = $this->filesystem->deleteFile(basename($full_path), dirname($full_path))) {
					if ($add_messages) {
						$this->addMessage($L['adminBackupDeleteComplete'] . ': ' . basename($full_path));
					}
				} else {
					if ($add_messages) {
						$this->addMessage($L['adminBackupDeleteProblem'] . ': ' . basename($full_path));
						$this->addMessage($this->printErrors());
						$this->clearErrors();
					}
				}
				return $delete_result;
				
			}
		
		} else {
			if ($add_messages) {
				global $L;
				$this->addMessage($L['adminBackupDeleteProblem'] . ': ' . basename($full_path));
			}
		}
	}
	
	function checkBackupDir() {
		$backup_dir = $this->getBackupPath();
		if (!$backup_dir) {
			return false;
		}
		if (file_exists($backup_dir) && @is_dir($backup_dir) && @is_readable($backup_dir)) {
			return true;
		}
		if (!file_exists($backup_dir)) {
			$create_dir_result = $this->filesystem->createDir(basename($backup_dir), dirname($backup_dir), true);
			$data_dir = $this->config->lookup('files.dir');
			if ($create_dir_result && $data_dir) {
				if ($this->filesystem->checkFileExists('.htaccess', $data_dir)) {
					$this->filesystem->copyFile($data_dir . '/.htaccess', false, $backup_dir);
				}
			}
		}
		return false;
	}
	function run() {
		global $L;
		@set_time_limit(1200); // 20 minutes
		if (isset($this->form_values['item']) && isset($this->form_values['item']['backup']) && $this->form_values['item']['backup']) {
			
			// currently calls backupFiles without args - will backup entire installation (site root and files dir)
			if ($backup_files_result = $this->backupFiles()) {
				$this->addMessage($L['adminBackupFileBackupComplete']);
				// $this->addMessage($L['adminBackupExportPath'] . ': ' . $this->getExportPath() . '');
			} else {
				$this->addMessage($L['adminBackupFileBackupProblem']);
				$this->addMessage($this->printErrors());
				$this->clearErrors();
				return;
			}
			
			if ($backup_database_result = $this->backupDatabase()) {
				$this->addMessage($L['adminBackupDbBackupComplete']);
				// $this->addMessage($L['adminBackupExportPath'] . ': ' . $this->getExportPath() . '');
			} else {
				$this->addMessage($L['adminBackupDbBackupProblem']);
				$this->addMessage($this->printErrors());
				$this->clearErrors();
				return;
			}
			
			if ($compress_backup = $this->compressBackup()) {
				$this->addMessage($L['adminBackupCompressionComplete']);
				// print($L['adminBackupExportPath'] . ': ' . $this->getExportPath() . '<hr />');
				// TODO: add error message if unsuccessful
				$this->deleteBackup(basename($this->getExportPath()), false);
			} else {
				$this->addMessage($L['adminBackupCompressionProblem']);
				$this->addMessage($this->printErrors());
				$this->clearErrors();
				return;
			}
			
			$this->addMessage($L['adminBackupName'] . ': ' . basename($this->getExportPath()) . '');
			
		} else if (isset($this->form_values['download']) && $this->form_values['download']) {
			$this->downloadFile($this->form_values['download']);
		} else if (isset($this->form_values['delete']) && $this->form_values['delete']) {
			// TODO: add error message if unsuccessful
			$this->deleteBackup($this->form_values['delete']);
		} else if (isset($this->form_values['archive']) && $this->form_values['archive']) {
			$this->setExportPath($this->form_values['archive']);
			if ($this->compressBackup($this->form_values['archive'])) {
				// print '<div>Successfully compressed backup: ' . $this->form_values['archive'] . '</div>';
				$this->addMessage($L['adminBackupCompressionComplete']);
				// TODO: add error message if unsuccessful
				$this->deleteBackup($this->form_values['archive'], false);
			} else {
				// print '<div>Could not compress backup: ' . $this->form_values['archive'] . '</div>';
				$this->addMessage($L['adminBackupCompressionProblem']);
			}
		} else {
			// Ensure we have a backup directory. If not, try to create it
			$this->checkBackupDir();
			
		}
	}
	
	function getBackupForm() {
		// TODO: future enhancements
		// - specify what to backup, options:
		// -- entire application (include check to see if data dir is in BCRM_DOCUMENT_ROOT)
		// -- data dir only
		// -- mail only
		// -- files only
		// -- database only
		// -- any combination of the above
		// Backup options:
		// -- compress backup (options: tar and/or gz, zip,...)
		// -- Retain only the latest backup (automatically purge old backup files/directories)
		// -- save backup using a standard name (eg browsercrm_backup.tar.gz) instead of formatted date (yyyymmddhhiiss.tar.gz)
		// -- append/prepend backup filename with a config or user specified label 
		//    (eg for 'clientname': clientname_backup_yyyymmddhhiiss.tar.gz)
		// Log all backups (un/successful) using the Logger service
		// Email or memo an Administrator when a backup is performed
		// Add a manifest to the archive
		global $SERVICES, $L;
		// --
		$table_type = 'viewTbl';
		$action = $SERVICES->getGlobal('php_self');;
		$js_validate = false; // No js validation
		$item = array();
		$error_msg = false;
		
		$field1 = array(
			'label' => '',
			'type' => 'hidden',
			'field_key' => 'backup', 
			'value' => 1
		);
		// --
		
		$content = '';
		
		$content .= $this->output->getBeginForm($action, $js_validate); 
		
		$content .= $this->output->getHtmlField($field1); 
			
		$content .=	$this->output->getBeginTable($table_type);

		// $content .= $this->getFieldRows($item, $error_msg);

		// $content .= $this->output->getSpacerRow();
		$content .= $this->output->getTextRow($L['adminBackupFormDescription']);
		$content .= $this->output->getTextRow($L['adminBackupFormDescription2']);
		
		$content .= $this->output->getSubmitRow();
		//$content .= $this->output->getSpacerRow();

		$content .=	$this->output->getEndTable();

		$content .= $this->output->getEndForm();
		
		return $content;
	}
	
	function getBackups() {
		$backup_dir = $this->getBackupPath();
		$backups = $this->filesystem->getDirContents($backup_dir, false, array('.htaccess'));
		// print_r($backups);
		return $backups;
	}
	
	function listBackups() {
		global $SERVICES, $L;
		// Display a table listing existing backups
		// Provide a link to remove the backup (file or directory)
		// If the backup is a file (ie compressed archive, eg .tar.gz) provide a download link
		// If the backup is a directory (and a file with the same name plus an extension does not exist) provide an Archive link
		

		$content = '';
		$backups = $this->getBackups();
		$backup_path = $this->getBackupPath();
		
		
		if (is_array($backups) && count($backups)) {
			ksort($backups);
			$content .= $this->output->getBeginTable('zeroTbl');
			$header = array(
				'filename' => array('label' => $L['adminBackupName']), 
				'size' => array('label' => $L['adminBackupFileSize']), 
				'action' => array('label' => $L['adminBackupAction']), 
				'delete' => array('label' => $L['adminBackupDelete'])
				
			);
			
			$content .= '<p>' . $L['adminBackupExistingBackups'] .'</p>';
			$content .= $this->output->getHeaderRow($header);
			
			$this_page = $SERVICES->getGlobal('php_self');
			foreach ($backups as $k => $v) {
				$row = array();
				$download_link = '';
				$file_size = false;
				if (isset($v['name']) && $v['name']) {
					if (@is_file($backup_path . '/' . $v['name']) && !@is_dir($backup_path . '/' . $v['name'])) {
						$download_link = '<a href="' . $this_page . '?download=' . $v['name'] . '">' . $L['adminBackupDownload'] . '</a>';
						// $fileinfo = array('location'=> $backup_path . '/' . $v['name']);
						// $this->filesystem->displayFile($fileinfo, true);
						$file_size = sprintf("%u", @filesize($backup_path . '/' . $v['name']));
						$file_size = floatVal($file_size / (1024*1024));
						$file_size = round($file_size, 2);
						$file_size = number_format($file_size, 2);
						$file_size .= ' Mb';
						$file_size = '<span style="float:right">' . $file_size . '</span>';
					} else {
						// archive link
						$download_link = '<a href="' . $this_page . '?archive=' . $v['name'] . '">' . $L['adminBackupArchive'] . '</a>';
					}
					$delete_link = '<a href="' . $this_page . '?delete=' . $v['name'] . '">' . $L['adminBackupDelete'] . '</a>';
					$row['filename'] = $v['name'];
					$row['size'] = $file_size;
					$row['action'] = $download_link;
					$row['delete'] = $delete_link;
					
					
					// $content .= '<div>' . $v['name'] . '' . ($download_link) .'</div>';
					$content .= $this->output->getTableRow($row);
				}
			}
			$content .= $this->output->getEndTable();
		}
		return $content;
	}
	
	function downloadFile($filename) {
		if ($filename) { 
			$complete_filename = $this->getBackupPath() . '/' . $filename;
			// print $complete_filename . '<hr />' . $filename . '</hr>'; exit();
			if (is_file($complete_filename) && is_readable($complete_filename)) {
				$fileinfo = array(
					'location'=> $complete_filename, 
					'file_name' => $filename
				);
				$download = true;
				// Phil: testing
				// if (isset($this->form_values['view_only']) && $this->form_values['view_only']) {
					// $download = false;
				// } 
				$this->filesystem->displayFile($fileinfo, $download);
				exit();
			}
		}
	}
	
	function display($content) {
		// print $content;
		$this->output->addToBody($content);
		$this->output->display();
	}
	
	function main() {
		global $L;
		$title_text = $L['adminBackupTitleText'];
		
		$content = '';
		
		$content .= $this->output->getBeginTable('mainTbl', true);
		
		$content .= $this->output->getTitleRow($title_text);
		
		$content .= $this->getDescriptionContent();
		
		$content .= $this->output->getBeginDataCell();
		
		// Display backup form
		$content .= $this->getBackupForm();
		
		// List Current Backups
		$content .= $this->listBackups();
		
		$content .= $this->output->getEndDataCell();
		
		$content .= $this->output->getEndTable(true);
		
		$this->display($content);
	}
}

$admin_backup_utility = new AdminBackupUtility();
$admin_backup_utility->run();
$admin_backup_utility->main();

?>
