<?php
require_once("../../prepend.inc");
/**
 * Create a new dropdown table.
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Admin
 * @author Phil Calder
 */

global $SERVICES;

$ua =& $SERVICES->getService('Authentication');
$ua->adminRestrict();

$add =& $SERVICES->getService('DataDefinition', '', 'AdminDataDefinition');

$SERVICES->getService('Language');
$SERVICES->language->getLanguage('Admin');

$SERVICES->getService('Config');

$output =& $SERVICES->getService('Output');

function getFormFields($item) {
	global $mod, $output, $L, $add;
	//field definitions
	$fields =  array( 
		array('label'=>'', 'field_key'=>'id', 'type'=>'hidden'), // not realy used
		array('label'=>'', 'field_key'=>'table', 'type'=>'hidden'), 
		array('label'=>'', 'field_key'=>'module', 'type'=>'hidden'), 
		array('label'=>'', 'field_key'=>'type', 'type'=>'hidden'),
		array('label'=>$L['adminCustomFieldsDropdownTable'], 'field_key'=>'table_name', 'type'=>'text', 'notnull'=>1)
	);
	
	foreach ($fields as $k => $v) {
		$fields[$k]['value'] = $item[$v['field_key']];
	}
	
	return $fields;	
} // getFormFields

function doSubmit() {
	global $SERVICES, $PHP_SELF, $L, $output, $add, $table, $type;
	
	$item = $_POST['item'];
	if ($_POST['submit']) {
		//server-side data validation goes here
		$fields = getFormFields($item);	//get field definitions
		$output->validateFields($item, $fields, $error_msg); //validate $item based on the definitions in $fields; $item values will get updated
		if (is_array($error_msg)) {		//if server-side validation errors, return to form
			main( $item['id'], $item, $error_msg);
			exit();
		}
		
		$table = $item['table_name'];
		
		if (empty($table)) {
			doRedirect($SERVICES->config->lookup('modules.Admin.modulelink.module_editor_index.link'));
			exit();
		}
		
		//any other processing before redirecting to the next stage
		$module = $item['module'];
		$table = $item['table'];
		bcrm_unset($item, 'table');
		
		$edit_link = $SERVICES->config->lookup('modules.Admin.module_edit.dropdown_edit.link');
		
		if ($item['table_name']) {
			// Attempt to create a blank dropdown table called 'ModuleName__table_name_dropdown'
			if ($dropdown_table_name = $add->createDropdownTable($item['table_name'], $module)) {
				$link = $edit_link . '?type=' . $item['type'] . '&table=' . $dropdown_table_name;
				doRedirect($link);
				exit();
			}
		}
		
		// error - could not create table
		$content = $output->getBeginTable('viewTbl');
		$content .= $output->getTextRow($L['adminCustomFieldsErrorCannotSetCustomField']);
		$content .= $output->getTextRow('<a href="' . $edit_link . '?type=' . $item['type'] . '&table=' . $table . '">' . $L['Back'] . '</a>');
		$content = $output->getEndTable();
		quickPage($content);
		exit();
	}
	
} //doSubmit


function doRedirect($nextpage) {
	global $SERVICES, $table;
	// print '<p>nextpage = ' . $nextpage . '</p>';
	
	if ($nextpage) {
		header('Location: ' . $nextpage);
	} else {
		// header('Location: admin_module_editor_index.php?contact_id=' . $contact_id . '&module=' . $module . '&table=' . $table);
		header('Location: ' . $SERVICES->config->lookup('modules.Admin.modulelink.module_editor_index.link') . '?table=' . $table);
	}	
	exit();
}


function main($id = '', $item = '', $error_msg = '') {
	global $PHP_SELF, $SERVICES, $output, $L, $table, $module, $type, $add;
	
	if (!$item) {  //item could be passed back from server-side data validation failure		
		$item = array();
		$item['table'] = $table;
		// $module = substr($table, 0, strpos($table, '__'));
		// $add->printDebug('module = ' . $module);
		$item['module'] = (isset($module) && $module ? $module : substr($table, 0, strpos($table, '__')));
		$item['type'] = $type;	
	}
	
	$title_menu = array(
		$output->getLinkHref(
			array(
				'href' => $SERVICES->config->lookup('modules.Admin.modulelink.module_editor_index.link'), 
				'text' => $SERVICES->config->lookup('modules.Admin.modulelink.module_editor_index.label')
			)
		), 
		$output->getLinkHref(
			array(
				'href' => $SERVICES->config->lookup('modules.Admin.module_edit.table_select.link') . $output->updateQueryString(array('type'=>'', 'module'=>$item['module'])), 
				'text' => $SERVICES->config->lookup('modules.Admin.module_edit.table_select.label')
			)
		)
	);	
	
	$content = '';
	$content .= $output->getBeginTable('viewTbl');
	// $content .= $output->getTitleRow($L['adminModuleEditIndexLabel'], $title_menu);
	$content .= $output->getTextRow($L['adminCustomFieldsEnterDropdownTableName']);
	$content .= $output->getFieldSpacerRow();
	//nested data table ------------------
	$content .= $output->getBeginNestedEditTable();
	//form
	$content .= $output->getBeginForm($PHP_SELF);  
	
	foreach (getFormFields($item) as $k => $v) {
		$content .= $output->getHtmlFieldRow($v, $error_msg[$v['field_key']]);
	}
	
	$content .= $output->getFieldSpacerRow();
	$content .= $output->getSubmitRow();
	$content .= $output->getEndForm();
	//end form

	$content .= $output->getEndNestedTable();
	//end nested data table ------------------
	$content .= $output->getEndTable();
	//
	
	$output->toolbar_link_mode = true;
	$output->setActionLinksMenu((array)$title_menu);
	$output->toolbar_link_mode = false;
	
	// $add->printDebug($item);
	
	$title_text = $L['adminModuleEditIndexLabel'];
	$padding = true;
	
	$window_layout = $output->getBeginTable('mainTbl', true);
	$window_layout .= $output->getTitleRow($title_text);
	// $window_layout .= $output->getMenuRow($title_menu);
	$window_layout .= $output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>';
	$window_layout .= $content;
	$window_layout .= '</td>' . $output->getEndRow(true);
	$window_layout .= $output->getEndTable(true);
	
	$output->addToBody($window_layout);
	$output->display();
	exit();
} // main

if (isset($_POST['submit']) || isset($_GET['delete'])) {
	doSubmit();
}

main();
?>
