<?php
require_once('../../prepend.inc');
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 */
 
global $SERVICES;
//quickPage('This process is not yet available');
//exit();
 
/**
 * Administrators can purge deleted records for selected modules.
 * 
 * @package UserAdmin
 * @author Phil Calder
 */
class AdminPurgeDeleted {
	
	function AdminPurgeDeleted() {
		$this->init();
	}
	
	function init() {
		global $SERVICES, $L;
		$lang =& $SERVICES->getService('Language');
		$lang->getLanguage('Admin');
		
		$this->config =& $SERVICES->getService('Config');
		$this->modules =& $SERVICES->getService('Modules');
		$this->output =& $SERVICES->getService('Output');
	}
	
	function getModules() {
		$modules = array();
		$modules = $this->config->lookup('modules.Admin.hard_delete');	
		// print '<div>modules:</div>'; print_r($modules); print '<hr>';
		return $modules;
	}
 
	/**
	 * Runs a data cleaning method for certain modules
	 * 
	 * @param mixed $modules Default false, or can be an array of modules to process
	 * key is module name, value is boolean (o - dont cleanup, 1 - cleanup)
	 * @return boolean True if no errors reported, false if any error/problem arises
	 */
	function runModulePurgeDeleted($modules = false) {
		global $L;
		
		if (!is_object($this->config) || !is_object($this->modules)) {
			return false;
		}
		
		if (!is_array($modules)) {
			$modules = array();
		}
		
		// Submit type:
		// 0 - do nothing
		// 'count' - count only, displays counts
		// 'purge' - purge items only (basic reporting, success/failure)
		// 'report' - purge items (extended reporting, showing counts per module/table)
		// default value should be 'count'. Selected value will be based on a dropdown in the form.
		$submit_type = 'purge'; // 'report'; // 'purge';
		
		$function_name = 'purgeDeleted';
		
		$content = '';
		
		if ( is_array($modules) && $function_name) {
			// print '<div>function_name: ' . $function_name . ', modules:</div>'; print_r(array_keys($modules)); print '<hr>';
			foreach ($modules as $k => $v) {
				
				// if (!is_array($v)) { // Should always trigger here, when form is submitted $v will be '1'
					// $v = $this->getModuleTables($k);
				// }
				// print '<div>1--module: ' . $k . '--</div>';
				if ( is_object($mods[$k] =& $this->modules->getModule($k)) ) {
					// print '<div>2--module: ' . $k . '--</div>';
					if (method_exists($mods[$k], $function_name)) {
						// print '<div>mod: ' . $mods[$k]->getModuleName() . ' class: ' . get_class($mods[$k]) . ' calling ' . $function_name . '</div>';
						$retval = $mods[$k]->$function_name($submit_type);
						
						// We only look boolean values here, or an array of results
						if ($retval === false) {
							// print '<div>[boolean] ' . $L['adminPurgeDeletedModule'] . ': ' . $mods[$k]->getModuleName() . ' ' . $L['adminPurgeDeletedFailed'] . '</div>';
							$content .= '<div>' . $L['adminPurgeDeletedModule'] . ': ' . $mods[$k]->getModuleName() . ' ' . $L['adminPurgeDeletedFailed'] . '</div>';
						} else if (is_array($retval)) {
							// print '<div>[array] module: ' . $k . ', retval: ' . $retval . '</div>'; print_r($retval); print '<hr>';
							$content .= '<div>' . $L['adminPurgeDeletedModule'] . ': ' . $mods[$k]->getModuleName() . '<br>';
							switch ($submit_type) {
								case 'purge':
									foreach ($retval as $table => $value) {
										$content .= '' . $table . ' - ' . ($value ? $L['adminPurgeDeletedSuccessful'] : $L['adminPurgeDeletedFailed']) . '<br>';
									}
								break;
								case 'count':
								default:
									foreach ($retval as $table => $value) {
										$content .= '' . $table . ' - ' . $value . '<br>';
									}
								break;
							}
							$content .= '</div>';
						} else {
							// print '<div>[other] module: ' . $k . ', retval: ' . $retval . '</div>'; print_r($retval); print '<hr>';
						}
					}
				}
			}
		}
		return $content;
	}
	
	function doSubmit() {
		if (isset($_POST['submit']) && $_POST['submit']) {
			$this->cleanup_modules = false;
			if ( $_POST['item']['modules'] && is_array($_POST['item']['modules']) ) {
				$this->cleanup_modules = array();
				foreach ($_POST['item']['modules'] as $k => $v) {
					$this->cleanup_modules[$v] = 1;
				}
			}
			// print '<div>modules:</div>'; print_r($this->cleanup_modules); print '<hr>';
			$this->returned = $this->runModulePurgeDeleted($this->cleanup_modules); 
			// print '<hr>';
		} else {
			// print '<div>not submitting</div>';
		}
	}
	
	function main() {
		global $L;
		$content = '';
		
		if (isset($this->returned) && $this->returned) {
			$content .= '<p>' . $L['adminPurgeDeletedStatus'] . '</p>';
			$content .= $this->returned;	
		}
		
		if (is_array($modules = $this->getModules())) {
			$content .= '<p>' . $L['adminPurgeDeletedFormExplanation'] . '</p>';
			
			$content .= '
			<form method=post>
			<select multiple style="width: 240px" size="7" class="" name="item[modules][]" >
			';
			if (is_array($modules)) {
				foreach ($modules as $k => $v) {
					if ($k) {
						$content .= '<option value="' . $k . '">' . $k . '</option>';
					}
				}
			}
			$content .= '
			</select>
			<br>
			<input type="submit" name="submit" value="' . $L['ButtonSubmit'] . '">
			</form>
			';
		}
		
		// $this->output->addToBody($content);
		// $this->output->display();
		// exit();
		$window_layout = $this->output->getBeginTable('mainTbl', true);
		$window_layout .= $this->output->getTitleRow($L['adminModulePurgeDeletedLabel'], array('bookmark'=>'off'));
		$window_layout .= $this->output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>';
		$window_layout .= $content;
		$window_layout .= '</td>' . $this->output->getEndRow(true);
		$window_layout .= $this->output->getEndTable(true);
		
		$this->output->addToBody($window_layout);
		$this->output->display();
		exit();
	}
}

$admin_purg_deleted = new AdminPurgeDeleted();

if (is_object($admin_purg_deleted)) {
	$admin_purg_deleted->doSubmit();
	$admin_purg_deleted->main();
} else {
	exit('Error: cannot initiate this process');
}

?>