<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Calendar
 */

/**
 * Appointment edit class
 * @package Calendar
 */
class CalendarItemWebMATEdit extends WebMATEdit {
	/**
	 * @var $new_item_user
	 * @access private
	 */
    var $new_item_user = false;
	
	/**
	 * Status key
	 * @var $status_needs_action
	 * @access private
	 */
	var $status_needs_action = 'action';
	
	/**
	 * Status key
	 * @var $status_accepted
	 * @access private
	 */
	var $status_accepted = 'accepted';
	
	/**
	 * Role key
	 * @var $role_owner
	 * @access private
	 */
	var $role_owner = 'Owner';
	
	/**
	 * Role key
	 * @var $role_attendee
	 * @access private
	 */
	var $role_attendee = 'Attendee';
	
	/**
	 * Role key
	 * @var $role_attendee
	 * @access private
	 */
	var $role_no_need_to_attend = 'NoNeedToAttend';
	
	function beforeDBWrite(&$item, &$fields, &$error_msg) {
		global $USER, $SERVICES;
		
		// Bosco 20050622  | Bug Fix | Changed to give multi lingual support	
		if (isset($this->form_values['submit']) && $this->form_values['submit']) {
			if ($item['alert_in_advance'] == -1) {
				$item['alert_end'] = 0;
				$item['alert_on'] = 0;
			} else {
				$item['alert_on'] = 1;
			}
			$this->new_item_user = array(
				 'user_id' => $item['user_id'],
				 'item_id' => $item['id'], //this is set in afterDBWrite if form_values['id'] is empty
				 // 'status_id'=> '', //this is set in afterDBWrite 
				 // 'role_id' =>  '', //this is set in afterDBWrite
				 'alert_in_advance' => $item['alert_in_advance'],
				 'alert_end' => $item['alert_end'],
				 'alert_on' => $item['alert_on'],                                                                                 
				 'last_update_user_id' => 0,
				 'last_update'=> $item['last_update'] 
			);
			//get Calendar__item_user table fields
			$new_item = array(
				 'id' => $item['id'], //this is set in afterDBWrite if form_values['id'] is empty
				 'creator_id' => $item['creator_id'],
				 'contact_id' => $item['contact_id'],
				 'category_id' => $item['category_id'],
				 'type_id' => $item['type_id'],
				 'title' => $item['title'],
				 'content' => $item['content'],
				 'date_start' => $item['date_start'],
				 'duration' => $item['duration'],
				 'last_update_user_id' => $USER['id'],
				 'last_update'=> $item['last_update'] //this is applied if item['last_update'] is available
				 
			);
			// if ($this->form_values['id']) {
			// } else {
				// $new_item['date_start'] = $SERVICES->offsetTimeZoneForRead($item['date_start']);
			// }
			$item = $new_item;	
		}
	}
	 
	function afterDBWrite(&$item, &$fields, &$error_msg, $id = false) {
	    global $USER;
		// Bosco 20050622  | Bug Fix | Changed to give multi lingual support	
		if  (isset($this->form_values['submit']) && $this->form_values['submit']) {
			//$new_item_user = $this->new_item_user;
			$db_functions =& $this->mod->getDatabaseTable('item_user');
			if ($this->form_values['item']['id']) {
				$db_functions->setDesignTimeFilters(array('item_id' => $id));
				$item_users = $db_functions->getItems();
			
				$old_item_user = false;
				if ($item_users) {
					foreach ($item_users as $id => $item_user) {
						$old_item_user[$item_user['user_id']] = $item_user;
					}
				}
				//insert data to the item_user table
				$user_ids =$this->new_item_user['user_id'];
			} else {
				$this->new_item_user['item_id'] = $id;
			}
			
			//insert data to the new_item_user table
			$user_ids = $this->new_item_user['user_id'];
			
			foreach ($user_ids as $key => $value){
				$add_item = $this->new_item_user;
				
				if ($old_item_user[$value]) {	
					$add_item = $old_item_user[$value];
					$add_item['last_update_user_id'] = 0;
					$add_item['seen'] = 0;
					$add_item['alert_in_advance'] = $this->new_item_user['alert_in_advance'];
					$add_item['alert_end'] = $this->new_item_user['alert_end'];
					$add_item['alert_on'] = $this->new_item_user['alert_on'];
					bcrm_unset($old_item_user, $value);
				}
				
				$add_item['user_id'] = $value;
				
				//check if the user is the owner
				// Phil 20060525 Using class vars
				if ($USER['user_id'] == $value) {
					$add_item['role_id'] = $this->role_owner; // Owner
					$add_item['status_id'] = $this->status_accepted; // Accepted
					$add_item['seen'] = 1;					
				} else {
					$add_item['role_id'] = $this->role_attendee; // Attendee
					$add_item['status_id'] = $this->status_needs_action; // Needs Action
				}
				// print_r($add_item); print '<hr>';												  
				$result = $db_functions->setItem($add_item);
			}
			
			if ($old_item_user) {
				// Phil 20060525 Using class vars
				foreach ($old_item_user as $user_id => $item_user) {
					$item_user['role_id'] = $this->role_no_need_to_attend; //No Need To Attend
					$item_user['seen'] = 0;
					// Set some fields to Nothing if the appointment is no loger assigned to the user  		2005/05/25
					$item_user['status_id'] = -1;  // dummy value which represent empty; 2005/05/25
					$item_user['alert_in_advance'] = -2; 	
					$item_user['alert_end'] = -2;
					$item_user['alert_on'] = 0;  	
					$db_functions->setItem($item_user);
				}
			}
		}
	}
	
	function beforeForm(&$id, &$item, &$error_msg) {
		//any other processing before the Form is created (ie. adding extra items)
		global $SERVICES,$USER;
		if ($this->form_values['id']) {
			//edits an appointment
			if ($item) {
				$db_functions =& $this->mod->getDatabaseTable('item_user');
				$db_functions->setDesignTimeFilters(array('item_id' => $item['id']));
				$item_users = $db_functions->getItems();
				if ($item_users) {
					foreach ($item_users as $id => $item_user) {
						// if ($item_user['role_id'] != 3) {    //if role is not "No Need To Attend" 
						// Phil 20060525 Using class vars
						if ($item_user['role_id'] != $this->role_no_need_to_attend) {
							$item['user_id'][] = $item_user['user_id'];
						}
					}
				}
			}	
		} else {
			//creates an appointment
			if ($this->form_values['date']) {
				$date_start_pieces = explode('-' , $this->form_values['date']);
				$date_start = $SERVICES->offsetTimeZoneForWrite(mktime($date_start_pieces[3],$date_start_pieces[4],0,$date_start_pieces[1],$date_start_pieces[2],$date_start_pieces[0]));
				$item['date_start'] = $date_start;
			} else {
				$item['date_start'] = time(); 
				//echo date('Y-m-d, H:i', time());//$SERVICES->offsetTimeZoneForRead(time())
			}
			
			if ($this->form_values['user_id']) {
				$item['user_id'] = $this->form_values['user_id'];
			} else {
				$item['user_id'] = $USER['id'];
			}
			// Phil 20060914 Added lookups for user prefs for new items
			global $SERVICES;
			$prefs =& $SERVICES->getService('Preferences');
			$item['alert_in_advance'] = $prefs->getUserPref('calendarDefaultAlertAdvance');
			$item['alert_end'] = $prefs->getUserPref('calendarDefaultAlertEnd');
		}
	}
}
?>