<?php
/** 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 */

/**
 * This class handles the database functions for the item table
 * 
 * @package Company
 * @author Phil Calder
 */
class itemCompanyDatabaseTable extends WebMATDatabaseTable {
	
	/**
	 * The constructor class for itemCompanyDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function itemCompanyDatabaseTable($module_name, $data_definition) {
		global $USER;
		$this->table_name = 'item';
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseTable
	}
	
	/**
	 * Sets custom data definition default values for certain fields, then calls parent function
	 */
	function setDataDefinition($data_definition){
		global $SERVICES;
		$SERVICES->getService('Preferences');
		
		if (is_array($data_definition['fields'])) { // Phil 20051107 - it should be, but better to check
			foreach($data_definition['fields'] as $k => $v){
				//TODO - Form values
				switch ($v['field_key']){
					case 'relationship_id':
						$data_definition['fields'][$k]['default_value'] = $SERVICES->preferences->getUserPref('companyRelationship');
					break;
					case 'industry_id':
						$data_definition['fields'][$k]['default_value'] = $SERVICES->preferences->getUserPref('companyIndustry');
					break;				
				}
			}
		}
		parent::setDataDefinition($data_definition);
	}
	
		
	/**
	 * Reassigns items to a different contact
	 * 
	 * Handled differently for this module. Instead of reassigning this method 
	 * removes the old record.
	 * 
	 * @param integer $contact_id the contact id you want to replace
	 * @param integer $new_contact_id the new contact id
	 */
	function reassignItemForContact($contact_id, $new_contact_id) {
		if ( is_array($this->columns) && in_array($this->getContactIdField(), $this->columns) ) {
			// $this->db->update($this->getTableName(), array($this->getContactIdField() => $new_contact_id), $this->getContactIdField() . '=' . $contact_id);
			// print '<div>reassignItemForContact REMOVING record for ' . $contact_id . '</div>';
			$this->db->update(
				$this->getTableName(), 
				array(
					'active' => '0', 
					'primary_record' => '0'
				), 
				"" . $this->getContactIdField() . "= '$contact_id'"
			);
		}
	}
	
	/**
	 * Checks user preference - allow contact delete, then calls parent
	 * @author Phil Calder
	 * @return boolean true is deletable, otherwise false
	 */
	function isDeletable( $id, $item = false ) {
		global $SERVICES;
		$SERVICES->getService('Preferences');
		if (! ($allow_delete = $SERVICES->preferences->getUserPref('crmAllowContactDelete'))) {
			return false;
		}
		
		return parent::isDeletable($id, $item);
	}
	
}
?>