<?php
require_once("../../prepend.inc");
/**
 * 
 * @license http://www.browsercrm.com/licence.php BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 */

includeWebMAT('view.inc');
includeWebMAT('contact_view.inc');

/**
 * Display a Contact View for the given contact.
 * 
 * @author Phil Calder
 * @package Contact
 */
class ContactViewManager extends WebMATView {
	
	/**
	 * @access private
	 */
	var $layout;
	var $container_name = 'contact-view-container';

	
	// var $content = '';
	// var $contact_piece_filename = '';
	// var $new_column_placeholder = '__placeholder__';
	
	/**
	 * Class Constructor function
	 *
	 * Calls init
	 * @see init()
	 * 
	 * @param  string $module  the current module name
	 * @param  string $data_option  the current data option used for DataDictionary
	 * @param  array $post the post array
	 * @param  array $get the get array
	 * @param  string $table_name the name of the table the page is using
	 * @param  array $language_overwrite has the keys title and description
	 */
	function ContactViewManager($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		$this->init($module, $data_option, $post, $get, $table_name, $language_overwrite);
	}
	
	/**
	 * Initialises the services and layout manager for the contact view.
	 * Handles duplicate check indexing.
	 * Handles contact permission redirecting.
	 */
	function init($module = false, $data_option = false, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		global $SERVICES, $L;
		// -- 
		$id = false;
		if (isset($this->form_values) && isset($this->form_values['id']) && $this->form_values['id']) {
			// We do need to set $_GET['contact_id']
			$id = $_GET['contact_id'] = $this->form_values['contact_id'] = $this->form_values['id'];
		} else if (isset($_GET) && isset($_GET['id'])) { 
			$id = $_GET['contact_id'] = $_GET['id'];
		}
		if (!$id) {
			header('Location: ../../modules/Contact/index.php'); // TODO: use a config lookup here
			exit();
		}
		
		$modules =& $SERVICES->getService('Modules');
		$this->mod =& $modules->getModule('Contact');
		// Yes, lang keys are for 'edit' (not view)
		if ($this->mod->isCompany($id)) {
			$language_overwrite['title'] = 'contactCompanyTitle_itemedit';
		} else {
			$language_overwrite['title'] = 'contactPersonTitle_itemedit';
		}
		
		// --
		
		
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
		
		
		
		$this->ua =& $SERVICES->getService('Authentication');
		$this->output =& $SERVICES->getService('Output');
		$SERVICES->getService('Preferences');
		$SERVICES->getService('Config');
		$SERVICES->getService('Language');
		$SERVICES->language->getLanguage('Contact');
		$SERVICES->getService('DateLib');
		$SERVICES->getService('Modules');
		$contact =& $SERVICES->modules->getModule('Contact');
		
		if (!is_object($this->db_functions)) {
			$this->db_functions =& $contact->getDatabaseTable('item');
		}
		
		// --
		// $_GET['dropdown_config'] = 'contact_view_setup';
		includeModule('Contact', 'inc/contact_view_layout.inc');
		$get = array();
		$get['dropdown_config'] = 'contact_view_setup';
		// $this->layout = new ContactViewWebMATAdminDropdown($post, $get, '', 'read');
		$this->layout = new ContactViewWebMATAdminDropdown($this->form_values, $get, '', 'read');
		
		// --
		
		if (isset($this->form_values['duplicate_check'])) {
			// print '<div>' . get_class($this) . '::init - id: ' . $id . ', duplicate_check: ' . ($this->form_values['duplicate_check'] ? 'yes' : 'no') . '</div>'; 
			// If we want to duplicate check this contact, 
			// we update it so that it has not been checked. That triggers the check later on.
			if ($this->form_values['duplicate_check']) {
				$contact->setIsDuplicateChecked($id, 0);
			} else {
				$contact->setIsDuplicateChecked($id, 1);
			}
		}
		
		// print '<p>' . get_class($this) . '::init finished</div>';	
	}
	
	/**
	 * Extended to return new company and new person links
	 *
	 * @author Phil Calder
	 * @todo Extend to include file set specific new links for adding an item and a Contact on one form
	 */
	function getNewLinks() {
		$contact_company = false;
		if (isset($this->form_values['id']) && $this->form_values['id']) {
			if ($this->mod->isCompany($this->form_values['id'])) {
				$contact_company = $this->form_values['id'];
			}
		}
		return array(
			'newCompany'   => $this->mod->getNewCompanyLink(), 
			'newPerson' => $this->mod->getNewPersonLink(false, $contact_company)
		);
	}
	
	/**
	 * The contact view is a non-standard page. This method builds the display for the page.
	 *
	 * Displays content and exits the application.
	 */
	function main() {
		$this->checkForBrowse();
		// $this->printDebug(get_class($this) . '::main - begin');
		$this->item = $this->getItem($this->form_values['id']);
		
		// Phil 20070618 required to display main contact piece
		$this->output->req_contact_js = true;
		$incl_css = '<link rel="stylesheet" type="text/css"  href="' . $this->output->getSkinWebPath() . 'css/contact.css">';
		$this->output->addToHead($incl_css);
		
		// Build a content array
		$content = array();
		// LastViewed
		//MCR 20070318
		//$content['lastViewed']          = $this->getPastLinksTable();
		// ---------------
		$content['beginMain']           = $this->getBeginMainContent();
		// Title (and links)
		$content['title']               = $this->getTitleContent();
		// Description (optional)
		$content['description']         = $this->getDescriptionContent();
		// Browse / paging
		$content['browsePaging_top']    = $this->getBrowsePagingContent();
		// ActionLinks
		$content['actionLinks']         = $this->getActionLinksContent();
		// ----
		$content['endMain']             = $this->getEndMainContent();

		//$content['beginDataCell']       = $this->getBeginDataCell();
		
		//new flag for container in webmat_item
	// 	$this->container_box_not_viewTbl = true;
		
		$this->container_not_table = true;
		// Main Content		
		$content['beginContainer']      = $this->getBeginContainer();
		$content['data']                = $this->getDataContent($item, $error_msg);
		$content['endContainer']      	= $this->getEndContainer();

		
		
		// Build contact item pieces
		$content['beginContainer2']     = '<div class="contact-view-container-bottom">';
		$content['pieces']              = $this->getContactData(); // . '<div>';
		$content['endContainer2']     	= '</div>';
		
		//$content['endDataCell']         = $this->getEndDataCell();
		
		// ----
		// Browse / paging
		// $content['browsePaging_bottom'] = $content['browsePaging_top']; // $this->getBrowsePagingContent();
		// $content['browsePaging_bottom']    = $this->getBrowsePagingContent();  // removed version 5
		// ---------------
		//$content['endMain']             = $this->getEndMainContent();
		
		// $this->printDebug(get_class($this) . '::main - end (calling display)');
		return $this->display($content) . '<hr>';
	}
	
	
	/**
	 * For now, only the standard display is enabled
	 *
	 * @return boolean true
	 */
	function isStandardDisplay() {	
		return true;
	}
	
	/**
	 * Returns the relvent contact data
	 *
	 * @see getContactRelatedPieces
	 * @see getContactEventHistory
	 */
	function getContactData() {
		// $this->printDebug(get_class($this) . '::getContactData');
		if (isset($this->form_values['event_history']) && $this->form_values['event_history']) {
			return $this->getContactEventHistory();
		} else if (isset($this->form_values['event_list']) && $this->form_values['event_list']) {
			return $this->getContactEventList();
		}
		// Default: the standard contact view pieces
		return $this->getContactRelatedPieces();
	}

	/**
	 * @return string The combined output of all contact related pieces the user is 'subscribed' to
	 */
	function getContactRelatedPieces() {
		// $this->printDebug(get_class($this) . '::getContactRelatedPieces - begin');
		$contact_view = array();
		$contact_view['config_pieces'] = $this->layout->getConfigPieces();
		// print '<div>pieces:</div>'; print_r($contact_view['config_pieces']); print '<hr>';
		if (!is_array($contact_view['config_pieces'])) {
			$contact_view['config_pieces'] = array();
		}
			
		$contact_view['items'] = $this->layout->getItems();
		// print '<div>items:</div>'; print_r($contact_view['items']); print '<hr>';
		
		if (!is_array($contact_view['items'])) {
			$contact_view['items'] = array();
		}
		// print '<div>$contact_view[items]</div>'; print_r($contact_view['items']); print '<hr>';

		$content = $content_other = ''; // build the contact view content	
		
		$standard_display = $this->isStandardDisplay();
		
		$tabs_content = '';
		$tabs = array();
		$first_tab = false;
		$active_tab = false;
		
		foreach ($contact_view['items'] as $k => $v) {
			// print '<div>item: ' . $k . '</div>'; print_r($v); print '<hr>';
			$do_count_items = (is_array($v) ? $v['count_items'] : 1);
			$do_count_alerts = (is_array($v) ? $v['count_alerts'] : 1);
			$do_show_items = (is_array($v) ? $v['show_items'] : 20);
			// print '<div>do_show_items: ' . $do_show_items . '</div>';
			
			$item_name = (is_array($v) ? $v['name'] : $v);
			if ($piece = $contact_view['config_pieces'][$item_name]) {
				
				$contact_piece_object = $this->getContactPieceObject($piece);
				
				// print '<div>***** ' . get_class($contact_piece_object) . '</div>';
				// $this->printDebug(get_class($this) . '::getContactData - instanciated class: ' . get_class($contact_piece_object));
					
				if (is_object($contact_piece_object)) {
					
					if (method_exists($contact_piece_object, 'setOptions')) {
						$contact_piece_object->setOptions($do_count_items, $do_count_alerts, $do_show_items);
					}
					
					$piece_identifier = $contact_piece_object->getPieceId();
					// print '<div>piece identifier: ' . $piece_identifier . ' class: ' . get_class($contact_piece_object) . '</div>';
					
					$title_text = $contact_piece_object->getTitleText();
					
					// $this->printDebug(get_class($this) . '::getContactData - count items...');
					$num_mod_found = 0;
					if ($standard_display) {
						if ($do_count_items) {
							// print '<div>do_count_items class: ' . get_class($contact_piece_object) . '</div>';
					
							if (method_exists($contact_piece_object, 'countItems')) {
								$num_mod_found = $contact_piece_object->countItems(); // Phil 20060322
							} else {
								$num_mod_found = $contact_piece_object->db_functions->countItems(); // Phil 20050709
							}
						}
					}
					
					// $this->printDebug(get_class($this) . '::getContactData - item count: ' . $num_mod_found);
					
					// $this->printDebug(get_class($this) . '::getContactData - has alerts...');
					$alert_text = '';
					$has_alerts = false;
					if ($do_count_alerts) {
						if (method_exists($contact_piece_object, 'countAlerts')) {
							$has_alerts = $contact_piece_object->countAlerts();
							// Handle in Output
							// $alert_text = ($has_alerts ? ' ' . $this->output->getFlagAlert($has_alerts) : ''); 
						}
					}
					// $this->printDebug(get_class($this) . '::getContactData - has alerts: ' . (strlen($alert_text) ? 'yes' : 'no'));
					
					//simply 1st time through loop
					if (!$first_tab) {
						$first_tab = 'tab' . $piece_identifier;
					}
					
					//first tab with items
					if ((!$active_tab) && $num_mod_found) {
						//$set_first_js = '<script type="text/javascript">alert(\'' . $piece_identifier . '\'); chngActv(\'' . $piece_identifier . '\');</script>';
						$active_tab = 'tab' . $piece_identifier;
					}
						
						
					if (!$standard_display) {
						// $contact_piece_content = '<div>Foo</div>'; //$contact_piece_object->main(true);
						$contact_piece_object->item_count = false;
						$contact_piece_content = $contact_piece_object->main(true);
						// $contact_piece_content = $contact_piece_object->getExpandableHeader(true);
					} else {
						// print '<div>' . get_class($contact_piece_object) . ' calling main</div>';
						$contact_piece_content = $contact_piece_object->main(true);
					}
					
					// Phil 20070427 Dont display any piece that has no content. 
					// Certain pieces apply only to Companies or People.
					// Note: may change this at a later date (ie call a method to determine whether to show the piece) 
					// when we implement dynamically loading pieces (AJAX)
					if (strlen($contact_piece_content)) {
						$tabs[] = array(
							'id' => 'tab' . $piece_identifier, 
							'title' => $title_text . (($num_mod_found > 0) ? ' ('. $num_mod_found . ')' : ''), 
							// 'disabled' => 0, 
							'is_alert' => ((int)$has_alerts ? 1 : 0) 
						);
					
						//$tabs_content .= $current_tab;
						//$firstTab = false;
						
						// Phil - when in print view mode, display a title for each tab
						if ($this->output->isPrintView()) {
							$tabs_content .= '
						<div id="tabTitle' . $piece_identifier . '" class="tab-title">' . $title_text . '</div>
						';
						}
						
						// Phil - The 'standard' contact view - pieces displayed one after the other
						$tabs_content .= '
						<div id="tab' . $piece_identifier . '" class="tab-content">
						';
						$tabs_content .= $contact_piece_content;
						$tabs_content .= '
						</div>
						';
					}
					// $this->printDebug(get_class($this) . '::getContactData - calling objects main method... done');
				}
				// $this->printDebug(get_class($this) . '::getContactData - end item for module: ' . $piece['module']);
			}
		}	
		
		$tabs_content = '
		<div id="contactTabs">
		' . $tabs_content . '
		</div>
		';
		
		//$this->output->initExt();
		$this->output->initTabs('contactTabs');
		foreach ($tabs as $tab) {
			$this->output->addTab($tab, 'contactTabs');
		}
		$this->output->setFirstTab($active_tab ? $active_tab : $first_tab, 'contactTabs');
		
		
		// tabs row
		$tabs_content = 
						//$this->output->getBeginRow()
						//. $this->output->getBeginCell(3, 'tabCell', 'th')
						$tabs_content
						//. $this->output->getEndCell('th')
						//. $this->output->getEndRow()
						;
		
		$content = 
		//$this->output->getBeginTable('contactPiecesTbl') . 
		$tabs_content . 
		//$this->output->getEndTable();
		$content .= $set_first_js; 
		
		// $this->printDebug(get_class($this) . '::getContactRelatedPieces - returning content');
		return $content;
	}
	
	function getContactPieceObject($piece) {
		$class = 'WebMATContactView';
		if ($piece['class']) {
			$class = $piece['class'];
		}
		$module = $piece['module'];
		$args = array();
		if (is_array($piece['args'])) {
			$args = $piece['args'];
		}
		$contact_piece_object = getWebMATObject( 
			$class, 
			$module, 
			$args
		);
		return $contact_piece_object;
	}
	
	/**
	 * @return string The combined output of all contact related pieces the user is 'subscribed' to
	 */
	function getContactEventHistory() {
		return $this->getContactEventData('history');
	}
	
	/**
	 * @return string The combined output of all contact related pieces the user is 'subscribed' to
	 */
	function getContactEventList() {
		return $this->getContactEventData('list');
	}
	
	/**
	 * @return string The combined output of all contact related pieces the user is 'subscribed' to
	 */
	function getContactEventData($type = 'history') {
		// $this->printDebug(get_class($this) . '::getContactEventData(' . $type . ') - begin');
		global $L;
		if ($type == 'history') {
			$config_key = 'contacteventhistoryconfig';
			$pref_name = 'event_history_search_values';
			$label_no_items = $L['contactEventHistoryNoItems'];
			$label_no_items_in_range = $L['contactEventHistoryNoItemsInRange'];
			$label_not_available = $L['contactEventHistoryNotAvailable'];
			$default_class = 'WebMATContactViewEventList';
			$label_modified = $L['contactEventLabelModified'];
		} else {
			$config_key = 'contacteventlistconfig';
			$pref_name = 'event_list_search_values';
			$label_no_items = $L['contactEventListNoItems'];
			$label_no_items_in_range = $L['contactEventListNoItemsInRange'];
			$label_not_available = $L['contactEventListNotAvailable'];
			$default_class = 'WebMATContactViewEventList';
			$label_modified = $L['Date'];
		}
		global $SERVICES, $L;
		$simple_layout = false;
		$items = $SERVICES->config->lookup('modules.Front.' . $config_key);
		// $this->printDebug(get_class($this) . '::getContactEventData config items: ' . count($items) . ' - begin');
		if (is_array($items) && count($items)) {
			$custom_search_content = $this->getCustomSearchContent($pref_name); // Load this BEFORE getting WebMAT objects
			
			$removed_v4 = array('Accounts', 'AddressEmail', 'AddressFax', 'AddressPhone', 'AddressPhysical', 'AddressPostal', 'AddressURL');
			
			// Get data
			$content = array();
			foreach ($items as $k => $piece) {
				// print '<p>================== item ' . $k . ' [' . $piece['class'] . '] ==================</p>';
				if ( 
					!is_array($piece) 
					|| !isset($piece['module']) 
					|| (!$this->ua->readAccess($USER['user_id'], $piece['module'])) 
					|| (in_array($piece['module'], $removed_v4))
				) {
					continue;
				}
				// $this->printDebug(get_class($this) . '::getContactEventData: item for module: ' . $piece['module'] . ' starting');
				$class = $default_class;
				if ($piece['class']) {
					$class = $piece['class'];
				}
				$module = $piece['module'];
				$args = array();
				if (is_array($piece['args'])) {
					$args = $piece['args'];
				}
				$args['get']['event_type'] = $type;
				$args['get']['contact_id'] = $this->form_values['id'];
				
				$contact_piece_object = getWebMATObject( 
					$class, 
					$module, 
					$args
				);
				// print '<p>class: ' . get_class($contact_piece_object) . '</p>';
				if (is_object($contact_piece_object)) {
					// $contact_piece_object->setEventType($type); // Using $args['get']['event_type'] instead
					// Phil Only display row if there is content for this piece
					if (is_array($contact_piece_content = $contact_piece_object->main(true))) {
						// $this->printDebug(get_class($this) . '::getContactEventData: item for module: ' . $piece['module'] . ', count: ' . count($contact_piece_content));
						$content = array_merge($content, $contact_piece_content);
					}
					unset($contact_piece_object);
				}
				// $this->printDebug(get_class($this) . '::getContactEventData: item for module: ' . $piece['module'] . ' finished');
			}
			
			// $this->printDebug(get_class($this) . '::getContactEventData: begin sorting content');
			// Sort
			$date = array();
			// $ids = array();
			// Obtain a list of columns
			foreach ($content as $key => $row) {
			   $date[$key]  = $row['_date'];
			   // $ids[$key] = $row['id'];
			}
			// Sort the data with date descending [, id ascending]
			array_multisort($date, SORT_DESC, SORT_NUMERIC, $content);
			// $this->printDebug(get_class($this) . '::getContactEventData: finished sorting content');
		
			
			// print_r(array_keys($content)); print '<hr>';
			$item_count = count($content); // Used for 'custom' paging
			$limit = $this->output->setPagingLimits('frontDefaultListSize', $item_count, false);
			// print_r($limit); print '<hr>';
			$content = array_slice($content, $limit['start'], $limit['count']);
			$paging_count = count($content); // New item count
			// $this->printDebug(get_class($this) . '::getContactEventData: item_count: ' . $item_count . ', paging_count: ' . $paging_count);
			// $this->printDebug(array_keys($content));
			// Build display content
			if (is_array($content) && $paging_count) {
				$datelib =& $SERVICES->getService('DateLib');
				$prefs =& $SERVICES->getService('Preferences');
				$company_depth_pref = $prefs->getUserPref('contactShowCompanyDepth', '0');
				
				$display_content = '';
				$display_content .= $custom_search_content;
				$display_content .= $this->output->getBeginTable('listTbl');
				$headers = array();
				
				$headers['module']   = array('label' => $L['contactEventLabelModule']);
				if ($company_depth_pref) {
					$headers['contact']  = array('label' => $L['contactEventLabelContact']);
				}
				$headers['modified'] = array('label' => $label_modified);
				$headers['label']    = array('label' => $L['contactEventLabelLabel']);
				$headers['detail']   = array('label' => $L['contactEventLabelDetail']);
				$display_content .= $this->output->getHeaderRow($headers, $sort);
				// $display_content .= '<tbody id="event_history">';
				$display_content .= $this->output->startDataBlock();
				// $datelib->getUserPrefDateFromTimestamp($date_value, 'Y/m/d', '-');
				$row_counter = 0;
				// $this->printDebug('getContactEventHistory.. looping through content'); //TODO: remove
				foreach ($content as $k => $row) {
					$row_counter++;
					$table_row = array();
					
					$date_value = $row['_date'];
					if ($date_value) {
						$date_value = $SERVICES->offsetTimeZoneForRead($date_value);
						$date_value = $datelib->getUserPrefDateFromTimestamp($date_value); // , 'Y/m/d', '-');
					} else {
						$date_value = '';
					}
					$table_row[] = $row['_module'];
					if ($company_depth_pref) {
						if ($row['_contact_id']) {
							$contact_link = $this->output->getLinkContact(
								array(
									'id'=> $row['_contact_id'], 
									'display_name' => $row['_contact_id_display_name']
								)
							);
						} else {
							$contact_link = '&nbsp;';
						}
						$table_row[] = $contact_link;
					}
					$table_row[] = $date_value;
					$table_row[] = $row['_label'];
					$table_row[] = $row['_detail'];
					
					$display_content .= $this->output->getTableRow($table_row, array(), array(), $row_counter);
				}
				// $this->printDebug('getContactEventHistory... finished looping through content'); //TODO: remove
				$display_content .= $this->output->endDataBlock();
				$display_content .= $this->output->getEndTable();
				
			} else {
				$range = $this->getUsersSearchDateRange($pref_name);
				if (is_array($range) && ( (isset($range['after']) && $range['after']) || (isset($range['before']) && $range['before']) ) ) {
					$label = $label_no_items_in_range;
				} else {
					$label = $label_no_items;
				}
				$simple_layout = true;
				$display_content = $custom_search_content;
				// $display_content .= '<br /><br />';
				
				$display_content .= $this->output->getBeginTable('listTbl');
				$display_content .= $this->output->startDataBlock();
				$display_content .= $this->output->getTextRow($label);
				$display_content .= $this->output->endDataBlock();
				$display_content .= $this->output->getEndTable();
			}
		
		} else {
			$simple_layout = true;
			$display_content = $label_not_available;
		}
		
		if ($simple_layout) {
			return $display_content;
		}
		
		if ($item_count) {
			 $paging_row = $this->output->getPagingRow('itemPagingCell');
		} else {
			$paging_row = '';
		}
		
		// 'contactPiecesTbl'
		$use_border = true;
		$title_text = $L['contactEventHistoryLabel'];
		$layout = '' . 
		$this->output->getBeginTable('mainTbl', $use_border) . 
		$this->output->getTitleRow($title_text, false) . 
		$paging_row . 
		$this->output->getBeginRow('', $use_border) . 
		$this->output->getBeginCell(2,'contactPiecesTbl') . 
		$display_content . 
		$this->output->getEndCell() . 
		$this->output->getEndRow($use_border) . 
		$paging_row . 
		$this->output->getEndTable($use_border);
		
		// $this->printDebug(get_class($this) . '::getContactEventData: returning layout content');
		return $layout;
	}
	
	function getDefaultOffsetEventDateAfter() {
		global $SERVICES;
		$pref_value = (int)$SERVICES->preferences->getUserPref('contactEventDefaultDateRangeAfter');
		if (!$pref_value) {
			$pref_value = 1; // ie 1 week	
		}
		$date = time() - ($pref_value*7*24*60*60); // x weeks ago
		$date = $SERVICES->offsetTimezoneForRead($date);	
		return $date;
	}
	
	function getUsersSearchDateRange($pref_name) {
		global $USER;
		$after = $before = $values = false;
		if (isset($USER['contact_event'][$pref_name]['after'])) {
			$after = $USER['contact_event'][$pref_name]['after'];
		}
		if (isset($USER['contact_event'][$pref_name]['before'])) {
			$before = $USER['contact_event'][$pref_name]['before'];
		}
		if ($before || $after) {
			$values = array('after' => $after, 'before' => $before);
		}
		return $values;
	}
	
	function setUsersSearchDateRange($pref_name, $values) {
		global $USER;
		$after = $before = false;
		if (isset($values['after'])) {
			$USER['contact_event'][$pref_name]['after'] = $values['after'];
		} else {
			$USER['contact_event'][$pref_name]['after'] = false;
		}
		if (isset($values['before'])) {
			$USER['contact_event'][$pref_name]['before'] = $values['before'];
		} else {
			$USER['contact_event'][$pref_name]['before'] = false;
		}
	}
	
	function getCustomSearchContent($pref_name = 'event_history_search_values') {
		global $SERVICES, $L;
		// print '<div>getCustomSearchContent ' . $pref_name . ' form_values[item]:</div>'; print_r($this->form_values['item']); print '<hr>';
		if (!$this->form_values['item']['before'] && !$this->form_values['item']['after'] && !isset($this->form_values['submit'])) {
			
			$values = $this->getUsersSearchDateRange($pref_name);
			
			if (is_array($values)) {
				$this->form_values['item']['after'] = $values['after'];
				$this->form_values['item']['before'] = $values['before'];
			}
			// print '<div>got values for pref ' . $pref_name . ':</div>'; print_r($values); print '<hr>';
			// Set a default search (items from up to 1 week ago)
			if (!is_array($values) && !$this->form_values['item']['after']) {
				$date = $this->getDefaultOffsetEventDateAfter();
				$this->form_values['item']['after'] = $date; 
				// print '<div>setting default values for pref ' . $pref_name . ':</div>'; print_r(array('after'=>$date)); print '<hr>';
				$this->setUsersSearchDateRange($pref_name, array('after'=>$date));
			}
		} else {
			// Values can be empty
			if (isset($this->form_values['item']['before']) || isset($this->form_values['item']['after'])) {
				// print '<div>' . get_class($this) . ' form_values[item] are set: after: ' . $this->form_values['item']['after'] . ', before: ' . $this->form_values['item']['before'] . '</div>';
				$new_pref = array(
					'after' => $this->form_values['item']['after'],
					'before' => $this->form_values['item']['before']
				);
				if (!is_numeric($new_pref['before'])) {
					$this->form_values['item']['before'] = $new_pref['before'] = $SERVICES->datelib->getTimestampFromUserPrefDate($new_pref['before']);
				}
				if (!is_numeric($new_pref['after'])) {
					$this->form_values['item']['after'] = $new_pref['after'] = $SERVICES->datelib->getTimestampFromUserPrefDate($new_pref['after']);
				}
				// print '<div>setting existing values for pref ' . $pref_name . ':</div>'; print_r($new_pref); print '<hr>';
				$this->setUsersSearchDateRange($pref_name, $new_pref);
			}
		}
		$after_field =array ( 
			'field_key' => 'after', 
			'label' => $L['LabelAfter'],  
			'type' => 'date', 
			'no_options' => true, // Dont show options dropdown
			'value' => ($this->form_values['item']['after'] ? $SERVICES->datelib->getMySqlDateFromTimestamp($this->form_values['item']['after']) : '')
		);
		$before_field =array ( 
			'field_key' => 'before', 
			'label' => $L['LabelBefore'],  
			'type' => 'date', 
			'no_options' => true, // Dont show options dropdown
			'value' => ($this->form_values['item']['before'] ? $SERVICES->datelib->getMySqlDateFromTimestamp($this->form_values['item']['before']) : '')
		);
		
		$search_filter_after = $this->output->getSearchFilterCell($after_field['field_key'], $after_field, 'after');
		$search_filter_before = $this->output->getSearchFilterCell($before_field['field_key'], $before_field, 'before');
		
		$custom_search_filter = array(
			$L['LabelAfter'],
			$search_filter_after, 
			$L['LabelBefore'],
			$search_filter_before, 
			$this->output->getHtmlSubmit($L['ButtonSubmit']), 
			$this->output->getHtmlSubmit($L['labelSearchFilterReset'], 'reset', '', 'reset')
		);
		$query = $new_query_asc = $this->output->updateQueryString(array('page_num'=>''));
		
		$content = '';
		$content .= $this->output->getBeginForm($SERVICES->getGlobal('php_self') . $query);
		// optional: could add ' width="100%" ' below
		$content .= '<table border="0" class="lFilterRow">';
		// $content .= '<tbody id="filterConstr">';
		$content .= $this->output->getTableRow($custom_search_filter, 'lFilterRow');
		// $content .= '</tbody>';
		$content .= $this->output->getEndTable(); 
		$content .= $this->output->getEndForm(); 
			
		return $content;
	}
	
	/**
	 * Sets the data option based on contact type, then excludes fields that dont apply to this contact type
	 */
	function setDataDefinition($data_option) {
		// print '<div>' . get_class($this) . '::setDataDefinition, data_option= ' . $data_option . '</div>';
		$this->is_person = $this->mod->isPerson($this->form_values['id']);
		$this->is_company = $this->mod->isCompany($this->form_values['id']);
		$this->is_account = $this->mod->isAccount($this->form_values['id']);
		
		if ($this->is_company) {
			$data_option = 'company_view';
		} else if ($this->is_person) {
			$data_option = 'person_view';
		}
		// print '<div>' . get_class($this) . '::setDataDefinition, id: ' . $this->form_values['id'] . ' data_option changed to: ' . $data_option . '</div>';
		
		parent::setDataDefinition($data_option);
		
		// --
		// Phil 200704 Should not really be necessary on a view page (no id)
		if (!$this->form_values['id']) {
			if (isset($this->form_values['contact_type'])) {
				$this->is_company = ($this->form_values['contact_type'] ? '1' : '0');
			} else {
				global $SERVICES;
				$SERVICES->getService('Preferences');
				$this->is_company = $SERVICES->preferences->getUserPref('contactNewContactType', '0');
			}
		}
		// --
		
		$exclude_fields = $exclude_contact_fields = $exclude_accounts_fields = $exclude_company_fields = array();
		// print_r($exclude_custom_table_fields); print '<hr>';
		$exclude_contact_fields[] = 'display_name'; // Will either use Company or Person name
		
		if ($this->is_company) {
			$exclude_person_fields = $this->dd->getFieldsBeginningWith('Person__item'); // ie Person__item.* and Person__item_custom.*
			$exclude_fields = bcrm_array_merge($exclude_contact_fields, $exclude_person_fields);
		} else if ($this->is_person) {
			$exclude_company_fields = $this->dd->getFieldsBeginningWith('Company__item'); // ie Company__item.* and Company__item_custom.*
			$exclude_accounts_fields[] = 'contact_account'; // Not used (20051027) but might be in future
			$exclude_fields = bcrm_array_merge($exclude_contact_fields, $exclude_accounts_fields, $exclude_company_fields);
		}
		$this->dd->setDictionaryExcludeFields($exclude_fields);
	}
	
	/**
	 * Gets content for the main contact display piece
	 *
	 * @param array $item the item array
	 * @param array $error_msg the error message array
	 * @return mixed string or array content
	 * @author Phil Calder
	 */
	function getFieldRows($item) {
		
		// sizes all in css
		
		// Designed to fit screen width 1024
		//$lhs_tbl_width = 300;
		//$rhs_tbl_width = 650;
		
		// Phil -
		// Either 
		// build a 3 column contact section (no tabs)
		// or
		// 1 column to display main contact data, and a (2 column) tabset
		
		global $SERVICES;
		$config =& $SERVICES->getService('Config');
		$lang =& $SERVICES->getService('Language');
		
		if (is_array($compulsary_config_pieces = $config->lookup('modules.Front.compulsarycontactviewconfig.Contact'))) {
			// print_r($compulsary_config_pieces); print '<hr>';
			foreach ($compulsary_config_pieces as $k => $piece) {
				// print '<div>piece: ' . $k . '</div>'; print_r($piece); print '<hr>';
				$do_count_items = (isset($piece['tab']) && isset($piece['tab']['do_count_items']) && $piece['tab']['do_count_items'] ? true : false);
				$standard_display = $this->isStandardDisplay();
				if (isset($piece['tab']) && is_array($piece['tab'])) {
					if (!isset($piece['tab']['section'])) {
						$contact_piece_object = $this->getContactPieceObject($piece);
						if (is_object($contact_piece_object)) {
							$piece['tab']['content'] = $contact_piece_object->main(true);
							// print '<div>setting content for tab ' . get_class($contact_piece_object) . ' ' . strlen($piece['tab']['content']) . '</div>';
						}
						// --
						$title_text = $contact_piece_object->getTitleText();
						// $this->printDebug(get_class($this) . '::getContactData - count items...');
						$num_mod_found = 0;
						if ($standard_display) {
							if ($do_count_items) {
								// print '<div>do_count_items class: ' . get_class($contact_piece_object) . '</div>';
						
								if (method_exists($contact_piece_object, 'countItems')) {
									$num_mod_found = $contact_piece_object->countItems(); // Phil 20060322
								} else {
									$num_mod_found = $contact_piece_object->db_functions->countItems(); // Phil 20050709
								}
								// print '<div>FINISHED do_count_items class: ' . get_class($contact_piece_object) . '</div>';
								
							}
						}
						// $piece['tab']['title'] = $lang->getLanguageText($piece['tab']['title'], $piece['module']);
						$title_text = $title_text . (($num_mod_found > 0) ? ' ('. $num_mod_found . ')' : '');
						$piece['tab']['title'] = $title_text;
					} else {
						// print '<div>section: ' . $piece['tab']['section'] . '</div>';
						$piece['tab']['title'] = $lang->getLanguageText($piece['tab']['title'], $piece['module']);
						$tabs[$k] = $piece['tab'];
					}
					
					// --
					
					
					$tabs[$k] = $piece['tab'];
				}
			}
			
		}
		// print '<div>tabs:</div>'; print_r($tabs); print '<hr>';
		$with_tabs = (boolean)is_array($tabs);
		// Phil 20070501 - these pieces now attached to the 'main' contact tabset
		
		// print '<div>tabs:</div>'; print_r($tabs);
		$this->setDataOptionForContactType();
		
		$contact_section_fields = $this->getContactSectionFields();
		$address_section_fields = $this->getAddressSectionFields();
		$postal_address_section_fields = $this->getPostalAddressSectionFields();
		$meta_section_fields = $this->getMetaSectionFields();
		// $custom_section_fields = $this->getCustomSectionFields(array($contact_section_fields, $address_section_fields, $meta_section_fields));
		// Meta and Other fields, and any custom fields defined in the data option
		$other_section_fields = $this->getOtherSectionFields(array($contact_section_fields, $address_section_fields, $postal_address_section_fields));
			
		$dd_item = $this->dd->getDataStructureWithData($item, $this->getPriorityEdit());  //specify alternate sort order
		
		// Phil 20070815 - duplicates parent method - WebMATView::getFieldRows
		if (is_array($dd_item)) {
			
			// print '<div>before CalculateValues</div>';
			$this->calculateValues($dd_item, $item);
			// print '<div>after CalculateValues</div>';
			
			foreach ($dd_item as $k => $field) {
				if (isset($field['hide_empty']) && $field['hide_empty']) {
					$value = $field['value'];
					switch ($field['type']) {	
					case 'dropdown':
						if ( empty($value) ) {
							// print '<div>contact field: ' . $k . ' value: "' . $field['value'] . '"</div>';
							bcrm_unset($dd_item, $k);
							continue;
						}
						break;
					case 'date':
						if ( empty($value) || ($value == '0000-00-00')) {
							// print '<div>contact field: ' . $k . ' value: "' . $field['value'] . '"</div>';
							bcrm_unset($dd_item, $k);
							continue;
						}
						break;
						
					default:
						if ( ($value !== "0") && empty($value) ) {
							// print '<div>contact field: ' . $k . ' value: "' . $field['value'] . '"</div>';
							bcrm_unset($dd_item, $k);
							continue;
						}
					}
				}
			}
		}
		
		$table_content =  '';	
		
		// $table_content .= $this->beginContactTable();
		//$table_content .= '
		//		<tr>
		//			<td>
		//			XXX
		//		';
		
		$table_content .= '
		
		<div id="topContainer">
		
		';
		
		$main_tab_title = ($this->getTitleText() ? $this->getTitleText() : 'Contact');
		
		$main_tab = array(
			'id' => 'tabMainContact', 
			'title' => $main_tab_title, 
			'section' => 'contact'
		);
		
		$main_contact_content = '';
		
		// Phil - when in print view mode, display a title for each tab
		if ($this->output->isPrintView()) {
			$main_contact_content .= '
		<div id="tabTitle' . $main_tab['id'] . '" class="tab-title">' . $main_tab['title'] . '</div>
		';
		}
		
		$main_contact_content .= '
			<div id="' . $main_tab['id'] . '" class="tab-content">
				<table id="tbl-' . $main_tab['id'] . '" class="viewTbl" border="0" cellpadding="0" cellspacing="0">
					' . $this->getSection($item, $dd_item, $contact_section_fields) . '
				</table>
			</div>
		';
		
		$main_tabset_name = 'contactTopLeftTabs';
		//$this->output->initExt();
		$this->output->initTabs($main_tabset_name);
		$this->output->addTab($main_tab, $main_tabset_name);
		$this->output->setFirstTab($main_tab['id'], $main_tabset_name);
		
		$table_content .= '
			<div id="' . $main_tabset_name . '">
				' . 
				$main_contact_content . 
				'
			</div>
		';
		// --
			
		// $table_content .= $this->beginNewColumn();
		// $table_content .= '
			// </td>
			// <td>
		// ';
		
		if ($with_tabs) {
			$tabs_content = '';
			foreach ($tabs as $k => $v) {
				$section_fields = $tab_content = false;
				// $tab_content = 'Hello world. I am ' . $v['id'] . '';
				// print '<div>id: ' . $v['id'] . ', title: ' . $v['title'] . ', section: ' . $v['section'] . '</div>';
				switch ($v['section']) {
				case 'main':
					$section_fields = $contact_section_fields;
				break;	
				case 'addresses':
					$section_fields = $address_section_fields;
				break;	
				case 'postal':
					$section_fields = $postal_address_section_fields;
				break;	
				
				case 'custom':
					$section_fields = $custom_section_fields;
				break;	
				case 'other':
					$section_fields = $other_section_fields;
				break;	
				}
				if ($section_fields || $v['content']) {
					if ($section_fields) {
						// print_r($v); print '<hr>';
						$split = ((isset($v['split']) && $v['split']) ? $v['split'] : false);
						$tab_content = $this->getSection($item, $dd_item, $section_fields, $split);
					} else {
						$tab_content = $v['content'];
					}
					if ($tab_content) {
						
						// Phil - when in print view mode, display a title for each tab
						if ($this->output->isPrintView()) {
							$tabs_content .= '
						<div id="tabTitle' . $v['id'] . '" class="tab-title">' . $v['title'] . '</div>
						';
						}
						
						$tabs_content .= '
						<div id="' . $v['id'] . '" class="tab-content">
							<table id="tbl-' . $v['id'] . '" class="viewTbl" border="0" cellpadding="0" cellspacing="0">
							' . 
							$tab_content . 
							'
							</table>
						</div>
						';
					} else {
						// print '<div>DISABLE: id: ' . $v['id'] . ', title: ' . $v['title'] . ', section: ' . $v['section'] . '</div>';
						$tabs[$k]['disable'] = true;
					}
				} else {
					bcrm_unset($tabs, $k);
				}
				
				
				// print '<div>Section---<br>' . $tab_content . '<br>---</div>';
				
				
			}
			// --
			//MCR example of how tabs could be added to top part of Contact form
			
			// Phil 20070501
			$tabset_name = 'contactTopTabs';
			reset($tabs);
			$current_tab = current($tabs);
			$first_tab = $current_tab['id'];
			$active_tab = false;
			//$this->output->initExt();
			$this->output->initTabs($tabset_name);
			foreach ($tabs as $tab) {
				// print '<div>adding tab:</div>'; print_r($tab);
				if (!$active_tab && (!isset($tab['disable']) || !$tab['disable'])) {
					$active_tab = $tab['id'];
				}
				$this->output->addTab($tab, $tabset_name);
				
			}
			$this->output->setFirstTab($active_tab ? $active_tab : $first_tab, $tabset_name);
			
			// Phil 20070501 table_content already contains individual div for each tab.
			// We just need to add them to the tabset
			$table_content .= '
			<div id="' . $tabset_name . '">
				' . $tabs_content . '
			</div>
			';
			
			$table_content .= '
			
			</div>
			
			'; //end topContainer
		
			$table_content = '
			<div id="contact-view-container-resizer">'
			 . $table_content . '
			 </div>
			 ';
			
			// Phil 20070618 height increased to 260
			$this->output->addOnReadyScript('
				var contactviewcontainer = new Ext.Resizable(\'contact-view-container-resizer\', {
					wrap:true,
					pinned:false,
					width:802,
					height:260,
					handles: \'s\',
					minHeight: 150,
					resizeChild:true,
					dynamic: true
				});
				');
				
			
			// --
			
		} else {
			// Build table content in sections: 
			// Contact section (person or company fields, and account fields)
			// Main addresses
			// Other contact fields (excl contact, address, custom)
			// Custom fields
			
			// TODO: postal address section (200707)
			
			$table_content .= $this->getSection($item, $dd_item, $address_section_fields);
			
			// $table_content .= $this->beginNewColumn();
			$table_content .= '
			</td>
			<td>
				<table>
		';
			$table_content .= $this->getSection($item, $dd_item, $other_section_fields);
			$table_content .= $this->getSection($item, $dd_item, $custom_section_fields);
			$table_content .= $this->getSection($item, $dd_item, $meta_section_fields);
		
			$table_content .= '
				</table>
				';
		
		}
		// $table_content .= $this->endContactTable($with_tabs);
		//$table_content .= '
		//			</td>
		//		</tr>
		//		';
		
		// wrap in styled box?
		// $table_content = '
		// <div class="x-box-tl"><div class="x-box-tr"><div class="x-box-tc"></div></div></div>
    	// <div class="x-box-ml"><div class="x-box-mr"><div class="x-box-mc">
		// ' . $table_content . '
		// </div></div></div>
		// <div class="x-box-bl"><div class="x-box-br"><div class="x-box-bc"></div></div></div>
		// ';
		
		
		//horrible hack to break tablethat contains this content
		// Phil do you have a better solution that doesn't take forever
		//$table_content = '</td></tr></tbody></table>' . $table_content . '<table cellpading=0 border=0 cellspacing=0><tbody><tr><td>'; 
		
		return $table_content;
	}
	
	/*
	 * builds field content (not necessarily table rows - depends if we are using tabs)
	 *
	 * @param array a field array
	 * @return string a table row
	 */
	// function getFieldRow($field, $with_tabs = false) {
		// $with_tabs = false;
		// if ($with_tabs) {
			// return $this->output->getStringFieldContent($field);
		// } else {
			// return $this->output->getStringFieldRow($field);
		// }
	// }
	
	/**
	 * Sets a data option variable dependant on contact type (Person or Company)
	 *
	 * @see getDataOptionForContactType
	 * @author Phil Calder
	 */
	function setDataOptionForContactType() {
		global $SERVICES;
		$dd =& $SERVICES->getService('DataDefinition');
		$tmp_data_option_name = ($this->is_person ? 'person_view' : 'company_view');
		$tmp_data_option_arr = $dd->getOptionByType($tmp_data_option_name, 'Contact__item');
		$data_option_values = false;
		$data_option_method = false;
		if (is_array($tmp_data_option_arr) && isset($tmp_data_option_arr['method']) && isset($tmp_data_option_arr['option_values']) ) {
			$data_option_values = explode(',', $tmp_data_option_arr['option_values']);
			$data_option_method = ($tmp_data_option_arr['method'] == 'include' ? 'include' : 'exclude');
		}
		// print '<div>data_option: ' . $tmp_data_option_name . ' method: ' . $tmp_data_option_arr['method'] . '</div>'; print_r($data_option_values); print '<hr />';
		$this->data_option_for_contact_type = array(
			'name' => $tmp_data_option_name, 
			'method' => $data_option_method, 
			'values' => $data_option_values
		);
	}
	
	/**
	 * Gets the data option variable to use dependant on contact type (Person or Company)
	 *
	 * @see setDataOptionForContactType
	 * @see showField
	 * @author Phil Calder
	 */
	function getDataOptionForContactType() {
		return $this->data_option_for_contact_type;
	}
	
	function showField($field_name) {
		$data_option = $this->getDataOptionForContactType();
		$data_option_method = $data_option['method'];
		$data_option_values = $data_option['values'];
		
		if ( ($data_option_values !== false) && ($data_option_method !== false) ) {
			if ( ($data_option_method === 'exclude') && in_array($field_name, $data_option_values) ) {
				// print '<div>WARNING: exclude this field ' . $field_name . '</div>';
				return false;
			} else if ( ($data_option_method === 'include') && !in_array($field_name, $data_option_values) ) {
				// print '<div>WARNING: do not include this field ' . $field_name . '</div>';
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Adds an address fields user defined label to the field label if both are included in the data structure
	 *
	 * @param array a field array
	 * @return string a table row
	 */
	function getFieldRow($field, $dd_item) {
		// add to address field labels, eg:
		// for email1_content, append the email1_label value to the email1_content label
		// dont display email1_label
		
		$map = array(
			'email1_content' => 'email1_label',
			'email2_content' => 'email2_label',
			'email3_content' => 'email3_label',
			'phone1_content' => 'phone1_label',
			'phone2_content' => 'phone2_label',
			'phone3_content' => 'phone3_label', 
			'messenger1_username' => 'messenger1_type', 
			'messenger2_username' => 'messenger2_type'
		);
		
		if (array_key_exists($field['field_key'], $map)) {
			if (isset($dd_item[$map[$field['field_key']]]) && $label = $this->output->getStringField($dd_item[$map[$field['field_key']]])) {
				$field['label'] .= ' - ' . $label;
			}
		} else if (in_array($field['field_key'], $map)) {
			return;
		}
		
		return parent::getFieldRow($field);
	}
	
	function getSection($item, $dd_item, $section_fields, $split_on = false) {
		$with_tabs = false;
		$table_content = false;
		
		$split_done = false;
		
		// print_r($dd_item);
		// print '<div>section_fields</div>'; print_r($section_fields);
		if (is_array($section_fields)) {
			foreach ($section_fields as $k => $v) {
				if (!$this->showField($k)) {
					continue;
				}
				if (isset($dd_item[$k]) && is_array($dd_item[$k])) {
					
					$dd_item[$k]['no_resize'] = 1; // Phil 20070618 - really for long_text fields
					$dd_item[$k]['no_spacer'] = 1; // Phil 20070618 - really for long_text fields
					
					if ($this->output->getHasTemplate()) {
						$table_content['fields'][$dd_item[$k]['field_key']] = $this->getFieldRow($dd_item[$k]);
					} else {
						if ($split_on && !$split_done && ($split_on == (substr($k, 0, strlen($split_on)))) ) {
							// print '<div>split on ' . $k . '</div>';
							$split_done = true;
							$table_content .= '
									</table>
								</td>
								<td width="50%">
								<!-- split -->
								<table class="viewTbl-split" border="0" cellpadding="0" cellspacing="0">
							';
						}
						// $table_content .= $this->getFieldRow($dd_item[$k], $with_tabs);
						$table_content .= $this->getFieldRow($dd_item[$k], $dd_item);
					}
				}
			}
		}
		
		if ($split_done) {
			// print '<div>split done - adding table content</div>';
			$table_content = '
			<tr>
				<td width="50%">
					<table class="viewTbl-split" border="0" cellpadding="0" cellspacing="0">
					' . $table_content . '
					</table>
				</td>
			</tr>
			';
		}
		
		return $table_content;
	}
	
	function getContactSectionFields() {
		$account_fields = $this->dd->getFieldsBeginningWith('acc_');
		
		if ($this->is_person) {
			$contact_fields = (array)$this->dd->getFieldsBeginningWith('Person__item');
			$contact_fields = array_merge($contact_fields, (array)$this->dd->getFieldsBeginningWith('contact_company'));
		} else {
			$contact_fields = (array)$this->dd->getFieldsBeginningWith('Company__item');
		}
		
		$custom_fields = $this->dd->getCustomFields();
		
		$section_fields = bcrm_array_merge(
			array_flip($contact_fields), 
			array_flip($account_fields), 
			array_flip($custom_fields)
		);
		return $section_fields;
	}
	
	function getAddressSectionFields() {
		// Look for fields: email1_, phone1_, fax1_, url1_
		// TODO: Possibly update dd_item to use [field]_label if it exists
		
		$address_fields = (array)$this->dd->getFieldsBeginningWith('contactby');
		$address_fields = array_merge($address_fields, (array)$this->dd->getFieldsBeginningWith('email'));
		$address_fields = array_merge($address_fields, (array)$this->dd->getFieldsBeginningWith('phone'));
		$address_fields = array_merge($address_fields, (array)$this->dd->getFieldsBeginningWith('fax'));
		$address_fields = array_merge($address_fields, (array)$this->dd->getFieldsBeginningWith('url'));
		// $address_fields = array_merge($address_fields, (array)$this->dd->getFieldsBeginningWith('addrpost'));
		// $address_fields = array_merge($address_fields, (array)$this->dd->getFieldsBeginningWith('addrstreet'));
		$address_fields = array_merge($address_fields, (array)$this->dd->getFieldsBeginningWith('skype'));
		$address_fields = array_merge($address_fields, (array)$this->dd->getFieldsBeginningWith('messenger'));
		
		$section_fields = array_flip($address_fields);
		return $section_fields;
	}
	
	function getPostalAddressSectionFields() {
		// Look for fields: addrpost%, addrstreet%
		// TODO: Possibly update dd_item to use [field]_l if it exists
		
		$address_fields = (array)$this->dd->getFieldsBeginningWith('addrpost');
		$address_fields = array_merge($address_fields, (array)$this->dd->getFieldsBeginningWith('addrstreet'));
		
		$section_fields = array_flip($address_fields);
		return $section_fields;
	}
	
	function getMetaSectionFields() {
		$permissions_fields = $this->dd->getPermissionsFields();
		$other_meta_fields = array(
			'creator_id',
			'legacy_id', 
			'unique_id', 
			'import_id'
		);
		$meta_fields = array_merge($other_meta_fields, (array)$this->dd->getMetaFields()); 
		
		$section_fields = bcrm_array_merge(
			array_flip($meta_fields), 
			array_flip($permissions_fields)
		);
		return $section_fields;
	}
	
	function getCustomSectionFields($exclude = array()) {
		$fields = (array)$this->dd->getCustomFields();
		$section_fields = array_flip($fields);
		$section_fields = $this->excludeFieldFromSection($exclude, $section_fields);
		// print '<div>section_fields:</div>'; print_r($section_fields); print '<hr>';
		return (array)$section_fields;
	}
	
	function getOtherSectionFields($exclude = array()) {
		$all_fields = (array)$this->dd->getFieldsBeginningWith();
		$section_fields = array_flip($all_fields);
		// print '<div>all_fields:</div>'; print_r($all_fields); print '<hr>';
		$section_fields = $this->excludeFieldFromSection($exclude, $section_fields);
		// print '<div>section_fields:</div>'; print_r($section_fields); print '<hr>';
		return (array)$section_fields;
	}
	
	function excludeFieldFromSection($exclude, $section_fields) {
		if (is_array($exclude)) {
			foreach ($exclude as $k => $v) {
				// print '<div>k=' . $k . '</div>'; print_r($v); print '<hr>';
				if (is_array($v)) {
					foreach ($v as $kk => $vv) {
						if (isset($section_fields[$kk])) {
							// print '<div>removing field: ' . $kk . '</div>';
							unset($section_fields[$kk]);
						}
					}
				}
				// print '<div>exclude:</div>'; print_r($v); print '<hr>';
			}
		}
		return (array)$section_fields;
	}
	
	/**
	 * @return boolean Whether the contact view display is one column or not (default false)
	 */
	function isOneColumnDisplay() {
		// Phil - Could allow a user pref here
		return false;
	}
	
	function beginContactTable($with_tabs = false) {
		$with_tabs = false;
		$content = '';
		// $content .= '<!-- beginContactTable -->';
		$content .= $this->output->getBeginRow();
		$content .= $this->output->getBeginCell(1, 'contactCol');
		if (!$with_tabs) {
			$content .= $this->output->getBeginTable('viewTbl');
		}
		return $content;
	}
	
	function beginNewColumn($with_tabs = false) {
		$with_tabs = false;
		$content = '';
		// $content .= '<!-- beginNewColumn -->';
		if (!$with_tabs) {
			$content .= $this->output->getEndTable();
		}
		$content .= $this->output->getEndCell();
		$cols = ($with_tabs ? 2 : 1);
		// $content .= $this->output->getBeginCell($cols, 'contactCol');
		$content .= '
		<td colspan="' . $cols . '">
		';
		if (!$with_tabs) {
			$content .= $this->output->getBeginTable('viewTbl');
		}
		return $content;
	}
	
	function endContactTable($with_tabs = false) {
		$with_tabs = false;
		$content = '';
		// $content .= '<!-- endContactTable -->';
		if (!$with_tabs) {
			$content .= $this->output->getEndTable();
		}
		$content .= $this->output->getEndCell();
		$content .= $this->output->getEndRow();
		return $content;
	}
	
	/**
	 * Gets the table links
	 *
	 * Adds a Company Depth link based on user preference
	 *
	 * @param array $item the item array
	 * @return array an array of links
	 */
	function getTableLinks($item) {
		$links = (array)WebMATView::getTableLinks($item);
		$links['company_depth'] = $this->getCompanyDepthToggleLinks($item['id']);
		$display_type_toggle_links = $this->getDisplayToggleLinks();
		$event_history = $this->getEventHistoryToggleLinks($item['id']);
		$event_list = $this->getEventListToggleLinks($item['id']);
		
		$links['contactNestedDropdownValue']['icon'] = 'page_misc.png';
		
		if (isset($links['contactNestedDropdownValue']) && is_array($links['contactNestedDropdownValue'])) {
			$links['contactNestedDropdownValue']['menu'] = array();
		}
		
		if (is_array($display_type_toggle_links)) {
			foreach ($display_type_toggle_links as $k => $v) {
				if (isset($links['contactNestedDropdownValue']['menu']) && is_array($links['contactNestedDropdownValue']['menu'])) {
					$links['contactNestedDropdownValue']['menu']['display_type' . $k] = $v;
				} else {
					$links['display_type' . $k] = $v;
				}
			}
		}
		
		if (isset($links['contactNestedDropdownValue']['menu']) && is_array($links['contactNestedDropdownValue']['menu'])) {
			if ($event_history) {
				$links['contactNestedDropdownValue']['menu']['event_history'] = $event_history;
			}
			if ($event_list) {
				$links['contactNestedDropdownValue']['menu']['event_list'] = $event_list;
			}
		} else {
			if ($event_history) {
				$links['event_history'] = $event_history;
			}
			if ($event_list) {
				$links['event_list'] = $event_list;
			}
		}
		
		if (isset($links['contactCheckForDuplicates']) && is_array($links['contactCheckForDuplicates'])) {
			$links['contactNestedDropdownValue']['menu']['contactCheckForDuplicates'] = $links['contactCheckForDuplicates'];
			bcrm_unset($links, 'contactCheckForDuplicates');
		}
		if (isset($links['contactLinkExportVcard']) && is_array($links['contactLinkExportVcard'])) {
			$links['contactNestedDropdownValue']['menu']['contactLinkExportVcard'] = $links['contactLinkExportVcard'];
			bcrm_unset($links, 'contactLinkExportVcard');
		}
		
		return $links;
	}
	
	/**
	 * @param integer $contact_id The current contact
	 * @return string Toggle link for loading contact view with/without full-company depth
	 */
	function getEventHistoryToggleLinks($contact_id) {
		global $L, $SERVICES;
		$content = '';
		
		if (isset($this->form_values['event_history']) && $this->form_values['event_history']) {
			$event_history_pref = 1;
		} else {
			$event_history_pref = 0;
		}
		
		if ($event_history_pref) {
			$href = $this->output->updateQueryString(array('event_history' => '0', 'event_list' => '0'));
			$text = $L['contactToggleEventHistoryTurnOff'];
			$title = $L['contactToggleEventHistoryTurnOffExplanation'];
		} else {
			$href = $this->output->updateQueryString(array('event_history' => '1', 'event_list' => '0'));
			$text = $L['contactLinkViewEventHistory']; //$L['contactToggleEventHistoryTurnOn'];
			$title = $L['contactToggleEventHistoryTurnOnExplanation'];
		}
		// $content .= $this->output->getLinkHref(
			// array(
				// 'href' => $href, 
				// 'text' => $text, 
				// 'title' => $title
			// ), 
			// true
		// );
		// Phil 20070507 Show link in toolbar
		$content = array(
			 'href' => $href,
			 'text' => $text,
			 'icon' => 'page_view.png',
			 'title' => $title
		 );
		return $content;
	}
	
	/**
	 * @param integer $contact_id The current contact
	 * @return string Toggle link for loading contact view with/without full-company depth
	 */
	function getEventListToggleLinks($contact_id) {
		global $L, $SERVICES;
		$content = '';
		
		if (isset($this->form_values['event_list']) && $this->form_values['event_list']) {
			$event_history_pref = 1;
		} else {
			$event_history_pref = 0;
		}
		
		if ($event_history_pref) {
			$href = $this->output->updateQueryString(array('event_list' => '0', 'event_history' => '0'));
			$text = $L['contactToggleEventListTurnOff'];
			$title = $L['contactToggleEventListTurnOffExplanation'];
		} else {
			$href = $this->output->updateQueryString(array('event_list' => '1', 'event_history' => '0'));
			$text = $L['contactLinkViewEventList'];
			$title = $L['contactToggleEventListTurnOnExplanation'];
		}
		// $content .= $this->output->getLinkHref(
			// array(
				// 'href' => $href, 
				// 'text' => $text, 
				// 'title' => $title
			// ), 
			// true
		// );
		// Phil 20070507 Show link in toolbar
		$content = array(
			 'href' => $href,
			 'text' => $text,
			 'icon' => 'page_view.png',
			 'title' => $title
		 );
		return $content;
	}
		
	/**
	 * @param integer $contact_id The current contact
	 * @return string Toggle link for loading contact view with/without full-company depth
	 */
	function getCompanyDepthToggleLinks($contact_id) {
		$content = '';
		
		if ($this->mod->isCompany($contact_id)) {
			global $L, $SERVICES;
			$prefs =& $SERVICES->getService('Preferences');
			$company_depth_pref = $prefs->getUserPref('contactShowCompanyDepth', '0');
			
			if (isset($this->form_values['contact_depth'])) {
				if ($this->form_values['contact_depth']) {
					if (!$company_depth_pref) {
						$prefs->setUserPref('contactShowCompanyDepth', '1');
						$company_depth_pref = 1;
					}
				} else {
					if ($company_depth_pref) {
						$prefs->setUserPref('contactShowCompanyDepth', '0');
						$company_depth_pref = 0;
					}
				}
			}
			
			if ($company_depth_pref) {
				$href = $this->output->updateQueryString(array('contact_depth' => '0'));
				$text = $L['contactToggleDepthCompanyOnly'];
				$title = $L['contactToggleDepthCompanyOnlyExplanation'];
			} else {
				$href = $this->output->updateQueryString(array('contact_depth' => '1'));
				$text = $L['contactToggleDepthCompanyDepth'];
				$title = $L['contactToggleDepthCompanyDepthExplanation'];
			}
			// $content .= $this->output->getLinkHref(
				// array(
					// 'href' => $href, 
					// 'text' => $text, 
					// 'title' => $title
				// ), 
				// true
			// );
			// Phil 20070507 Show link in toolbar
			$content = array(
				 'href' => $href,
				 'text' => $text,
				 'icon' => 'page_view.png',
				 'title' => $title
			 );
		}
		return $content;
	}
	
	function getDisplayToggleLinks() {
		global $L;
		if (is_object($this->output)) {
			$link_href = $php_self . $this->output->updateQueryString(array('alt_data_option' => '0'));
			$text = $L['LinkStandardView'];
			// $standard = $this->output->getLinkHref(
				// array(
					// 'href' => $link_href,
					// 'text' => $text, 
					// 'tag_extras' => $tag_extras
				// )
			// );
			// Phil 20070507 Show link in toolbar
			$standard = array(
				 'href' => $link_href,
				 'text' => $text,
				 'icon' => 'page_view.png',
				 'title' => $title
			 );
			
			$link_href = $php_self . $this->output->updateQueryString(array('alt_data_option' => '1'));
			$text = $L['LinkAlternativeView'];
			// $alternative = $this->output->getLinkHref(
				// array(
					// 'href' => $link_href,
					// 'text' => $text, 
					// 'tag_extras' => $tag_extras
				// )
			// );
			$alternative = array(
				 'href' => $link_href,
				 'text' => $text,
				 'icon' => 'page_view.png',
				 'title' => $title
			 );
			
			// $content .= $standard . ' ' . $alternative;
			return array($standard, $alternative);
		}
		return $content;
	}
	
	/**
	 * @deprecated version 5
	 */
	function setContactPieceFilename() {
	}
	
	/**
	 * @deprecated version 5
	 */
	function getContactPieceContent() {
		return false;
	}
}

$contact_view_manager = new ContactViewManager('Contact', 'contact_view');
$contact_view_manager->main();
?>