<?php
/**
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Contact
 */

/**
 * Displays a contact view list of the current contacts addresses
 *
 * @package Contact
 * @author Phil Calder
 */
class ContactAddressesWebMATContactView extends WebMATContactView {
	
	/**
	 * @private
	 */
	var $show_sub_contact_view = true;
	
	var $edit_link = 'edit_addresses';
	
	/**
	 * Determines whether to show sub-contacts
	 *
	 * Used by {@link main}. 
	 * If we arent showing sub-contacts (ie there arent any), main will return an empty string
	 * Calls parent constructor
	 */
	function ContactAddressesWebMATContactView($module, $data_option, $post, $get) {
		// Phil 200704 Moved to setPieceFilter // $this->show_sub_contact_view = $this->showSubContactView($get['id']);
		parent::WebMATContactView($module, $data_option, $post, $get);
		global $L;
		$this->setTitleText($L['contactAddressAddresses']);
	}
	
	/**
	 * Sets piece related flags
	 *
	 * Will only call parent init if disable_webmat_init is no set or not true
	 */
	function init($module, $data_option, $post = false, $get = false) {
		parent::init($module, $data_option, $post, $get);
		$this->edit_link = 'edit_addresses';
	}
	
	// Phil 200704 No longer required
	// 
	// Force refresh
	// 
	// @param boolean $refresh Ignored - will be set to true
	// @author Phil Calder
	// 
	// function setDbFunctions($refresh = false) {
		// // print '<div>Address piece ' . get_class($this) . '::setDbFunctions(' . ($refresh ? 'true' : 'false'). ')</div>';
		// // Phil 20060717 We need to force a refresh here to ensure 
		// // the db_function object uses the correct data definition
		// $refresh = true;
		// parent::setDbFunctions($refresh);
	// }

	/**
	 * Either an empty string or the content for this contacts sub-contact table
	 *
	 * Sets title text
	 *
	 * @return string Always returns a string (can be empty)
	 */
	function main($return_string = true) {
		// print '<div>' . get_class($this) . ' in main contact_id: ' . $this->contact_id . ' return_string: ' . ($return_string ? 'yes' : 'no') . '</div>';
		// if (!$this->show_sub_contact_view) {
			// return '';
		// }
		// Phil 20051101 - display_name will become a contact link so we dont want to trim it
		$this->dd->setNoTrimOption(); 
		return parent::main($return_string);
	}
	
	/**
	 * Custom - reorders the contact piece display so that the contact link is first
	 *
	 * @author Phil Calder
	 */
	function addContactField() {
		global $SERVICES;
		$dd =& $this->getDataDefinition();
		$fields = $this->db_functions->getDataDefinitionFields();
		if (isset($fields['display_name']) && $fields['display_name']) { 
			// $dd->setDictionaryIncludeField($fields[$contact_id_field], true);
			$new_order = array('display_name'=>array());
			$dd->data_dictionary = $this->arrayMergeReplace($new_order, $dd->data_dictionary);
		}
	}
	
	/**
	 * @param integer Contact id
	 * @return boolean true if this is a company
	 */
	function showSubContactView($contact_id = false) {
		$contact_id = ($contact_id ? $contact_id : $this->contact_id);
		if ($contact_id) {
			 if ($this->mod->isCompany($contact_id)) {
				global $SERVICES;
				$prefs =& $SERVICES->getService('Preferences');
				return (boolean)$prefs->getUserPref('contactShowCompanyDepth', '0');
			}
		}
		return false;
	}
	
	/**
	 * Returns the contact module object
	 */
	function &getContactModule() {
		global $SERVICES;
		$SERVICES->getService('Modules');
		$contact =& $SERVICES->modules->getModule('Contact');
		return $contact;
	}
	
	/**
	 * @return object The contact module item database table object
	 */
	function &getContactModuleTable() {
		global $SERVICES;
		$SERVICES->getService('Modules');
		$contact =& $SERVICES->modules->getModule('Contact');
		$db_functions =& $contact->getDatabaseTable('item');
		return $db_functions;
	}
	
	/** 
	 * @return integer Always return 0 (no alerts for this piece)
	 */
	function countAlerts() {
		return 0;
	}
	
	/**
	 * Sets design time filter for contacts from a given company based on the current contact
	 */
	function setPieceFilter() {
		if (!isset($this->db_functions)) {
			// $this->db_functions =& $this->mod->getDatabaseTable('item');
			$this->db_functions =& $this->getContactModuleTable();
		}
		if ( isset($this->db_functions) && is_object($this->db_functions) ) {
			$this->db_functions->where_statement = false; // Phil 20050708, caching whereStatement does not work here
			$this->db_functions->resetDesignTimeFilters();
			
			// if ($this->show_sub_contact_view) {
			if ($this->showSubContactView()) {
				$query = array('contact_company' => array('where'=> ' (Contact__item.contact_company = "' . $this->form_values['id']. '")')); 
			} else {
				$query = array('id' => array('where'=> ' (Contact__item.id = "' . $this->form_values['id']. '")'));
			}
			$this->db_functions->setDesignTimeFilters( $query );			
			
			
		}
	}
	
	/**
	 * The Address piece displays a custom list table, separated into blocks for each address type (email, phone, etc)
	 *
	 * @param array $item An item array (optional for view pages)
	 * @param array $error_msg error message (optional for view pages)
	 * @return string Content
	 * @author Phil Calder
	 */
	function getDataContent() {
		$cells = array();
		
		// $cells['__links'] = ''; // view, edit, etc, links
		// $cells['__flags'] = ''; // alert, unseen/edited, etc
		
		
		
		
		
		$cells = array_merge($cells, $this->getDataDictionary());
		$cells = $this->excludeCalculated($cells); // flags calculated fields
		
		// print '<div>' . get_class($this) . ' ' . $this->getModuleName() . ' cells:</div>'; print_r($cells); print '<hr>';
		// Get header cells for each address piece
		$pieces = array(
			'email', 
			'phone', 
			'fax', 
			'url', 
			'addr', 
		);
		$piece_fields = $piece_cells = array();
		
		foreach ($pieces as $k => $v) {
			$piece_fields[$v] = (array)$this->dd->getFieldsBeginningWith($v);
			
			$new_order = array('display_name'=> 'display_name');
			$piece_fields[$v] = $this->arrayMergeReplace($new_order, $piece_fields[$v]);
			// print '<div>piece: ' . $v . '</div>'; print_r($piece_fields[$v]); print '<hr>';
		}
		foreach ($piece_fields as $kk => $vv) {
			// print '<div>piece_field for ' . $kk . ':</div>'; print_r($vv); print'<hr>';
			if (is_array($vv)) {
				foreach ($vv as $kkk => $vvv) {
					$piece_cells[$kk][$vvv] = (array)$cells[$vvv];
				}
			
			}
			
			// print '<div>piece_cells: ' . $v . '</div>'; print_r($piece_cells[$v]); print '<hr>';
		}
		// print '<div>piece_cells:</div>'; print_r($piece_cells); print '<hr>'; exit();
		
		// print '<div>' . get_class($this) . ' ' . $this->getModuleName() . '</div>'; print_r($piece_cells); print '<hr>'; exit();
		// print '<div>' . get_class($this) . ' ' . $this->getModuleName() . '</div>'; print_r($cells); print '<hr>'; exit();
		
		// Create several header rows
		// - links, flags, contact display name, urls
		// email addresses
		// phone and fax numbers
		// addresses

		// $sort = $this->sortTable($this->form_values['sort']);
		// print_r($sort);
		$sort = array('field' => 'display_name', 'order' => 'ASC');
 		$contact_id = $this->form_values['contact_id'];
		
		if (!isset($this->item_count) || !$this->item_count) { // Phil 20070212 can sometimes already have been called (eg contact view, desktop)
			// print '<div>List, calling db_functions->countItems [item_count: ' . $this->item_count . ']</div>'; 
			$this->item_count = $this->db_functions->countItems($contact_id); // only needed for setPagingLimits, and showCheckboxMenu
		} // else {
			// print '<div>[cached] item_count: ' . $this->item_count . '</div>'; 
		// }
		
		// TODO - break this down according to set
		$piece_header = array();
		foreach ($piece_cells as $k => $v) {
			$piece_header[$k] = $this->getSubTableHeader($v, $sort);
			// print '<div>piece_header: ' . $k . ' <pre>' . $piece_header[$k] . '</pre></div>'; print '<hr>';
		}
		
		// $table_content = $this->getSubTableHeader($cells, $sort); //Syed 20050704
		// print '<pre>' . $table_content . '</pre>';

		if ($this->paging_overwrite['limit']) {
			// print '<div>paging_overwrite[limit]' . $this->paging_overwrite['limit'] . '</div>';
			$limit = 'LIMIT 0, ' . $this->paging_overwrite['limit'];
		} else {
			$limit = $this->output->setPagingLimits($this->pref_name . 'ListSize', $this->item_count);
		}
		if ($this->paging_overwrite['sort']) {
			// print '<div>paging_overwrite[sort]' . $this->paging_overwrite['sort'] . '</div>';
			$sort = $this->paging_overwrite['sort'];
		}
		
		$table_content .= $this->startDataBlock();
		
		$this->beforeGetItems($sort, $limit);
		
		if (!$this->useGetItemsIterator()) { // Phil - this piece does not use an iterator
			
			$items = $this->getItems($sort, $limit);
			// print '<div>items:</div><pre>' . print_r($items, 1) . '</pre><hr>';
			$table_content .= $this->afterGetItems($items);
	
			if (is_array($items) && sizeof($items)) {
				// $alert_ids = false;
				// if ($this->showAlertStatus()) { // Phil 20051114 Will not calculate or show alert flag if this is false
					// $alert_ids = $this->db_functions->getAlertIds($limit, array_keys($items)); //passing item ids
				// }
				// Phil 20051031 Tell Output if we dont want to trim fields
				$no_trim = $this->dd->getNoTrim();
				$current_no_trim = $this->setNoTrim($no_trim);
				
				foreach ($items as $k => $item) {
					// print_r(array_keys($item));
					// $table_content .= $this->buildListTable($item, $alert_ids);
					$item_content[$k] =  $this->buildStringFieldRow($item);
				}
				$this->setNoTrim($current_no_trim); // Phil 20051031 Reset the no trim value
				
				// print '<div>piece_header:</div>'; print_r($piece_header); print '<hr>';
				// print '<div>piece_cells:</div>'; print_r($piece_cells); print '<hr>';
				// print '<div>item_content:</div>'; print_r($item_content); print '<hr>';
				$piece_content = array();
				foreach ($piece_cells as $k => $pc) {
					 // print '<div>piece_cells: ' . $k . '</div>'; print_r($pc); print '<hr>';
					// $piece_content[$k] = array();
					// foreach ($pc as $key => $field) {
						// // print '<div>key: ' . $key . ' field:</div>'; print_r($field); print '<hr>';
						// foreach ($item_content as $k_item => $v_item) {
							// // print '<div>key: ' . $key . ' k_item: ' . $k_item . '</div>'; print_r($v_item); print '<hr>';
							// if (isset($v_item[$key])) {
								// // print '<div>adding piece_content[' . $k . '][' . $key . '][] = ' . $v_item[$key] . '</div>';
								// $piece_content[$k][$key][] = $v_item[$key];
								// // continue;
							// }
						// }
					// }
					$piece_content[$k] = array();
					foreach ($item_content as $k_item => $v_item) {
						foreach ($pc as $f_key => $field) {
						// print '<div>key: ' . $f_key . ' field:</div>'; print_r($field); print '<hr>';
						
							// print '<div>f_key: ' . $f_key . ' k_item: ' . $k_item . '</div>'; print_r($v_item); print '<hr>';
							//if (isset($v_item[$f_key])) {
								// print '<div>adding piece_content[' . $k . '][' . $f_key . '][] = ' . $v_item[$f_key] . '</div>';
								$piece_content[$k][$k_item][$f_key] = $v_item[$f_key];
								// continue;
							//}
						}
					}
				}
				// print_r($piece_content);
				
				// print '<div>piece_header:</div>'; print_r($piece_header); print '<hr>';
				// print '<div>piece_content:</div>'; print_r($piece_content); print '<hr>';
				if (isset($piece_content) && is_array($piece_content) ) {
					foreach ($piece_header as $k => $v) {
						// print '<div>header ' . $k . '</div>'; print_r($v); print '<hr>';
						// Add address rows for this piece
						if (isset($piece_content[$k]) && is_array($piece_content[$k])) {
							
							// Add a header row for this address piece
							$table_content .= $v;
							// Add rows of data
							$row = array();
							foreach ($piece_content[$k] as $kk => $table_row) {
								$this->data_row_counter++;
								$style_flag = array();
								$table_content .= $this->output->getTableRow($table_row, $style_flag, $this->cell_formatting, $this->getHtmlIdPrefix() . $this->data_row_counter);
							}
						}
					}
				}
				
			} else {
				global $L;
				$table_content .= $this->output->getTextRow($L['msgNoItemsForView'] , sizeof($cells)+1);
			}
		}
		
		

		$table_content .= $this->endDataBlock();
		return $table_content;
	}
	
	function useGetItemsIterator() {
		return false;
	}
	
	/**
	 * disabled
	 *
	 * @param array $item the items array
	 * @param array $alert_ids an array of alert ids
	 * @return string returns a table row with data.
	 */
	function buildListTable($item, $alert_ids = false, $style = false) {
		return false;
	}
	
	
	function getNewLink() {
		return false;
	}
	
	/**
	 * Disabled
	 *
	 * @return boolean false
	 * @author Phil Calder
	 */
	function getBrowseQueryDataOption() {
		return false;
	}
}
?>