<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Contact
 */

includeModule('Contact', 'inc/item_edit.inc');

/**
 * The Contact module Address edit class
 *
 * Sets a custom generated data definition 'on the fly' for all address fields
 * 
 * @package Contact
 * @author Phil Calder
 */
class ContactAddressesWebMATEdit extends WebMATEdit { //ContactItemWebMATEdit { 
	
	/**
	 * If the data option parameter is 'custom', 
	 * this method will generate a data definition on the fly for all custom fields for the Contact module.
	 * Otherwise the parent method is called.
	 * 
	 * @param string $data_option the data_option is used to set the data option
	 */
	function setDataDefinition($data_option) {
		$data_option = 'addresses'; // Will have been altered to either person_edit or company_edit.
		// print '<div>' . get_class($this) . '::setDataDefinition(' . $data_option . ')</div>'; print '<hr>';
		// see also item_edit.inc init method
		
		if ( ($data_option == 'addresses') ) {
			$this->dd = $this->getAddressDataDefinition($data_option, $this->form_values['id'], true); // Include display name
			// print_r($this->dd); print '<hr>';
			if (is_object($this->dd)) {
				return;
			}
			// print '<div>dd not an object</div>';
		}
		
		parent::setDataDefinition($data_option);
	}
	
	/**
	 * Data definition created on the fly for this modules Custom Fields
	 * 
	 * Used by the custom field view and edit pieces (contact view page)
	 *
	 * @param string $data_option
	 * @param mixed $contact_id Optional integer, defaults to false
	 * @return array data definition
	 */
	function getAddressDataDefinition($data_option, $contact_id = false, $include_display_name = false) {
		global $SERVICES, $L;
		// print '<div>data_option: ' . $data_option . '</div>';
		
		$dd_service =& $SERVICES->getService('DataDefinition');
		// if (!is_array($data_definition=$this->getExtendedCustomDataDefinition())) {
			// $data_definition = array();
			// $data_definition['fields'] = $dd_service->getCustomFields('Contact__item');
		// }
		$data_definition = array();
		$data_definition['fields'] = $dd_service->getFields('Contact__item');
		$data_definition['fields']['id'] = array(
			'label'     => $L['Contact'], 
			'field_key' => 'id', 
			'type'      => 'contact', 
			'value'     => $contact_id, 
			'read_only' => 1
		);
		$data_definition['data_options']['addresses'] = array(
			'method' => 'include',
			'values' => $include_keys = array_keys($data_definition['fields']), 
			'options' => array(
				'dropdown_values' => 1, 
				'exclude_calculated'=>0, 
				'exclude_custom'=>0, 
				'exclude_foreign'=>0, 
				'exclude_hidden'=>0, 
				'no_trim'=>1
			)
		);
		
		$dd = getWebMATObject(
			'WebMATDataDictionary', 
			$this->getModuleName(), 
			array(
				'data_definition' => $data_definition, 
				'data_option' => $data_option
			)
		);
		
		// Address field names --
		$address_fields = (array)$dd->getFieldsBeginningWith('email');
		$address_fields = array_merge($address_fields, (array)$dd->getFieldsBeginningWith('phone'));
		$address_fields = array_merge($address_fields, (array)$dd->getFieldsBeginningWith('fax'));
		$address_fields = array_merge($address_fields, (array)$dd->getFieldsBeginningWith('url'));
		$address_fields = array_merge($address_fields, (array)$dd->getFieldsBeginningWith('addrpost'));
		$address_fields = array_merge($address_fields, (array)$dd->getFieldsBeginningWith('addrstreet'));
		$address_fields = array_merge($address_fields, (array)$dd->getFieldsBeginningWith('contactby'));
		$address_fields = array_merge($address_fields, (array)$dd->getFieldsBeginningWith('skype'));
		$address_fields = array_merge($address_fields, (array)$dd->getFieldsBeginningWith('messenger'));
		
		// Remove postal and physical display name fields
		foreach ($address_fields as $k => $v) {
			if ( ($v == 'addrpost1_displayname') || ($v == 'addrstreet1_displayname') ) {
				// print $v . '<hr>';
				bcrm_unset($address_fields, $k);
			}
		}
		
		$address_fields[] = 'id';
		// -- 
		$dd->setDictionaryIncludeFields($address_fields);
		
		return $dd;
	}
}
?>