<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Contact
 */

/**
 * Contact view piece for Contact custom fields
 *
 * As of vesion 5 custom fields can be displayed in the main contact section.
 * This piece displays all custom fields regardless of whether they are displayed in the main contact section.
 *
 * @package Contact
 * @author Phil Calder
 */
class ContactCustomWebMATShortView extends WebMATShortView {
	
	var $num_custom_fields = 0;
	
	/**
	 * Returns an empty string if there are no custom fields in the data definition
	 */
	function main($return_string = true) {
		// print __FILE__ . ' ' . get_class($this) . '<hr>';
		// The id field is always present - we need to know if there are other fields
		if (is_object($this->dd) && (count($this->dd->data_dictionary)>1)) { 
			// global $L;
			// $this->setTitleText($L['contactAdminCustomFields']);
			$this->begin_table_arg = 'contactPieceTable'; // Phil 20060130
			$result = parent::main($return_string);
			// Phil 20070507 Remove db functions from module cache when finished
			if (isset($this->mod->table_objects_array)) {
				bcrm_unset($this->mod->table_objects_array, 'item');
			}
			return $result;
		} else {
			return '';	
		}
	}
	
	/**
	 * Disabled
	 *
	 * @return boolean false
	 */
	function countItems() {
		if (isset($this->num_custom_fields) && (int)$this->num_custom_fields) {
			return (int)$this->num_custom_fields;
		}
		return false;	
	}
	
	/**
	 * Returns the custom fields edit link if there is one
	 */	
	function getMenuItems($contact_id) {
		if ($this->db_functions->isEditable($contact_id)) {
			return $this->getCustomFieldsEditLink($contact_id);
		}
		return '';
	}
	
	/**
	 * If the data option parameter is 'custom', 
	 * this method will generate a data definition on the fly for all custom fields for the Contact module.
	 * Otherwise the parent method is called.
	 * 
	 * @param string $data_option the data_option name used for the data definition
	 */
	function setDataDefinition($data_option) {
		// print '<div>ContactCustomWebMATShortView::setDataDefinition(' . $data_option . ')</div>'; print '<hr>';
		if ( ($data_option == 'custom') ) {
			$this->dd = $this->mod->getCustomDataDefinition($data_option, $this->form_values['id']); // Dont include display name
			if (is_object($this->dd)) {
				// print '<div>************ setDataDefinition(' . $data_option . ')</div>'; print_r($this->dd); print '<hr>';
				// $this->db_functions = $this->mod->getDatabaseTable('item'); // New copy
				if (is_object($this->db_functions)) {
					$data_definition['fields'] = $this->dd->data_dictionary;
					// print '<div>custom fields:</div>'; print_r($data_definition['fields']); print '<hr>';
					if (is_array($data_definition['fields']) && sizeof($data_definition['fields'])) {
						$this->num_custom_fields = 0;
						foreach ($data_definition['fields'] as $k => $v) {
							if ( (!isset($v['hidden']) || $v['hidden']) && (!isset($v['type']) || ($v['type'] != 'hidden')) ) {
								$this->num_custom_fields++;
							}
						}
					}
					$data_definition['data_options']['list'] = $this->dd->data_options;
					$this->db_functions->setDataDefinition($data_definition);
				}
				// print '<div>returning because data def is custom</div>'; print_r($this->dd); print '<hr>';
				return;
			}
		}
		// print '<div>calling parent</div>';
		parent::setDataDefinition($data_option); // returns above if option is 'custom'
	}
	
	/**
	 * Calls getCustomEditLink
	 */
	function getEditLink($id) {
		return $this->getCustomFieldsEditLink($id);
	}
	
	/*
	 * Force db function refresh
	 */
	// function setDbFunctions($refresh = false) {
		// print '<div>' . get_class($this) . '::setDbFunctions(' . ($refresh ? 'true' : 'false') . ')</div>';
		// // parent::setDbFunctions(true);
		// parent::setDbFunctions($refresh);
	// }
	
	function getTitleText() {
		global $L;
		return $L['contactAdminCustomFields'];
	}
}
?>