<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 */

/**
 * This class handles the database functions for the Contact module item table
 * 
 * This module is an extend verison of WebMATDatabaseTable 
 * 
 * @author Phil Calder
 * @package Contact
 */
class itemContactDatabaseTable extends WebMATDatabaseTable {
	
	/**
	 * The constructor class for itemContactDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function itemContactDatabaseTable($module_name, $data_definition) {
		global $USER;
		$this->table_name = $module_name . '__item';
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseTable
		
		$this->setContactIdField('id'); // The contact id field for this table is 'id', not 'contact_id'
	}
	
	
	// function calculateValues(& $dd_item, $item) {
		// if (is_array($dd_item)) {
			// // print '<div>keys:</div>'; print_r(array_keys($dd_item)); print '<hr>';
			// // print '<div>item:</div>'; print_r($item); print '<hr>';
			// 
			// $contact_id_field = $this->getContactIdField();
			// $contact_id = $item[$contact_id_field];
			
			// foreach ($dd_item as $k => $v) {
				// if (isset($v['calculated']) && $v['calculated']) {
				// }
			// }
		// }
	// }
	
	/**
	 * Overridden to handle user pref for created after date
	 *
	 * @return string the alert where statement
	 */
	function getAlertWhere() {
		if (!isset($this->alert_where_processed) || !$this->alert_where_processed) {
			global $SERVICES, $USER;
			
			$this->alert_where_processed = array();
			
			$alert_array = $SERVICES->config->lookup($this->getFilesetConfigPath() . 'alert');
			
			$prefs =& $SERVICES->getService('Preferences');
			$pref_interval = $prefs->getUserPref('contactCreatedLessThanDaysAgo'); // 7 * 60 * 60 * 24;
			$created_array = false;
			
			if (!$pref_interval) {
				// print '<div>!pref_interval: ' . $pref_interval . '</div>';
				return $this->alert_where_processed = '0'; // ie dont display alerts
			} else if ($pref_interval == 'All') {
				// print '<div>[All] pref_interval: ' . $pref_interval . '</div>';
				// Show all Contacts matching other criteria (no 'created' criteria)
				$created_array = false;
			} else {
				// print '<div>processing pref_interval: ' . $pref_interval . '</div>';
				$current_date = mktime (0,0,0,date('m'),date('d'),date('Y'));
				// print '<div>current date: ' . $current_date . ', interval: ' . $pref_interval . '</div>';
				// $start_date = (intVal($current_date) - intVal($pref_interval));
				// Phil 20070131 corrected - pref_interval is num days, needs to be num seconds
				$start_date = intVal($current_date) - (intVal($pref_interval) * 24 * 3600);
				// print '<div>current date: ' . date("Y-m-d", $current_date) . ', start date: ' . date("Y-m-d", ($start_date) ) . '</div>';
				
				$created_array = array('where' => 'created >= (' . $start_date . ')');
			}
			if (is_array($alert_array)) { 
				if (is_array($created_array)) {
					$alert_array['created'] = $created_array;
				}
				$has_seen = $this->getHasSeenTable(); // Phil 20051007
				$where = array();
				foreach($alert_array as $k => $v) {
					if (in_array($k, $this->columns) || ($has_seen && $k == 'seen')) {
						// print '<div>k=' . $k . '</div>';
						$where_piece = $this->processDesignTimeFilter($k, $v);
						if ($has_seen && $k == 'seen') {
							$where_piece = str_replace($this->getTableName() . '.', $this->getTableName() . '_seen.', $where_piece);
							$where_piece = '(' . $where_piece . ' OR ' . str_replace('=\'0\'', ' IS NULL', $where_piece) .')';
						}
						$where[] = $where_piece;
					}
				}
			}
			
			if (!is_array($where)) {
				$where = explode(' AND ', $where);
			}
			
			//Might be able to be removed
			foreach ($where as $k => $v) {
				if ($v && (substr($v, 0, strlen($this->getTableName())) != $this->getTableName()) && (substr($v, 0, 1) != '(') ) {
					$where[$k] = $this->getTableName() . '.' . trim($v);
				}
			}
			
			$where = implode (' AND ', $where);
			
			$this->alert_where_processed = $this->replacePlaceholders($where); // Phil 20051007
		}
		// print '<div>$this->alert_where_processed = ' . $this->alert_where_processed . '</div>';
		return $this->alert_where_processed;
		
	}
	
	/**
	 * Sets include_where to true
	 * @author Phil Calder
	 */
	function getItem($id = false, $order_by = false, $include_where = false, $refresh = false) {
		// print '<div>getItem(' . $id . ')</div>';
		return parent::getItem($id, $order_by, true, $refresh);
	}
	
	/**
	 * Processes a row for foreign fields
	 *
	 * Calls parent method, then recalculates Company, Person, and Account values based on contact type.
	 *
	 * @param array $foreign_fields The contact fields to look for
	 * @return array The processed row
	 * @author Phil Calder
	 */
	function processGetRowForeignFields($row, $foreign_fields) {
		$row = parent::processGetRowForeignFields($row, $foreign_fields);
		// foreign fields
		if (count($foreign_fields)) {
			// print '<div>foreign_fields</div>'; print_r($foreign_fields); print '<hr>';
			// print '<div>contact: ' . $row[$this->getContactIdField()] . '</div>'; print_r($row); print '<br>';
			global $SERVICES;
			if (!is_object($mod =& $SERVICES->modules->getModule($this->module_name))) {
				return $row;
			}
			
			$is_person = $mod->isPerson($row[$this->getContactIdField()], $row);
			if ($is_person) {
				// print '<div>' . $row[$this->getContactIdField()] . ' is a person</div>';
				// Not a Company. Recalculate all Company foreign fields and set as readonly.
				$company_fields = $this->dd->getFieldsBeginningWith('Company__item');
				// print '<div>company fields</div>'; print_r($company_fields);
				
				$is_account = $mod->isAccount($row[$this->getContactIdField()], $row);
				if (!$is_account) {
					// print '<div>' . $row[$this->getContactIdField()] . ' is not an account</div>';
					// Not an Account. Recalculate all Account foreign fields and set as readonly.
					$account_fields = $this->dd->getFieldsBeginningWith('acc_');
					// print '<div>account fields</div>'; print_r($account_fields);
					// $contact_account = $mod->getContactAccountFor($row[$this->getContactIdField()], false, $row);
					// print '<div>contact account</div>'; print_r($contact_account);
				}
				
			} 
			$is_company = $mod->isCompany($row[$this->getContactIdField()], $row);
			if ($is_company) {
				// print '<div>' . $row[$this->getContactIdField()] . ' is a company</div>';
				// Not a Person. Recalculate all Person foreign fields and set as readonly.
				$person_fields = $this->dd->getFieldsBeginningWith('Person__item');
				// print '<div>person fields</div>'; print_r($person_fields);
				$contact_person = $mod->getContactPersonFor($row[$this->getContactIdField()], false, $row);
				// print '<div>contact person</div>'; print_r($contact_person);
			}
			// print '<hr>';
			
			foreach ($foreign_fields as $k => $v) {
				
				$info = explode('.', $v);
				$table = $info[0];
				$field_name = $info[1];
				
				if ($is_company || ($is_account && !$is_person)) {
					if (is_array($contact_person) && in_array($v, $person_fields) && array_key_exists($field_name, $contact_person) ) {
						$row[$v] = $contact_person[$field_name];
						// print '<div>Company->person SET ' . $v . ' to ' . $row[$v] . ' [for ' . $field_name . ']</div>';
					}
				} else if ($is_person) {
					if (!$is_account) {
						// print '<div>Person is not an account</div>';
						// if (is_array($contact_account) && in_array($v, $account_fields) && array_key_exists($field_name, $contact_account) ) {
							// $row[$v] = $contact_account[$field_name];
							// /// print '<div>Person->account SET ' . $v . ' to ' . $row[$v] . ' [for ' . $field_name . ']</div>';
						// }
						if (is_array($contact_company) && in_array($v, $contact_company) && array_key_exists($field_name, $contact_company) ) {
							$row[$v] = $contact_company[$field_name];
							// print '<div>Person->company SET ' . $v . ' to ' . $row[$v] . ' [for ' . $field_name . ']</div>';
						}
					}
					
				}
			}
			return $row;
		}
		return $row;
	}
	
	/**
	 * @return string $table The name or alias of the Contact item table
	 */
	function getContactTable() {
		return 'Contact__item';
	}
	
	/**
	 * Calls parent to delete item, then initiates cascading delete
	 */
	function deleteItem($id) {
		parent::deleteItem($id);
		global $SERVICES;
		$contact =& $SERVICES->modules->getModule('Contact');
		$contact->cascadeDelete($id);
	}
		
	/**
	 * Custom: group permissions for Contact module only
	 * 
	 * Includes Web Form user contact id check (Phil 20050916)
	 *
	 * @param array the current where statement
	 * @return array the current where statement with the new where statement appended
	 */
	function getDisplayByGroupWhere($where_array) {
		// print '<div>getDisplayByGroupFlag: ' . $this->getDisplayByGroupFlag() . '</div>';
		if ($this->getDisplayByGroupFlag() == 0) { // No permissions group filtering
			return $where_array;
		}
		
		global $SERVICES, $USER;
		$web_form_string = '';
		if (is_object($this->ua) && ($this->ua->isWebFormUser() && $this->ua->isWebFormUserAllowedToViewOwnContact()) && $this->ua->users->getUsersContactId()) {
			$web_form_string = "OR " . $this->getContactTable() .".id = '" . $this->ua->users->getUsersContactId() . "' "; 
		} 

		$where_array[] = "
			(" . $this->getContactTable() .".view_group_id" . " IN (" . $USER['group_string'] . ") 
			OR " . $this->getContactTable() .".edit_group_id" . " IN (" . $USER['group_string'] . ") 
			OR " . $this->getContactTable() .".owner_group_id" . " IN (" . $USER['group_string'] . ") 
			OR " . $this->getContactTable() .".admin_group_id" . " IN (" . $USER['group_string'] . ")
			" . $web_form_string . ")
		";
		// print_r($where_array); print '<p>--------------</p>';
		return $where_array;
	}
	
	/**
	 * Set the data definition array
	 *
	 * Sets default value for lock permissions based on user preference
	 *
	 * @param array $data_definition An array containing the data definition (fields and options)
	 */
	function setDataDefinition($data_definition) {
		// print '<div>' . get_class($this) . '</div>';
		global $SERVICES,$L;
		$SERVICES->getService('Language');
		$SERVICES->getService('Preferences');
		
		$this->mod =& $SERVICES->modules->getModule('Contact');
		
		if (is_array($data_definition['fields'])) { // Phil - it should be, but better to check
			foreach($data_definition['fields'] as $k => $v){
				switch ($v['field_key']){
					case 'lock_permissions':
						$data_definition['fields'][$k]['default_value'] = $SERVICES->preferences->getUserPref('contactDefaultLockState');
					break;
					// TODO: add default preferences
					// case 'addrpost1_city':
					// case 'addrstreet1_city':
						// $data_definition['fields'][$k]['default_value'] = $SERVICES->preferences->getUserPref('contactDefaultAddressCity');
					// break;
					// case 'addrpost1_state':
					// case 'addrstreet1_state':
						// $data_definition['fields'][$k]['default_value'] = $SERVICES->preferences->getUserPref('contactDefaultAddressState');
					// break;
					// case 'addrpost1_postcode':
					// case 'addrstreet1_postcode':	
						// $data_definition['fields'][$k]['default_value'] = $SERVICES->preferences->getUserPref('contactDefaultAddressZip');	
					// break;
					// case 'addrpost1_country':
					// case 'addrstreet1_country':	
						// $data_definition['fields'][$k]['default_value'] = $SERVICES->preferences->getUserPref('contactDefaultAddressCountry');					
					// break;
				
					//--
				}
			}
		}
		parent::setDataDefinition($data_definition);
	}
	
	/**
	 * Checks user preference - allow contact delete, then calls parent or returns false
	 * @author Phil Calder
	 */
	function isDeletable( $id, $item = false ) {
		global $SERVICES;
		$SERVICES->getService('Preferences');
		if (! ($allow_delete = $SERVICES->preferences->getUserPref('contactAllowContactDelete'))) {
			return false;
		}
		
		return parent::isDeletable($id, $item);
	}
	
	/**
	 * Reassigns items to a different contact
	 *
	 * This function differ from its parent in that we handle Contact reassignments differently.
	 * Company contacts are updated to point to the new contact id, as are accounts. 
	 * @param integer $contact_id the contact id you want to replace
	 * @param integer $new_contact_id the new contact id
	 * @author Phil Calder
	 */
	function reassignItemForContact($contact_id, $new_contact_id) {
		if ( is_array($this->columns) ) {
			if ( in_array('contact_company', $this->columns) ) {
				$this->db->update(
					$this->getTableName(), 
					array('contact_company' => $new_contact_id), 
					"contact_company='$contact_id'"
				);
			}
			if ( in_array('contact_account', $this->columns) ) {
				$this->db->update(
					$this->getTableName(), 
					array('contact_account' => $new_contact_id), 
					"contact_account='$contact_id'"
				);
			}
		} 
	}
}
?>