<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 */

includeModule('Contact', 'inc/item_edit.inc');

/**
 * The Contact module Public edit class
 *
 * See the WebForms '/pub' directory
 * @package Contact
 * @author Phil Calder
 */
class ContactItemWebMATPublicEdit extends ContactItemWebMATEdit {
	
	/**
	 * Allow/prevent permissions fields from being displayed
	 * Applied on top of data option setting 
	 * (ie will prevent permissions fields from being displayed even if they are included by the data option).
	 * @access private
	 */
	var $allow_permissions = false;
	
	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		global $SERVICES;
		$this->ua =& $SERVICES->getService('Authentication');
		$this->ua->clientRestrict(); // Requires client login
		
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
		$this->output->setShowContactSelector(false);
		$this->output->setShowContactPopup(false); // disables contact popup link option
		// print '<p>setting</p>';
		$this->setContact();
		global $L;
		$this->language_overwrite['title'] = $L['contactTitle_edit'];
		$this->setPublicExcludeFields();
	}
	
	// function main($id = '', $item = '', $error_msg = '') {
		// print '<div>main id=' . $id . ' ' . $item . ' ' . $error_msg . '</div>'; print_r($item); print '<hr>'; print_r($error_msg); print '<hr>';
		// parent::main($id, $item, $error_msg);
	// }
	
	function isBrowseable() {
		return false;
	}
	function getBrowseLinks() {
		return false;
	}
	function getNewLink() {
		return false;
	}
	function getNewMenuContent() {
		return false;
	}
	function addSecondMenuLinks() {
		return;
	}
	function getNextPageRow() {
		return false;
	}
	function markAsSeen($item, $extra_values = false) {}
	
	function getTitleRowLinks() { 
		return array(
			'help' => -1, 
			'bookmark' => 'off', 
			'index' => $this->getIndexLink(), 
			'logout' => $this->getLogoutLink(), 
			
		);
	}
	
	function getLastViewedPages() {
		return array();
	}
	function getLastViewedRow() {
		return '';
	}
	function getLastViewedLinks() {
		return array();
	}
	
	function setPublicExcludeFields() {
		if (!$this->allow_permissions) {
			$exclude = array(
				'lock_permissions', 
				'view_group_id', 
				'edit_group_id', 
				'owner_group_id', 
				'assigned_to_group_id', 
				'admin_group_id'
			);
			$this->dd->setDictionaryExcludeFields($exclude);
		}	
	}
	
	function setContact() {
		if ($this->ua->isWebFormUser()) {
			// print '<p>setContact is web form user</p>';
			// $contact_id_field = 'id'; // 'contact_id'
			$contact_id_field = $this->db_functions->getContactIdField();
			if ($id = $this->ua->users->getUsersContactId()) {
				// print '<p>setting id = ' . $id . '</p>';
				$this->form_values[$contact_id_field] = $id;
			}
		}
		// print '<p>setContact id = ' . $this->form_values[$contact_id_field] . '</p>';
	}
	
	/**
	 * Alters certain fields to remove links
	 *
	 * @param array $field The field array from a data definition, required for the Output Service getStringField function.
	 * @return string representation of this field, as defined in the Output Service
	 * @author Phil Calder
	 */
	function getStringField($field) {
		if ( 
			($field['type'] == 'email') || 
			($field['type'] == 'url') || 
			($field['type'] == 'contact')
		) {
			$field['no_link'] = 1; //'text';
		}
		return parent::getStringField($field);
	}
	
	function getHtmlFieldRow($field, $error_msg) {
		// print '<div>getHtmlFieldRow</div>';
		if ( 
			($field['type'] == 'email') || 
			($field['type'] == 'url') || 
			($field['type'] == 'contact')
		) {
			$field['no_link'] = 1; //'text';
			// print '<div>no link</div>';
		}
		return parent::getHtmlFieldRow($field, $error_msg);
	}
	
	function getTableLinks($item) {
		$links = parent::getTableLinks($item); // --------------------------
		bcrm_unset($links, 'list');
		return $links;
	}
	
	/**
	 * Builds the View link
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return string containing the link
	 */
	function getViewLink($id = false) {
		return $this->output->getLinkView(
			'contact_view.php' // todo: make this a config lookup
		);
	}
	/**
	 * Builds the View link
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return string containing the link
	 */
	function getEditLink($id = false) {
		return $this->output->getLinkEdit(
			'contact_edit.php' // todo: make this a config lookup
		);
	}
	
	/**
	 * Disables the delete link
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return string An empty string
	 */
	function getDeleteLink_q($id = false) {
		return '';
	}
	
	function isDeletable($id = false) {
		// print '<div>Delete disabled for web form</div>';
		return false;	
	}
	
	/*
	 * Adds a logout link
	 */
	// function getExtraItemLinks() {
		// $links = (array)parent::getExtraItemLinks();
		// 
		// if ($logout = $this->getLogoutLink()) {
			// $links[] = $logout;
		// }
		// return $links;
	// }
	
	/**
	 * @return mixed false or a logout link
	 */
	function getLogoutLink() {
		if ($this->ua->isWebFormUserLoggedIn()) {
			global $L;
			return '<a href="?logout=1">' . $L['logout'] . '</a>';	
		}
		return false;
	}
	
	/**
	 * @return string a link to the index page
	 */
	function getIndexLink() {
		global $L;
		return '<a href="index.php">' . $L['LinkHome'] . '</a>';
	}
	
	// Phil: see Contact/inc/item_edit.inc
	/**
	 * Set contact, then calls parent to change data definition based on contact type
	 */
	function setDataDefinition($data_option) {
		$this->setContact(); // Phil 20051114 sets form values id
		// print '<div>setDataDefinition for contact ' . $this->form_values['id'] . '</div>';
		parent::setDataDefinition($data_option);
	}
	
}
?>