<?php
/**
 * The Contact mass create item select menu
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('list.inc');

/**
 * The Contact mass create item select menu class
 * 
 * @package WebMAT
 * @author Phil Calder
 */
class ContactMassCreateSelectWebMATMenu extends WebMATMenu {
	
	/**
	 * Constructor function sets all the class variables
	 *
	 * @param  string  $module  the current module name
	 * @param  array  $post  the post array
	 * @param  array  $get  the get array
	 * @param  array $language_overwrite has the keys title and description
	 */
	function ContactMassCreateSelectWebMATMenu($module, $post, $get, $menu_links, $language_overwrite = false) {
		$this->WebMATStandardMenu($module, $post, $get, $menu_links, $language_overwrite);
	}
	
	/**
	 * Overrides the WebMATList constructor function to disable it
	 */
	function WebMATList() {}
	
	function setContactSessionId($session_id) {
		$this->contact_session_id = $session_id;
	}
	
	function getContactSessionId() {
		return $this->contact_session_id;
	}
	
	// function getConfigMenuLinks($suffix = 'menu_links') {}
	
	function getLinks() {
		// print '<div>getLinks</div>';
		// TODO build from config settings (see WebMATMenu::getLinks)
		$session_id = $this->getContactSessionId();
		// $this->menu_links = array(
			// 'tasks' => array(
				// 'href' => '../../modules/Tasks/mass_create.php?contact_session_id=' . $session_id, 
				// 'text' => 'Task'
			// )
		// );
		$this->menu_links = $this->getConfigMenuLinks($session_id);
		
		return $this->menu_links;
	}
	
	/**
	 * Adds links to each module that supports the Mass Create / Use Contacts functionality.
	 *
	 * Based on a config setting modules.Contact.mass_create.[ModuleName].[identifier] = array('label' => 'URL')
	 * 
	 * @return array An array of extra config links
	 */
	function getConfigMenuLinks($session_id) {
		global $USER;
		// modules.Contact.mass_create.[ModuleName].[identifier] = array('label' => 'URL');
		if (!isset($this->config_menu_links)) { // Phil - need to check not set - may be set but not have any extra links
			global $SERVICES;
			$config =& $SERVICES->getService('Config');
			$language =& $SERVICES->getService('Language');
			
			// $table_name = $this->getTableName();
			// $extra_links = $config->lookup('modules.' .  $this->module_name . '.file_set.' . $table_name . '.' . $suffix);
			$extra_links = $config->lookup('modules.Contact.mass_create');
			// print '<div>extra links for [modules.' .  $this->module_name . '.file_set.' . $table_name . '.' . $suffix . ']</div>'; print_r($extra_links); print '<hr>';
			$links = array();
			
			if (is_array($extra_links)) {
				$ua =& $SERVICES->getService('Authentication');
				
				foreach ($extra_links as $module_name => $pieces) {
					// print '<div>module_name=' . $module_name . ', v=' . $pieces . '</div>'; print_r($pieces); print '<hr>';
					if (is_array($pieces) && $ua->writeAccess($USER['user_id'], $module_name)) {
						foreach ($pieces as $piece => $data) {
							foreach ($data as $label => $href) {
								// print '<div>label=' . $label . ', href=' . $href . '</div>';
								$separator = '?';
								if (strpos($href, '?')) {
									$separator = '&';
								}
								$text = $language->getLanguageText($label, $module_name);
								$links[] = array(
									'href' => $href . $separator . 'contact_session_id=' . $session_id, // TODO: check the href does not already have a query string. If it does use & instead of ?
									'text' => $text
								);
							}
						}
					}
				}
			}
			$this->config_menu_links = $links;
		}
		return $this->config_menu_links;
	}
	
	function getMenuLinksTitle() {
		global $L;
		// Select a module
		$content = $L['contactLabelSelectModule'];
		return $content;
	}
	
	// function getTitleText() {
		// return 'Mass Create';
		// return $this->getMenuLinksTitle();
	// }
}
?>