<?php
/**
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Documents
 */

class DocumentsItemWebMATList extends WebMATList {
	/**
	 * @var array all the folders
	 */
	var $folders = array();

	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		global $SERVICES;
		parent::init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false);
		if (!count($this->getFolders())) {
			global $L, $SERVICES;
			$SERVICES->language->getLanguage('Documents');
			quickPage($L['documentsNoHomeFolder'] .  '<br>');
			exit();
		}
	}
	
		
	function getViewLink() {
		global $L;
		$link_href = $this->getFileSetHref('version_list');
		// print_r($link_href);
		//$this->db_functions->parent_items = null;  //this needs to be considered.
		
	    //before getParentItems 
		//$parent_item = $this->db_functions->getParentItems();
		//print_r($parent_item); echo '<hr>';print_r($parent_item['mime_type_id']);
		//if (!$this->mod->isBrowserViewable($parent_item['mime_type_id'])) {
		return $this->output->getLinkHref(array('text' => $L['documentsViewVersions'], 'href' => $link_href));
	}
	
	/**
	 * Builds the edit link
	 *
	 * Uses 'custom' edit link text (Edit Properties)
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return string containing the link
	 */
	function getEditLink($id = false) {
		global $L;
		$link_href = $this->getFileSetHref($this->edit_link);
		return $this->output->getLinkEdit($this->getModulePath() . $link_href, $L['documentsLabelEdit']);
	}
	
	/**
	 * Adds breadcrums for the folder navigation
	 * 
	 * @return string breadcrums each as a link
	 */
	function addToDescription() {
		// Phil 20060223 todo: this doesnt display the parent folder 
		// when we are in a sub-folder that itself has more than one sub-folder
		
		if (!$this->form_values['contact_id']) {
			$breadcrumbs = array_reverse($this->getParents());
			// $content = $this->output->getTextRow($this->output->getLinkBreadcrumbs($breadcrumbs));
			// Phil 20060223
			$padding = true;
			$content = $this->output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>';
			$content .= $this->output->getLinkBreadcrumbs($breadcrumbs);
			$content .= '</td>' . $this->output->getEndRow(true);
			
			if ($children = $this->getChildren($id = false)) {
				// $content .= $this->output->getTextRow($children);
				// Phil 20060223
				$padding = true;
				$content = $this->output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>';
				$content .= $children;
				$content .= '</td>' . $this->output->getEndRow(true);
			}
			
			
			
			
			return $content;
		} else {
			global $SERVICES;
			$SERVICES->getService('Preferences');
			$SERVICES->preferences->setUserPref($this->pref_name . 'Filter', 'fnone');
		}
	}
	
	
	/**
	 * gets the folders class vairable
	 */
	function getFolders() {
		if (!count($this->folders)) {
			$this->folders = array();
			$fields = 'id, name, parent_id';
			$tables = 'Documents__folder';
			$result = $this->db_functions->db->select($fields, $tables);
			while ($row = $this->db_functions->db->getRow($result)) {
				$this->folders[$row['id']] = array('name' => $row['name'], 'parent_id' => $row['parent_id']);
			}
		}
		return $this->folders;
	}
	
	
	/**
	 * gets all the parent folders for the current folder
	 * 
	 * @param integer the folder id (optional)
	 * @param array an array of previous parents this includes the child  (optional)
	 * @return array and array containing the child and all the parents with the 
	 * child first (with the keys 'text', 'href')
	 */
	function getParents($id = false, $folders = array()) {
		$all_folders =& $this->getFolders();
		if (!$id) {
			$id = $this->filters->getFilterInfo('value');
		}
		$folder = $all_folders[$id];
		$link_href = str_replace('%id%', $id ,$this->getFileSetHref('filter_list'));
		$folders[] = array('href' => $link_href, 'text' => $folder['name'] );
		if ($folder['parent_id'] != 0) {
			$folders = $this->getParents($folder['parent_id'], $folders);
		}
		return $folders;
	}
	
	
	/**
	 * gets all the parent folders for the current folder
	 * 
	 * @param integer the folder id (optional)
	 * @param array an array of previous parents this includes the child  (optional)
	 * @return array and array containing the child and all the parents with the 
	 * child first (with the keys 'text', 'href')
	 */
	function getChildren($id = false) {
		$children = '';
		if (!$id) {
			$id = $this->filters->getFilterInfo('value');
		}
		$all_folders =& $this->getFolders();
		foreach ($all_folders as $k => $v) {
			if ($v['parent_id'] == $id) {
				$link_href = str_replace('%id%', $k ,$this->getFileSetHref('filter_list'));
				$children .= '&gt; ' . $this->output->getLinkHref(array('href' => $link_href, 'text' => $v['name'] )) . '<br>';
			}
		}
		return $children;
	}
	
	function getTableLinks($item) {
		$links = parent::getTableLinks($item);
		// print '<div>table links for item mime type id: ' . $item['mime_type_id']. '</div>'; print_r($links); print '<hr>';
		if (!$this->mod->isBrowserViewable($item['mime_type_id'])) {
			bcrm_unset($links, 'documentsViewDocument');
			// $links['documentsViewDocument'] = '&nbsp;';
		}
		return $links;
	}

	/**
	 * Overrides the method in webmat_list.inc
	 * 
	 */
    function afterDataDictionary() {
		// Bosco 20050726 | To display the option of Standard and Custom list views
		global $SERVICES,$L;
		
		$link = $SERVICES->getGlobal('php_self');
		$query_string = $SERVICES->getGlobal('query_string');
		$query_string = preg_replace('/&{0,1}view_type=[a-z]*/', '', $query_string);
		$link.= ($query_string != "") ? "?" . $query_string . "&" : "?";

		if ($this->form_values['view_type'] == 'custom' || (empty($this->form_values['view_type']) && $SERVICES->preferences->getUserPref($this->pref_name . 'ViewType', 0) == 'custom')) {
			foreach ($this->dd->data_dictionary as $k => $v) {
				if ($v['field_key'] == 'folder_id') {
					bcrm_unset($this->dd->data_dictionary, $k);
				}
			}
			$SERVICES->preferences->setUserPref($this->pref_name . 'ViewType', 'custom');
			$this->addSecondMenuLinks(array('href' => $link . 'view_type=standard', 'text' => $L['DocumentsListViewTypeStandard']));
		} else {
			$this->db_functions->disableFilter();
			unset($this->filters->filter_info);
			$SERVICES->preferences->setUserPref($this->pref_name . 'ViewType', 'standard');
			$this->addSecondMenuLinks(array('href' => $link . 'view_type=custom', 'text' => $L['DocumentsListViewTypeCustom']));
		}	
	}
	
	/**
	 * Forces list to NOT use an iterator for getItems
	 *
	 * ie process and return the entire items array
	 * Added 200512xx. This modules list page relies on bulk processing 
	 * of items in the {@link afterGetItems} method
	 *
	 * @return boolean false
	 * @author Phil Calder
	 */
	function useGetItemsIterator() {
		return false;
	}
}

?>