<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 */

/**
 * The edit class for Documents versions
 *
 * @package Documents
 */
class DocumentsVersionWebMATEdit extends WebMATEdit {
	/**
	 * @var array $version_file the version file
	 */
	var $version_file;
	/**
	 * @var array $make_current 
	 */
	var $make_current;
	
	
	/**
	 * sets the value for $item['make_current_version']
	 */
	function beforeForm(&$id, &$item, &$error_msg) {
		global $USER;
	
		// Bosco 20050706 | If id is not passed, set the id (For current version check bug)
		if (!$id) {
			if(!($id = $item['id'])) {
				if (!($id = $this->form_values['id'])) {
					$id = $this->form_values['duplicate'];
				}
			}
		}
		// Bosco 20050706 | Changed to make the check box checked if file is the current version of document
		if ($id) {
			$this->db_functions->setMasterIds($item['item_id']);
			$parent_item = $this->db_functions->getParentItems();
			if ($parent_item['version_id'] == $item['id']) {
				//bcrm_unset($fields, 'make_current_version');// Bosco 20050706 | commented, code does nothing
				$item['make_current_version'] = 1; // Bosco 20050706 | Added to tick the check box
			} else {
				$item['make_current_version'] = 0;
			}
		} else {
			$item['make_current_version'] = 1;
		}
		
		// Bosco | 20050707 | Changed to make MIME_TYPE and Version Info read only.	
		if (empty($parent_item)) {		
			$this->db_functions->setMasterIds($item['item_id']);
			$parent_item = $this->db_functions->getParentItems();
		}
		// Bosco | 20050702 | Added to change MIME_Type field as read only.	
		if(!$this->isDocumentAdmin($parent_item)) {
			$this->dd->changeAttributes(array('field_key' => 'mime_type_id', 'read_only' => 1));
		}
		// Bosco | 20050702 | Added to change Version field as read only if the USER is not the OWNER or ADMIN.	
		if(!($this->isDocumentAdmin($parent_item) || $this->isDocumentOwner($parent_item))) {
			$this->dd->changeAttributes(array('field_key' => 'version', 'read_only' => 1));
		}

	}

	/**
	 * Checks if the current user is Owner of the document
	 * @author Phil 20050727
	 * @param array The parent_item array
	 * @return bool true if owner, false otherwise
	 */
	function isDocumentOwner(&$parent_item) {
		
		if (is_object($item_db_functions =& $this->mod->getDatabaseTable('item'))) {
			
			return $item_db_functions->isOwner($parent_item['id'], $parent_item);
			
		}
		
		return false;
		
	}	
	
	/**
	 * Checks if the current user is the admin of the document
	 * @author Phil 20050727
	 * @param array The parent_item array
	 * @return bool true if Admin, false otherwise
	 */
	function isDocumentAdmin(&$parent_item) {
		
		if (is_object($item_db_functions =& $this->mod->getDatabaseTable('item'))) {
			
			return $item_db_functions->isAdmin($parent_item['id'], $parent_item);
			
		}
		
		return false;
		
	}	
	
	/**
	 * removes version_file from the items array and
	 *
	 * @param array $item
	 * @param array $fields
	 * @param array $error_msg 
	 */
	function beforeDBWrite(&$item, &$fields, &$error_msg) {
		global $L,$SERVICES;
		//print_r($item);
		if (!$item['id']) {
			$this->version_file = array();
			$this->version_file = $item['version_file'];     // where is the $item['version_file'] from    --Cecil
			bcrm_unset($item, 'version_file');
			$item['size'] = $this->version_file['size'];          
			$item['file_name'] = $this->version_file['name'];
		}
		
		$this->make_current = $item['make_current_version'];
		bcrm_unset($item, 'make_current_version');
	}
	
	/**
	 * any processing required after adding to the db
	 *
	 * @param array $item
	 * @param array $fields
	 * @param array $error_msg 
	 */
	function afterDBWrite(&$item, &$fields, &$error_msg, $id = false) {  //passed by reference
		//print_r($this->version_file);
		if (is_array($this->version_file) && !$this->version_file['error']) {
			//print_r($this->version_file);
			global $SERVICES;
			$SERVICES->getService('FileSystem');
			$mime_type_id = $this->mod->getMimeType($this->version_file['type'],	$this->version_file['name']);
			$SERVICES->filesystem->moveFile($id, 'files', $this->version_file['location']);
			// Bosco 20050708 | Added uploaded_on Field
                        $uploaded_on = time();
			// Bosco 20050817 | Added to store file size,file name, extension in parent record
			$file_size = $this->version_file['size'];
			$file_name = $this->version_file['name'];
			$this->db_functions->setItem(array('id' => $id, 'mime_type_id' => $mime_type_id, 'uploaded_on' => $uploaded_on));
		} else {
			$mime_type_id = $item['mime_type_id'];
			// Bosco 20050708 | Added uploaded_on Field
                        $uploaded_on = $item['uploaded_on'];
			// Bosco 20050817 | Added to store file size, name, extension in parent record
			$file_size = $item['size'];
			$file_name = $item['name'];
		}
		
		if ($this->make_current) {
			$this->db_functions->setMasterIds($item['item_id']);
			$parent_item['version_id'] = $id;
			$parent_item['mime_type_id'] = $mime_type_id;
			// Bosco 20050708 | Added uploaded_on Field
			$parent_item['uploaded_on'] = $uploaded_on;
			// Bosco 20050817 | Added to store file size, name, extension in parent record
			$file_ext  = array_pop(explode('.', $file_name));
			$parent_item['file_extension'] = $file_ext; 
			$parent_item['file_name'] = basename($file_name, '.' . $file_ext);
			$parent_item['file_size_bytes'] = $file_size;
			$parent_id = $this->db_functions->updateParentItem($parent_item);
		}
	}
	
	function editTableLinks(& $links, $id, $item) {
		$parent_item = $this->db_functions->getParentItems();
		if (!$this->mod->isBrowserViewable($parent_item['mime_type_id'])) {
			$links['documentsViewDocument'] = '';
		}
		if (!count($this->db_functions->getEditableParents())) {
			bcrm_unset($links, 'edit');
			bcrm_unset($links, 'delete');
		} else {
			if ($parent_item['version_id'] == $id) {
				$links['delete'] = '';
			}
		}
		
	}
}
?>