<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Front
 */

/**
 * Manages information for display regarding system and administrative messaging
 * 
 * @package Front
 * @author Phil Calder
 */ 
class FrontAbout {
	
	/**
	 * @access private
	 */
	function getLicenceId() {
		global $SERVICES;
		return $SERVICES->getLicenceId();
	}
	
	/**
	 * @access private
	 */
	function getLicensedUsers() {
		global $SERVICES;
		return $SERVICES->getNumLicensedUsers();
	}
	
	/**
	 * @access private
	 */
	function getCurrentUsers() {
		global $SERVICES;
		return $SERVICES->getCurrentUsers();
	}
	
	/**
	 * @access private
	 */
	function getCurrentUsersCount() {
		global $SERVICES;
		return $SERVICES->getCurrentUsersCount();
	}
	
	/**
	 * @access private
	 */
	function getExpiryDate() {
		global $SERVICES;
		return $SERVICES->getExpiryDate();
	}
	
	/**
	 * @access private
	 */
	function getRenewalDate() {
		global $SERVICES;
		return $SERVICES->getRenewalDate();
	}
	
	/**
	 * @access private
	 */
	function isAdminUser() {
		global $SERVICES, $USER;
		$ua =& $SERVICES->getService('Authentication');
		if ( $ua->adminAccess( $USER['user_id'], 'UserAdmin') ) {
			return true;
		}	
		return false;
	}
	
	function getCurrentUsersMessage() {
		global $USER, $L;
		$users = $this->getCurrentUsers();
		$id = $USER['user_id'];
		
		$item = array();
		
		$item[] = $L['frontCurrentUsers'] . ': ';
		
		$users_list = '';
		if (is_array($users)) {
			foreach ($users as $k => $v) {
				if ($k == $id) {
					$users_list .= ' ' . $v . '*,';
				} else {
					$users_list .= ' ' . $v . ',';
				}
			}
		}
		$users_list = trim($users_list, ',');
		$item[] = $users_list;
		// $item[] = '';
		return $item;
	}
	
	function main() {
		global  $USER, $SERVICES, $L;
		
		$SERVICES->getService('Language');
		$SERVICES->language->getLanguage('Admin');
		$SERVICES->language->getLanguage('Licence');
		
		// 'Front' creates module and loads Front language
		$SERVICES->getService('Modules');
		$mod =& $SERVICES->modules->getModule('Front'); 
		$output =& $SERVICES->getService('Output');
		
		$admin_user = $this->isAdminUser();
		
		$items[] = array(
			$L['frontSystemInfo'], 
			false,
			'header'
		);
		// Current version
		$items[] = array($L['labelVersion'], $SERVICES->getCodeVersion());

		if ($admin_user) {
			if (@file_exists('../../setup/index.php')) {
				$SERVICES->language->getLanguage('Setup');
				$SERVICES->getService('Preferences');
						
				$items[] = array(
					'', 
					'<a href="../../setup/index.php">' . $L['setupTitleSetup'] . '</a> ' .
					'<a href="../../setup/index.php" target="_blank" title="' . $L['LinkOpenInNewWindow'] . '">[+]</a>'
				);
				//TODO
				$items[] = array(
					'', 
					'<a href="../../setup/index.php?step=2&check_php_config=1&report_only=1&lang=' . $SERVICES->preferences->getUserPref('cfgLanguage') . '">' . $L['frontAboutWebServerSettings'] . '</a> ' .
					//$L['frontAboutWebServerSettings'] .
					'<a href="../../setup/index.php?step=2&check_php_config=1&report_only=1&lang=' . $SERVICES->preferences->getUserPref('cfgLanguage') . '" target="_blank" title="' . $L['LinkOpenInNewWindow'] . '">[+]</a>'
				);
			}
		}	
			
		// Server and User time
		$items[] = array( $L['frontCurrentServerTime'], date("Y-m-d H:i"));
		$items[] = array( $L['frontCurrentUserTimeInTimezone'], date("Y-m-d H:i", $SERVICES->offsetTimeZoneForRead(time())));

		// Current users
		$items[] = $this->getCurrentUsersMessage();
		
		// Licence Information
		$items[] = array(
			$L['licenceViewProperties'], 
			false,
			'header'
		);
		
		// Licence ID
		// dont display if __12345__
		$items[] = array( $L['licId'], (($this->getLicenceId() && $this->getLicenceId() != '__12345__') ? $this->getLicenceId() : '&nbsp;'));
		
		//show Expiry Date if exists
		// expiry date
		//echo 'exp'. $this->getExpiryDate();
		if ( ($expiry_date = $this->getExpiryDate()) ) {
			if ($this->getAlert('expiry_date')) {
				$expiry_alert = 'rAlert';
			}
			
			$SERVICES->getService('Preferences');
			$items[] = array( 
				$L['licExpiryDate'], 
				date($SERVICES->preferences->getUserPref('cfgShortDateFormat'), $expiry_date), 
				$expiry_alert
			);
			
			if ($expiry_alert == 'rAlert') {
				$link = '<a href="' . $SERVICES->getContactUrl('buy') . '" target="_blank">' . $L['frontToOrder'] . '</a>';
				$items[] = array('', $link);
			}
		}
		
		//renewal date message/warning (usually annual)
		if ($admin_user && ($renewal_date = $this->getRenewalDate()) ) {
			if ($this->getAlert('renewal_date')) {
				$renewal_alert = 'rAlert';
			}
			
			$items[] = array( 
				$L['licRenewalDate'], 
				date($SERVICES->preferences->getUserPref('cfgShortDateFormat'), $renewal_date), 
				$renewal_alert
				//($renewal_date ? $renewal_date : '&nbsp;')
			);
			
			if ($renewal_alert == 'rAlert') {
				$link = '<a href="' . $SERVICES->getContactUrl('renew') . '">' . $L['frontToRenew'] . '</a>';
				$items[] = array('', $link);
			}
		}
		
		// Licensed and Current users, [with warning and buy link]
		if ($admin_user) {
			$current_users = $this->getCurrentUsersCount();
			if (!$current_users) {
				$current_users = '0';
			}
			$licensed_users = $this->getLicensedUsers();
			if (!$licensed_users) {
				$licensed_users = '0';
			}
			
			$style_flag = '';
			$users_warning = '';
			
			if ($this->getAlert('user_count_exceeded')) {
				$users_warning = '
				<a href="' . $SERVICES->getContactUrl($SERVICES->isLicenceFileExisting() ? 'user_count_exceeded' : 'buy') . '"> ' . 
				($SERVICES->isLicenceFileExisting() ? $L['licenceGetUpgrade'] : $L['licenceGetNewKey']) . '</a>';
				
				$style_flag = 'rAlert';
			} elseif ($this->getAlert('user_count_reached')) {
				$users_upgrade_prompt = '<a target="_blank" href="' . $SERVICES->getContactUrl($SERVICES->isLicenceFileExisting() ? 'upgrade_users' : 'buy') . '"> ' . 
				($SERVICES->isLicenceFileExisting() ? $L['licenceGetUpgrade'] : $L['licenceGetNewKey']) . '</a>';
			}
			
			$items[] = array( 
				$L['licUsers'], 
				$licensed_users
			);
			
			$items[] = array( 
				$L['frontInstalledUsers'], 
				$current_users, 
				$style_flag
			);
			
			if (strlen($users_warning)) {
				$items[] = array( 
					$L['licenceLimitExceeded'], 
					$users_warning, 
					$style_flag
				);
			} else if (strlen($users_upgrade_prompt)) {
				$items[] = array( 
					$L['licenceLimitReached'], 
					$users_upgrade_prompt,
					$style_flag,
				);
			}
			
			//if (setup dir exists)
			//TODO  view licence
			//if (licence dir exists)
			if (@file_exists('../../licence/view.php')) {
				$items[] = array(
					'', 
					'<a href="../../licence/view.php?framed=1">' . $L['licenceViewProperties'] . '</a> ' .
					'<a href="../../licence/view.php?framed=1" target="_blank" title="' . $L['LinkOpenInNewWindow'] . '">[+]</a>'
				);
			}
			
			//TODO  send error_log to support
		}
		
		if ($admin_user) {
			$bad_passwords = $mod->getBadPasswordsWarning('array');
			if ($bad_passwords) {	
				$items[] = array(
					$L['frontWarningInfo'], 
					'',
					'header'
				);
				
				$items[] =  array(
					$L['frontDefaultPasswordWarning'], 
					''
				);
				
				foreach ($bad_passwords as $k => $v) {
					$items[] = array(
						$v['username'],  
						'<a href="../../modules/UserAdmin/password.php?user_id=' . $v['id'] . '">' . $L['adminModuleLabel'] .' | ' .	$L['frontChangePassword'] . '</a>', 
						'rAlert'
					);
				}
			}
		}
		
		$content = '';
		
		//$content .= $output->getBeginTable('settingsTbl');
		//$content .= $output->startDataBlock();
		if (is_array($items)) {
			// print_r($items);
			$first_time_flag = 1;
			foreach($items as $k => $v) {
				$style_flag = '';
				if (isset($v[2]) && $v[2]) {
					$style_flag = $v[2];
				}
				bcrm_unset($v, 2);
				
				if ($style_flag == 'header') {
					// todo: Add/change class to display as a list page header 
					// eg blue skin: blue bg, white bold text, margin top and bottom
					// possibly use:
					// output getBeginRow, getTableHeaderCell, getEndRow
					
					if (!$first_time_flag) {
						//$content .= '<tr class="spacer"><td colspan="3"><div>&nbsp;</div></td></tr>';
						$content .= $output->getEndTable();
						$content .= '<p>&nbsp;</a>';
					}
					
					//<tr class="double"><th colspan="3">' . $v[0] . '</th></tr>
					$content .= '
					
					<h2>' . $v[0] . '</h2>
					';
					$content .= $output->getBeginTable('settingsTbl');
					$first_time_flag = 0;
				} else if (empty($v[1])) {
					//$content .= '<tr><td colspan="3">' . $v[0] . '</td></tr>';
					$content .= $output->getTextRow($v[0], 3);
				} else {
					$v['__flags'] = '';
					$content .= $output->getTableRow($v, $style_flag);
				}
			}
		}
		//$content .= $output->endDataBlock();
		
		
		$content .= $output->getEndTable();
		
		//todo
		$content .= '
			<div align="center" style="padding:20px;" class="">
				<a href="http://www.browsercrm.com/content/view/181/63/" target="_blank"><img src="' . $USER['skin_web_path']. 'img/gophp5.png" title="BrowserCRM supports GoPHP5" border="0"></a>
			</div>
			';
		
		
		// ---
		$title = $L['frontAbout'];
		$padding = true;
		
		$window_layout = $output->getBeginTable('', true);
		$window_layout .= $output->getTitleRow($title);
		$window_layout .= $output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>';
		$window_layout .= $content;
		$window_layout .= '</td>' . $output->getEndRow(true);
		$window_layout .= $output->getEndTable(true);
		
		$output->addToBody($window_layout);
		$output->display();
		exit();
	}
	
	/**
	 * Some alerts are only generated for Adin users
	 *
	 * @param string $name The alert name
	 * @return boolean Whether the chosen alert is active
	 * @author Phil Calder
	 */
	function getAlert($name = '') {
		switch ($name) {
			case 'expiry_date':
				if ($expiry_date = $this->getExpiryDate()) {
					//print '<div>$expiry_date = ' . $expiry_date . '</div>';
					//21 days
					return ($expiry_date < (time() + (21*24*3600)) );
				}
			break;
			case 'renewal_date':
				if ($renewal_date = $this->getRenewalDate()) {
					// 21 days - long notice for (annual) renewal
					return ($renewal_date < (time() + (21*24*3600)) );
				}
			break;
			/*MCR 20060511
			//let's only implement expiry date for now, since we can now send an updated lic
			case 'expire_in_days':
				//print '<div>Expire in days: ' . $this->getExpireInDays() . '</div>';
				return ($this->getExpireInDays() < 5);
			break;
			*/
			case 'user_count_exceeded':
				if ($this->isAdminUser()) {
					$current_users = $this->getCurrentUsersCount();
					$licensed_users = $this->getLicensedUsers();
					// print '<div>getAlert(user_count): ' . ( ((int)$current_users > (int)$licensed_users) ? 'alert' : 'ok') . '</div>';
					return ((int)$current_users > (int)$licensed_users);
				}
			break;
			
			case 'user_count_reached':
				if ($this->isAdminUser()) {
					$current_users = $this->getCurrentUsersCount();
					$licensed_users = $this->getLicensedUsers();
					// print '<div>getAlert(user_count): ' . ( ((int)$current_users > (int)$licensed_users) ? 'alert' : 'ok') . '</div>';
					return ((int)$current_users >= (int)$licensed_users);
				}
			break;
		}
		return false;
	}
	
	/**
	 * @return boolean Whether there are menu alerts for this class
	 * @author Phil Calder
	 */
	function getMenuAlerts() {
		$menu_alerts = array(
			'expiry_date', 
			'expire_in_days', 
			'user_count_exceeded'
		);
		foreach ($menu_alerts as $k => $v) {
			if ($this->getAlert($v)) {
				// print '<div>alert: ' . $v . ' is true</div>';
				return true;
			}
		}
		return false;
	}
}
?>