<?php
/**
 * A collection of classes extended from WebMATDatabaseTable
 * 
 * You need an extended class for each table in you module not including the 
 * dropdown tables.
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Template
 */


/**
 * This class handles the database functions for the item table
 * 
 * This module is an extend verison of WebMATDatabaseTable with the table_name 
 * set and the alert_where set.
 * 
 * @author Matthew Corich
 */
class itemKnowledgeBaseDatabaseTable extends WebMATDatabaseTable {
	
	/**
	 * The constructor class for itemTemplateDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function itemKnowledgeBaseDatabaseTable($module_name, $data_definition) {
		global $USER;
				
		// $this->table_name = 'item';
		// TODO: check this is not already prefixed
		 $this->table_name = $module_name . '__' . 'item';
		
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseTable
	}
	
	function eventsSeen($id) {}
	function crmCascadingDelete($contact_id) {}
	
	function getItemsForDropdown($id, $active=1, $order_by = '', $limit = '') {    //contact id
		global $SERVICES, $USER;
		//set a blank array to begin with
		$items = array();
		
		if (is_array($order_by)) {
			$sort = $order_by['field'] . ' ' .$order_by['order'];
		}
		$where = '';
		
		$SERVICES->getService('Config');
		// if ($SERVICES->config->lookup('modules.' . $this->module_name .'.file_set.item.display_by_group') > 0) {
		if ($this->getDisplayByGroupFlag() > 0) { // Phil 20051013
			$where = " AND (
					0 
					OR (view_group_id IN (" . $USER['group_string'] . ")) 
					OR (edit_group_id IN (" . $USER['group_string'] . "))  
					OR (owner_group_id IN (" . $USER['group_string'] . "))  
					OR (admin_group_id IN (" . $USER['group_string'] . "))  
				)";
		}
		$result = $this->db->select('*', 'KnowledgeBase__item', "dropdown_id='$id' AND active='$active'" . $where, $sort, '', $limit);
		if ($this->db->hasError())	$this->db->printError();
		while ($row = $this->db->getRow($result)) {
			$items[] = $row;
		}
		return $items;            
	}
	
	// used in pub folder
	// returns count for given dropdown id (ie category)
	function getItemCount($id = 0) {
		global $SERVICES, $USER;
		$result = 0;
		$where = '';
		
		$SERVICES->getService('Config');
		// if ($SERVICES->config->lookup('modules.' . $this->module_name .'.file_set.item.display_by_group') > 0) {
		if ($this->getDisplayByGroupFlag() > 0) { // Phil 20051013
			$where = " AND (
					0 
					OR (view_group_id IN (" . $USER['group_string'] . ")) 
					OR (edit_group_id IN (" . $USER['group_string'] . "))  
					OR (owner_group_id IN (" . $USER['group_string'] . "))  
					OR (admin_group_id IN (" . $USER['group_string'] . "))  
				)"; 
		}
		// print '<div>item count for: ' . $id . '</div>';
		if ($id) {
			$result = $this->db->quickget('count(*)', 'KnowledgeBase__item', "dropdown_id='$id' AND active=1" . $where);
		} else {
			$result = 0;
		}
		
		return $result;
	}
	
	
	function isCategoryEditable($id, $category_id = false) {
		global $USER, $SERVICES;
		if (!$category_id) {
			$category_id = $this->db->quickGet('dropdown_id','KnowledgeBase__item',"id = '$id'");
		}
		if (!$category_id) {
			return false;
		}
		if ( in_array('edit_group_id', $this->db->getColumns('KnowledgeBase__dropdown')) ) {
			$check = $this->db->quickget(
				'id',
				'KnowledgeBase__dropdown',
				"id = $category_id AND (
					0 
					OR (view_group_id IN (" . $USER['group_string'] . ")) 
					OR (edit_group_id IN (" . $USER['group_string'] . "))  
				)" 
			);
			if ($this->db->hasError()) {
				$this->db->printError();
				return false;
			} else {
				if ($check != $category_id) {
					return false;
				}
			}
		}
		return true;
	}
	
	
	function isCategoryViewable($id, $category_id = false) {
		global $USER, $SERVICES;
		if (!$category_id) {
			$category_id = $this->db->quickGet('dropdown_id','KnowledgeBase__item',"id ='$id'");
		}
		if (!$category_id) {
			return false;
		}
		// print_r($this->db->getColumns('KnowledgeBase__dropdown'));
		
		if ( in_array('view_group_id', $this->db->getColumns('KnowledgeBase__dropdown')) ) {
			$check = $this->db->quickget(
				'id',
				'KnowledgeBase__dropdown',
				"id = $category_id AND (
					0 
					OR (view_group_id IN (" . $USER['group_string'] . ")) 
					OR (edit_group_id IN (" . $USER['group_string'] . "))  
				)" 
			);
			if ($this->db->hasError()) {
				$this->db->printError();
				return false;
			} else {
				if ($check != $category_id) {
					return false;
				}
			}
		}
		return true;
	}
	
	/**
	* checks if the an DB record is editable
	* 
	* @pram intger $id the item id
	* @return bool
	*/
	function isEditable( $id ) {
		$id = $this->cleanId($id); // Phil 20070305
		global $USER, $SERVICES;
		$ua =& $SERVICES->getService('Authentication');
		if (!$ua->writeAccess($USER['user_id'],'KnowledgeBase')) {
			return false;
		}
		if (!$this->isCategoryEditable( $id )) {
			return false;
		}

		// Bosco 20050711 | Changed to get correct display_by_group config value
		// $SERVICES->getService('Config');
		// if (!$id || $SERVICES->config->lookup($this->getFilesetConfigPath() . 'display_by_group') == 0 ) {
		if (!$id || $this->getDisplayByGroupFlag() == 0) { // Phil 20051013
			return true;
		}
		// Bosco 20050711 | Changed to get correct edit permissions
		if ( in_array('edit_group_id', $this->columns) && in_array('owner_group_id', $this->columns) && in_array('admin_group_id', $this->columns) ) {
			$check = $this->db->quickget(
				'id',
				'KnowledgeBase__item',
				"id = $id AND (
					0 
					OR (edit_group_id IN (" . $USER['group_string'] . "))  
					OR (owner_group_id IN (" . $USER['group_string'] . "))  
					OR (admin_group_id IN (" . $USER['group_string'] . "))  
				)" 
			);
			if ($this->db->hasError()) {
				$this->db->printError();
				return false;
			} else {
				if ($check != $id) {
					return false;
				}
			}
		}
		return true;
	}
	
	/**
	 * checks if the an DB record is viewable
	 * 
	 * @pram intger $id the item id
	 * @return bool
	 */
	function isViewable( $id ) {
		$id = $this->cleanId($id); // Phil 20070305
		global $USER, $SERVICES;
		$ua =& $SERVICES->getService('Authentication');
		if (!$id) {
			return false;
		}
		if (!$ua->readAccess($USER['user_id'],'KnowledgeBase')) {
			return false;
		}
		if (!$this->isCategoryViewable( $id )) {
			return false;
		}
		// Bosco 20050711 | Changed to get exact view permissions
		if ( in_array('view_group_id', $this->columns) && in_array('edit_group_id', $this->columns) && in_array('owner_group_id', $this->columns) && in_array('admin_group_id', $this->columns )) {
			global $USER;
			$check = $this->db->quickget(
				'id',
				'KnowledgeBase__item',
				"id = $id AND (
					0 
					OR (view_group_id IN (" . $USER['group_string'] . "))  
					OR (edit_group_id IN (" . $USER['group_string'] . "))  
					OR (owner_group_id IN (" . $USER['group_string'] . "))  
					OR (admin_group_id IN (" . $USER['group_string'] . "))  
				)" 
			);
			if ($this->db->hasError()) {
				$this->db->printError();
				return false;
			} else {
				if ($check != $id) {
					return false;
				}
			}
		}
		return true;
	}
}
?>