<?php
/**
 * This acts as an interface to whichever mailer class is being used.
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Mail
 */

includeExternal('phpmailer', 'class.phpmailer.php');

class BCRMMailer extends PHPMailer {

	var $mod; // reference to the mail module
	
	/**
	 * Default constructor
	 */
	function BCRMMailer() {
		global $SERVICES, $USER;
		
		$this->mod = & $SERVICES->modules->getModule('Mail');
		
		//$this->mailer->SetLanguagePath('other/language/');
		
		//setStoreParts
		$this->SetStoreParts();
		
		//setLanguagePath
		$this->SetLanguagePath(getExternalDir() . '/phpmailer/language/');
		
		//set Lang
		$SERVICES->getService('Preferences');
		$lang = $SERVICES->preferences->getUserPref('cfgLanguage');
		$this->SetLang($lang);	
		
	}
	
	
	/**
	 * Loops through all the attachments and adds them to the email
	 *
	 * @param array $item
	 */
	function processAttachments(&$item) {
		global $SERVICES;
		
		//echo '<p><h1>bcrm_MAILER</H1>';print_r($item); 
		//echo '<p>SESSION HASH: ';		print_r($_SESSION[$item['hash']]);
		
		if ($item['hash'] && is_array($_SESSION[$item['hash']]['files'])) {
			foreach ($_SESSION[$item['hash']]['files'] as $k => $v) {
				// check for process flag - attachment has already been processed
				//MCR 20051010 if (!$v['process']) {
					//$this->AddEmbeddedImage($v['location'], md5($v['name']), $v['name'], 'base64', $v['type']);
					$this->AddAttachment($v['location'], $v['name']);
				//}
				//echo '<p>file: '; print_r($v); echo '</p>';
			}
			$item['attachments'] = 1;
		}
		
		if ($item['hash'] && is_array($_SESSION[$item['hash']]['strings'])) {
			foreach ($_SESSION[$item['hash']]['strings'] as $k => $v) {
				// check for process flag - attachment has already been processed
				//MCR 20051010 if (!$v['process']) {
				$this->AddStringAttachment($v['body'], $v['name']);
					//, $v['content-transfer-encoding'], 
					//				  $v['content-type']);
				//}
				//echo '<p>Strings: '; print_r($v); echo '</p>'; 
			}
			$item['attachments'] = 1;
		}
		
		if ($item['hash'] && is_array($_SESSION[$item['hash']]['documents'])) {
			foreach ($_SESSION[$item['hash']]['documents'] as $k => $v) {
				// process document attachments
				// get file path
				//echo $v['version_id'] . '<br>';
				$path = $SERVICES->filesystem->getFilePath($v['version_id'], 'files'); //'Documents'
				//echo $path;
				// attach file
				//$this->AddEmbeddedImage($path, md5($v['filename']), $v['filename'], 'base64', $v['type']);
				$this->AddAttachment($path, $v['filename']);
				
				// // check for process flag - attachment has already been processed
				// if (!$v['process']) {
					// $this->mail->AddEmbeddedImage($v['location'], md5($v['name']), $v['name'], 'base64', 
									  // $v['type']);
				// }
				//echo '<p>Documents module / Files: '; print_r($v); echo '</p>';
			}
			$item['attachments'] = 1;
		}
		
		bcrm_unset($item, 'hash');
		bcrm_unset($item, 'curr_attachments');
		bcrm_unset($item, 'attachment');
		bcrm_unset($item, 'attach');
	}
	
	
	/**
	 * Sets up to get ready for sending an email
	 *
	 * @param array $item
	 * @param bool $include_signature determines whether the signature is included in the content
	 *
	 * @return array Returns a list of all email addresses that the email will be sent to.
	 */
	function fillEmailMessage(& $item, $include_signature = true) {
		$ret_val = array();
		// process attachments
		$this->processAttachments($item);
		
		// add the 'to' addresses
		$to = $this->mod->splitAddresses($item['recipient']);
		if (!empty($to) && is_array($to)) {
			foreach ($to as $k => $v) {
				$this->AddAddress($v['address'], $v['from']);
				$ret_val[] = $v['address'];
			}
		}
		
		// add the 'cc' addresses
		$cc = $this->mod->splitAddresses($item['cc']);
		
		if (!empty($cc) && is_array($cc)) {
			foreach ($cc as $k => $v) {
				$this->AddCC($v['address'], $v['from']);
				$ret_val[] = $v['address'];
			}
		}
		
		//echo "<br>vaddr<hr>";print_r($v);
		// add the 'bcc' addresses
		$bcc = $this->mod->splitAddresses($item['bcc']);
		if (!empty($bcc) && is_array($bcc)) {
			foreach ($bcc as $k => $v) {
				$this->AddBCC($v['address'], $v['from']);
				$ret_val[] = $v['address'];
			}
		}
		
		// Set subject
		if (get_magic_quotes_gpc()) {
			$this->Subject = stripSlashes($item['subject']);
		} else {
			$this->Subject = $item['subject'];
		}

		// check if magic_qutoes is on, if so, we need to strip the slashes
		// from the input, since it's not actually being put in a db
		// this is a further step to the above addresses, because those shouldn't
		// have slashes at all, whereas these could
		// - subject is taken care of futher down, since it goes in the db
		if (get_magic_quotes_gpc()) {
			$content = stripSlashes($item['content']);
			//MCR 20050805 $content = str_replace("\n", "<br />", trim($content)); //dumb SOB that put this here in the first place
			$signature = stripSlashes($item['signature']);
		} else {
			$content = $item['content'];
			$signature = $item['signature'];
		}
		
		// should we include the signature?
		if ($include_signature) {
			//$this->Body = $content . "\n\n" . $signature;
			// $this->Body = $content . trim($signature);//Saravanakumar :: 20050708
			// Phil 20050722 - readded newline before signature
			//$this->Body = $content .  "\n" . trim($signature);// Phil
			$this->Body = $content .  "\n" . $signature; //MCR 20050805 don't trim 
										//cause sig might deliberately have spacer lines
		} else {
			$this->Body = $content;
		}
		return $ret_val;
	}
	
	/**
	 * @access private
	 * @author Phil Calder
     */
    function setIsCreateForSend() {
		$this->is_create_for_send = true;
	}
	
	/**
	 * @access private
	 * @author Phil Calder
     * @return boolean
     */
    function isCreateForSend() {
		if (isset($this->is_create_for_send) && $this->is_create_for_send) {
			return true;
		}
		return false;
	}
	
	/**
     * Overridden to handle tagline
	 * @access private
	 * @author Phil Calder
     * @return string
     */
    function CreateBody() {
		if ($this->isCreateForSend() && $this->includeTagline()) {
			$this->Body = $this->Body .  "\n\n" . $this->getTagline(); 
		}
		return parent::CreateBody();
	}
	
	/**
	 * @return boolean whether to include a tagline to outgoing email
	 * @access private
	 * @author Phil Calder
	 */
	function includeTagline() {
		global $SERVICES;
		// Preferences created in constructor
		return $SERVICES->preferences->getUserPref('mailIncludeTagline');
	}
	
	/**
	 * @return string email tagline
	 * @access private
	 * @author Phil Calder
	 */
	function getTagline() {
		global $SERVICES;
		$SERVICES->getService('Config');
		return $SERVICES->config->lookup('modules.Mail.tagline');
		// return 'Sent by BrowserCRM http://www.browsercrm.com';
	}
	
	
	/**
	 * Configures this->mail to be ready for sending email.
	 *
	 * @param array $item
	 * @param array $sendAccount the account we're using to send
	 */
	function configure(& $item, & $sending_account) {
		global $SERVICES;
		// if it's an html email
		if($this->mod->isHtmlPart($item['content'])){
			$this->isHTML(true);
		}
		
		$ssl_host_prefix = 'ssl://'; // Phil 20070125

		switch ($sending_account['transport_method']) {
			case 1:
				$this->isMail();
				break;
			case 2:
				$this->isSendMail();
				break;
			case 3:
				$this->isSMTP();
				$this->Host = $sending_account['host'];
				
				// Phil 20070125 Adding ssl and port
				$this->Port = ($sending_account['port'] ? (int)$sending_account['port'] : $this->Port);
				if (isset($sending_account['use_ssl']) && $sending_account['use_ssl']) {
					$this->Host = $ssl_host_prefix . str_replace($ssl_host_prefix, '', $this->Host);
				}
				
				break;
			case 4:
				$this->SMTPAuth = true;
				$this->isSMTP();
				
				$this->Host = $sending_account['host'];
				$this->Username = $sending_account['login'];
				$this->Password = base64_decode($sending_account['password']);
				
				// Phil 20070125 Adding ssl and port
				$this->Port = ($sending_account['port'] ? (int)$sending_account['port'] : $this->Port);
				if (isset($sending_account['use_ssl']) && $sending_account['use_ssl']) {
					$this->Host = $ssl_host_prefix . str_replace($ssl_host_prefix, '', $this->Host);
				}
				
				break;
		}
		
		// Set From email and name
		$this->FromName = $sending_account['from_name'];
		$this->From = $sending_account['email_address'];
		
		// Set charset based on user pref
		$mail_enc = $SERVICES->preferences->getUserPref('mailEncoding');		
		$this->CharSet = $mail_enc;
	
		// Set return-path email
		$this->Sender = $sending_account['email_address'];
	}
	
	
	/**
	 * adds the addresses to the mailer, as well as a continuous address array
	 *
	 * @param array $address Array of addresses to add
	 * @param array $addresses Reference to a continue array of addresses
	 * @param string $func Name of the function so use (to, bcc, cc)
	 *
	function addAddresses($address, &$addresses, $func) {
		$function = 'Add' . $func;
		if (!empty($address)) {
			foreach ($address as $k => $v) {
					$this->$function($v['address'], $v['from']);
					$addresses[] = $v['address'];
			}
		}
	}*/

	function getError() {
		return $this->ErrorInfo;
	}
	
	/**
	 * Overridden to handle ssl connections, eg ssl://smtp.server.net
	 *
     * Initiates a connection to an SMTP server.  Returns false if the 
     * operation failed.
     * @access private
     * @return bool
     */
    function SmtpConnect() {
        if($this->smtp == NULL) { $this->smtp = new SMTP(); }

        $this->smtp->do_debug = $this->SMTPDebug;
        $hosts = explode(";", $this->Host);
        $index = 0;
        $connection = ($this->smtp->Connected()); 

        // Retry while there is no connection
        while($index < count($hosts) && $connection == false)
        {
			
			if (strstr($hosts[$index], "://")) { // Phil 20070125 Added this first check
				// eg ssl://
				$host = $hosts[$index];
                $port = $this->Port;
			} else 
            if(strstr($hosts[$index], ":"))
                list($host, $port) = explode(":", $hosts[$index]);
            else
            {
                $host = $hosts[$index];
                $port = $this->Port;
            }

            if($this->smtp->Connect($host, $port, $this->Timeout))
            {
                if ($this->Helo != '')
                    $this->smtp->Hello($this->Helo);
                else
                    $this->smtp->Hello($this->ServerHostname());
        
                if($this->SMTPAuth)
                {
                    if(!$this->smtp->Authenticate($this->Username, 
                                                  $this->Password))
                    {
                        $this->SetError($this->Lang("authenticate"));
                        $this->smtp->Reset();
                        $connection = false;
                    }
                }
                $connection = true;
            }
            $index++;
        }
        if(!$connection)
            $this->SetError($this->Lang("connect_host"));

        return $connection;
    }
}
?>