<?php
/**
 * A collection of classes extended from WebMATDatabaseTable
 * 
 * You need an extended class for each table in you module not including the 
 * dropdown tables.
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Mail
 */

/**
 * This class handles the database functions for the item table
 * 
 * This module is an extend verison of WebMATDatabaseTable with the table_name 
 * set and the alert_where set.
 * 
 */
class itemMailDatabaseTable extends WebMATDatabaseTable {
	
	/**
	 * to filter on a particular contact or contacts
	 * @access private
	 */
	var $mail_contact_id = false;
	
	/**
	 * The constructor class for itemMailDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function itemMailDatabaseTable($module_name, $data_definition) {
		global $USER;		
		$this->table_name = 'item';
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseTable
	}
	
	/**
	 * Calls helper method to decode item fields: subject, recipient, sender, cc, bcc
	 *
	 * @author Phil Calder
	 */
	function getItem($id, $order_by = false, $include_where = false) {
		// print '<div>' . get_class($this) . '::getItem(' . $id . ')</div>';
		$result = parent::getItem($id, $order_by, $include_where);
		if ($result !== false) {
			$result = $this->processRowDecoding($result);
		}
		return $result;
	}
	
	/**
	 * Calls helper method to decode item fields: subject, recipient, sender, cc, bcc
	 *
	 * @author Phil Calder
	 */
	function processRow($id, $row) {
		if ($id && $this->isViewable( $id, $row )) { // Phil
			$this->addBrowseQueryIndex($id);
			$item = $this->processRowDecoding($row);
		}
		return $item;
	}
	
	/**
	 * Decodes item fields: subject, recipient, sender, cc, bcc
	 *
	 * @author Phil Calder
	 */
	function processRowDecoding($item) {
		$item['subject'] = $this->decodeString($item['subject']);
		$item['subject'] =  quoted_printable_decode($item['subject']);
		
		$item['sender'] = $this->decodeString($item['sender']);
		$item['sender'] =  quoted_printable_decode($item['sender']); 
		
		$item['recipient'] = $this->decodeString($item['recipient']);
		$item['recipient'] =  quoted_printable_decode($item['recipient']);
		
		$item['cc'] = $this->decodeString($item['cc']);
		$item['cc'] =  quoted_printable_decode($item['cc']);
		
		$item['bcc'] = $this->decodeString($item['bcc']);
		$item['bcc'] =  quoted_printable_decode($item['bcc']);
		return $item;
	}
	
	/**
	 * Similar to decode_iso88591 but handles an entire string
	 *
	 * Useful for decoding email fields (subject, recipient, sender, cc, bcc)
	 * Use in conjunction with (before) quoted printable decoding
	 */
	function decodeString($input) {
		if (!strlen($input)) {
			return $input;
		}
		// print '<p>decodeString(' . $input . ')</p>'; exit();
		// Remove white space between encoded-words
		$input = preg_replace('/(=\?[^?]+\?(q|b)\?[^?]*\?=)(\s)+=\?/i', '\1=?', $input);
		
		// For each encoded-word...
		while (preg_match('/(=\?([^?]+)\?(q|b)\?([^?]*)\?=)/i', $input, $matches)) {
		
			$encoded  = $matches[1];
			$charset  = $matches[2];
			$encoding = $matches[3];
			$text     = $matches[4];
		
			switch (strtolower($encoding)) {
				case 'b':
					$text = base64_decode($text);
					break;
		
				case 'q':
					$text = str_replace('_', ' ', $text);
					preg_match_all('/=([a-f0-9]{2})/i', $text, $matches);
					foreach($matches[1] as $value)
						$text = str_replace('='.$value, chr(hexdec($value)), $text);
					break;
			}
		
			$input = str_replace($encoded, $text, $input);
		}
		return $input;
	}
	
	/**
	 * @param $contact_ids Either an array of contact ids, or an array containing a 'where' key and dtf where clause
	 * @return array An array of email addresses for the specified contact(s)
	 * @author Phil Calder
	 */
	function getEmailAddressesFor($contact_ids) {
		// print '<div>----------- getEmailAddressesFor</div>'; print_r($contact_ids); print '<hr>';
		$items = array();
		if (is_array($contact_ids) && count($contact_ids)) {
			if (isset($contact_ids['where'])) {
				$contact_id_where = $contact_ids['where'];
				$contact_id_where = str_replace('contact_id', 'id', $contact_id_where);
			} else {
				$contact_ids_str = implode(', ', $contact_ids);
				$contact_id_where = "id IN(" . $contact_ids_str . ")";
			}
			$result = $this->db->select(
				'email1_content, email2_content, email3_content', 
				'Contact__item',
				"active=1 AND " . $contact_id_where
			);
			// print '<div>contact_id_where: ' . $contact_id_where . '</div>'; 
			if ($result) {
				while ($row = $this->db->getRow($result)) {
					// print '<div>row:</div>'; print_r($row); print '<hr>';
					if (isset($row['email1_content']) && $row['email1_content']) {
						$items[] = $row['email1_content'];
					}
					if (isset($row['email2_content']) && $row['email2_content']) {
						$items[] = $row['email2_content'];
					}
					if (isset($row['email3_content']) && $row['email3_content']) {
						$items[] = $row['email3_content'];
					}
					
				}
			}
		}
		// print '<div>getEmailAddressesFor returning items:</div>'; print_r($items); print '<hr>';
		return $items;
	}
	
	/**
	 * Custom functionality for the Mail module if there is a Mail contact id (ie Contact filter)
	 *
	 * Pre-fetches and filters on email addresses for all contacts, as opposed to filtering on contact ids
	 * 
	 * @author Phil Calder
	 */
	function processGetItemsContactFields(&$tables, &$fields) {	
		// print '<div>*********** processGetItemsContactFields</div>';
		if ($this->mail_contact_id) {
			// print __FILE__ . '<hr>';
			$tables = $this->getTableName();
		 	$columns = array();
			$columns[] = 'DISTINCT ' . $this->getTableName() . '.*';
			$fields = implode(', ', $columns);
			$this->setCountItemsFields('DISTINCT ' . $this->getTableName() . '.id');
			
			// print '<p>$this->mail_contact_id</p>'; print_r($this->mail_contact_id); print '<hr>';
			$email_addresses = $this->getEmailAddressesFor((array)$this->mail_contact_id);
			// print '<div>contacts: ' . (is_array($this->mail_contact_id) ? implode(',', $this->mail_contact_id) : $this->mail_contact_id) . ' addresses:</div>'; print_r($email_addresses); print '<hr>'; // exit();
			if (is_array($email_addresses) && count($email_addresses)) {
				$email_address_str = "'" . implode("', '", $email_addresses) . "'";
			
				$email_address_filter = array(
					'__special__' => array(
						'where' => "Mail__item_email.address IN(" . $email_address_str . ")"
					)
				);
				// print '<p>email_addresses:</p>'; print_r($email_addresses); print '<hr>';
				// print '<p>calling setDesignTimeFilters</p>'; print_r($email_address_filter); print '<hr>';
			
				$this->setDesignTimeFilters($email_address_filter);			
				
				$tables .= ' LEFT JOIN Mail__item_email ON ' . $this->getTableName() . '.id = Mail__item_email.item_id';
			} else {
				// Phil - there are no emails. Technically, would be better to not even run the query.
				// print '<div>no email addresses [' . $this->mail_contact_id . ']</div>';
				$email_address_filter = array(
					'__special__' => array(
						'where' => "1=0"
					)
				);
				$this->setDesignTimeFilters($email_address_filter);
			}
			
			$contact_fields = '_contact_id'; // Phil 20050819 - Need to return at least one contact field
			
			return $contact_fields;
		} else {
			// print '<div>no contact id(S) [' . $this->mail_contact_id . ']</div>';
			return parent::processGetItemsContactFields($tables, $fields);
		}
	}
	
	/**
	 * @author Phil Calder
	 */
	function setDesignTimeFilters($filter) {
		// print '<div>$$ setDesignTimeFilters:</div>'; print_r($filter); print '<hr>';
		if ($filter['contact_id']) {
			$this->mail_contact_id = $filter['contact_id'];
			// $this->contact_id_filter = $this->filters->getSubContactIds(); // $filter['contact_id'];
			// print '<div>setDesignTimeFilters setting mail_contact_id: ' . $this->mail_contact_id . ' contact_id_filter:</div>'; print_r($this->contact_id_filter); print '<hr>'; 
		}
		parent::setDesignTimeFilters($filter);
	}

	/**
	 * Puts an item into the trash
	 * 
	 * @param integer $id the item id
	 */
	function trashItem($id) {
		global $SERVICES, $L;
		$this->setPrimaryRecordDelete($id); // SEC 20/1/05
		//soft delete - set active to 0
		$this->db->update($this->getTableName(), array('is_trash' => '1'), "id='$id'"); 
		if ($this->db->hasError()) {
			$this->db->printError();
		} else {
			$this->setLog($L['logDelete'] .' - '. $L['mailDelete'], 'id=' . $id . ' ' . $this->module_name . ' ' . $this->getTableName());
		}
	}

	/**
	* Set the data deinition array
	*
	* @param array $data_definition An array containing the data definition (fields and options)
	*/
	function setDataDefinition($data_definition){
		global $L , $SERVICES;
		$SERVICES->getService('Output');
		if (is_array($data_definition['fields'])) { // Phil 20051107 - it should be, but better to check
			foreach($data_definition['fields'] as $k => $v){
				switch ($v['field_key']){
					case 'attachments':
						$data_definition['fields'][$k]['list_values'] = array($L['No'],$L['Yes']);
						break;
						
					case 'is_closed':
						$data_definition['fields'][$k]['list_values'] = array($L['Yes'],$L['No']);
						break;
					
					case 'is_replied':
						$data_definition['fields'][$k]['list_values'] = array($L['No'],$L['Yes']);
						break;
						
					case 'checked_out':
						$data_definition['fields'][$k]['list_values'] = array($L['No'],$L['Yes']);
						break;
	
					case 'action_required':
						$data_definition['fields'][$k]['list_values'] = array($L['No'],$L['Yes']);
						break;
				}
			}
		}
		parent::setDataDefinition($data_definition);
	}

	/**
	 * over-rides and uses the parent version to do a customized call
	 * reason:
	 * there may be a *very* very rare chance that the uidl will be a string
	 * disguised as a number ... for example:
	 * 42332487000001e9 - this 'can' be a number, due to the 'e', php will interpret
	 * this as a floating value (which is technically correct) and then when it goes
	 * to add it do the database, it does not put quotes around it, which we 
	 * definetly don't want since it 'should' be a string .... this is a very
	 * rare case, but it has happened, so, better safe than sorry, we explicitly
	 * set it as a string.
	 *
	 * @param array $item the item as an array
	 */
	function setItem($item, $set_seen = true){
		// only do this if we have 'md5_hash'
		if (array_key_exists('md5_hash', $item)) {
			// set & override
			$original_val = $this->db->setFieldChecking(true);
			
			if (array_key_exists('md5_hash', $item)) {
				// this will prefix '%fn_prefix%' to ensure proper handling.
				$item['md5_hash'] = $this->db->prefixSpecialFieldValue($item['md5_hash'], true);
			}
			
			// call parent to do as normal
			$return_val = parent::setItem($item, $set_seen);
			// now set it back
			$this->db->setFieldChecking($original_val);
			// and finally return
			return $return_val;
			
		// otherwise, do it the normal way
		} else {
			return parent::setItem($item, $set_seen);
		}
	}
	
	/**
	 * Custom string for this module
	 *
	 * @see getUnseenWhere
	 * @author Phil Calder
	 */
	function getUnseenWhereString() {
		return "( Mail__item.active=1 AND seen IS NULL)"; // Phil: took string directly from (removed) getUnseenWhere in this class
	}
	
	/**
	 * Custom string for this module
	 *
	 * @see getModifiedWhere
	 * @author Phil Calder
	 */
	function getModifiedWhereString() {
		return "( Mail__item.active=1 AND ( (seen IS NULL) OR (seen NOT IN(1,3)) ) )"; 
	}	
	
	// Returns true if mail is marked as action required, false if action is not required..nothing if no id is passed
	// Bosco 20060328 | Added Function
	function isMarkedAsActionRequired($id) {
		if (!empty($id)) {
			$item = $this->getItem($id);
			if ($this->db->hasError()) {
					$this->db->printError();
			}
			if ($item['action_required'] == '1') {
					return true;
			} else return false;
		}
		return;
	}
}

class smtp_accountMailDatabaseTable extends WebMATDatabaseTable {
	
	/**
	 * The constructor class for itemMailDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function smtp_accountMailDatabaseTable($module_name, $data_definition) {
		global $USER;
		$this->table_name = 'smtp_account';
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseTable
	}
	
	/**
	 * Overridden to allow web form users to access the default account
	 *
	 * @author Phil Calder
	 */
	function isViewable($id) {
		$id = $this->cleanId($id); // Phil 20070305
		global $SERVICES;
		$users =& $SERVICES->getService('Users');
		if ($users->isWebFormUser()) {
			$prefs =& $SERVICES->getService('Preferences');
			$default_account = $prefs->getUserPref('mailSMTP');
			if ($id && ($id == $default_account) ) {
				return true;
			}
		}
		return parent::isViewable($id);
	}
	
	/**
	 * Overridden to allow web form users to access the default account
	 *
	 * @author Phil Calder
	 */
	function isEditable($id) {
		$id = $this->cleanId($id); // Phil 20070305
		global $SERVICES;
		$users =& $SERVICES->getService('Users');
		if ($users->isWebFormUser()) {
			$prefs =& $SERVICES->getService('Preferences');
			$default_account = $prefs->getUserPref('mailSMTP');
			if ($id && ($id == $default_account) ) {
				return true;
			}
		}
		return parent::isViewable($id);
	}
}

class item_emailMailDatabaseTable extends WebMATDatabaseTable {
	
	/**
	 * The constructor class for itemMailDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function item_emailMailDatabaseTable($module_name, $data_definition) {
		global $USER;		
		$this->table_name = 'item_email';
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseTable
	}
}

class pop_accountMailDatabaseTable extends WebMATDatabaseTable {
	
	/**
	 * The constructor class for itemMailDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function pop_accountMailDatabaseTable($module_name, $data_definition) {
		global $USER;		
		$this->table_name = 'pop_account';
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseTable
	}
}

class item_seenMailDatabaseTable extends WebMATDatabaseTable {
	
	/**
	 * The constructor class for itemMailDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function item_seenMailDatabaseTable($module_name, $data_definition) {
		global $USER;
		$this->table_name = 'item_seen';
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseTable
	}
}

class uidlMailDatabaseTable extends WebMATDatabaseTable {
	
	/**
	 * The constructor class for itemMailDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function uidlMailDatabaseTable($module_name, $data_definition) {
		global $USER;
		$this->table_name = 'uidl';
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseTable
	}
	
	/**
	 * over-rides and uses the parent version to do a customized call
	 * reason:
	 * there may be a *very* very rare chance that the uidl will be a string
	 * disguised as a number ... for example:
	 * 42332487000001e9 - this 'can' be a number, due to the 'e', php will interpret
	 * this as a floating value (which is technically correct) and then when it goes
	 * to add it do the database, it does not put quotes around it, which we 
	 * definetly don't want since it 'should' be a string .... this is a very
	 * rare case, but it has happened, so, better safe than sorry, we explicitly
	 * set it as a string.
	 *
	 * @param array $item the item as an array
	 */
	function setItem($item){
		// set & override
		$original_val = $this->db->setFieldChecking(true);
		
		if (array_key_exists('uidl', $item)) {
			// this will prefix '%fn_prefix%' to ensure proper handling.
			$item['uidl'] = $this->db->prefixSpecialFieldValue($item['uidl'], true);
		}
		// call parent to do as normal
		$return_val = parent::setItem($item);
		// now set it back
		$this->db->setFieldChecking($original_val);
		// and finally return
		return $return_val;
	}
}
?>