<?php
/**
 * The custom edit class for the Mail module
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Mail
 * @author Phil Calder, Simon Cook, Brenton LeTrabb
 */

global $SERVICES;
$SERVICES->getService('FileSystem');

/**
 * The custom edit class for the Mail module
 * 
 * @package Mail
 * @author Phil Calder, Simon Cook, Brenton LeTrabb
 */
class MailItemWebMATEdit extends WebMATEdit {

	var $container_name = 'mail-view-container';

	/**
	 * turn off default Output js validation
	 * @var integer $form_js_validate sets javascript state for the form, 0 is off
	 * @access private
	 */
	var $form_js_validate = 0;
	/**
	 * @var $recipient_addresses
	 * @access private
	 */
	var $recipient_addresses;
	/**
	 * @var $mailer
	 * @access private
	 */
	var $mailer;
	/**
	 * @var $msg
	 * @access private
	 */
	var $msg;
	/**
	 * @var $hash_iterator
	 * @access private
	 */
	var $hash_iterator; //hash_iterator is to solve duplication bug because this class runs twice
	/**
	 * @var $send_accounts
	 * @access private
	 */
	var $send_accounts;
	/**
	 * @var $parent_item
	 * @access private
	 */
	var $parent_item = array(); // the original item, for use after we create the reply item
	/**
	 * @var $description_message
	 * @access private
	 */
	var $description_message = '';  //TODO remove
	/**
	 * @var $description_status
	 * @access private
	 */
	var $description_status = 'description';  //description or error

	/**
	 * any processing
	 */
	function getItem($id = false, $item = false) {
		// echo '<p>Mail/inc/item-edit.inc::getItem   ---  outside if (!$item) {</p>';
		if (!$item) {
			$item = parent::getItem($id, $item);
			// before we can send anything, we need to ensure we *can* send anything
			
			// print '<div>[1] MailItemWebMATEdit getItem(' . $id . ')</div>'; print_r($item); print '<hr>';
			
			$this->fillSendAccountsArray();
			$this->hash = $this->setHash($item);
	
			//echo $this->hash;
			
			//decoded_mail is calculated where there is an id, ie where we are forwarding or using an existing message
			$decoded_mail = $this->mod->getDecodedMime($item['id']);
	
	
			$this->setupValidateComposeJavascript();// Saravanakumar 20050616 setupValidateRecipientJavascript() changed to setupValidateComposeJavascript()
			//$this->setupValidateRecipientJavascript();
			
			//$this->setupVerifyLeavingJavascript();//Saravanakumar :: 20050711 :: Commented to avoid js message on attach local file.
			// because item['send_account'] gets overwritten when we setup the from accounts,
			// and because we need to know who we're replying to, we set the recipients first
			$this->setRecipients($item, $decoded_mail);
			// print '<div>[2] MailItemWebMATEdit getItem(' . $id . ')</div>'; print_r($item); print '<hr>';
			
		     
			$this->setFromAccountDropdown($item); // setup the 'from account' stuff		
			// print '<div>[3] MailItemWebMATEdit getItem(' . $id . ')</div>'; print_r($item); print '<hr>';
			
			$this->setSubject($item);
			
			$this->setInitialAttachments($item, $decoded_mail);
			
			$this->setContent($item, $decoded_mail);
		}
		//print_r($item);
		// Senthil 20050624  Checks the Rich Text Editor content type from user pref. - Begin		
		// if ($SERVICES->preferences->getUserPref('mailRichTextEditorSwitch') == 1) {
			// $this->setupRichTextEditorJavascript(); 
		// }
		// Senthil 20050624  Checks the Rich Text Editor content type from user pref. - End
		return $item;
	}
	
	function beforeForm(&$id, &$item, &$error_msg) {
		$this->setupAttachmentJavascript($item);
	}

	/**
	 * this override for Mail, is to display 
	 * CheckedOut messages and phpMailer errors etc
	 */
	function getActionLinksContent() {
		global $L, $SERVICES;
		$parent_content = parent::getActionLinksContent();
		if ($this->description_message) {
			$padding = true;
			//TODO class based on $this->description_status
			if ($this->description_status == 'error') {
				$content = $this->output->getValidationWarning($this->description_message);
			} else {
				$content = '<b>' . $this->description_message . '</b>';
			}
			$content = $this->output->getBeginRow('', true) .
					'<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>' .
					$content .
					'</td>' .
					$this->output->getEndRow(true);
					
			$content .= $this->output->getSpacerRow();
		}
		return $parent_content . $content;
	}
	
	/**
	 * Retrieves or creates a md5 hash, sets it in the $item array as well
	 *
	 * @param array  $item
	 * @return string $md5 the created/retrieved hash
	 */
	function setHash(&$item) {
		$ret_val = '';
		// if (isset($this->form_values['hash'])) {
			// $this->mod->spellcheckReturn($item, $this->form_values['hash']);
			// $ret_val = $this->form_values['hash'];
		if (isset($item['hash'])) {
			//$this->mod->spellcheckReturn($item, $this->form_values['hash']);
			$ret_val = $item['hash'];
		} else {
			// create the hash for this mail
			//hash_iterator is to solve duplication bug because this class runs twice 
			$ret_val = md5(time() + $this->hash_iterator);
			$this->hash_iterator++;
		}
		
		//echo 'creating hash' . $ret_val;
		$item['hash'] = $ret_val;
		return $ret_val;
	}
	
	/**
	 * Set the recipients (to, cc, bcc)
	 *
	 * @param array $item
	 * @param string $decoded_mail mime decoded_mail content (optional)
	 */
	function setRecipients(&$item, $decoded_mail = false) {}
	
	/**   
	 * sets the Compose content type. as RichText or Plain Type
	 *
	 * @param array $item 
	 */
	function setComposeType(&$item) {}
	
	/**
	 * sets the subject. intended to be extended
	 *
	 * @param array $item
	 */
	function setSubject(&$item) {
		//MCR 20051007
		//we need to know subject prior to RE: or FWD: being added
		$this->original_subject = $item['subject'];
		//now extend
	}
	
	/**
	 * Sets up the body/content of the email, should it be required
	 *
	 * @param array $item
	 * @param string $decoded_mail mime decoded_mail content (optional)
	 * this function builds the body for reply/forward; is overridden for edit
	 */
	function setContent(&$item, $decoded_mail = false) {
		//function setContent(& $item, $decoded_mail = false) {}
		//added overwrite to blank in edit_edit.inc
		global $SERVICES, $L;
			
		// if we're forwarding as an attachment, don't want to display it in the content, it's an attachment
		if ($this->form_values['as_attachment']) {
			return;
		}
		
		$content = $item['content']; // in case nothing happens below
		if ($decoded_mail) { 
			//$content = $this->mod->get Content($decoded_mail);
			//MCR 20051025
			$display_part = $this->mod->getDisplayPart($decoded_mail);
			$content = $display_part['body'];
			//MCR 20051011
			// this should all be replaced by a function, in Service?,
			// which takes content (either plain or html) and 
			// returns a quoted 'body' which also may be either plain or html
			
			// strip tags (if is html)
			//if ($this->mod->isHtmlPart($content)) {
			
			//MCR 20051025
			//TODO
			// This is entirely based on an assumption that we are targetting a textarea edit field
			// rather than rich text
			// Leave in for now - VERY temporarily
			
			if ($display_part['ctype_secondary'] == 'html') {
				//echo 'stripping';
				$content = strip_tags($content);
			}
			
			//Saravanakumar :: 20050707 :: Below lines are commented because of not formatting correctly
			// Add the > symbols at the beginning of each line
			//$content = str_replace("\n", "\n> ", $content);
			// Add some blank lines at the beginning
			//$content = "\n\n\n" . '> ' . $content;
			
			//Saravanakumar :: 20050708 :: Formating the original message for both 'plaintext' and 'RTE'
			//Getting header information of original message
			$msg_info = $this->mod->getWroteOnLabel($decoded_mail);
			
			//Removing trailing '<br />' added as default
			if (substr($content,0,3)) {
				$content = str_replace('<br /><br /><br />',"",$content);
			}
			
			//$content = str_replace("\n",  "", $content);
			//MCR 20060503
			//if ($SERVICES->preferences->getUserPref('mailRichTextEditorSwitch') == 0) {
				//create plain text body
				$content = trim($content);  //MCR remove trailing line endings
				$content = str_replace("<br />&gt;", "<br />", $content);
				$content = str_replace("<p>", "", $content);
				$content = str_replace("</p>", "<br />", $content);
				$content = str_replace("<br />", "\n", $content);
				$content = str_replace("&nbsp;", "", $content);
				$content = str_replace("\n", "\n> ", $content);
				$content = '> ' . $content;
				$content = "\n\n\n" . $msg_info . "\n\n" . $content . "\n\n";
				
				//echo '**' . $content . '**';
				//$content = strip_tags($content, "<br>");
			// } else {
				// //create rich text body
				// $content = str_replace("<br />&gt;", "<br />", $content);  ////MCR 2005-07-24 - what is this for? and what about reply to reply?
				// $content = str_replace("<br>", "<br />", $content);
				// $content = str_replace("<p>", "", $content);
				// $content = str_replace("</p>", "<br />", $content);
				// $content = str_replace("<br />", "<br />>", $content);
				// $content = "<br /><br /><br />" . $msg_info ."<br />>".$content;
			// }
			
		} 
		$item['content'] = $content;
	}
	
	/**
	 * Sets up the initial attachments for the email, should it be required
	 *
	 * @param array $item
	 * @param string $decoded_mail mime decoded_mail content (optional)
	 */
	function setInitialAttachments(&$item, $decoded_mail = false) {
		global $SERVICES;
		//echo '<p>setInitialAttachments</P>'; echo count($_SESSION[$item['hash']]['strings']);
		//echo $_SESSION[$item['hash']['fwd_as_attachment_processed']];
		
		// we are forwarding a message as an attachment
		// this is called when we first add the attachment
		
		if ($this->form_values['as_attachment']) {
			// location, name, type, size
			if (!isset($_SESSION[$item['hash']]['fwd_as_attachment_processed'])) {
				$file['location'] = $SERVICES->filesystem->getFilePath($this->form_values['id'], 'mail');
				//echo '<br>' . $this->original_subject . '<br>' . $file['location'] . '<br>';
				//MCR 20051007 fix naming
				//$file['name'] = 'message.txt';
				$file['name'] = $this->original_subject . '.eml';
				$file['type'] = 'message/rfc822';  //MCR 20051010 'text/plain';
				$file['size'] = $SERVICES->filesystem->getFileSize($this->form_values['id'], 'mail');
				//set flag so this is only done once
				$_SESSION[$item['hash']]['fwd_as_attachment_processed'] = 1;
				// finally add the file to the list
				$_SESSION[$item['hash']]['files'][] = $file;
			}
		} else if ($decoded_mail) {	//MCR 20051007 'else' added here to prevent 'forward as attachment' together with usual attachments
			//echo 'getting attachments: ' . $decoded_mail . '<br>'; print_r($item);
			$attachments = $this->mod->getAttachmentsWithinMessage($decoded_mail);
			//echo 'GOT FROM MOD attachments: '; sizeof($attachments); print_r($attachments);
			// If we have an attachment add it to the session attachment list so it gets included
			// This only applies to forwarding a message with attachments
			foreach ($attachments as $k => $v) {
				//echo 'here ' . $v['filename'];
				$attach_array = array();
				$attach_array['name'] = $v['filename'];
				//TODO size calculation blatantly wrong
				$attach_array['size'] = $v['size'];
				//$attach_array['process'] = 1; 
				$attach_array['body'] = $v['body'];
				//$_SESSION[$item['hash']]['files'][] = $attach_array;
				$_SESSION[$item['hash']]['strings'][] = $attach_array;
			}
		}
		
		// Check the session hash for any attachments we may already have
		// This could happen when we are forwarding or returning from a spell check where
		// we had attachments before the spellcheck began
		
		$list_a = array();
		// File attachments
		//echo count($_SESSION[$item['hash']]);
		
		if (is_array($_SESSION[$item['hash']]['files'])) {
			foreach ($_SESSION[$item['hash']]['files'] as $k => $v) {
				$list_a[] = $v['name'] . ' (' . $v['size'] . ')';
				//echo '<p> heree 1' . $v['name'] . ' (' . $v['size'] . ')' . '</P>';
			}
		}
		
		//print_r($_SESSION[$item['hash']]['strings']);
		//echo count($_SESSION[$item['hash']]['strings']);
		// String attachments
		if (is_array($_SESSION[$item['hash']]['strings'])) {
			foreach ($_SESSION[$item['hash']]['strings'] as $k => $v) {
				$list_a[] = $v['name'] . ' (' . $v['size'] . ')';
				//echo '<p> heree 2 ' . $v['name'] . ' (' . $v['size'] . ')' . '</P>';
			}
		}
		// Document attachments
		if (is_array($_SESSION[$item['hash']]['documents'])) {
			foreach ($_SESSION[$item['hash']]['documents'] as $k => $v) {
				$list_a[] = $v['name'] . ' (' . $v['size'] . ')';
				//echo '<p> heree 3</P>';
			}
		}		
		//print_r($list_a);
		if (!empty($list_a)) {
			// $list = implode(', ', $list_a);
			$list = implode("\r\n", $list_a);
			$this->dd->changeAttributes(array('field_key' => 'curr_attachments', 'rows' => (sizeof($list_a)-1)));
		} else {
			$list = '';
		}
		
		$item['curr_attachments'] = $list;
		//echo $item['curr_attachments'];
		//echo '<p>fwd attachments';	print_r($item['curr_attachments']);	print_r($_SESSION[$item['hash']]);
	}
	
	/**
	 * Sets up the javascript used for updating the 'from' sender when it's respective
	 * account is chosen from the dropdown menu
	 */
	function setupFromAccountJavascript() {
		global $SERVICES ; //Senthil 20050707
		// quick double check
		if (!is_array($this->send_accounts)) {
			$this->fillSendAccountsArray();
		}
		
		// store signatures in jscript array
		$array_code = 'var from = new Array(' . sizeof($this->send_accounts) . ');' . "\n";
		$array_code .= 'var sigs = new Array(' . sizeof($this->send_accounts) . ');' . "\n";
		$count = 0;
		foreach ($this->send_accounts as $k => $v) {
			$array_code .= "from[$count] = \"" . $v['from_name'] . ' <' . $v['email_address'] . '>' . "\";\n";
			// Senthil 20050707  remove the HTML Tags for Text Area
			// if ($SERVICES->preferences->getUserPref('mailRichTextEditorSwitch') == 1) {
				// $array_code .= "sigs[$count] = \"" . preg_replace(array("/\n/","/\r/"),array('\\n',''), addslashes($v['signature'])) . "\";\n";
			// } else {
				// $sign = preg_replace(array("/\n/","/\r/","'<br />'"),array('\\n','','\\n'),($v['signature']));
				// Phil 20070505 We need to addslashes
				$sign = preg_replace(array("/\n/","/\r/","'<br />'"),array('\\n','','\\n'),(addslashes($v['signature'])));
				$array_code .= "sigs[$count] = \"" .strip_tags($sign). "\";\n";
			//}	
		 	$count ++;
		}
		
		$code_function = '
		<script type="text/javascript" language="Javascript">
		' . $array_code . '
		
		// this will update the from & signature text items
		function updateContent() {
			// Change the signature to the new accounts one
			document.forms.mainform.elements["item[signature]"].value = sigs[document.forms.mainform.elements["item[send_account]"].selectedIndex];
			document.forms.mainform.elements["item[sender]"].value = from[document.forms.mainform.elements["item[send_account]"].selectedIndex];
			';

		//MCR 20060503
		// if ($SERVICES->preferences->getUserPref('mailRichTextEditorSwitch') == 1) {	
		 	 // $code_function .= ' if (__htmlareas[0]) { 						__htmlareas[0].setHTML(sigs[document.forms.mainform.elements["item[send_account]"].selectedIndex]); }';
		// }
	
		$code_function .='}
		</script>'
		;
		$this->output->addToHead($code_function);
		//$this->setSignature($item,$array_code,$xx);
	}

	//Bit of javascript to validate the reciptients: ensures at least 1 addy is provided
	// function setupValidateRecipientJavascript() {
		// global $L;
		// $code = '
		// <script type="text/javascript" language="Javascript">
		// function validateRecipients() {
			// var message = \'\'; 
			// if ( (document.forms.mainform.elements["item[recipient]"].value == \'\') && 
					// (document.forms.mainform.elements["item[cc]"].value == \'\') && 
					// (document.forms.mainform.elements["item[bcc]"].value == \'\') ) {
				// message = \''. $L['mailEmptyRecipientError'] .'\' + \'\n\';
			// }
			// if (message == \'\') {
				// disableLeaveCheck();
				// return true;
			// } else {
				// alert(message);
				// return false;
			// }
		// }
		// </script>
		// ';
		// 
		// $this->output->addToHead($code);
	// }
	
	/**
	 * Saravanakumar :: 20050616  
	 * setupValidateRecipientJavascript() function is modified to validate Recipient and Subject fields
	 * with trim functionality and function name changed from setupValidateRecipientJavascript() to 
	 * setupValidateComposeJavascript()
	 **/
	 //Saravanakumar :: 20050708 :: Function altered to have atleast one address from To or CC or BCC.
	function setupValidateComposeJavascript() {
		global $L;

		
		// Note: this is what it looked like before the 'disableLeaveCheck()' function call was removed.
		// This was provided as a reference, just in case it's added back in.
		// ~BBartel 20050912
			// if (message == \'\') { 
				// disableLeaveCheck();
				// return true;
			// }

		// * The Trim(), Ltrim() and Rtrim() functions are added in js_functions.js file 
		// <script type="text/javascript" src="'. $this->output->getJsWebPath() .'js_functions.js"></script>
		$code = '
		<script type="text/javascript" language="Javascript">
		// Removes leading and ending whitespaces
		function trim (str) {
		  str = ( (this != window) ? this : str);
		  return str.replace(/^\s+/g, \'\').replace(/\s+$/g, \'\');
		}

		function validateForm() {
			var message = \'\';
			var recipient = document.forms.mainform.elements["item[recipient]"].value;
			var cc = document.forms.mainform.elements["item[cc]"].value;
			var bcc = document.forms.mainform.elements["item[bcc]"].value;
			var subject = document.forms.mainform.elements["item[subject]"].value;
			if (!trim(recipient) && !trim(cc) && !trim(bcc)) {
				message = \''. $L['mailNoAddress'] .'\' + \'\n\';
			}
			if (message == \'\') { 
				return true;
			}
			else {
				alert(message);
				return false;
			}
		}
		</script>';
		$this->output->addToHead($code);
	}

	/**
	 * This puts in a bit of javascript for a rich text editor.
	 * Note: the textarea *must* have an id set (see item_data_definition.inc)
	 * To be able to swap in/out different editors, we simply call a separate js
	 * include file which will take care of it. That way, we just edit that file
	 * and not worry about coming in here to the code.
	 * NB: This is just calling it from it's relative position, which assumes we are
	 * within BCRM_ROOT/modules/Mail/ for browsing, doing relative addressing.body_signature
	 * Senthil  create the new object for adding two RTE in singale page thats xinha_editors1 20050624
	 */
	function setupRichTextEditorJavascript() {
		$code = '<script type="text/javascript" src="'. $this->output->getJsWebPath() .'rich_text_editor.js"></script>
		
		<script type="text/javascript" language="Javascript">
		// (preferably absolute) URL (including trailing slash) where Xinha is installed
		_editor_url  = "' . $this->output->getJsWebPath() .'RichTextEditor/";
		_editor_lang = "en";      // And the language we need to use in the editor.

		// name of the textarea that will get the editor
		xinha_editors = ["message_body"];
		xinha_editors_main = ["body_signature"];
		</script>

		<script type="text/javascript" src="' . $this->output->getJsWebPath() .'RichTextEditor/htmlarea.js"></script>
		';
		$this->output->addToHead($code);
	}
	
	/**
	 * Bit of javascript to check if a user's leaving the edit page & display a message
	 */
	function setupVerifyLeavingJavascript() {
		global $L;

		$code = '
		<script type="text/javascript" language="Javascript">
		window.onbeforeunload = verifyLeaving;
		
		var checkLeaving = true;
		
		function disableLeaveCheck() {
			checkLeaving = false;
		}
		
		function verifyLeaving() {
			if (checkLeaving) {
				message = \''. $L['mailLeavingPageWillLoseEmail'] .'\';
	
				return message;
			}
		}
		</script>
		';
		
		$this->output->addToHead($code);
	}

	/**
	 * Javascript primarily for the popups for attachments. Sets it up in a sneaky way:
	 * puts in as an item in the $item array, very sneaky, and probably bad
	 */
	function setupAttachmentJavascript(&$item){
		global $L;

		// Setup the javascript popups for attachments and documents
		//Saravanakumar :: 2005-06-14 :: Popup window size altered
		$selector_options = array(
			'url' => 'add_attachment.php?hash=' . $item['hash'],
			'window_name' => 'attachment_selector',
			'resizable' => true,
			'scrollbars' => false,
			'width' => 615, 
			'height' => 250
		);
		$this->output->addJSPopup('attachmentSelector', $selector_options);
		$select_link = $this->output->getLinkHref(
			array(
				'tag_extras' =>' onClick="javascript:attachmentSelector();" ',
				'href' => 'javascript:;',
				'text' => $L['mailAttachLocalFile'],
			)
		);
		
		
		$selector_options = array(
			'url' => 'add_document.php?hash=' . $item['hash'],
			'window_name' => 'document_selector',			
			'resizable' => true,
			'scrollbars' => false,
			'width' => 640,
			'height' => 600
		);
		$this->output->addJSPopup('documentSelector', $selector_options);
		$select_link2 = $this->output->getLinkHref(
			array(
				'tag_extras' =>' onClick="javascript:documentSelector();" ',
				'href' => 'javascript:;',
				'text' => $L['mailAttachFromDocuments'],
			)
		);
		
		$item['attach'] = $select_link . ' | ' . $select_link2;
		
		$this->dd->changeAttributes(array('field_key' => 'curr_attachments', 'no_resize' => 1)); // Phil 20070306
		$this->dd->changeAttributes(array('field_key' => 'curr_attachments', 'read_only' => 1));
	}

	/**
	 * Populates the send_accounts variable - gets all accounts that we can edit
	 *
	 * @param bool $warning_message If true, will display a message saying there are no accounts (if applicable) and exit the entire script.
	 */
	function fillSendAccountsArray($warning_message = true){
		// Saravanakumar :: 2005-06-14 :: Global Variable Declared to get the value of "$L['mailCreateAccount']"
		global $L; 
		$smtp_account_table =& $this->mod->getDatabaseTable('smtp_account');
		
		// get all send accounts we can view
		$this->send_accounts = $smtp_account_table->getItems();
		
		if (is_array($this->send_accounts)) {  //MCR 20050309, added this if block
			// now loop through, and only get those that we can edit.
			foreach ($this->send_accounts as $k => $v) {
				if (!$smtp_account_table->isEditable($v['id'], $v)) {
					bcrm_unset($this->send_accounts, $k);
				}
			}
		} else {
			$this->send_accounts= array();
		}
		
		// see if there are no accounts, if we want the warning message, we'll display it.
		if ($warning_message && ((!is_array($this->send_accounts)) || empty($this->send_accounts))) {
			$create_account_link = $this->output->getLinkHref(array('href' => '../Mail/account_menu.php', 'text' => $L['mailCreateAccount']));
			quickPage($L['mailNoAccount'] . '<br> <br> ' . $create_account_link . ' | '. $this->output->getLinkBack($_SESSION['return_page']));
			exit();
		}	
	}

	/**
	 * This will get all the send accounts and setup the dropdown for them and the 'from' address.
	 * Also handles calling the javascript functions.
	 *
	 * @param array $item 
	 */
	function setFromAccountDropdown(&$item) {
		global $SERVICES;
		
		// quick double check
		if (!is_array($this->send_accounts)) {
			$this->fillSendAccountsArray();
		}
		
		if (is_array($this->send_accounts)) {
			$dd_value = array();
			foreach ($this->send_accounts as $k => $v) {				
				$dd_value[$v['id']] = $v['name'];	
			}
		}			
		
		$this->setupFromAccountJavascript();
		
		// get userPref send account
		$default_id = $SERVICES->preferences->getUserPref('mailSMTP');
		
		// print '<div>user pref default_id: ' . $default_id . '</div>';
		// print '<div>setFromAccountDropdown</div>'; print_r($item); print '<hr>';

		if (!isset($item['id'])) {
			// new mail
			if ($default_id) {
				if (!(isset($this->send_accounts[$default_id]))) {
					reset($this->send_accounts);
					$default = current($this->send_accounts);
				} else {
					$default = $this->send_accounts[$default_id];
				}
				if ($setSig) {
					$item['signature'] = $default['signature'];
				}
				$item['send_account'] = $default['id'];
				$item['sender'] = $default['from_name'] . ' <' . $default['email_address'] . '>';
			} else {
				reset($this->send_accounts);
				$v = current($this->send_accounts);
				$item['send_account'] = $v['id'];
				if ($setSig) {
					$item['signature'] = $v['signature'];
				}
				$item['sender'] = $v['from_name'] . ' <' . $v['email_address'] . '>';
			}
		} else {
			// print '<div>reply/fwd</div>';
			// print '<div>this->send_accounts[$item[send_account_id]][id]: ' . $this->send_accounts[$item['send_account_id']]['id'] . '</div>'; print_r($this->send_accounts[$item['send_account_id']]); print '<hr>';
			// reply/forward
			if (!isset($item['send_account_id']) || !$item['send_account_id'] || !isset($this->send_accounts[$item['send_account_id']]) || !$this->send_accounts[$item['send_account_id']] ) {
				$get_account_id = false;
				if (isset($item['send_account_id']) && $item['send_account_id']) {
					$get_account_id = $item['send_account_id'];
				} else {
					$get_account_id = $default_id;
				}
				$mail_u =& $this->mod->getDatabaseTable('smtp_account');
				// $default = $mail_u->getItem($item['send_account_id']);
				$default = $mail_u->getItem($get_account_id);
				// print '<div>db default: ' . $default['id'] . '</div>';
				if (!$default) {
					reset($this->send_accounts);
					$default = current($this->send_accounts);
					// print '<div>using first account, default: ' . $default['id'] . '</div>';
				}
				unset($mail_u);
			} else {
				$default = $this->send_accounts[$item['send_account_id']];
				// print '<div>using send_account_id account: ' . $default['id'] . '</div>';
			}
			$item['send_account'] = $default['id'];
			if ($setSig) {
				$item['signature'] = $default['signature'];
			}
			$item['sender'] =  $default['from_name'] . ' <' . $default['email_address'] . '>';
		}

		// finally, update the data definition stuff
		$this->dd->changeAttributes(array('field_key' => 'sender', 'read_only' => 1));
		$this->dd->changeAttributes(array('field_key' => 'send_account', 'dropdown_values' => $dd_value));
		$this->dd->changeAttributes(array('field_key' => 'send_account', 'tag_extras' => 'onChange="updateContent()"'));
	}
	
	/**
	 * Checks out a message
	 *
	 * @param array $item
	 * @param bool $warning_message Defaulted to true, if it is true, it'll exit the script and display a message
	 */
	function checkoutMessage(&$item, $warning_message = true) {
		// verify we actually have a valid id
		if (!isset($item['id']) && !$this->form_values['id']) {
			return;
		}

		// Check the item we are replying to is not already checked out (in the process of being replied to by someone else)
		if ($item['checked_out'] && !$this->form_values['cont'] && $warning_message && ($item['sent_status'] != 'mailTemplates')) {
			global $L;
			//TODO get time and user who checked it out
			//skip message is current user or old time
			//print_r($item);
			$continue_link = $this->output->getLinkHref(array('href' => $_SERVER['PHP_SELF'] . '?' . $_SERVER['QUERY_STRING'] . '&cont=1', 'text' => $L['mailContinue']));

			$this->description_message = $L['mailCheckedOut'];
			$this->description_status = 'description';
				
			//quickPage($L['mailCheckedOut'] . '<br> <br> '. $this->output->getLinkBack() .' | ' . $continue_link);
			//exit();
		}
		
		// Update this message -- it is now checked out
		// use the proper id 
		if (isset($item['id'])) {
			$update['id'] = $item['id'];
		} else {
			$update['id'] = $this->form_values['id'];
		}

		$update['checked_out'] = 1;
		//$update['checked_out_time'] = time(); - to be implemented in the future
		$this->db_functions->setItem($update);
	}

	/**
	 * Checks a message back in
	 *
	 * @param array $item
	 */
	function checkinMessage(&$item) {
		// Update this message -- it is now checked out
		$update['id'] = $item['id'];
		$update['checked_out'] = 0;
		//$update['checked_out_time'] = time(); - to be implemented in the future

		$this->db_functions->setItem($update);
	}

	/**
	 * Converts to special chars, while still maintain the proper brackets
	 *
	 * @param string $str String to convert
	 *
	 * @return string Converted string
	 */
	function htmlToSpecialChars($str){
		// global function in ~/services/Services.inc - all references should be
		// altered to use that function. - *HOWEVER*, we want to keep angled
		// brackets and quotes, so keep using this one for now. change later!
		$trans_table = array_flip(get_html_translation_table(HTML_ENTITIES));
		return strtr($str, $trans_table);
	}
	
	/**
	 * Operations before we actually write to the db, included in here is sending
	 */
	function beforeDBWrite(&$item, &$fields, &$error_msg) {
		global $L, $SERVICES;
		// Phil 20070125 Possibility of PHP Warnings using SSL
		$current_error_reporting = error_reporting(0);
		//echo 'fvhash' . $this->form_values['hash'] . '<br>';
		//echo 'hash' . $item['hash'] . '<br>';
		//echo 'SESShash' . $_SESSION[$item['hash']] . '<br>';
		//print_r($_SESSION[$item['hash']]);
		//$item['hash'] will be unset as message is sent
		$temp['hash'] = $item['hash'];
		
		//save item in case we have to return to form
		//because $item gets processed
		$original_item = $item;
		
		// if we're doing a spell check, this function will redirect us there, thus ignoring the rest of this function
		
		// if ($this->form_values['submitSpellCheck']) {
		// 	$this->doSpellcheck($item);
		// 	return;
		// }
		
		// setup so we know which account to use. - will exit out if none found
		// print '<div>get send account...</div>';
		$sending_account = $this->getSendAccount($item);
		// print_r($sending_account); print '<hr>';
		
		// Phil 20060714 Adding checks to ensure we are allowed to use 
		// the transport method for this send account
		$this->isTransportMethodAllowed($sending_account['transport_method'], $item);
		
		// we may need to use this to update it later on.
		// in some cases (like sending a drafted email), they should be appropriate
		if ($item['parent_id']) {
			$this->parent_item['id'] = $item['parent_id'];
		} else if ($this->form_values['id']) {
			// use the form_value var, and not item, cause for some reason, it's not set.
			$this->parent_item['id'] = $this->form_values['id'];
		}
		
		// remove slashes from address fields
		$item['recipient'] = stripslashes($item['recipient']);
		$item['cc'] = stripslashes($item['cc']);
		$item['bcc'] = stripslashes($item['bcc']);

		// add sender to item
		$item['sender'] = $sending_account['from_name'] . ' <' . $sending_account['email_address'] . '>';
		$item['send_account_id'] = $item['send_account'];
		
		// print '<div>call configureMailer(item, sending_account):</div>'; print_r($item); print_r($sending_account); print '<hr>';
		// get $this->mailer ready for sending.
		$this->configureMailer($item, $sending_account);

		// get the addresses the message will be sent to.
		$this->recipient_addresses = $this->fillEmailMessage($item);
			
		// now comes the main portion where we actually do some sending! (if applicable)
		if ($this->form_values['submitSend']) {
			// extra header info (thread_id) - maybe future stuff
			//$extra_header = array('label' => 'BCRM-Thread-ID', 'value' => 2);
			if ($_SESSION[$temp_hash]['sent']) {
				//already sent, make notification
				quickPage($L['mailAlreadySent'] . '<br> <br> ' . $this->output->getLinkBack($_SESSION['return_page']));
				exit();
				// return false;
				// TODO add link to a 'resend' option
			}

			// *SEND* the email
			$this->msg = $this->doSend();
			// $this->msg returned false, meaning there was an error, what was the problem?
			if (!$this->msg && (!isset($this->return_string) || !$this->return_string)) { // Phil 20050506 - same as main - can return message as string OR display and exit
				//Output::validateFields  $error_msg[$v['field_key']] = $validated['error'];
				$this->description_message = $this->mailer->getError();
				$this->description_status = 'error';
				
				//$this->main( $item['id'], $temp_item, $error_msg);
				//$error_msg['recipient'] = $this->mailer->getError();
				//TODO all fields; loop
				foreach ($original_item as $k => $v) {
					$original_item[$k] = stripslashes($original_item[$k]); 
				}
				
				// $original_item['recipient'] = stripslashes($original_item['recipient']);
				// $original_item['cc'] = stripslashes($original_item['cc']);
				// $original_item['bcc'] = stripslashes($original_item['bcc']);
				// $original_item['message_body'] = stripslashes($original_item['message_body']);
				// $original_item['sig'] = stripslashes($original_item['sig']);
				
				$this->main( $item['id'], $original_item, $error_msg);
				
				//quickPage($this->mailer->ErrorInfo . '<br> <br> ' . $this->output->getLinkBack());
				//can get parts and do something: retry option
				exit();
				
			} else {
				//MCR 20051026
				if ($temp_hash) {
					//required to garbage collect on _SESSION
					bcrm_unset($_SESSION, $temp_hash);
					$_SESSION[$temp_hash]['sent'] = true;   //used as a 'sent' marker to prevent double-clicks / double sends
				}
			}
			
			// track who actually sent it
			global $USER;
			$item['sent_by_user_id'] = $USER['id'];
			
			// here we check if we've just sent a template, if so, we actually want to keep the template, so we unset the id
			// (as opposed to a draft where we actually send the draft.)
			if ($item['sent_status'] == 'mailTemplates') {
				bcrm_unset($item, 'id');				
			}
			
			// made it this far, so set the status
			$item['sent_status'] = 'mailSent';
			$item['action_required'] = 0; // why should there be an action on a sent item?

		} else {
			// probably going in as a template or draft
			$this->msg = $this->mailer->buildMessage();
			
			// set some status
			if ($this->form_values['submitSaveDraft']) {
				$item['sent_status'] = 'mailDraft';
			} else if ($this->form_values['submitSaveTemplate']) {
				$item['sent_status'] = 'mailTemplates';
			}
		}
		
		// If the send was successful above, we will continue on through here, otherwise, this doesn't even get executed
		
		// strip content component out of $item for writing to DB - since they don't exist in the db
		bcrm_unset($item, 'content');
		bcrm_unset($item, 'signature');
		bcrm_unset($item, 'send_account');

		// some final items to set.
		$item['date'] = time();
		$item['is_opened'] = 1;
		
		// defaults?
		$item['view_group_id'] = $sending_account['message_view_group_id'];
		$item['edit_group_id'] =  $sending_account['message_edit_group_id'];
		$item['owner_group_id'] =  $sending_account['message_owner_group_id'];
		$item['admin_group_id'] =  $sending_account['message_admin_group_id'];
		
		$current_error_reporting = error_reporting($current_error_reporting);
	}
	
	/**
	 * @author Phil Calder
	 */
	function isTransportMethodAllowed($transport_method, $item) {
		// print '<div>transport_method: ' . $transport_method . '</div>';
		$is_enabled = $this->mod->isTransportMethodEnabled($transport_method);
		if ($is_enabled) {
			return true;
		}
		
		// set description message, status, then call main
		global $L;
		$this->description_status = 'error';
		$this->description_message = $L['mailSmtpAccountTransportMethodNotAllowed'] . ': ';
		
		switch ($transport_method) {
			case 1:
				$this->description_message .= $L['mailSMTPDropdownPHPMail'];
			break;
			case 2:
				$this->description_message .= $L['mailSMTPDropdownSendmail'];
				
			break;
			case 3:
				$this->description_message .= $L['mailSMTPDropdownSMTP'];
			break;
			case 4:
				$this->description_message .= $L['mailSMTPDropdownSMTPAuthenticated'];
			break;
			default:
				$this->description_message .= $L['mailSMTPDropdownUnknownTransportMethod'];
			break;	
		}
		
		$this->main($item['id'], $item, $error_msg);
		exit();
	}

	/**
	 * Sets up to get ready for sending an email
	 *
	 * @param array $item
	 *
	 * @return array Returns an array of emails that the message will be sent to
	 */
	function fillEmailMessage(&$item) {
		$include_signature = !$this->form_values['submitSaveDraft'] && !$this->form_values['submitSaveTemplate'];
		// Write addresses to item_email table <-- afterDBWrite
		return $this->mailer->fillEmailMessage($item, $include_signature);
	}


	/**
	 * Gets the account for sending
	 *
	 * @param array $item
	 * @param bool $warning_message If true, will display a message saying there are no accounts (if applicable) and exit the entire script.
	 *
	 * @return array $sendAccount the array of the sending account
	 */
	function getSendAccount(&$item, $warning_message = true) {
		// Set mail transport method
		$smtp_account_table =& $this->mod->getDatabaseTable('smtp_account');

		$sending_account = $smtp_account_table->getItem($item['send_account']);

		// if it doesn't exist, we leave
		if (!$sending_account && $warning_message) {
			$this->displayNoSendAccountMessage(); // Phil 20061121, created function here so we can override
		}

		return $sending_account;
	}
	
	/**
	 * Displays message and exits
	 *
	 * Child classes can override
	 * @author Phil Calder
	 */
	function displayNoSendAccountMessage() {
		global $L;
		$create_account_link = $this->output->getLinkHref(array('href' => '../Mail/account_menu.php', 'text' => $L['mailCreateAccount']));
		
		quickPage($L['mailNoAccount'] . '<br> <br> ' . $create_account_link . ' | '. $this->output->getLinkBack($_SESSION['return_page']));
		exit();
		return;
	}
	
	
	/**
	 * Configures this->mailer to be ready for sending email.
	 *
	 * @param array $item
	 * @param array $sendAccount the account we're using to send
	 */
	function configureMailer(&$item, &$sending_account) {
		
		// see if something might already be set
		if (!is_object($this->mailer)) {
			$this->mailer = new BCRMMailer();
		}
		$this->mailer->configure($item, $sending_account);
	}


	/**
	 * Executed after the database has been written to
	 */
	function afterDBWrite(&$item, &$fields, &$error_msg, $id = false) {  //passed by reference
		global $SERVICES, $L;
		//any other processing after posting to database
		
		// Write addresses to item_email table
		if ($this->form_values['submitSend']) {
			$this->writeRecipientAddressesToDB($id);
		}
		
		// write email to file
		if ($this->writeEmailToFile($id)) {
			//success - set the message size
			//$path = $SERVICES->filesystem->getDirectoryPath('mail');
			$msg_size = $SERVICES->filesystem->getFileSize($id, 'mail');
			if ($msg_size > 0) {
				$this->db_functions->setItem(array('id' => $id, 'message_size' => $msg_size));
			}
		}
					
		// after we're done everything, we're gonna update the original parent, should it exist
		if ($this->parent_item['id']) {
			$this->parent_item['checked_out'] = 0;
			$this->parent_item['action_required'] = 0;

			// these would most likely be more suited in the child classes.
			// *however* there is an exceptional case where a reply or forward has saved as a draft
			// and then sent at a later date, it this situation, we *need* to know who the parent was.
			// and deal with it appropriately - unfortunately, only place is here.
			if ($this->form_values['submitSend']) {
				if ($item['message_send_type'] == 'reply') {
					$this->parent_item['is_replied'] = 1;
				} else if ($item['message_send_type'] == 'forward') {
					$this->parent_item['has_been_forwarded'] = 1;
				}
			}
			
			$this->db_functions->setItem($this->parent_item);
		}
		
		// Phil 20070116 todo: user pref to bypass success message
		if (!$this->getShowSentItemConfirmation()) {
			$this->redirectToList(); // will exit
		}
		
		if ($this->form_values['submitSaveDraft']) {
			$success_msg = $L['mailDraftSuccess'];
		} else if ($this->form_values['submitSaveTemplate']) {
			$success_msg = $L['mailTemplateSuccess'];
		} else {
			$success_msg = $L['mailSuccess'];
		}
		
		//link to this item
		//TODO get id & view.php link
		//$item_link_id = $this->db_functions->getInsertId();
		//echo '***' . $item_link_id;
		$content = '';
		$content .= '<p>' . $this->output->getLinkHref(array('href' => $this->getListPageUrl(), 'text' => $L['ListView'])) . '</p>';
		if ($id) {
			//$content .= '<p>id: ' . $id . '</p>';
			$link_href = $this->getModulePath() . $this->getFileSetHref('view');
			$link_href = str_replace("%id%",$id,$link_href);
			
			$content .= '<p>' . $this->output->getLinkHref(array('href' => $link_href, 'text' => $L['LabelView'] . ': ' . stripslashes($item['subject']))) . '</p>';
		}
		
		//link to send new mail to same address(es)
		
		//link to send same mail to same address(es)
		
		
		//if sent to contacts, links to those contacts, 
		
		
		//get Filter for above message and apply to this link
		
		//TODO 3rd link to current list view
		
		
		
		// finally, display or return success message
		if (!isset($this->return_string) || !$this->return_string) { // Phil 20050506 - same as main - can return message as string OR display and exit
			$this->displaySendSuccessMessage($success_msg, $content); // Phil 20061121
		}
		return ($success_msg . '<br> <br> '. $link);
	}
	
	/**
	 * Displays message and exits
	 *
	 * Override to prevent the script exiting
	 * @author Phil Calder
	 */
	function displaySendSuccessMessage($success_msg, $content = '') {
		quickPage($success_msg . '<br> <br> ' . $content);
		exit();
	}

	/**
	 * Insert into the db, the addresses to whom the email was sent
	 *
	 * @param int $id The item id.
	 */
	function writeRecipientAddressesToDB($id){

		// if there aren't any, or not set, no point in continuing
		if (!isset($this->recipient_addresses) || !is_array($this->recipient_addresses) || (count($this->recipient_addresses) <= 0)) {
			return;
		}

		$item_email_db =& $this->mod->getDatabaseTable('item_email');

		foreach($this->recipient_addresses as $k => $addy) {
			if ($addy) {
				$item_email_db->setItem(array('item_id' => $id, 'address' => $addy));
			}
		}
	}

	/**
	 * Write the email to a file
	 *
	 * @param int $id
	 *
	 * @return bool true or false for successful writing
	 */
	function writeEmailToFile($id) {
		global $SERVICES;
		
		// if no message, then don't continue
		if (!$this->msg || !isset($this->msg) || !$id) {
			return false;
		}
		
		//$path = $SERVICES->filesystem->getDirectoryPath('mail');
		$msg = stripslashes($this->msg['header']) . "\n\n" . stripslashes($this->msg['body']);
		
		if ($SERVICES->filesystem->writeFileContents($id, 'mail', $msg)) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Create the submit row for the email message.
	 *
	 * @param var $extra_submit Optionally throw in more buttons in addition to the defaults
	 */
	 //Saravanakumar :: 20050616 :: validateRecipients() changed to validateForm()
	function getSubmitRow($extra_submit = false) {
		global $L;
		
		$content = '
		<tr><th>&nbsp;</th><td align="left">' . 
			$this->output->getHtmlSubmit($L['mailSend'], 'submitSend', 'onClick="return validateForm();"') . ' &nbsp; ' .
			//$this->output->getHtmlSubmit($L['mailSpellCheck'], 'submitSpellCheck', 'onClick="disableLeaveCheck();"') . ' &nbsp; ' . //Saravanakumar :: 20050708 :: as Matt told
			// taken out since function's no longer used: 'onClick="disableLeaveCheck();"'
			// would normally appear as: getHtmlSubmit($L['mailSaveDraft'], 'submitSaveDraft', 'onClick="disableLeaveCheck();"') 
			$this->output->getHtmlSubmit($L['mailSaveDraft'], 'submitSaveDraft') . ' &nbsp; ' .
			$this->output->getHtmlSubmit($L['mailSaveTemplate'], 'submitSaveTemplate');
		
		if ($extra_submit) {
			if (is_array($extra_submit)) {
				foreach ($extra_submit as $k => $btn) {
					$content .= ' &nbsp; '. $btn;
				}
			} else {
				$content .= ' &nbsp; '. $extra_submit;
			}
		}

		$content .= '	</td>
		</tr>';

		// this corresponds to a previous function
		$content .= '
		<script type="text/javascript" language="Javascript">
			updateContent();
		</script>';

		return $content;
	}
	
	/**
     * Checks if for a delete or update runs the submit function
	 * (this may need some rethinking)
	 */
	function run() {
		//Session set here so that it only happens once per edit
		if (isset($this->form_values['submitSpellCheck']) 
		|| (isset($this->form_values['submitSend'])) 
		|| (isset($this->form_values['submitSaveDraft'])) 
		|| (isset($this->form_values['submitSaveTemplate'])) 
		|| (isset($this->form_values['delete']))) {
			
			if (!$this->form_values['delete']) {
				$this->setRefererAsReturnPage();
				$this->submit = 1;
			}
			$this->doSubmit();
		} else {
			$this->setRefererAsReturnPage();
		}
	}
	
	function getShowSentItemConfirmation() {
		global $SERVICES;
		$prefs =& $SERVICES->getService('Preferences');
		$show_sent_message = $prefs->getUserPref('mailShowSentItemConfirmation');
		return (int)$show_sent_message;
	}
	
	/**
	 * Special delete function for mail: There's 'delete' and then there's 'trash' (2 levels)
	 */
	function doSubmitDelete() {
		global $L;
		$this->item = $this->db_functions->getItem($this->delete);

		if ($this->item['is_trash']) {			
			$this->db_functions->deleteItem($this->delete);				
		} else {
			$this->db_functions->trashItem($this->delete);
		}
		$this->afterDelete($this->item);
	}
  
	/**
	 * Redirects to the spellchecker
	 *
	 * @param array $item
	 */
	function doSpellcheck(&$item) {
		// create the session
		$this->createItemSession($item);	
		header('Location: spellcheck.php?hash=' . $item['hash']);
		exit();
	}
	
	/**
	 * Puts vital information into a session.
	 *
	 * @param array $item
	 */
	function createItemSession(&$item) {
		// double check on the hash
		if (!isset($item['hash'])) {
			$this->setHash($item);
		}
		
		$_SESSION[$item['hash']]['id'] = $item['id'];
		$_SESSION[$item['hash']]['recipient'] = $item['recipient'];
		$_SESSION[$item['hash']]['cc'] = $item['cc'];
		$_SESSION[$item['hash']]['bcc'] = $item['bcc'];
		$_SESSION[$item['hash']]['subject'] = $item['subject'];
		$_SESSION[$item['hash']]['content'] = $item['content'];
		$_SESSION[$item['hash']]['signature'] = $item['signature'];
		$_SESSION[$item['hash']]['return_script_name'] = $_SERVER['SCRIPT_NAME'];  //MCR TODO this looks dodgy
	}
	
	/**
	 * Gets vital information from a session.
	 *
	 * @param array $item
	 */
	function retrieveItemSession(&$item) {
		// double check on the hash
		if (!isset($item['hash']) || !$this->form_values['hash']) {
			return;
		}
		
		if (isset($item['hash'])) {
			$hash = $item['hash'];
		} else {
			$hash = $this->form_values['hash'];
		}
		
		$item['id'] = $_SESSION[$hash]['id'];
		$item['recipient'] = $_SESSION[$hash]['recipient']; 
		$item['cc'] = $_SESSION[$hash]['cc'];
		$item['bcc'] = $_SESSION[$hash]['bcc'];
		$item['subject'] = $_SESSION[$hash]['subject'];
		$item['content'] = $_SESSION[$hash]['content'];
		$item['signature'] = $_SESSION[$hash]['signature'];
	}

	/**
	 * Create, edit, remove the links, as appropriate for the given item (overrides parent)
	 *
	 * @param array $links The array of links for the item
	 * @param int $id
	 * @param array $item
	 */
	function editTableLinks(&$links, $id, $item) {
		// anything that inherits this, should setup their own, otherwise fall back on an empty list (ex: new)
		foreach ($links as $k => $v) {
			$links[$k] = '';
		}
	}
	
	/**
	 * Determines whether the item is editable by the current user
	 *
	 * @param integer $id the item id
	 * @param mixed item - item array or false
	 * @return boolean true if editable, false if not (note: new items are editable)
	 * @author Phil Calder
	 */
	function isEditable($id, $item = false){
		if ($id) { // id passed in is for an existing item
			return $this->db_functions->isEditable($id, $item);
		}
		return true;
	}
	
	/**
	 * Sends the mail.
	 * 
	 * @return 'false' if an error, or the message if success
	 * @author Matthew Ross, Phil Calder
	 */
	function doSend() {
		$this->mailer->setIsCreateForSend();
		if ($this->mailer->Send()) {
			$parts = $this->mailer->GetMessageParts();
			// Phil 20061102
			if (isset($parts['body']) && $this->mailer->includeTagline()) {
				// $parts['body'] = str_replace($this->mailer->getTagline(), '', $parts['body']);
				$tagline = $this->mailer->getTagline();
				$tmp_body = rtrim($parts['body']);
				$start_pos = strlen($tmp_body) - (strlen($tagline));
				if (substr($tmp_body, $start_pos) == $tagline) {
					$parts['body'] = substr($tmp_body, 0, $start_pos);
				}
			}
			return $parts;
		} else {
			return false;
		}
	}
}
?>