<?php
/**
 * The Mail template select menu
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('list.inc');

/**
 * The Mail template select menu class
 * 
 * @package WebMAT
 * @author Phil Calder
 */
class MailItemWebMATMenu extends WebMATMenu {
	
	/**
	 * Constructor function sets all the class variables
	 *
	 * @param  string  $module  the current module name
	 * @param  array  $post  the post array
	 * @param  array  $get  the get array
	 * @param  array $language_overwrite has the keys title and description
	 */
	function MailItemWebMATMenu($module, $post, $get, $menu_links, $language_overwrite = false) {
		$this->WebMATStandardMenu($module, $post, $get, $menu_links, $language_overwrite);
		// print __FILE__ . '<hr>';
	}
	
	/**
	 * Overrides the WebMATList constructor function to disable it
	 */
	function WebMATList() {}
	
	function getLinks() {
		// print '<div>getLinks</div>';
		if (isset($_GET['id']) && $_GET['id']) {
			$id = $_GET['id'];
			$this->menu_links = $this->getMailTemplateReplyLinks($id);
		} else {
			$this->menu_links = $this->getMailTemplateNewLinks();
		}
		return $this->menu_links;
	}
	
	function getMenuLinksTitle() {
		global $L;
		// Select a template
		// 
		return $L['mailLabelSelectTemplate'] . '<br><br>';
	}
	
	function getTemplateMenuItems() {
		$dtf = array(
			'sent_status' => array(
				'where' => "Mail__item.sent_status = 'mailTemplates' AND Mail__item.is_trash = 0 AND Mail__item.active = 1"
			)
		);
		$db_functions = $this->mod->getDatabaseTable('item'); // no &
		$db_functions->resetDesignTimeFilters();
		$db_functions->setDesignTimeFilters($dtf);
		
		$items = $db_functions->getItems();
		
		if (is_array($items)) {
			foreach ($items as $k => $v) {
				if (!$db_functions->isEditable($k)) {
					// print '<div>unset ' . $k . '</div>';
					bcrm_unset($items, $k);
				}
			}
		}
		return $items;
	}
	
	function getMailTemplateReplyLinks($id) {
		$content = array();
			
		$items = $this->getTemplateMenuItems();
		
		global $L;
		$template_text = $L['mailSentStatusTemplate'];
		if (is_array($items)) {
			foreach ($items as $k => $v) {
				// print_r($v); print '<hr>';
				$content[$k] = array(
					'href' => 'reply.php?id=' . $id . '&template=' . $v['id'] . '"', 
					'text' => (strlen($v['subject']) ? $v['subject'] : $template_text . ' ' . $v['id'])
				);
			}
		}
		return $content;
	}
	
	function getMailTemplateNewLinks() {
		$contact_id = ((isset($_GET['contact_id']) && $_GET['contact_id']) ? $_GET['contact_id'] : '');
		$content = array();
			
		$items = $this->getTemplateMenuItems();
		
		global $L;
		$template_text = $L['mailSentStatusTemplate'];
		if (is_array($items)) {
			foreach ($items as $k => $v) {
				// print_r($v); print '<hr>';
				$content[$k] = array(
					'href' => '../../modules/Mail/edit.php?id=' . $v['id'] . '&contact_id=' . $contact_id . '"', 
					'text' => (strlen($v['subject']) ? $v['subject'] : $template_text . ' ' . $v['id'])
				);
			}
		}
		return $content;
	}
}
?>