<?php
/**
 * Mail module view
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Mail
 */

/**
 * Mail view class
 * 
 * @package Mail
 */
class MailItemWebMATView extends WebMATView {
	
	var $container_name = 'item-container-wide';
	
	/**
	 * Need to set up a few items
	 *
	 * @param array $dd_item
	 * @param array $item
	 */
	function calculateValues(& $dd_item, $item) {
		global $SERVICES, $L;
		$decoded_mail = $this->mod->getDecodedMime($item['id']);
		
		$content = '';
		$html = false;
		$attachments = array();
		
		//MCR 20051025
		// multipart/alternative messages come in order of least to most faithful to original content
		// limited multipart/alternative support:
		// only the following will be decoded and used in the order they appear here
		// For example, if there is a html part we use that first, and so on.
		// $text_types = array(
					// 'plain' => 0, 
					// 'enriched' => 1, 
					// 'richtext' => 2, 
					// 'html' => 3
					// );
		// $current_part = '';
		
		$attachments = $this->mod->getAttachmentLinks($decoded_mail, $item['id']);
		
		//MCR 20051025
		//$content = $this->mod->get Content($decode);
		$display_part = $this->mod->getDisplayPart($decoded_mail);
		
		//MCR 20051025
		// if ($this->mod->isHtmlPart($content)) {
			// $html = true;
		// }
		
		// see if there are any attachments, if not, don't need the label
		if (count($attachments) > 0) {
			$dd_item['attach']['value'] = '<table' . $this->output->getTagContent('zeroTbl') . 'c>' . implode('', $attachments) . '</table>';
		} else if (array_key_exists('attach', $dd_item)) {
			bcrm_unset($dd_item, 'attach');
		}
		
		//if ($html) {
		if ($display_part['ctype_secondary'] == 'html') {
			$dd_item['content']['type'] = 'iframe';
			$dd_item['content']['rows'] = 400;
			$dd_item['content']['width'] = 750;
			$dd_item['content']['value'] = 'content.php?id=' . $item['id'];
		} else {
			$dd_item['content']['type'] = 'free';
			
			//$dd_item['content']['value'] = nl2br($content);
			// Phil 20070412 removed //MCR 20051006 this is still uswise to be bypassing Output
			// $dd_item['content']['value'] = nl2br(bcrm_htmlspecialchars($display_part['body'], ENT_QUOTES));
			
			// Phil 20070412 Use Output Service to calculate text email content
			$tmp_content = array(
				'field_key' => 'content',
				'type' => 'long_text',
				'value' => $display_part['body']
			);
			$dd_item['content']['value'] = $this->output->getStringField($tmp_content);
		}
		
		$address_type = array('recipient', 'sender', 'cc', 'bcc');
		
		foreach ($address_type as $k => $v) {
			$addresses = $this->mod->splitAddresses($dd_item[$v]['value']);
			$result = array();
			foreach ($addresses as $kk => $vv) {
				// find potential contact name and link etc
				//MCR 20060503 nowrap
				$result[] = '<tr><td>' . str_replace(' ', '&nbsp;', $vv['from']) . ($vv['from'] ? '&nbsp;' : '') .  '&lt;<a href="edit.php?address=' . $vv['address'] . '">' . $vv['address'] . '</a>&gt;</td><td nowrap> &nbsp; &nbsp; &nbsp; ' . $this->getContactLinks($vv['address']) . '</td></tr>';				
			}			
			if (sizeof($result)) {
				$dd_item[$v]['value'] = '<table' . $this->output->getTagContent('zeroTbl') . '>' . implode('', $result) . '</table>';
			}
		}		
		
		// see if there are any previous actions
		$action_value = $this->mod->getPreviousActionStatement($item['id']);
		if ($action_value) {
			$dd_item['previous_actions']['value'] = $action_value;
		} else if (array_key_exists('previous_actions', $dd_item)) {
			bcrm_unset($dd_item, 'previous_actions'); // get rid of it - no need if nothing to display
		}
		
		$dd_item['recipient']['no_html_translate'] = 1;
		$dd_item['cc']['no_html_translate'] = 1;
		$dd_item['bcc']['no_html_translate'] = 1;
		
		$this->db_functions->calculateValues($dd_item, $item);
	}

	/**
	 * Returns a string of contact links, or string containing links to create a new contact with the given email address
	 *
	 * @param string $address The email address
	 * @todo version 5 - Implement 'Add Address To Contact'
	 */
	function getContactLinks($address) {
		global $SERVICES;
		$contacts = $this->mod->getEmailFromAddress($address, false); // Return multiple
		$contact_link = array();
		if (is_array($contacts) && count($contacts)) {
			foreach ($contacts as $key => $value) {
				$contact_link[] = $this->output->getLinkContact($value['contact_id'], 'new');
			}
		} else {
			global $SERVICES, $L;
			$SERVICES->getService('Language');
			$SERVICES->language->getLanguage('Mail');
			// $contact_link[] = ' <a href="../Contact/new_contact.php?new[AddressEmail][content]=' . $address . '">Create Contact</a> | <a href="../AddressEmail/edit.php?item[content]=' . $address . '">Add Address to Contact</a>';
			$new_contact_type = $SERVICES->preferences->getUserPref('contactNewContactType', '0');
			if ($new_contact_type == 0) {
				$contact_type = 'Person';
			} else {
				$contact_type = 'Company';
			}
			$contact_link[] = ' <a href="../../modules/Contact/contact_edit.php?contact_type=' . $new_contact_type . '&item[email1_content]=' . $address . '">' . $L['mailCreateContact'] . '</a>';
			// TODO: version 5 - add this to above link // ' | <a href="../../modules/AddressEmail/edit.php?item[content]=' . $address . '">' . $L['mailAddAddressToContact'] . '</a>';
		}
		//remove blanks
		foreach ($contact_link as $k => $v) {
			if (!$v) {
				unset($contact_link[$k]);
			}
		}
		return implode(', ', $contact_link);
	}
	
	/**
	 * Create, edit, remove the links, as appropriate for the given item (overrides parent)
	 *
	 * @param array $links The array of links for the item
	 * @param int $id
	 * @param array $item
	 */
	function editTableLinks(&$links, $id, $item) {
		global $L, $SERVICES;
		// special case: if we have 'original', then it's a new window from a forward or reply, and we just want to view it, no links
		if ($this->form_values['parent']) {
			foreach ($links as $k => $v) {
				$links[$k] = '';
			}
			return;
		}
		
		// Phil 20070508
		// Reorganising table links:
		// New List Hide Delete (defined and set elsewhere, we wont be changing these except possibly turning Delete into Delete From Trash)
		// Reply (sub menu: Reply All, Forward, Forward as Attachment, Reply with Template [sub menu: Template list])
		// Settings
		// Mark as ...
		// View Source (sub menu: New Window, Download Source)
		
		// Phil 20060821
		// 'mailReplyAll', 'mailReply'
		
		if ($id) { 
			if ($item['sent_status'] == 'mailDraft') {
				// print '<div>Draft</div>';
				$links['mailReply'] = array(
					'href' => $this->getFileSetHref($this->edit_link), 
					'text' => $L['mailContinue'], 
					'icon' => 'page_misc.png',
					'title' => $L['mailContinue']
				);
				
			} else if($item['sent_status'] == 'mailTemplates') {
				// print '<div>Template</div>';
				$links['mailReply'] = array(
					'href' => $this->getFileSetHref($this->edit_link), 
					'text' => $L['mailUse'], 
					'icon' => 'page_misc.png',
					'title' => $L['mailUse']
				);
				
			} else if (isset($links['mailReply']) && $links['mailReply']) {
				// here we'll do some checks on the item, to setup the proper links
				// firstly, deal with the sent_status
				
				// And add the Reply menu
				// print '<div>reply link:</div>'; print_r($links['mailReply']); print '<hr>';
				// Temporarily removed - needs rewritting for v5
				// $links['mailReply'] = $this->getReplyWithTemplateLink($id) . $links['mailReply'];
				$links['mailReply']['menu'] = array();
				// $links['mailReply']['menu']['replyAll'] = array(
					// 'href' => $this->getFileSetHref('forward'), 
					// 'text' => $L['mailForward'], 
					// 'icon' => 'page_misc.png',
					// 'title' => $L['mailForward']
				// );
				if (isset($links['mailReplyAll']) && is_array($links['mailReplyAll'])) {
					$links['mailReply']['menu']['mailReplyAll'] = $links['mailReplyAll'];
				}
				
				$links['mailReply']['menu']['forward'] = array(
					'href' => $this->getFileSetHref('forward'), 
					'text' => $L['mailForward'], 
					'icon' => 'page_misc.png',
					'title' => $L['mailForward']
				);
				$links['mailReply']['menu']['forward_attachment'] = array(
					'href' => $this->getFileSetHref('forward_attachment'), 
					'text' => $L['mailForwardAsAttachment'], 
					'icon' => 'page_misc.png',
					'title' => $L['mailForwardAsAttachment']
				);
				
				$templates = $this->mod->getTemplateToolbarMenuItems($links['mailReply']);
				if (is_array($templates) && count($templates)) {
					$links['mailReply']['menu']['reply_template'] = array(
						'href' => $this->getFileSetHref('reply'), // each sub menu item will have: . '&template=%template_id%', 
						'text' => $L['mailSentStatusTemplate'], 
						'icon' => 'page_misc.png',
						'title' => $L['mailSentStatusTemplate'], 
						'menu' => $templates
					);
					$links['mailReply']['menu']['reply_template']['menu'] = $this->replaceLinkVariables($id, $item, $links['mailReply']['menu']['reply_template']['menu']);
				}
				
				$links['mailReply']['menu'] = $this->replaceLinkVariables($id, $item, $links['mailReply']['menu']);
				
				bcrm_unset($links, 'mailReplyAll');
				bcrm_unset($links, 'forward');
				bcrm_unset($links, 'forward_attachment');
			}
		}
		
		// Bosco 20060328 | Disable links by checking db field
		$required_status = $this->db_functions->isMarkedAsActionRequired($id);
		if ($required_status === true) {
			bcrm_unset($links, 'mailMarkAsRequired');
		} else if ($required_status === false) {
			bcrm_unset($links, 'mailMarkAsNotRequired');
		}
		
		

		if ($item['is_trash']) {
			// need to also reset the $links['delete'] value, should it need it
			if (array_key_exists('delete', $links)) {
				$links['delete'] = $this->mod->getDeleteFromTrashLink($id, $item['is_trash']);
			}
		}
		
		// check if we can edit the properties
		if (array_key_exists('mailSettings', $links) && !$this->db_functions->isEditable($id, $item)) {
			$links['mailSettings'] = '';
		} else {
			$links['mailSettings']['menu'] = array();
			
			$links['mailSettings']['menu']['view_source'] = array(
				'href' => $this->getFileSetHref('view_source'), 
				'text' => $L['mailViewSource'], 
				'icon' => 'page_misc.png',
				'title' => $L['mailViewSource']
			);
			
			$links['mailSettings']['menu']['download_source'] = array(
				'href' => $this->getFileSetHref('download_source'), 
				'text' => $L['mailDownloadSource'], 
				'icon' => 'page_misc.png',
				'title' => $L['mailDownloadSource']
			);
			
			$links['mailSettings']['menu'] = $this->replaceLinkVariables($id, $item, $links['mailSettings']['menu']);
			
			/*
			$links['view_source'] = $SERVICES->output->getLinkHref(
				array(
					'href' => $this->getFileSetHref('view_source'), 
					'text' => $L['mailViewSource']
				)
			) . $SERVICES->output->getLinkHref(
				array(
					'href' => $this->getFileSetHref('view_source'), 
					'text' => '[+]', 
					'title' => $L['LinkOpenInNewWindow'], 
					'target' => '_blank'
				)
			);
			//$current .= ' <a href="attachment.php?id=' . $id . '&part=' . ($k + 1) . '" target="_blank" title="' . $L['LinkOpenInNewWindow'] . '">[+]</a>';
			
			$links['download_source'] = $SERVICES->output->getLinkHref(
				array(
					'href' => $this->getFileSetHref('download_source'), 
					'text' => $L['mailDownloadSource']
				)
			); //MCR 20060504
			*/
		
		}
		
		bcrm_unset($links, 'edit');
		
		// print '<div>links after:</div>'; print_r($links); print '<hr>';
	}
	
	/**
	 * @author Phil Calder
	 */
	function getReplyWithTemplateLink($id) {
		$this->output->req_contactMenu_js = true;
		$mail_template_menu_link = $this->output->getLinkHref(
			array(
				'href' => 'javascript:mailTemplateReplyMenu(\'' . 'mtrMen' . $id . '\',\'' . $id . '\');',
				'img' => $this->output->getFlagMenu('mtrMen' . $id),
				'tag_extras' => ' onMouseOut="hM(\'contactMen\')" '
			),
			false // no language text check
		);
		return $mail_template_menu_link;
	}
	
	/**
	 * since view also doubles as the hide/unhide vessel, we need to override
	 * if we're simply hiding or unhiding, we don't want to do anything other
	 * than just that.
	 *
	 * @param $item array that contains the item's info.
	 */
	function markAsSeen($item) {
		// if we're not hiding/unhiding, then set it to open
		if (!$this->form_values['hide'] && !$this->form_values['unhide']) {
			$db_functions =& $this->mod->getDatabaseTable('item');
			$update['id'] = $item['id'];
			$update['is_opened'] = 1;
			$db_functions->setItem($update);
		}
		parent::markAsSeen($item);
	}

	// Bosco 20060328 | Added funtions for Mark mail as action required / Not required
	function markAsRequired($id) {
		if (!empty($id)){
			$item = array('id' => $id, 'action_required' => 1);
			$this->db_functions->setItem($item);
		}
	}

	function markAsNotRequired($id) {
		if (!empty($id)) {
			$item = array('id' => $id, 'action_required' => 0, 'is_opened' => 1);
			$this->db_functions->setItem($item);
		}
	}
}
?>