<?php
/**
 * Class for Email-Merge functionality
 * 
 * Key methods to look at here are:
 * @see doSubmit()
 *
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Mail
 * @author Phil Calder
 */

includeModule('Mail', 'inc/item_edit.inc');

class MailMailmergeWebMATEdit extends MailItemWebMATEdit {

	/**
	 * Parses the ids_session for contacts
	 * 
	 * @return array $contact_ids array of contact ids (which may be empty)
	 * @author Phil Calder
	 */
	function getContactsForMailmerge() {
		$contact_ids = array();
		// Phil: Need to look for ids_session AND contact_session_id
		if (isset($this->form_values['ids_session']) && $this->form_values['ids_session'] && is_array($_SESSION[$this->form_values['ids_session']])) {
			$contact_ids = array_unique($_SESSION[$this->form_values['ids_session']]);
		} else if (isset($this->form_values['contact_session_id']) && $this->form_values['contact_session_id'] && is_array($_SESSION[$this->form_values['contact_session_id']])) {
			$contact_ids = array_unique($_SESSION[$this->form_values['contact_session_id']]);
		}
		return (array)$contact_ids;
	}
	
	/**
	 * Removed other submit buttons (spellcheck etc). Disabled leave-checking for the submit button.
	 *
	 * @param var $extra_submit Optionally throw in more buttons in addition to the defaults
	 */
	function getSubmitRow($extra_submit = false) {
		global $L;
		
		// 20060316 changed str_repeat from to 1 from 2
		$content = '
		<tr>
			' . str_repeat('<td>&nbsp;</td>', 1) . ' 
			<td align="left">' . 
			//$this->output->getHtmlSubmit($L['mailSend'], 'submitSend', 'onClick="return disableLeaveCheck();"') . ' &nbsp; ' .
			$this->output->getHtmlSubmit($L['mailSend'], 'submitSend') . ' &nbsp; ' .
			// $this->output->getHtmlSubmit($L['mailSpellCheck'], 'submitSpellCheck', 'onClick="disableLeaveCheck();"') . ' &nbsp; ' . 
			//$this->output->getHtmlSubmit($L['mailSaveDraft'], 'submitSaveDraft', 'onClick="disableLeaveCheck();"') . ' &nbsp; ' .
			//$this->output->getHtmlSubmit($L['mailSaveTemplate'], 'submitSaveTemplate', 'onClick="disableLeaveCheck();"') . 
			'';
		
		if ($extra_submit) {
			if (is_array($extra_submit)) {
				foreach ($extra_submit as $k => $btn) {
					$content .= ' &nbsp; '. $btn;
				}
			} else {
				$content .= ' &nbsp; '. $extra_submit;
			}
		}

		$content .= '	</td>
		</tr>';

		// this corresponds to a previous function
		$content .= '
		<script type="text/javascript" language="Javascript">
			updateContent();
		</script>';

		return $content;
	}
	
	/**
	 * @deprecated version 4.5
	 * @see getDescriptionContent
	 */
	function getDescriptionRow() {
		return $this->getDescriptionContent();
	}
	
	/**
	 * Adds contact and email address counts to the standard description row
	 * 
	 * @param array $item the item array
	 * @param array $error_msg the error message array
	 * @return string the content for the form
	 * @see getContactCountRow()
	 * @see getEmailAddressCountRow()
	 */
	function getDescriptionContent() {
		$content = parent::getDescriptionContent();
		
		$content .= $this->getContactCountRow();
		$content .= $this->getEmailAddressCountRow();
		
		return $content;
	}
	
	/** 
	 * Returns a table row containing the contact count
	 * @see getDescriptionRow()
	 */
	function getContactCountRow() {
		global $L;	
		
		$contact_ids = $this->getContactsForMailmerge();
		
		$description = $L['mailEmailmergeNumberOfContacts'] . ': ' . count($contact_ids);
		
		$contact_count = count($contact_ids);
		if ($contact_count) {
			$description .= ' ' . $this->getContactListLink();
		}
		
		$padding = true;
		$content = $this->output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>';
		$content .= $description;
		$content .= '</td>' . $this->output->getEndRow(true);
		
		return $content;
	}
	
	/**
	 * Link to the Contact list page, filtered on contact ids
	 */
	function getContactListLink() {
		global $L;
		// $session_id = $this->getContactSessionId();
		if (isset($this->form_values['ids_session']) && $this->form_values['ids_session']) {
			$session_id = $this->form_values['ids_session'];
		} else if (isset($this->form_values['contact_session_id']) && $this->form_values['contact_session_id']) {
			$session_id = $this->form_values['contact_session_id'];
		}
		// TODO config lookup
		return '<a href="../../modules/Contact/index.php?ids_session=' . $session_id . '&filter=fnone&qfilter=&search_filter=off">' . $L['LabelView'] . '</a>';
	}
	
	/** 
	 * Returns a table row containing the email address count
	 * @see getDescriptionRow()
	 */
	function getEmailAddressCountRow() {
		global $L;
		
		$contact_ids = $this->getContactsForMailmerge();
		
		$valid_email_addresses = 0;
		
		foreach ($contact_ids as $k => $v) {
			$address = $this->mod->getEmailFromContact($v);	
			if (is_array($address) && isset($address['address'])) { // Must have email 'address', 'from' name is optional
				$valid_email_addresses++;	
			}
		}
		
		$description = $L['mailEmailmergeNumberOfEmailAddresses'] . ': ' . $valid_email_addresses;
		$padding = true;
		$content = $this->output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>';
		$content .= $description;
		$content .= '</td>' . $this->output->getEndRow(true);
		
		return $content;
	}
	
	/**
	 * Creates an object for each email address to send to.
	 * Prints a summary of contact and email sent then exits
	 */
	function doSubmit() {
		
		$content = $this->output->getBeginTable();
		// $content = $this->getContactAndEmailAddressCount();
		$contact_ids = $this->getContactsForMailmerge();
		$content .= $this->getContactCountRow();
		$number_sent = $valid_email_addresses = 0;
		
		if (is_array($contact_ids)) {
			foreach ( $contact_ids as $k => $v) {
				// print '<div>send to: ' . $v . '</div>';
				$form_values = $this->form_values;
				
				$address = $this->mod->getEmailFromContact($v);	
				
				if (is_array($address) && isset($address['address'])) { // Must have email 'address', 'from' name is optional
					
					// $form_values['item']['recipient'] = $address;
					if (!empty($address)) {
						$addr_string = '';
						if ($address['from']) {
							$addr_string .= $address['from'] . ' ';
						}
						$addr_string .= '<' . $address['address'] . '>';
						$form_values['item']['recipient'] = $addr_string;
					}
					
					if ($form_values['item']['recipient']) {
						
						bcrm_unset($form_values['item'], 'hash'); // Phil 20060317
						
						$valid_email_addresses++;
						
						$mail_edit_object = getWebMATObject(
							'WebMATEdit', 
							'Mail', 
							array(
								'data_option' => 'edit',
								'piece_identifier' => 'mailmergesingleedit', 
								'post' => $form_values
							)
						);
						
						$mail_edit_object->return_string = true; // Prevent class from displaying output and exiting
						// $mail_edit_object->setReturnString(true);// Phil 20060316
						$mail_edit_object->run();
						$mail_edit_object->return_string = true; // Prevent class from displaying output and exiting
						$ignored_content = $mail_edit_object->main(); // We wont actually use the content here, but its good practice to at least store it
						$number_sent++;
						// print '<p>number sent: ' . $number_sent . '</p>';
					}
				}
			}
				
		} 
		
		global $L;
		$content = '';
		$padding = true;
		
		$description = $L['mailEmailmergeNumberOfContactsWithEmailAddresses'] . ': ' . $valid_email_addresses . ' ' . $this->getContactListLink();
		
		$content .= $this->output->getBeginTable();
		
		$content .= $this->output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>';
		$content .= $description;
		$content .= '</td>' . $this->output->getEndRow(true);
		
		$description = $L['mailEmailmergeNumberOfEmailsSent'] . ': ' . $number_sent;
		
		$content .= $this->output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>';
		$content .= $description;
		$content .= '</td>' . $this->output->getEndRow(true);
		
		$content .= $this->output->getEndTable();
		
		$this->output->addToBody($content);
		$this->output->display();
		exit();
		
	}
	
}

?>