<?php
/**
 * Mail reply
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Mail
 */

includeModule('Mail', 'inc/item_edit.inc');

/**
 * Reply to an email
 * 
 * @package Mail
 */
class MailReplyWebMATEdit extends MailItemWebMATEdit {

	/**
	 * Check out message, get previous actions
	 */
	function getItem($id = false, $item = false) {
		if (!$item) {
			$item = parent::getItem($id, $item);
			
			$this->checkoutMessage($item);
			// see if there are any previous actions
			$action_value = $this->mod->getPreviousActionStatement($item['id']);
			if ($action_value) {
				$item['previous_actions'] = $action_value;
			} else if (array_key_exists('previous_actions', $this->dd->data_dictionary)) {
				bcrm_unset($this->dd->data_dictionary, 'previous_actions'); // get rid of it - no need if nothing to display
			}
			
			$this->loadTemplateData($item);
		}
		// print '<div>item:</div>'; print_r($item); print '<hr>';
		return $item;
	}
	
	/**
	 * Set the recipients (to, cc, bcc)
	 *
	 * @param array $item
	 * @param string $decode mime decode content (optional)
	 */
	function setRecipients(& $item, $decode = false) {
		global $SERVICES;
		
		// Strip the address the message was delivered to (our address) from the list of all recipients
		$toAddresses = $this->mod->splitAddresses($item['recipient']);
		$ccAddresses = $this->mod->splitAddresses($item['cc']);
		
		// reset & set
		$item['cc'] = '';
		$item['bcc'] = '';
		//Saravanakumar :: 20050707 :: Checking reply-to in headers
		if ($decode->headers['reply-to']) {
			$item['recipient'] = stripslashes($this->htmlToSpecialChars($decode->headers['reply-to']));	
		} else {
			$item['recipient'] = stripslashes($this->htmlToSpecialChars($item['sender']));
		}
		
		// Reply all? - if not, no worries, we've already set the recipients
		if ($this->form_values['all'] && $decode) {
			
			$cc = array();
			//Saravanakumar :: 20050706 :: To find the exact sender address without name
			$senderNameAddress = $this->mod->splitAddresses($item['sender']);
			$senderAddress = $senderNameAddress[0]['address'];
			// do the 'to's
			if (is_array($toAddresses)) {
				foreach ($toAddresses as $k => $v) { 
					// we check the 'to' and 'delivered-to', so we don't send it to back ourselves (cause we'll be in the list)
					
					//Saravanakumar :: 20050706 :: Commented because of Logical mistakes, it's changed to compare with Sender address
					//if ($v['address'] != $decode->headers['delivered-to'] && $v['address'] != $decode->headers['to']) {
					if ($v['address'] != $decode->headers['delivered-to'] && $v['address'] != $senderAddress) {
						// Construct the "nice" from field to add to the CC
						$cc[] = trim($v['from']) . ' <' . $v['address'] . '>'; 
						//2005/05/18   add a space  trim the string
					}
				}
			}
			
			// do the 'cc's
			if (is_array($ccAddresses)) {
				foreach ($ccAddresses as $k => $v) {
					// we check the 'to' and 'delivered-to', so we don't send it to back ourselves (cause we'll be in the list)
					if ($v['address'] != $decode->headers['delivered-to'] && $v['address'] != $senderAddress) {
						// Construct the "nice" from field to add to the CC
						$cc[] = trim($v['from']) . ' <' . $v['address'] . '>';	//2005/05/18   add a space  trim the string
					}
				}
			}
			
			//Saravanakumar :: 20050706 :: Remove duplicate addresses
			if (is_array($cc)) {
				for ($i = 0; $i <= count($cc); $i++) {
					for ($j = $i+1; $j <= count($cc); $j++) {
						if ($cc[$i] == $cc[$j]) {
							bcrm_unset($cc, $j);
						}
					}
				}
			}
			
			//Remove the trailing ',' and ';'
			foreach ($cc as $k => $v) {
				if (substr(trim($v),0,1) == ',' || substr(trim($v),0,1) == ';') {
					$cc[$k] = str_replace(substr(trim($v),0,1), '', $v);
				}
			}
			
			if (!empty($cc)) {
				$item['cc'] = trim(implode(', ', $cc));
			}
		}
		
		$default_bcc = $SERVICES->preferences->getUserPref('mailDefaultBccAddresses');
		if ($default_bcc) {
			$item['bcc'] = $default_bcc;
		}
		
	}
	
	/**
	 * sets the subject
	 *
	 * @param array $item
	 */
	function setSubject(& $item) {
		parent::setSubject($item);  //MCR 20051007
		// Add the RE or FW prefixes to the subject 
		//MCR 20051007  
		//TODO we can add something so that after 3 or 4 'RE'it just shows '...' or something
		//(only do this once so we don't end up with RE: RE: RE: etc)
		//if (strpos($item['subject'], 'RE') === FALSE) {
			//TODO add language
			$item['subject'] = 'RE: ' . $item['subject'];
		//}
	}
	
	function setInitialAttachments(& $item, $decoded_mail = false) {
	}
	
	/**
	 * Operations before we actually write to the db, included in here is sending
	 */
	function beforeDBWrite(&$item, &$fields, &$error_msg) {  //passed by reference
		parent::beforeDBWrite($item, $fields, $error_msg);
		// if we've made it this far, continue with a few 'reply' customizations
		// the parent is who the original id was, so include it
		$item['parent_id'] = $this->form_values['id'];
		$item['message_send_type'] = 'reply';
		$item['is_opened'] = 1;
		// Make sure we are going to get a new row in the table for this (this _IS_ afterall, an edit page)
		bcrm_unset($item, 'id');
	}

	/**
	 * Create, edit, remove the links, as appropriate for the given item (overrides parent)
	 *
	 * @param array $links The array of links for the item
	 * @param int $id
	 * @param array $item
	 */
	function editTableLinks(& $links, $id, $item) {
		global $L, $SERVICES;
		// forwarding is essentially creating a new item, so we don't need any links
		foreach ($links as $k => $v) {
			if ($k != 'list') {
				bcrm_unset($links, $k);
			}
		}
		// but maybe we want to view the original?
		$links['mailViewParent'] = $SERVICES->output->getLinkHref(array('href' => $this->getFileSetHref('view_parent'), 'text' => $L['mailViewParent'], 'target' => '_new'));
	}
	
	/**
	 * Over-ride the parent to disply a more appropriate title
	 */
	function getTitleText() {
		global $L;
		return $L['mailTitle_replyedit'];
	}
	
	/**
	 * @author Phil Calder
	 */
	function loadTemplateData(&$item) {
		if (isset($this->form_values['template']) && $this->form_values['template']) {
			// $template_item = parent::getItem($this->form_values['template']);
			if ($this->isEditable($this->form_values['template'])) {
				$template_data = $this->mod->getDecodedMime($this->form_values['template']);
				// print '<div>loadTemplateData item id: ' . $item['id'] . '</div>'; print_r($item); print '<hr>';
				// print '<div>loadTemplateData template id: ' . $this->form_values['template'] . '</div>'; print_r($template_item); print '<hr>';
				// print '<div>loadTemplateData template id: ' . $this->form_values['template'] . '</div>'; print_r($template_data); print '<hr>';
				if (is_object($template_data)) {
					// print '<div>setting item content</div>';
					$item['content'] = $template_data->body . $item['content'];
					if (!strlen($item['subject'])) {
						// print '<div>setting item subject</div>';
						$item['subject'] = $template_data->subject;
					}
				}
			}
			bcrm_unset($this->form_values, 'template'); // ensures this block is only executed once
		}
		return $item;
	}
}
?>