<?php
require_once('../../prepend.inc');
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 */
 
 
/**
 * Provides garbage collection interface for WebMAT modules
 *
 * Administrators can initiate a data cleanup for selected modules.
 *
 * @package UserAdmin
 * @author Phil Calder
 */
class AdminDataCleanup {
	
	function adminDataCleanup() {
		$this->init();
	}
	
	function init() {
		global $SERVICES, $L;
		$lang =& $SERVICES->getService('Language');
		$lang->getLanguage('UserAdmin');
		
		$this->config =& $SERVICES->getService('Config');
		$this->modules =& $SERVICES->getService('Modules');
		$this->output =& $SERVICES->getService('Output');
	}
	
	function getCleanupModules() {
		$cleanup_modules = array();
		$cleanup_modules = $this->config->lookup('modules.Admin.cleanup');	
		if (isset($cleanup_modules['Contact']) && $cleanup_modules['Contact']) {
			$value = $cleanup_modules['Contact'];
			bcrm_unset($cleanup_modules, 'Contact');
			$cleanup_modules['Contact'] = $value;
		}
		return $cleanup_modules;
	}
	
	/**
	 * Runs a data cleaning method for certain modules
	 * 
	 * @param mixed $cleanup_modules Default false, or can be an array of modules to process
	 * key is module name, value is boolean (o - dont cleanup, 1 - cleanup)
	 * @return boolean True if no errors reported, false if any error/problem arises
	 */
	function runModuleCleanupData($cleanup_modules = false, $full_cleanup = false) {
		global $L;
		
		if (!is_object($this->config) || !is_object($this->modules)) {
			return false;
		}
		
		if (!is_array($cleanup_modules)) {
			$cleanup_modules = array();
			// $cleanup_modules = $this->getCleanupModules();// Phil - this would return all modules - better to force the user to select modules
		}
		// print_r($cleanup_modules); print '<hr>';
		
		$content = '';
		
		// print '<div>cleanup modules</div>'; print_r($cleanup_modules); print '<hr>';
		
		if ( is_array($cleanup_modules) ) {
			foreach ($cleanup_modules as $k => $v) {
				if ( $v && is_object($mods[$k] =& $this->modules->getModule($k)) ) {
					if (method_exists($mods[$k], 'cleanupData')) {
						// print '<div>mod: ' . $mods[$k]->getModuleName() . ' class: ' . get_class($mods[$k]) . ' calling cleanupData [full: ' . $full_cleanup . ']</div>';
						$retval = $mods[$k]->cleanupData($full_cleanup);
						
						// We only look for boolean values here (any other value including NULL will be ignored)
						if ($retval === true) {
							$content .= '<p>' . $L['useradminAdminCleanupDataModule'] . ': ' . $mods[$k]->getModuleName() . ' ' . $L['useradminAdminCleanupDataSuccessful'] . '</p>';
						} else if ($retval === false) {
							$content .= '<p>' . $L['useradminAdminCleanupDataModule'] . ': ' . $mods[$k]->getModuleName() . ' ' . $L['useradminAdminCleanupDataFailed'] . '</p>';
						}
						// print '<div>retval: ' . $retval . '</div>';
					}
				}
			}
		}
		return $content;
	}
	
	function doSubmit() {
		if (isset($_POST['submit']) && $_POST['submit']) {
			
			// print '<div>POST:</div>'; print_r($_POST); print '<hr>';
			$full_cleanup = ( (isset($_POST['item']['full']) && $_POST['item']['full']) ? true : false);
			
			$this->cleanup_modules = false;
			if ( $_POST['item']['modules'] && is_array($_POST['item']['modules']) ) {
				$this->cleanup_modules = array();
				foreach ($_POST['item']['modules'] as $k => $v) {
					$this->cleanup_modules[$v] = 1;
				}
			}
			
			// print '<div>modules:</div>'; print_r($this->cleanup_modules); print '<hr>';
			// print '<div>full_cleanup: ' . ($full_cleanup ? 'yes' : 'no') . '</div>'; print '<hr>';
			
			$this->returned = $this->runModuleCleanupData($this->cleanup_modules, $full_cleanup); 
			// print '<hr>';
		} else {
			// print '<div>not submitting</div>';
		}
	}
	
	function main() {
		global $L;
		
		$content = '';
		// $content .= '
		// <html>
		// <body>
		// ';
		
		if (isset($this->returned) && $this->returned) {
			$content .= '<p>' . $L['useradminAdminCleanupDataStatus'] . '</p>';
			$content .= $this->returned;	
		}
		
		if (is_array($cleanup_modules = $this->getCleanupModules())) {
			
			$content .= '<p>' . $L['useradminAdminCleanupDataFormExplanation'] . '</p>';
			
			$content .= '
			<form method=post>
			<select multiple style="width: 240px" size="7" class="" name="item[modules][]" >
			';
			if (is_array($cleanup_modules)) {
				foreach ($cleanup_modules as $k => $v) {
					if ($v) {
						$content .= '<option value="' . $k . '">' . $k . '</option>';
					}
				}
			}
			$content .= '
			</select>
			<br />
			<div>' . $L['useradminAdminCleanupDataFull'] . '</div>
			<div><input type="checkbox" name="item[full]" value=1></div>
			<br />
			<input type="submit" name="submit" value="' . $L['ButtonSubmit'] . '">
			</form>
			';
		}
		
		// Phil 20060207 Adding new UI layout (mimics WebMATList)
		$window_layout = $this->output->getBeginTable('mainTbl', true);
		$window_layout .= $this->output->getTitleRow($L['useradminAdminCleanupData'], array('bookmark'=>'off'));
		$window_layout .= $this->output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>';
		$window_layout .= $content;
		$window_layout .= '</td>' . $this->output->getEndRow(true);
		$window_layout .= $this->output->getEndTable(true);
		
		$this->output->addToBody($window_layout);
		$this->output->display();
		
		exit();
	}
}

$admin_data_cleanup = new AdminDataCleanup();

if (is_object($admin_data_cleanup)) {
	$admin_data_cleanup->doSubmit();
	$admin_data_cleanup->main();
} else {
	exit('Error: cannot initiate data cleanup process');
}


?>