<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 */

global $login;
$login['host'] = 1;
$login['username'] = '_pub';
$login['password'] = 'public';
$login['webform'] = true;

if (isset($bcrm_pub_root)) {
	$bcrm_root = substr($bcrm_pub_root, 0, (strlen($bcrm_pub_root) - 4));
	// print $bcrm_root . '<hr>'; exit();
	require_once($bcrm_root . "/prepend.inc");
} else {
	require_once("../prepend.inc");
}

/** 
 * @author Phil Calder
 */
class BrowserCRMWebForm {
	
	/**
	 * @access private
	 */
	var $config_file = 'webform.config.php';
	
	/**
	 * @access private
	 */
	var $custom_config_file = 'custom.webform.config.php';
	
	/**
	 * @access private
	 */
	var $web_form_identifier = false;
	
	/**
	 * @access private
	 */
	var $config = array();
	
	/**
	 * @access private
	 */
	var $web_form_include_path = '';
	
	/**
	 * @access private
	 */
	var $form_values = array();
	
	/**
	 * @access private
	 */
	var $mapping = array();
	
	/**
	 * @access private
	 */
	var $contact_info = array();
	
	/**
	 * @access private
	 */
	var $messages = array(
		'form_unavailable'    => 'Form unavailable', 
		'process_failed'      => 'Process failed', 
		'process_successful' => 'Process successful', 
		'required_fields_error' => 'Please return to the form and complete all required fields.', 
		'login' => 'Login', 
		'logout' => 'Logout', 
		
	);
	
	/**
	 * @access private
	 */
	var $template_path = 'inc/webform.template.inc.php';
	
	function BrowserCRMWebForm($args = array()) {
		$this->init($args);
	}
	
	function init($args = array()) {
		// WebForms need an identifier. Used to find WebFrom config settings.
		if (isset($args['identifier'])) {
			// already set
		} else if (is_array($_GET['args']) && isset($_GET['args']['identifier'])) {
			$args['identifier'] = $_GET['args']['identifier'];
		} else {
			// Default contact form
			$args['identifier'] = 'default';
		}
		
		$this->setWebFormIncludePath();
		$this->setStandardConfigFile();
		$this->setCustomConfigFile();
		$this->setConfigSettings();
		$identifier = (is_array($args) && isset($args['identifier']) ? $args['identifier'] : get_class($this));
		$this->setWebFormIdentifier($identifier);
		$this->setFormValues();
		$this->setMapping();
	}
	
	function main($id = '', $item = '', $error_msg = '') {
		// print 'main ';
		// TODO: handle error message
		// if ($error_msg) {
			// $this->doProcessFailed($id, $item, $error_msg); // Will exit
		// }
		
		if ($this->isSubmit()) { // check $this->form_values?
			$result = $this->doSubmit();
			
			if ($result) {
				$this->doProcessSuccess($id, $item, $error_msg);
			} else {
				$this->doProcessFailed($id, $item, $error_msg);
			}
		} else {
			if ($this->webFormIsViewable()) {
				$this->doDisplay($id, $item, $error_msg);
			} else {
				$this->doProcessFailed($id, $item, $error_msg);
			}
		}
	}
	
	/**
	 * @access private
	 */
	function setWebFormIncludePath() {
		global $bcrm_pub_root;
		if (isset($bcrm_pub_root)) {
			$this->web_form_include_path = $bcrm_pub_root . '/';
		}
	}
	
	/**
	 * @access private
	 */
	function getWebFormIncludePath() {
		return $this->web_form_include_path;
	}
	
	
	/**
	 * Override to use a different standard config file
	 *
	 * @access private
	 */
	function setStandardConfigFile() {
		// $this->config_file = 'webform.config.php';
	}
	
	/**
	 * Override to use a different custom config file
	 */
	function setCustomConfigFile() {
		// $this->custom_config_file = 'custom.webform.config.php';
	}
	
	/**
	 * @access private
	 */
	function setConfigSettings() {
		$webform_config_file = $this->getWebFormIncludePath() . $this->config_file;
		$custom_webform_config_file = $this->getWebFormIncludePath() . $this->custom_config_file;
		// print '<p>config: ' . $this->config_file . '</p>';
		// print '<p>custom config: ' . $this->custom_config_file . '</p>';
		$config = array();
		
		// Populate $config variable
		if (file_exists($webform_config_file) && is_readable($webform_config_file)) {
			// print '<p>config file:' . $webform_config_file . '</p>';
			@include($webform_config_file);
			// print_r($config); print '<hr>';
		}
		
		// Populate $custom_config variable
		if (file_exists($custom_webform_config_file) && is_readable($custom_webform_config_file)) {
			@include($custom_webform_config_file);
		}
		
		// merge into single config variable
		if (is_array($config) && isset($custom_config) && is_array($custom_config) ) {
			global $SERVICES;
			if (is_object($SERVICES->config) && method_exists($SERVICES->config, 'arrayMergeReplace')) {
				$config = $SERVICES->config->arrayMergeReplace($config, $custom_config);
			}
		}
		
		// Store class variable
		$this->config = $config;
	}
	
	function getConfig() {
		return (array)$this->config;
	}
	
	function setWebFormIdentifier($identifier) {
		$this->web_form_identifier = $identifier;
	}
	
	function getWebFormIdentifier($identifier) {
		return $this->web_form_identifier;
	}
	
	function setFormValues() {
		$this->form_values = array();
		// Where to look for the form data:
		// - 'post': the $_POST variable
		// - 'get': the $_GET variable
		// - 'request': combined post and get
		// - 'var': look for a variable called 'bcrm_form_data'
		// - 'config': look for config key 'bcrm_form_data' (eg $config['default']['bcrm_form_data'])
		if (isset($this->config[$this->web_form_identifier]) && is_array($this->config[$this->web_form_identifier])) {
			switch ($this->config[$this->web_form_identifier]['data_source']) {
				case 'post':
					$this->form_values = (array)$_POST;
				break;
				case 'get':
					$this->form_values = (array)$_GET;
				break;
				case 'request':
					$this->form_values = (array)$_REQUEST;
				break;
				case 'var':
					global $bcrm_form_data;
					$this->form_values = (array)$bcrm_form_data;
				break;
				case 'config':
				if (isset($this->config[$this->web_form_identifier]['bcrm_form_data'])) {
					$this->form_values = (array)$this->config[$this->web_form_identifier]['bcrm_form_data'];
				} else {
					$this->form_values = (array)$this->config['default']['bcrm_form_data'];
				}
				break;
			}
		}
	}
	
	function getFormValues() {
		return (array)$this->form_values;	
	}
	
	function setMapping() {
		$this->mapping = array();
		if (isset($this->config[$this->web_form_identifier]['mapping'])) {
			$this->mapping = (array)$this->config[$this->web_form_identifier]['mapping'];
		}
	}
	
	function getMapping() {
		return $this->mapping;
	}
	
	function setContactInfo($identifier = 'company', $value = false) {
		$this->contact_info[$identifier] = $value;
	}
	
	function getContactInfo($identifier = false) {
		if ($identifier) {
			if (is_array($this->contact_info) && isset($this->contact_info[$identifier])) {
				return $this->contact_info[$identifier];
			} else {
				return false;
			}
		}
		return $this->contact_info;
	}
	
	/** 
	 * Determines whether the form has been submitted
	 */
	function isSubmit() {
		if ($is_submit = $this->config[$this->web_form_identifier]['is_submit']) {
			if ($is_submit === true) {
				// print '<div>is_submit: yes [1]1</div>';
				return true;
			} else if ($is_submit === false) {
				// print '<div>is_submit: no [1]</div>';
				return false;
			}
			if (isset($this->form_values[$is_submit]) && $this->form_values[$is_submit]) {
				// print '<div>is_submit: yes [2]</div>';
				return true;
			}
		}
		// print '<div>is_submit: no [2]</div>';
		return false;
	}
	
	/**
	 * Uses field mapping to process each entity. 
	 *
	 * If more than one entity is processed, the result of the first processed entity is returned.
	 *
	 * @see processEntity
	 *
	 * @return mixed false or success return value (often an int).
	 */
	function doSubmit() {
		if ($this->isClientLoginRequired()) {	
			$this->clientLogin();
		}
		
		$this->doCheckRequiredFields();
		
		$mapping = $this->getMapping();
		$args = array();
		
		$first = true;
		foreach ($mapping as $k => $v) {
			$tmp_result = $this->processEntity($k, $v);
			if ($first) {
				$result = $tmp_result;
				$first = false;
			}
		}
		return $result;
	}
	
	function doCheckRequiredFields() {
		if (isset($this->config[$this->web_form_identifier]['check_required']) && $this->config[$this->web_form_identifier]['check_required']) {
			$mapping = $this->getMapping();
			$err_required_fields = array();
			foreach ($mapping as $k => $v) {
				// Check all required fields with a src have a value
				if (is_array($v) && isset($v['data'])) {
					foreach ($v['data'] as $kk => $vv) {
						if (isset($vv['required']) && $vv['required']) {
							// print '<p>required field: ' . $vv['src'] . '</p>';
							if (isset($vv['src']) && (!$this->form_values[$vv['src']] && !$vv['default'])) {
								// print '<p>required field: ' . $vv['src'] . ' has no value [value: ' . $this->form_values[$vv['src']]. ', default: ' . $this->form_values[$vv['src']]. ']</p>';
								$err_required_fields[] = $vv['src'];
							}
						}
					}
				}
				
			}
			if (count($err_required_fields)) {
				$this->doDisplayRequiredFieldsError($err_required_fields);
			}
		}
	}
	
	function isClientLoginRequired() {
		return ($this->config[$this->web_form_identifier]['require_login'] ? true : false);
	}
	
	function isClientLoggedIn() {
		global $SERVICES;
		$auth =& $SERVICES->getService('Authentication');
		if ($auth->isWebFormUserLoggedIn()) {
			return true;
		}
		return false;
	}
	
	function getClientLoginLinks() {
		$content = '';
		if ($this->isClientLoggedIn()) {
			global $SERVICES;
			$auth =& $SERVICES->getService('Authentication');
			$client_name = $auth->getWebFormUserLoginName();
			if ($client_name) {
				$content .= $client_name . ' | ';
			}
			$content .= '<a href="clients.php?logout=1">' . $this->messages['logout'] . '</a>';
		} else {
			$content .= '<a href="clients.php">' . $this->messages['login'] . '</a>';
		}
		return $content;
	}
	
	function clientLogin() {
		global $SERVICES;
		$auth =& $SERVICES->getService('Authentication');
		$auth->clientRestrict(); // Requires client login
		$this->isClientLoggedIn();
	}
	
	/**
	 * Passes entity variable to relevant method (based on name variable)
	 *
	 * @return mixed return value, usually an int or false.
	 */
	function processEntity($name, $entity) {
		// print '<div>processEntity ' . $name . '</p>'; print_r(array_keys($entity));
		$item = array();
		$id = false;
		if (isset($name) && $name && is_array($entity) && isset($entity['data']) && is_array($entity['data'])) {
			foreach ($entity['data'] as $k => $v) {
				if ( isset($v['dest']) && ( isset($v['src']) || isset($v['default'])) ) {
					$item[$v['dest']] = ($v['src'] && $this->form_values[$v['src']] ? $this->form_values[$v['src']] : $v['default']);
					// if (isset($v['required']) && $v['required'] && !$item[$v['dest']]) {
						// print '<p>Required field ' . $v['dest'] . ' has no value</p>';
					// }
				}
			}
		}
		
		if (isset($entity['attach']) && $entity['attach']) {
			$contact_id = (int)$this->getContactInfo($entity['attach']);
			// print '<p>attach to ' . $entity['attach'] . ' [' . $contact_id . ']</p>';
		}
		switch ($name) {
			case 'company':
			case 'company1':
			case 'company2':
				$id = $this->addCompany($item);
			break;
			case 'person':
			case 'person1':
			case 'person2':
				$id = $this->addPerson($item, $contact_id);
			break;
			case 'note':
			case 'note1':
			case 'note2':
			case 'notes':
				$id = $this->addNote($item, $contact_id);
			break;
			case 'task':
			case 'task1':
			case 'task2':
			case 'tasks':
				$id = $this->addTask($item, $contact_id);
			break;
			case 'opportunity':
			case 'opportunity1':
			case 'opportunity2':
			case 'opportunities':	
				$id = $this->addOpportunity($item, $contact_id);
			break;
		}
		return $id;
	}
	
	function addCompany($item) {
		// print_r($item); print '<hr>'; print '<p><a href="javascript:history.go(-1);">Back</a></p>'; exit();
		$data_option = 'company_edit';
		$company_args = array('item' => (array)$item);
		$company_object = getWebMATObject(
			'WebMATEdit', 
			'Contact', 
			array(
				'data_option'=>$data_option, 
				'get' => array('contact_type' => 1), 
				'post' => $company_args, 
				// 'piece_identifier' => 'combineditem'
			)
		);
		// print '<div>company_object: ' . get_class($company_object) . '</div>'; exit();
		
		$company_object->doSubmitBegin();
	
		$company_object->doSubmitPermissions();
		// No validation - handled by external (referring) script
		// $company_object->doSubmitValidate();
		$company_id = $company_object->doSubmitProcess();
		$this->setContactInfo('company', $company_id);
		$company_object->doSubmitPrintErrors($company_id);
		$company_object->doSubmitEnd($company_id);
		return $company_id;
	}
	
	function addPerson($item, $contact_company = false) {
		$data_option = 'person_edit';
		$is_sub_contact = false;
		if ($contact_company) {
			$is_sub_contact = true;	
			$item['contact_company'] = $contact_company;
		}
		$person_args = array('item' => (array)$item);
		$person_object = getWebMATObject(
			'WebMATEdit', 
			'Contact', 
			array(
				'data_option'=>$data_option, 
				'get' => array(
					'contact_type' => 0, 
					'is_sub_contact' => $is_sub_contact 
				), 
				'post' => $person_args,  
				// 'piece_identifier' => 'combineditem'
			)
		);
		$person_object->doSubmitBegin();
		$person_object->doSubmitPermissions();
		// No validation - handled by external (referring) script
		// $person_object->doSubmitValidate();
		$person_id = $person_object->doSubmitProcess();
		$this->setContactInfo('person', $person_id);
		$person_object->doSubmitPrintErrors($person_id);
		$person_object->doSubmitEnd($person_id);
		
		return $person_id;
	}
	
	function addNote($item, $contact_id = false) {
		if ($contact_id) {
			$item['contact_id'] = $contact_id;
		}
		$note_object = getWebMATObject(
			'WebMATEdit', 
			'Note', 
			array(
				'data_option'=>'edit', 
				'get' => array(), 
				'post' => array()
			)
		);
		
		$item = $note_object->getDefaultItem($item);
		$id = $note_object->setItem($item);
		// print '<p>note id: ' . $id . '</div>';
		return $id;
	}
	
	function addTask($item, $contact_id = false) {
		if ($contact_id) {
			$item['contact_id'] = $contact_id;
		}
		$task_object = getWebMATObject(
			'WebMATEdit', 
			'Tasks', 
			array(
				'data_option'=>'edit', 
				'get' => array(), 
				'post' => array()
			)
		);
		
		$item = $task_object->getDefaultItem($item);
		$id = $task_object->setItem($item);
		// print '<p>task id: ' . $id . '</div>';
		return $id;
	}
	
	function addOpportunity($item, $contact_id = false) {
		if ($contact_id) {
			$item['contact_id'] = $contact_id;
		}
		$edit_object = getWebMATObject(
			'WebMATEdit', 
			'Opportunities', 
			array(
				'data_option'=>'edit', 
				'get' => array(), 
				'post' => array()
			)
		);
		
		$item = $edit_object->getDefaultItem($item);
		$id = $edit_object->setItem($item);
		// print '<p>opportunity id: ' . $id . '</div>';
		return $id;
	}
	
	function webFormIsViewable() {
		if (isset($this->config[$this->web_form_identifier]['view']) && $this->config[$this->web_form_identifier]['view']) {
			return true;
		}
		return false;
	}
	
	function doProcessSuccess() {
		switch ($this->config[$this->web_form_identifier]['on_success']) {
			case 'return':
				print '1';
				exit();
			break;
			case 'var':
				global $bcrm_result;
				$bcrm_result = true;
			break;
			case 'redirect':
				if (isset($this->config[$this->web_form_identifier]['redirect_success']) && $this->config[$this->web_form_identifier]['redirect_success']) {
					header('Location: ' . $this->config[$this->web_form_identifier]['redirect_success']);
					exit();
				}
			case 'display':
				if (isset($this->config[$this->web_form_identifier]['display_success']) && $this->config[$this->web_form_identifier]['display_success']) {
					if (file_exists($this->config[$this->web_form_identifier]['display_success']) && is_readable($this->config[$this->web_form_identifier]['display_success'])) {
						@include($this->config[$this->web_form_identifier]['display_success']);
						exit();
					}
				}
			case 'include':
				if (isset($this->config[$this->web_form_identifier]['display_success']) && $this->config[$this->web_form_identifier]['display_success']) {
					if (file_exists($this->config[$this->web_form_identifier]['display_success']) && is_readable($this->config[$this->web_form_identifier]['display_success'])) {
						$content = '';
						if (function_exists('file_get_contents')) {
							$content = file_get_contents($this->config[$this->web_form_identifier]['display_success']);
						}
						$page_content = $this->getErrorPage();
						$page_content = str_replace('%content%', $content, $page_content);
						print $page_content;
						exit();
					}
				}
			default:
				$content = $this->messages['process_successful'];
				$page_content = $this->getErrorPage();
				$page_content = str_replace('%content%', $content, $page_content);
				print $page_content;
			break;
		}
		exit();
	}
	
	function doProcessFailed() {
		switch ($this->config[$this->web_form_identifier]['on_success']) {
			case 'return':
				print '0';
				exit();
			break;
			case 'var':
				global $bcrm_result;
				$bcrm_result = false;
			break;
			case 'redirect':
				if (isset($this->config[$this->web_form_identifier]['redirect_failed']) && $this->config[$this->web_form_identifier]['redirect_failed']) {
					header('Location: ' . $this->config[$this->web_form_identifier]['redirect_failed']);
					exit();
				}
			case 'display':
				if (isset($this->config[$this->web_form_identifier]['display_failed']) && $this->config[$this->web_form_identifier]['display_failed']) {
					if (file_exists($this->config[$this->web_form_identifier]['display_failed']) && is_readable($this->config[$this->web_form_identifier]['display_failed'])) {
						@include($this->config[$this->web_form_identifier]['display_failed']);
						exit();
					}
				}
			case 'include':
				if (isset($this->config[$this->web_form_identifier]['display_failed']) && $this->config[$this->web_form_identifier]['display_failed']) {
					if (file_exists($this->config[$this->web_form_identifier]['display_failed']) && is_readable($this->config[$this->web_form_identifier]['display_failed'])) {
						$content = '';
						if (function_exists('file_get_contents')) {
							$content = file_get_contents($this->config[$this->web_form_identifier]['display_failed']);
						}
						$page_content = $this->getErrorPage();
						$page_content = str_replace('%content%', $content, $page_content);
						print $page_content;
						exit();
					}
				}
				
			default:
				$content = $this->messages['process_failed'];
				$page_content = $this->getErrorPage();
				$page_content = str_replace('%content%', $content, $page_content);
				print $page_content;
			break;
		}
		exit();
	}
	
	function doDisplay($id = '', $item = '', $error_msg = '') {
		if ($this->isClientLoginRequired()) {
			// print '<p>Login required</p>';	
			$this->clientLogin();
		}
		
		
		switch ($this->config[$this->web_form_identifier]['view_type']) {
			case 'redirect':
				if (isset($this->config[$this->web_form_identifier]['view_url']) && $this->config[$this->web_form_identifier]['view_url']) {
					header('Location: ' . $this->config[$this->web_form_identifier]['view_url']);
					exit();
				}
			case 'display':
				if (isset($this->config[$this->web_form_identifier]['view_display']) && $this->config[$this->web_form_identifier]['view_display']) {
					if (file_exists($this->config[$this->web_form_identifier]['view_display']) && is_readable($this->config[$this->web_form_identifier]['view_display'])) {
						@include($this->config[$this->web_form_identifier]['view_display']);
						exit();
					}
				}
			case 'include':
				if (isset($this->config[$this->web_form_identifier]['view_include']) && $this->config[$this->web_form_identifier]['view_include']) {
					if (file_exists($this->config[$this->web_form_identifier]['view_include']) && is_readable($this->config[$this->web_form_identifier]['view_include'])) {
						$content = '';
						if (function_exists('file_get_contents')) {
							$content = file_get_contents($this->config[$this->web_form_identifier]['view_include']);
						}
						$page_content = $this->getViewPage();
						$page_content = str_replace('%content%', $content, $page_content);
						print $page_content;
						exit();
					}
				}
			
			default:
				$content = $this->messages['form_unavailable'];
				$page_content = $this->getErrorPage();
				$page_content = str_replace('%content%', $content, $page_content);
				print $page_content;
			break;
			
		}
	}
	
	function doDisplayRequiredFieldsError($fields) {
		// print_r($fields);
		if (isset($this->config[$this->web_form_identifier]['on_failed_required']) && $this->config[$this->web_form_identifier]['on_failed_required']) {
		
			switch('on_failed_required') {
				case 'redirect':
					if (isset($this->config[$this->web_form_identifier]['on_failed_required_url']) && $this->config[$this->web_form_identifier]['on_failed_required_url']) {
						header('Location: ' . $this->config[$this->web_form_identifier]['on_failed_required_url']);
						exit();
					}
				break;
				case 'display':
				default:
				$content = $this->messages['required_fields_error'];
				$page_content = $this->getErrorPage();
				$page_content = str_replace('%content%', $content, $page_content);
				print $page_content;
				exit();
				break;
			}
		}
	}
	
	function getViewPage() {
		return $this->getTemplatecontent();
	}
	
	function getErrorPage() {
		return $this->getTemplatecontent();
	}
	
	function getTemplateContent() {
		if (function_exists('file_get_contents') && file_exists($this->template_path) && is_readable($this->template_path)) {
			$content = file_get_contents($this->template_path);
			if ($this->includeFixedLinksContent()) {
				$fixed_links = $this->getFixedLinksContent();
			}
			$content = str_replace('<!--%fixed_links%-->', $fixed_links, $content);
		} else {
			$content = '
			<html>
			<body>
			<p>%content%</p>
			<p><a href="javascript:history.go(-1);">Back</a></p>
			</body>
			</html>
			';
		}
		return $content;
		
	}
	
	function includeFixedLinksContent() {
		return true; // TODO: config setting	
	}
	
	function getFixedLinksContent() {
		$content = $this->getClientLoginLinks();
		return $content;
	}
	
	function getIndexLinks($client_only = false) {
		global $config;
		$config = $links = array();
		
		$config = $this->getConfig();
		// if (function_exists('getWebFormConfig')) {
			// $config = getWebFormConfig();
		// }
		
		if (isset($config['index']['links']) && is_array($config['index']['links'])) {
			$links = $config['index']['links'];
		}
		// print '<p>links:</p>'; print_r($links); print '<hr>';
		if (is_array($links) && ($client_only || !$this->isClientLoggedIn())) {
			foreach ($links as $k => $v) {
				if (is_array($v) && isset($v['identifier'])) {
					$identifier = $v['identifier'];
					if (
						(isset($v['require_login']) && $v['require_login']) || 
						isset($config[$identifier]) && 
						isset($config[$identifier]['require_login']) && 
						$config[$identifier]['require_login']) {
						// print '<p>require login for k=' . $k . ' identifier=' . $v['identifier'] . '</p>';
						if (!$client_only) {
							unset($links[$k]);
						}
					} else {
						if ($client_only) {
							unset($links[$k]);
						}
					}
				}
			}
		}
		return $links;
	}
}

/**
 * Legacy code - newer WebForms extend a common base class
 *
 * @return array The web form config settings
 * @author Phil Calder
 */
function getWebFormConfig() {
	if (class_exists('BrowserCRMWebForm')) {
		$bcrmWebForm = new BrowserCRMWebForm();
		return $bcrmWebForm->getConfig();
	}
	return array();
}
?>
