<?php
/**
 * Stores all the configuration options for WebForms.
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebForms
 * @author Phil Calder
 */
 
/**
 * To add custom config settings, use the file ./custom.webform.config.php
 * Populate an array called $custom_config
 * To use a different config file for each web form, 
 * override that classes setCustomConfigFile method
 */

$config = array(
	// Config settings for the Web Form index file
	// Display links to the following WebForms 
	// (depending on client login status/requirement)
	'index' => array(
		'links' => array(
			'contact' => array( 
				'link' => 'contact.php', 
				'name' => 'Contact Us', 
				'identifier' => 'webform.contact'
			), 
			'kb_view' => array(
				'link' => 'kb.php', 
				'name' => 'Knowledgebase', 
				'identifier' => 'webform.kbviewer'
			), 
			'clients' => array(
				'link' => 'clients.php', 
				'name' => 'Clients', 
				'identifier' => 'webform.clients'
			), 
			// 'contact_view' => array(
				// 'link' => 'contact_view.php', 
				// 'name' => 'View contact details', 
				// 'identifier' => 'webform.client_view'
			// ), 
			// 'jobs_list' => array(
				// 'link' => 'jobs_list.php', 
				// 'name' => 'View job list', 
				// 'identifier' => 'webform.client_jobs'
			// )
			// Not completed
			// 'signup' => array(
				// 'link' => 'signup.php', 
				// 'name' => 'Signup'
			// ), 
			// Not completed
			// 'unsubscribe' => array(
				// 'link' => 'unsubscribe.php', 
				// 'name' => 'Unsubscribe'
			// )
		)
	), 
	//  
	'default' => array(
		// Where to look for the form data:
		// - 'post': the $_POST variable
		// - 'get': the $_GET variable
		// - 'request': combined post and get
		// - 'var': look for a variable called 'bcrm_form_data'
		// - 'config': look for config key 'bcrm_form_data' (eg $config['default']['bcrm_form_data'])
		'data_source' => 'post', 
		'view' => true, // Page / form can be viewed
		'process' => true,  // Page / form can process (ie submit)
		'is_submit' => 'submit', // look in data source for this field. Value can also be true or false.
		'mapping' => false,  // or array('entity_name' => array('attach' => true|false, 'data' => array('src' => form_field_name, 'dest' => bcrm_field_name, 'default' => default_value)))
		'on_success' => 'display', // 'redirect': look in data source for redirect_success, 'display': print message, 'var': set $bcrm_result = true, 'return': print '1'
		'on_failure' => 'display', // 'redirect': look in data source for redirect_failure, 'display': print message, 'var': set $bcrm_result = false, 'return': print '0'
		// 'bcrm_form_data' => array() // Used if data_source is 'config'
		
	), 
	'webform.clients' => array(
		// This class requires client login. 
		// Dont specify in config since we want to display this link 
		// even if client not logged in
		//'require_login' => true
		'process' => true, // page cannot be 'submitted'
		'data_source' => 'post', 
		'view' => true, // Page can be viewed
		'view_type' => 'display', 
		// 'links' => array(
			// 'contact_view' => array(
				// 'link' => 'contact_view.php', 
				// 'name' => 'View contact details'
			// ), 
			// 'jobs_list' => array(
				// 'link' => 'jobs_list.php', 
				// 'name' => 'View job list'
			// )
		// )
	), 
	'webform.kbviewer' => array(
		'process' => false, // page cannot be 'submitted'
		'data_source' => 'get', 
		'view' => true, // Page can be viewed
		'view_type' => 'display',
		'require_login' => true
	),
	'webform.contact' => array(
		// Where to look for the form data:
		// - 'post': the $_POST variable
		// - 'get': the $_GET variable
		// - 'request': combined post and get
		// - 'var': look for a variable called 'bcrm_form_data'
		// - 'config': look for config key 'bcrm_form_data' (eg $config['default']['bcrm_form_data'])
		'data_source' => 'post', 
		
		'view' => true, // Page / form can be viewed
		// If page can be viewed, specify type and url to load or file to include
		// 'view_type' => 'display', 
		// view_type 'display', include the following file:
		// 'view_display' => 'inc/contact.form.inc.php', 
		// view_type 'redirect', redirect to the following URL:
		// 'view_url' => 'contact.form.php', 
		'view_type' => 'include', 
		// view_type 'include', include the following file:
		'view_include' => 'inc/contact.form.inc.php', 
		
		'process' => true,  // Page / form can process (ie submit)
		'is_submit' => 'submit', // look in data source for this field to determine if form has been submitted. 
		
		'check_required' => true, // check that all required fields have been completed.
		'on_failed_required' => 'display', // Display an error message when all required fields are not completed
		// when on_failed_required is redirect: 
		// 'on_failed_required_url' => 'contact.required.failed.php', 
		
		// Options: 'redirect', 'display', 'var', 'return', 'include'
		'on_success' => 'include', 
		'on_failure' => 'include', 
		
		// Path to file. Used when 'on_success' is set to 'display' or 'include'
		'display_success' => 'inc/contact.success.inc.php', 
		// Path to file. Used when 'on_failure' is set to 'display' or 'include'
		'display_failed' => 'inc/contact.failed.inc.php',
		
		// URL. Used when 'on_success' is set to 'redirect'
		// 'redirect_success' => 'contact.success.php', 
		// URL. Used when 'on_failure' is set to 'redirect'
		// 'redirect_failed' => 'contact.failed.php', 
		
		'mapping' => array(  // or array(form_field_name => bcrm_field_name)
			'company' => array(
				'attach' => false, // Companies are top level records
				'data' => array(
					array(
						'src'    => 'company', 
						'dest'   => 'Company__item.display_name', 
						'required' => true
					), 
					array(
						'src'    => 'company_email', 
						'dest'   => 'email1_content'
					), 
					array(
						'src'    => 'company_phone', 
						'dest'   => 'phone1_content'
					),
					array(
						'src'    => 'company_url', 
						'dest'   => 'url1_content'
					),
					array(
						'src'    => 'company_address_houseno', 
						'dest'   => 'addrpost1_houseno'
					), 
					array(
						'src'    => 'company_address_housesuffix', 
						'dest'   => 'addrpost1_housesuffix'
					), 
					array(
						'src'    => 'company_address_street', 
						'dest'   => 'addrpost1_street'
					), 
					array(
						'src'    => 'company_address_line2', 
						'dest'   => 'addrpost1_line2'
					), 
					array(
						'src'    => 'company_address_line3', 
						'dest'   => 'addrpost1_line3'
					), 
					array(
						'src'    => 'company_address_city', 
						'dest'   => 'addrpost1_city'
					),  
					array(
						'src'    => 'company_address_postcode', 
						'dest'   => 'addrpost1_postcode'
					), 
					array(
						'src'    => 'company_address_state', 
						'dest'   => 'addrpost1_state'
					),
					array(
						'src'    => 'company_address_country', 
						'dest'   => 'addrpost1_country'
					),
					array(
						'default' => 1, 
						'dest'  => 'acc_is_alive'
					),
					array(
						'default' => 10, // Web site
						'dest'  => 'acc_source'
					),
					array(
						'default' => 2, // Prospect
						'dest'  => 'acc_type_id'
					),
					array(
						'default' => 1, 
						'dest'  => 'lock_permissions'
					)
				)
			),
			'person' => array(
				'attach' => 'company', 
				'data' => array(
					array(
						'src' => 'first_name', 
						'dest'  => 'Person__item.first_name', 
						'required' => true
					), 
					array(
						'src'    => 'last_name', 
						'dest' => 'Person__item.last_name', 
						'required' => true
					), 
					array(
						'src'    => 'person_email', 
						'dest'   => 'email1_content'
					), 
					array(
						'src'    => 'person_phone', 
						'dest'   => 'phone1_content'
					),
					array(
						'src'    => 'person_fax', 
						'dest'   => 'fax1_content'
					),
					array(
						'default' => 1, 
						'dest'  => 'lock_permissions'
					)
				)
			),
			'tasks' => array(
				'attach' => 'company',
				'data' => array(
					array(
						'default' => 'WebForm Task', 
						'dest'  => 'label', 
					),
					array(
						'src' => 'message', 
						'dest'  => 'content', 
					), 
					array(
						'default' => 1, // Send a letter
						'dest'  => 'type_id', 
					),
					array(
						'default' => 0, // [unrestricted]
						'dest' => 'assigned_to_group_id'
					)
				)
			)
		)
	),
	'webform.client_view' => array(
		'require_login' => true
	), 
	'webform.client_jobs' => array(
		'require_login' => true
	)
);
?>