<?php
/**
 *
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Services
 */
 
define('SALT_LENGTH', 9);
 
includeService('Groups');
includeService('Users');

/**
 *  The Authentication service manages logins and access levels to resources.
 *
 * @author Phil Calder
 * @package Services
 */
class Authentication extends Service {
	
	/**
	 * @access private
	 */
	var $db = '';
	/**
	 * @access private
	 */
	var $cache = '';
	/**
	 * @access private
	 */
	var $levels = '';
	/**
	 * @access private
	 */
	var $groups = '';
	/**
	 * @access private
	 */
	var $is_web_form = -1;

	/**
	 * Constructor initialises db, cach, levels, users, and groups.
	 */
	function Authentication() {
		global $SERVICES;
		$this->db =& $SERVICES->getService('Database');
		$this->cache = array();
		$this->levels = array(
			'read'  => 1, 
			'write' => 2,
			'admin' => 4
		);
		$this->groups =& $SERVICES->getService('Groups');
		$this->users =& $SERVICES->getService('Users');
		$this->log =& $SERVICES->getService('Log');
		$this->config =& $SERVICES->getService('Config'); // Phil 20051007 for web form user lookups
		// $this->printDebug('Authentication::Authentication, session id: ' . session_id());
	}
	
	/**
	 * Redirects to a login page if there is no valid user.
	 * Passes login information to a user login function for validation.
	 * Updates session stats if applicable.
	 * This function is generally called once per page load to verify there is a user logged in.
	 */
	function init() {
		global $USER, $login, $BCRM_DOCUMENT_ROOT;
		// $this->printDebug('Authentication::init start');
		global $SERVICES;
		$SERVICES->isValidLicence();
		
		if (empty($login)) {
			// $this->printDebug('Authentication::init, login empty');
			// Phil 20060418
			$remember_me_enabled = $this->isRememberMeEnabled();
			$has_remember_me_cookie = $this->hasRememberMeCookie();
			if ($remember_me_enabled && $has_remember_me_cookie) {
				
				//moved to index.inc.php
				// $do_redirect = $this->checkLicence('__unknown__');
				// if ($do_redirect) {
					// $this->redirectAbout();
				// }
				
				// Phil 20070115 USER may already be set (with user prefs). Check whether username is set.
				if (isset($USER) && is_array($USER) && isset($USER['username']) && $USER['username']) {
					// Already logged in with this cookie
					if (!$this->users->checkUser()) {
						$this->log->logAction('User logged out due to invalid remember me session id');
						// $this->printDebug('Authentication::init calling authenticate (user logged out due to invalid session id)');
						$this->authenticate(8);
						exit();
					}
				}
				// $this->printDebug('Authentication::init calling login');
				$this->login(array());
			} 
			
			if (empty($USER) || (!isset($USER['document_root'])) || ($USER['document_root'] != $BCRM_DOCUMENT_ROOT)) {
				// $this->printDebug('Authentication::init calling authenticate (no user or document root)'); 
				// $this->printDebug($USER);
				$this->authenticate(1);
				exit();
			} else {
				
				if (
				(!$this->users->checkUser()) 
				&& (strpos($BCRM_DOCUMENT_ROOT, base64_decode('L2hvbWUvYnJvd3NlcmMvcHVibGljX2h0bWwvZGVtbw==')) === false) 
				) {
					
					$this->log->logAction('User logged out due to invalid session id');
					// $this->printDebug('Authentication::init calling authenticate (user logged out due to invalid session id)');
					$this->authenticate(2);
					exit();
				}
				$this->users->setLastLoad();
			}
			
		} else {
			
			// Phil 20070816
			if (isset($login['rebuild_cache']) && $login['rebuild_cache']) {
				$config =& $SERVICES->getService('Config');
				$lang =& $SERVICES->getService('Language');
				$db =& $SERVICES->getService('Database');
				
				$config->removeCachedConfig();
				$config->init();
				$lang->removeCache();
				$db->rebuildCache();
			}
			
			// $this->printDebug('Authentication::init, login NOT empty');
			if (isset($login['u']) && $login['u']) {
				$login['username'] = $login['u'];
				bcrm_unset($login, 'u');
			}
			
			if (isset($login['p']) && $login['p']) {
				$login['password'] = $login['p'];
				bcrm_unset($login, 'p');
			}
			
			if (isset($login['c']) && $login['c']) {
				$login['host'] = $login['c'];
				bcrm_unset($login, 'c');
			}
			
			if (empty($login['disable_email_check'])) {
				$login['disable_email_check'] = '';
			}
			
			// $this->printDebug('Authentication::init calling checkLicence');
			//moved to Layout: $do_redirect = $this->checkLicence($login['username']);
			// $this->printDebug('Authentication::init calling login');
			
			$this->login($login); // Phil 20050909
		
			if ($login['skin']) {
				$SERVICES->getService('Screen');
				$SERVICES->screen->setSkin('/skins/' . $login['skin']. '/');
			}
			
			//moved to Layout: 
			// if ($do_redirect) {
				// $this->redirectAbout();
			// }
		}
		// $this->printDebug('Authentication::init end');
	}
	
	/**
	 * @access private
	 */
	function redirectAbout() {
		// $this->printDebug('redirectAbout');
		global $SERVICES;
		$modules =& $SERVICES->getService('Modules');
		if ($front =& $modules->getModule('Front')) {
			// Load the frameset with content frame showing the About page
			$link = $front->getAbsoluteLinkTo('../index.php?url=modules/Front/about.php');
			header('Location: ' . $link);
			exit();
		}
	}
	
	/**
	 * Internal
	 *
	 * This method can halt the application for non-admin users
	 *
	 * @return boolean false = OK, true = prompt the (admin) user to take action
	 * @access private
	 */
	function checkLicence($username) {
		global $SERVICES, $L;
		
		$licensed_users = $SERVICES->getNumLicensedUsers();
		$current_users  = $SERVICES->getCurrentUsersCount();
		
		if ($current_users > $licensed_users) {
			if ($this->hasRememberMeCookie()) {
				$this->resetRememberMeCookie();
			}
			global $USER;
			if ( (strtolower($username) == 'admin') || (isset($USER['username']) && strtolower($USER['username']) == 'admin') ) {
				// $L['errAdminExceeededLicensedUserCount'] = 'Unable to login due to licensing issues. Please resolve user count';
				// $message = $L['errAdminExceeededLicensedUserCount'];
				return true;
			} else if ( in_array($username, $this->users->getSystemUsers()) ) {
				return false;
			} else {
				if ($lang =& $SERVICES->getService('Language')) {
					$lang->getLanguage('Front');
					$lang->getLanguage('Login');
				}
				// $back_link = '<a href="javascript:history.go(-1)">' . $L['Back'] . '</a>';
				$login_link = '<a href="' . $SERVICES->getGlobal('php_self') . '?logout=1&forget=1' . '">' . $L['loginLogin'] . '</a>';
				
				$message = $L['errNonAdminExceeededLicensedUserCount'];
				
				$message .= '<br />';
				$message .= '<br />';
				// $message .= $back_link;
				// $message .= ' | ';
				$message .= $login_link;
				
				$SERVICES->displayMessage($message);
				// quickPage($message);
				exit();
			}
		}
		return false;
	}
	
	/**
	 * Displays a login page and exits.
	 */
	function authenticate($msg_id = '', $public = false) {
		global $BCRM_DOCUMENT_ROOT, $auth_message;
		
		$auth_message = ''; // No message by default
		
		if ($msg_id) {
			global $SERVICES, $L;
			$SERVICES->getService('Language');
			$SERVICES->language->getLanguage('_Global');
			switch ($msg_id) {
				case 1: // No USER variable set
					$auth_message = ''; // No message
				break;
				case 2: // Invalid session logout
					$auth_message = $L['authErrInvalidSessionLogout'];
				break;
				case 3: // No user/password combination
					$auth_message = $L['authErrNoUserPassCombo'];
				break;
				case 4: // No read access
					$auth_message = $L['authErrNoReadAccess'];
				break;
				case 5: // No write access
					$auth_message = $L['authErrNoWriteAccess'];
				break;
				case 6: // No admin access
					$auth_message = $L['authErrNoAdminAccess'];
				break;
				case 7: // No admin access
					$auth_message = $L['authErrClientLoginRequired'];
				break;
				case 8: // Invalid session logout (remember me)
					$auth_message = $L['authErrInvalidSessionLogout'];
				break;
			}
		}
		
		if (!$public) {
			// $this->printDebug('Authentication::init include login)');
			include($BCRM_DOCUMENT_ROOT . '/login.inc.php');
		} else {
			// $this->printDebug('Authentication::init include pub login)');
			include($BCRM_DOCUMENT_ROOT . '/pub/login.php');
		}
		exit();
	}
	
	/**
	 * @access private
	 * @return string The SQL password function name
	 */
	function getSqlPasswordFunctionName() {
		// $version = $this->db->getVersion();
		// $this->printDebug('version: ' . $version);
		// if ($version >= 4.1) {
			// $this->printDebug('version >= 4.1: ' . $version);
			// return 'old_password';
		// }
		// $this->printDebug('version < 4.1: ' . $version);
		return 'password';
	}
	
	/**
	 * @author Phil Calder
	 * @access private
	 */
	function generateHash($plain_text, $salt = null) {
		$passwd_alg = 'SMD5'; // default alg is md5 with salt
		if ($salt === null) {
			$salt = substr(md5(uniqid(rand(), true)), 0, SALT_LENGTH);
		} else {
			if ($salt[5] == '}') {
				$passwd_alg = substr($salt, 1,4);
				$salt = substr($salt, 6);
			}
			$salt = substr($salt, 0, SALT_LENGTH);
		}
		
		if ($passwd_alg == 'SSHA') {
			$result = '{' . $passwd_alg . '}' . $salt . sha1($salt . $plain_text);
		} else {
			$result = '{' . $passwd_alg . '}' . $salt . md5($salt . $plain_text);
		}
		return $result; //$salt . sha1($salt . $plainText);
	}

	/**
	 * @access private
	 * @return string The random number to be used in 'remember_me' cookie
	 */
	function generateRandom() {
		$length = 128;
		$seeds = 'abcdefghijklmnopqrstuvwqyz0123456789';
		
		// Seed mt_srand
		list($usec, $sec) = explode(' ', microtime());
		$seed = (float) $sec + ((float) $usec * 100000);
		mt_srand($seed);
		
		// Generate 128 bit Random Number
		$random_number = '';
		$seeds_count = strlen($seeds);
		for ($i = 0; $length > $i; $i++) {
				$random_number .= $seeds{mt_rand(0, $seeds_count - 1)};
		}
		return $random_number;
	}

	/**
	 * Sets 'remember_me' cookie
	 *
	 * There is code in the prepend file that manipulates this cookie as well.
	 * The cookie is also used in the login page.
	 * @access private
	 */
	function setRememberMeCookie($username, $id='') {
		$cookie_separator = '~#~';
		$random_id = '';
		$random_id = $this->generateRandom();
		
		$result = false;
		// set cookie in db
		if (!empty($id)) {
			$result = $this->db->update('System__cookies', array('random_id' => $random_id), 'id = ' . $id);	
			// $this->db->printError();
		} else {
			$result = $this->db->insert('System__cookies', array('random_id' => $random_id, 'username' => $username));
			// $this->db->printError();			
		}
		
		// Reset cookie
		$this->resetRememberMeCookie();
		
		// set cookie in client machine
		$cookie_value = $random_id . $cookie_separator . $username;
		setcookie('remember_me', $cookie_value, time() + 60 * 60 * 24 * 30, '/');
		
		// $this->printDebug('setRememberMeCookie(' . $username . ', ' . $id . ')');
		// $this->printDebug('setRememberMeCookie cookie_value: ' . $cookie_value);
		// $this->printDebug('username = ' . $username . ', id = ' . $id . ', random_id: ' . $random_id . ', insert/update result: [' . $result . '] ' . ($result ? 'true' : 'false') . '');
	}	
	
	/**
	 * Sets 'remember_me' cookie
	 *
	 * @access private
	 */
	function resetRememberMeCookie() {
		// Reset cookie
		// setcookie('remember_me', ''); 
		// Phil - Use same parameter list as in setRememberMeCookie
		setcookie('remember_me', '', time()-10, '/');
	}
	
	/**
	 * Based on config and database check
	 *
	 * @return boolean Whether the 'Remember Me' login functionality is enabled or not
	 * @author Phil Calder
	 * @access private
	 */
	function isRememberMeEnabled() {
		$remember_me_enabled = (bool)$this->config->lookup('modules.UserAdmin.auth.remember_me_enabled');
		// print '<div>isRememberMeEnabled pref=' . ($remember_me_enabled ? 'true' : 'false') . '</div>';
		if (!$this->db->tableExists('System__cookies')) {
			$remember_me_enabled = false;
			// print '<div>isRememberMeEnabled table=' . ($remember_me_enabled ? 'true' : 'false') . '</div>';
		}
		return $remember_me_enabled;
	}
	
	/**
	 * @return boolean Whether there is a 'Remember Me' cookie
	 * @author Phil Calder
	 * @access private
	 */
	function hasRememberMeCookie() {
		return ( (isset($_COOKIE['remember_me']) && $_COOKIE['remember_me']) ? true : false);
	}
	
	/**
	 * @return mixed The 'Remember Me' cookie, or false
	 * @author Phil Calder
	 * @access private
	 */
	function getRememberMeCookie() {
		if ($this->hasRememberMeCookie()) {
			return $_COOKIE['remember_me'];
		}
		return false;
	}
	
	/**
	 * Internal function to verify a user is logged in correctly.
	 *
	 * Now passes all login info in one parameter: login. Other parameters (password, disable_email_check) kept for backwards compatibilty.
	 *
	 * @param mixed $login Either a login array containing all login info, or a string - current users username or the username supplied at the login page.
	 * @param string $password The current users password, or the password supplied at the login page.
	 * @return boolean true, or halts (calls {@link authenticate})
	 * @access private
	 */
	function login($login, $password = '', $disable_email_check = '') {
		if (is_array($login)) {
			$username = (isset($login['username']) ? $login['username'] : false);
			$password = (isset($login['password']) ? $login['password'] : $password);
			$disable_email_check = (isset($login['disable_email_check']) ? $login['disable_email_check'] : $disable_email_check);
		}
		global $USER;
		
		if ($this->isWebForm() && $this->isWebFormUser()) {
			if (!$this->isWebFormUserLoggedIn()) { 
				$this->webFormUserLogin($login); // May or may not be successful
			}
		} else {
			
			$remember_me_session_id = false;
			$using_remember_me_login = false;
			$remember_me_enabled = $this->isRememberMeEnabled();
			// if ($remember_me_enabled && $_COOKIE['remember_me'] && empty($username) && empty($password)) {
			if ($remember_me_enabled && $this->getRememberMeCookie() && empty($username) && empty($password)) {
				// $this->printDebug('login - enabled, cookie, no username etc');
				$cookie_separator = '~#~';
				$cookie_array = explode($cookie_separator, $_COOKIE['remember_me']);
				// $this->printDebug('login - cookie_array'); $this->printDebug($cookie_array);
				$db_cookie = $this->db->quickGet(
					'*',
					'System__cookies',
					'random_id = \'' . $cookie_array[0] . '\' AND username = \'' . $cookie_array[1] . '\''
				);
				// $this->printDebug('db cookie'); $this->printDebug($db_cookie); $this->db->printError();
				if (is_array($db_cookie)) {
					// $this->printDebug('db cookie is an array, looking for db user [' . $db_cookie['username'] . ']');
					$db_user = $this->db->quickGet(
						'*',
						'System__users',
						'status = 1 AND username = "' . $db_cookie['username'] . '"'
					);
				}
				if (isset($db_user) && is_array($db_user)) {
					$this->setRememberMeCookie($db_cookie['username'], $db_cookie['id']);
					$using_remember_me_login = true;
					$remember_me_session_id = $db_user['session_id'];
				} else {
					// $this->printDebug('login returning false - remember me but no db_user (cookie array: ' . $cookie_array[0] . ',  ' . $cookie_array[1]  . ') USER:'); $this->printDebug($USER);
					return false;
				}
			} 
			
			if (!$using_remember_me_login) {
				// Phil pre 20070115
				// $db_user = $this->db->quickGet( // Phil 20060814 added status to query
					// '*',
					// 'System__users',
					// 'username = "' . $username . '" AND status = 1 AND password = ' . $this->getSqlPasswordFunctionName() . '("' . $password . '")'
				// );
				
				$stored_hash = $this->db->quickGet(
					'password', 
					'System__users', 
					'`username` = "' . $username . '" AND `status` = 1'
				);
				
				// Phil 20070115 Handle old password format (pre 4.620)
				if (strlen($stored_hash) < 47) {
					if ( (strlen($stored_hash) == 16) && ($this->db->getVersion() >= '4.1') ) {
						$check_passwd_result = $this->db->sqlQuery('SELECT OLD_PASSWORD("' . $password . '")');
					} else {
						$check_passwd_result = $this->db->sqlQuery('SELECT PASSWORD("' . $password . '")');
					}
					if ($check_passwd_result) {
						$check_passwd = $this->db->getRow($check_passwd_result);
						if (is_array($check_passwd) && sizeof($check_passwd) ) {
							$check_passwd = current($check_passwd);
							if ($check_passwd == $stored_hash) {
								$stored_hash = $this->generateHash($password);
								$update_result = $this->db->update(
									'System__users', 
									array(
										'password' => $stored_hash
									), 
									"`username` = '$username'", 
									'LIMIT 1'
								);
								// $this->db->printError();
							}
						}
					}
				}
				// --
				
				$generated_password = $this->generateHash($password, $stored_hash);
				
				$db_user = $this->db->quickGet( // Phil 20060814 added status to query
					'*',
					'System__users',
					'`username` = "' . $username . '" AND `status` = 1 AND `password` = "' . $generated_password . '"'
				);
				
				if ($remember_me_enabled && isset($login['remember_me']) && $login['remember_me'] && is_array($db_user)) {
					$this->setRememberMeCookie($db_user['username']);
				}
			}
			if (!isset($db_user) || !is_array($db_user)) {
				// $this->printDebug('Authentication::login, no such user/password in db');
				$this->authenticate(3);
			}
			
			if (isset($db_user['id'])) {
				$db_user['user_id'] = $db_user['id'];	
			}
			
			// Phil 20060929
			// About to poulate the USER variable (session)
			// Need to retain alert and browse queries for users logged in with
			// 'Remember Me' enabled
			$alert_queries = $browse_queries = $user_prefs = false;
			if (isset($USER['alert_queries']) && is_array($USER['alert_queries'])) {
				$alert_queries = $USER['alert_queries'];
			}
			if (isset($USER['browse_query']) && is_array($USER['browse_query'])) {
				$browse_queries = $USER['browse_query'];
			}
			if (isset($USER['prefs']) && is_array($USER['prefs'])) {
				$user_prefs = $USER['prefs'];
			}
			$USER = $db_user;
			
			if ($alert_queries) {
				$USER['alert_queries'] = $alert_queries;
			}
			if ($browse_queries) {
				$USER['browse_query'] = $browse_queries;
			}
			if ($user_prefs) {
				$USER['prefs'] = $user_prefs;
			}
			// $this->printDebug('Authentication::login has set USER:');
			// $this->printDebug($USER);
		}
		
		// Phil 20060514 Adding check to see if we need to update the last login
		// passes existing db session id (can be false)
		$this->users->setLogin($remember_me_session_id); // update session id and last login
		$this->users->refresh();
		//setSkin
		$this->users->setSessionSkin($USER['skin']);
		// Phil 20060111
		// set Javascript directory path
		$this->users->setSessionJsPath($USER['skin']);
		// set disable email check
		return true;
	}
	
	/**
	 * @param $user_id Optional user id, defaults to current user.
	 * @return an array of privileges, key is the module name, value is the privilege level (1-7), see $this->levels
	 */
	function getPrivileges($user = '') {
		if (empty($user)) {
			global $USER;
			$user = $USER['user_id'];
		}
		
		if (empty($user)) {
			return array();
		}
		
		if (!isset($this->cache[$user]['priv'])) {
			// $this->printDebug('not cached ' . $user);
			$this->cache[$user]['priv'] = '';
			// $this->printDebug('Getting module privs ' . $user);
			$result = $this->db->select(
				'System__modules.name as name, BIT_OR(System__module_group.priv) as priv',
				'System__modules, System__module_group, System__user_group',
				'System__modules.id = System__module_group.module_id 
				AND 
				System__user_group.group_id = System__module_group.group_id 
				AND 
				System__user_group.user_id = ' . $user,
				'',
				'GROUP BY System__user_group.user_id, System__module_group.module_id'
			);
			
			while($row = $this->db->getRow($result)) {
				$this->cache[$user]['priv'][$row['name']] = $row['priv'];
			}
			// $this->printDebug('Finished getting module privs ' . $user);
		
		} // else {
			// $this->printDebug('already cached ' . $user);
		// }
		// $this->printDebug('Authentication, getPrivileges(' . $user . '):');
		// $this->printDebug($this->cache[$user]['priv']);
		return $this->cache[$user]['priv'];
	}
	
	/**
	 * Retrieves the access level to a module for a given user. Primarily an internal function.
	 *
	 * Note: This function returns true if the given module (or the 'current module') does not exist, 
	 * ie we dont place restrictions on code outside of this application, such as WebForms or web site integration. 
	 * Authentication functions called within modules should include the module name to restrict access to
	 * so that the restriction will take effect even if called from outside the application.
	 *
	 * @param $user, the users id.
	 * @param $module Optional module name, defaults to current module, see $this->getCurrentModule
	 * @param $level Optional access level, defaults to 'read' access, see $this->levels
	 * @return boolean true or false, a flag determined by the users right to access the module with the requied access level.
	 *
	 * @see readRestrict()
	 * @see writeRestrict()
	 * @see adminRestrict()
	 * @see readAccess()
	 * @see writeAccess()
	 * @see adminAccess()
	 *
	 */
	function access($user, $module = '', $level = '') {
		global $USER;
		
		if (empty($user)) {
			$user = $USER['user_id'];
		}
		
		if (empty($module)) {
			$module = $this->getCurrentModule();
		}
		// $this->printDebug('module: ' . $module);
		
		// Phil 20050908
		if ( $this->isWebFormUser() && !$this->isWebFormUserAllowedInternalAccess() && !$this->isWebForm() ) {
			return false;
		}
		
		global $SERVICES;
		$SERVICES->getService('Modules');
		
		if (!$SERVICES->modules->exists($module)) {
			// $this->printDebug('Authentication, access returning true because module ' . $module . ' does not exist');
			return true; // Dont authenticate against non-existant modules
		}
		
		if (empty($level)) {
			$module = $this->levels['read'];
		}
		// $this->printDebug('Authentication, access: user = ' . $user . ', module = ' . $module . ', level = '  .$level);
		
		if (!isset($this->cache[$user]['priv'][$module])) {
			$this->cache[$user]['priv'] = $this->getPrivileges($user);
		}
		
		if (isset($this->cache[$user]['priv'][$module]) && ($this->cache[$user]['priv'][$module] & $level)) {
			$retval = 'true';
		} else {
			$retval = 'false';
		}
		
		// $this->printDebug('Authentication, access: returning ' . $retval . ' (cache: ' . (isset($this->cache[$user]['priv'][$module]) ? $this->cache[$user]['priv'][$module] : '--') . ', level: ' . $level . ')');
		return (isset($this->cache[$user]['priv'][$module]) && ($this->cache[$user]['priv'][$module] & $level));
	}
	
	/**
	 * Determine if a user has read access to a module
	 *
	 * @param $user The user id to check access for.
	 * @param $module The module name.
	 * @return true or false, if the user has read access to the specified module.
	 *
	 */
	function readAccess($user, $module) {
		return $this->access($user, $module, $this->levels['read']);
	}
	
	/**
	 * Determine if a user has write access to a module
	 *
	 * @param $user The user id to check access for.
	 * @param $module The module name.
	 * @return true or false, if the user has write access to the specified module.
	 *
	 */
	function writeAccess($user, $module) {
		return $this->access($user, $module, $this->levels['write']);
	}
	
	/**
	 * Determine if a user has admin access to a module
	 *
	 * @param $user The user id to check access for.
	 * @param $module The module name.
	 * @return true or false, if the user has admin access to the specified module.
	 *
	 */
	function adminAccess($user, $module) {
		return $this->access($user, $module, $this->levels['admin']);
	}
	
	/**
	 * Lock out a user who does not have read access to the given module.
	 *
	 * Requires the user to 'log in' if their privileges do not allow them to 'read' from the given module.
	 * @param $module Optional, the module name (defaults to current module).
	 *
	 */
	function readRestrict($module = '') {
		// $this->printDebug(get_class($this) . '::readRestrict(' . $module . ') start');
		// print '<div>' . get_class($this) . '::readRestrict(' . $module . ') start</div>';
		global $USER;
		if (empty($module)) {
			// print '<div>[calling getCurrent] ' . get_class($this) . '::readRestrict(' . $module . ') start</div>';
		
			global $SERVICES;
			$SERVICES->getService('Modules');
			$module = $SERVICES->modules->getCurrent();
			// $this->printDebug('ReadRestrict current module: ' . $module . '');
		}
		if (!$this->readAccess($USER['user_id'], $module)) {
			$this->printDebug('Authentication readRestrict in effect user id: ' . $USER['user_id'] . ' module: ' . $module);
			$this->log->logAction('Authentication readRestrict in effect user id: ' . $USER['user_id'] . ' module: ' . $module);
			$this->authenticate(4);
		}
		// $this->printDebug(get_class($this) . '::readRestrict(' . $module . ') end');
	}
	
	/**
	 * Lock out a user who does not have write access to the given module.
	 *
	 * Requires the user to 'log in' if their privileges do not allow them to 'write' to the given module.
	 * @param $module Optional, the module name (defaults to current module).
	 *
	 */
	function writeRestrict($module = '') {
		global $USER;
		// print '<div>' . get_class($this) . '::readRestrict(' . $module . ') start</div>';
		if (!$this->writeAccess($USER['user_id'], $module)) {
			// $this->printDebug('Authentication writeRestrict in effect user id: ' . $USER['user_id'] . ' module: ' . $module);
			$this->log->logAction('Authentication writeRestrict in effect user id: ' . $USER['user_id'] . ' module: ' . $module);
			$this->authenticate(5);
		}
	}
	
	/**
	 * Lock out a user who does not have admin access to the given module.
	 *
	 * Requires the user to 'log in' if their privileges do not give them 'admin' access to the module.
	 * @param $module Optional, the module name (defaults to current module).
	 *
	 */
	function adminRestrict($module = '') {
		global $USER;
		// $this->printDebug('Authentication adminRestrict called for ' . $module);
		if (!$this->adminAccess($USER['user_id'], $module)) {
			// $this->printDebug('Authentication adminRestrict in effect user id: ' . $USER['user_id'] . ' module: ' . $module);
			$this->log->logAction('Authentication adminRestrict in effect user id: ' . $USER['user_id'] . ' module: ' . $module);
			// $this->printDebug('Debug: would authenticate here ' . $module);
			$this->authenticate(6);
		}
	}
	
	/**
	 *
	 * @author Phil Calder
	 */
	function clientRestrict() {
		// TODO: Phil - $login variable (below) not set anywhere.
		global $USER;
		// $this->printDebug('USER'); $this->printDebug($USER);
		if ($this->isWebForm() && $this->isWebFormUser()) {
			if (!$this->isWebFormUserLoggedIn()) { 
				if (!$this->webFormUserLogin($login)) {
					$this->authenticate(7, true);
				}
			}
		}
	}
	
	/**
	 * @return integer value for read access
	 */
	function getReadLevel() {
		return $this->levels['read'];
	}
	
	/**
	 * @return integer value for write access
	 */
	function getWriteLevel() {
		return $this->levels['write'];
	}
	
	/**
	 * @return integer value for admin access
	 */
	function getAdminLevel() {
		return $this->levels['admin'];
	}
	
	/**
	 * Returns the current module (based on the location on the calling script) if any.
	 *
	 * @return module name or ''
	 * @access private
	 */
	function getCurrentModule() {
		global $SERVICES;
		$SERVICES->getService('Modules');
		return $SERVICES->modules->getCurrent();
	}
	
	/**
	 * @access private
	 */
	function isWebForm() {
		if ($this->is_web_form === true) {
			return $this->is_web_form;
		}
		$this->is_web_form = false;
		
		global $login;
		if (isset($login['webform']) && $login['webform']) {
			// global $SERVICES;
			// $php_self = $SERVICES->getGlobal('PHP_SELF');
			// if ((strpos($php_self, '/pub') !== false) || (strpos($php_self, '\\pub') !== false)) {
				// $this->is_web_form = true;
			// }
			$this->is_web_form = true;
		}
		return $this->is_web_form;
	}
	
	/**
	 * @access private
	 */
	function isWebFormUser() {
		return $this->users->isWebFormUser();
	}
	
	/**
	 * When false, web form user cannot view own contact record
	 *
	 * @return boolean Whether the web form user is allowed to view own contact record
	 */
	function isWebFormUserAllowedToViewOwnContact() {
		// $this->printDebug('view: ' . $this->config->lookup('modules.UserAdmin.webformuser.view_own_contact') . '');
		return $this->config->lookup('modules.UserAdmin.webformuser.view_own_contact');
	}
	
	/**
	 * When false, web form user cannot edit own contact record
	 *
	 * @return boolean Whether the web form user is allowed to edit own contact record
	 */
	function isWebFormUserAllowedToEditOwnContact() {
		// $this->printDebug('edit: ' . $this->config->lookup('modules.UserAdmin.webformuser.edit_own_contact') . '');
		return $this->config->lookup('modules.UserAdmin.webformuser.edit_own_contact');
	}
	
	/**
	 * When false, web form user can only access pub directory
	 *
	 * @return boolean Whether the web form user is allowed to browse modules inside the application
	 */
	function isWebFormUserAllowedInternalAccess() {
		return $this->config->lookup('modules.UserAdmin.webformuser.allowed_internal_access');
	}
	
	/**
	 * 
	 */
	function isWebFormUserLoggedIn() {
		global $USER;
		// $this->printDebug('isWebFormUserLoggedIn USER:'); $this->printDebug($USER);
		if ($USER['client_email'] && $USER['client_password']) {
			if (!isset($this->is_web_form_user_logged_in) || !$this->is_web_form_user_logged_in) {
				$client_user = $this->db->quickget(
					'count(*)',
					'ClientLoginManager__item',
					'primary_record=1 AND content = "' .$USER['client_email'] . '" AND password = "' . $USER['client_password'] . '"'
				);
				// $this->printDebug('client_user'); $this->printDebug($client_user);
				$this->is_web_form_user_logged_in = $client_user;
			}
			// $this->printDebug('is_web_form_user_logged_in'); $this->printDebug($this->is_web_form_user_logged_in);
			return $this->is_web_form_user_logged_in;
		}
		return false;
	}
	
	/**
	 * 
	 */
	function getWebFormUserLoginName() {
		global $USER;
		if ($this->isWebFormUserLoggedIn()) {
			return $USER['client_email'];
		}
		return false;
	}
	
	/**
	 *
	 */
	function webFormUserLogin($login) {
		// $this->printDebug('webFormUserLogin'); $this->printDebug($login);
		global $USER;
		if ($login['client_email']) {
			// $this->printDebug('webFormUserLogin - setting client email');
			$USER['client_email'] = $login['client_email'];
		}
		if ($login['client_password']) {
			// $this->printDebug('webFormUserLogin - setting client password');
			$USER['client_password'] = $login['client_password'];
		}
		
		if ($USER['client_email'] && $USER['client_password'] && $this->db->tableExists('ClientLoginManager__item')) {
			$USER['client_contact_id'] = $this->db->quickget(
				'contact_id',
				'ClientLoginManager__item',
				'primary_record=1 AND content = "' . $USER['client_email'] . '" AND password = "' . $USER['client_password'] . '"'
			);
			// $this->printDebug('$USER[client_contact_id]: ' . $USER['client_contact_id'] . ''); $this->printDebug($USER);
			// $this->printDebug('<b>webFromUserLogin returning: ' . $USER['client_contact_id'] . '</b>');
			return $USER['client_contact_id'];
		}
		// $this->printDebug('<b>webFromUserLogin returning false</b>');
		return false;
	}
} // class Authentication
?>