<?php
/**
 *
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @author Brenton Bartel
 * @package Services
 */

includeSmarty('Smarty.class.php');

/**
 * This TemplateEngine class handles the use of templates.
 * Note: to meet a deadline, the 'Smarty.class.php' file was altered to extend Services,
 * but this file should be the one that extends services, then has the proper wrapping functions.
 * So, for the moment this is just used for naming.
 *
 * @author Brenton Bartel
 * @package Services
 */
class TemplateEngine extends Smarty {

	/**
	 * @access private
	 */
	var $template_path = ''; // path to the template we will use.

	/**
	 * The TemplateEngine constructor
	 */
	function TemplateEngine() {
		global $SERVICES;

		parent::Smarty();

		$this->cache_dir = $SERVICES->config->lookup('templates.cache_dir');
		$this->compile_dir = $SERVICES->config->lookup('templates.compile_dir');
	}


	/**
	 * Sets the path to the template.
	 *
	 * @param string Path from BCRM document root.
	 */
	function setTemplatePath($path) {
		$this->template_path = $path;
	}


	/**
	 * Overrides the Smarty function of the same name. We're tracking the template path
	 * from the begining so we don't have to remember it later on, so here we're just
	 * calling the parent version and using our var.
	 *
	 * @param string Path of the template file
	 * @param integer Cache Id
	 * @param integer Compile Id
	 */
	function display($resource_name = '', $cache_id = null, $compile_id = null) {

		if ($resource_name == '') {
			$resource_name = $this->template_path;
		}

		Smarty::display($resource_name, $cache_id, $compile_id);
	}


	/**
	 * Overrides the Smarty function of the same name. We're tracking the template path
	 * from the begining so we don't have to remember it later on, so here we're just
	 * calling the parent version and using our var.
	 *
	 * @param string Path of the template file
	 * @param integer Cache Id
	 * @param integer Compile Id
	 *
	 * @return bool Value if the template has been cached or not
	 */
	function is_cached($tpl_file = '', $cache_id = null, $compile_id = null) {

		if ($tpl_file == '') {
			$tpl_file = $this->template_path;
		}

		return Smarty::is_cached($tpl_file, $cache_id, $compile_id);
	}

} // class TemplateEngine

?>