<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Services
 */

includeService('Output');

/**
 * this class creates a tree view table from an array
 *
 * @author Matthew Corich
 * @package Services
 */
class TreeView {
	/**
	 * @var array $tree the array that stores the direcroty tree
	 */
	var $tree = array();
	/**
	 * @var array $icons and array of icons (one for each type)
	 */
	var $icons = array();
	/**
	 * @var string $sort_by the field key you want to sort on
	 */
	var $sort_by = false;
	/**
	 * object the service output object
	 */
	var $output = false;
	
	
	/**
	 * The constructor for the tree view class this sets the calls variables
	 * 
	 * @param array $tree the tree array see $this->setTree() for more details
	 * @param array $icons the icons array see $this->setIcons() for more details (optional)
	 * @param string $sort_by string $sort_by the field key you want to sort on (optional)
	 */
	function TreeView($tree, $icons = false, $sort_by = false) {
		global $SERVICES;
		$this->setTree($tree);
		$this->setIcons($icons);
		$this->setSortBy($sort_by);
		$this->output =& $SERVICES->getService('Output');
	}
	
	
	/**
	 * sets the $tree class variable
	 * 
	 * @param array $tree is a multidimensional array the has all the informaion 
	 * on the the structure of the tree view the keys are listed below:
	 *  - 'name' string the name lable for the leaf
	 *  - 'type' string the type of item also used for the array icon
	 *  - 'link' string (optional) the link for the name and icon
	 *  - 'extras' array (optional) used to add extra information on the row 
	 *     each array item will appearer in a different column
	 *  - 'sub_items' array (optional) new instance of the tree array that will 
	 *     be nested below the leaf
	 * @return boolean true if $this->tree has been set otherwise false
	 */
	function setTree($tree) {
		if (is_array($tree)) {
			$this->tree = $tree;
			return true;
		} else {
			return false;
		}
	}
	
	
	/**
	 * returns the $tree class variable
	 * 
	 * @return array the $tree class variable or an blank array
	 */
	function getTree() {
		if ($this->tree) {
			return $this->tree;
		} else {
			return array();
		}
	}
	
	
	/**
	 * sets the $icons class variable
	 * 
	 * @param array $icons the tree array (optional) (with the 'type' as the key and the 
	 * icon image name as the value. NOTE: the images must be in the skins folder)
	 */
	function setIcons($icons = false) {
		if (is_array($icons)) {
			$this->icons = $icons;
		} else {
			$this->icons = array(
				'folder_open' => 'folder_open.gif', 
				'folder_closed' => 'folder_closed.gif' 
			);
		}
	}
	
	
	/**
	 * returns the $icons class variable
	 * 
	 * @return array the $icons class variable or an blank array
	 */
	function getIcons() {
		if ($this->icons) {
			return $this->icons;
		} else {
			return array();
		}
	}
	
	
	/**
	 * Sets the sort by variable
	 * 
	 * @param string $sort_by the key you want to sort on  (optional)
	 * @return boolean true if the sort is a valid array key
	 */
	function setSortBy($sort_by = false) {
		if ($sort_by) {
			$tree = $this->getTree();
			foreach ($tree as $k => $v) {
				if (isset($v[$sort_by])) {
					$this->sort_by = $sort_by;
					return true;
				}
			}
			$this->sort_by = false;
		}
		return false;
	}
	
	
	/**
	 * returns the $sort_by class variable
	 * 
	 * @return array the $sort_by class
	 */
	function getSortBy() {
		return $this->sort_by;
	}
	
	
	/**
	 * builds the tree
	 * 
	 * @param array $tree the tree array see $this->setTree() for more details
	 * @param integer $level how meny levels you are down.
	 * @return string the tree in a subtable
	 */
	function buildTree($tree = false,  $level = 0) {
		if (!$tree) {
			$tree = $this->getTree();
		}
		if ($this->getSortBy()) {
			usort($tree, array($this, '_compareAlphaString'));
		}
		$content = '';
		if ($level) {
			$spacer_width = $level*25;
			$spacer_image = $this->output->getImageTag('tree_spacer.gif', 'spacer', 
				'width="' . $spacer_width . '" height="1"');
		} else {
			$spacer_image = '';
		}
		foreach ($tree as $k => $v) {
			$img_link = $this->output->getLinkHref( array('href' => $v['link'], 
				'text' => $this->getIconImage($v), 'title' => $v['name']) );
			$text_link = $this->output->getLinkHref( array('href' => $v['link'], 'text' => $v['name']) );
			$table_row = array($spacer_image . $img_link . ' ' . $text_link);
			if (is_array($v['extras'])) {
				$table_row = bcrm_array_merge($table_row, array('&nbsp;'), $v['extras']);
			}
			$content .= $this->output->getTableRow($table_row) . "\n";
			if ($v['sub_items']) {
				$content .= $this->buildTree($v['sub_items'],  ++$level);
			}
		}
		return $content;
	}
	
	
	/**
	 * returns the tree view in a nested table
	 */
	function getTreeTable() {
		return $this->output->getBeginNestedTable() . $this->buildTree(). $this->output->getEndNestedTable();
	}
	
	
	/**
	 * returns the "<img>" tag a leaf item
	 * 
	 * @param array the leaf item from $this->buildTree();
	 * @return string the <img> link from the icon list.
	 */
	function getIconImage($leaf) {
		$icons = $this->getIcons();
		if ($icons[$leaf['type']]) {
			return $this->output->getImageTag($icons[$leaf['type']], $leaf['type'], 'border="0" align="bottom"');
		} else {
			return $this->output->getImageTag('document.gif', $leaf['type'], 'border="0"');
		}
	}	
	
	/**
	 * the callback cmp_function used to sort the array;
	 * 
	 * @see TreeView->buildTree();
	 */
	function _compareAlphaString($a, $b){
		$sort_by = $this->getSortBy();
		if ($a[$sort_by] < $b[$sort_by]) {
			return true;
		}
		return false;
	}

}
?>