<?php
/**
 * Creates a dropdown admin page
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
**/

// includeWebMAT('webmat.inc');
includeWebMAT('display.inc');

class WebMATStandardAdminDropdown extends WebMATDisplay {
	
	/**
	 * @var array the combined post and get arrays
	 * @access private
	 */
	var $form_values = array();
	/**
	 * @var array from the module.config
	 * @access private
	 */
	var $dropdown_config;
	/**
	 * @var object the module class
	 * @access private
	 */
	var $mod;
	/**
	 * @var array a cut down data definition containing only the field
	 */
	var $mini_data_definition;
	/**
	 * @var object the Output class
	 * @access private
	 */
	var $output;
	/**
	 * @var object the dropdown object
	 * @access private
	 */
	var $dropdown;
	
	/**
	 * The constructor function. Sets most of the class variables
	 * 
	 * @param array $post the $_POST array
	 * @param array $get the $_GET array
	 * @param array $mini_data_definition a cut down version of the data definition
	 */
	function WebMATStandardAdminDropdown($post, $get, $mini_data_definition = false, $access_level = '') {
		global $SERVICES, $L;
		// Phil 20050404
		$this->accessRestrict('', $access_level); // defaults to Admin access required for the current module
		$SERVICES->getService('Config');
		// $this->form_values = array_merge_recursive($get, $post); //joins the $_get and $_post values into one array
		$this->form_values = $this->arrayMergeReplace($get, $post);
		$this->mini_data_definition = $mini_data_definition;
		$this->output =& $SERVICES->getService('Output'); //Creates a new output object
		$this->setDropdownConfig();
		$this->dropdown = $this->getDropDownObject();
		$this->toolbar_link_mode = true;
		$this->new_menu_links[] = $this->getNewLink();
		//$this->mod =& $SERVICES->modules->getModule($this->dropdown_config['module']);
		$this->module_name_lower = strtolower( substr($this->dropdown_config['module'], 0, 1) ) . substr($this->dropdown_config['module'], 1);
	}
	
	/**
	 * Set the access restrict level for this admin page
	 *
	 * @param string $module Optional module name (default is current module)
	 * @param integer $level Optional (default is admin)
	 */
	function accessRestrict($module = '', $level = 'admin') {
		global $SERVICES;
		$ua =& $SERVICES->getService('Authentication');
		// Phil 20060625 Added module to access calls
		switch ($level) {
			case 'read':
				$ua->readRestrict($module); //halt page execution for users who do not have Admin access for this module
			break;
			case 'write':
				$ua->writeRestrict($module); //halt page execution for users who do not have Admin access for this module
			break;
			case 'admin':
			default:
				$ua->adminRestrict($module); //halt page execution for users who do not have Admin access for this module
			break;
		}
	}
	
	/**
	 * Checks if for a delete or updatruns the submit function
	 */
	function run() {
		if (isset($this->form_values['submit']) || isset($this->form_values['delete'])) {
			$this->doSubmit();
		}
		//show the edit page if an id has been supplied
		if (isset($this->form_values['id'])) {
			$this->edit( $this->form_values['id'] );
		}
	}
	
	/**
	 * Establish the module, table, and field to use for this dropdown
	 *
	 * @param array $config_values Optional array to overwrite the module, table, and field values.
	 * whether this is specified or not, the 'default' values will be loaded first
	 *
	 * This is usually only called by a constructor.
	 */
	function setDropdownConfig($config_values = array()) {
		global $SERVICES;
		// This is the deafult
		if ($this->form_values['dropdown_config']) { //sets the dropdown config
			$_SESSION['dropdown_config'] = $SERVICES->config->lookup('modules.Admin.modulelink.' . 
				$this->form_values['dropdown_config']);
		}
		$this->dropdown_config = $_SESSION['dropdown_config'];
		if (!isset($this->dropdown_config['field']) || !$this->dropdown_config['field']) { //Sets dropdown_config['field'] to name if null
			$this->dropdown_config['field'] = 'name';
		}
		
		// Override
		if (is_array($config_values)) {
			foreach ($config_values as $k => $v) {
				$this->dropdown_config[$k] = $v;
			}
		}
	}
	
	/**
	 * Creates a cut down Data Dictionary
	 *
	 * @param array $item the current item from the dropdown table.
	 * @return array an array of items with formating information
	 */
	function getFormFields($item) {
		global $L;
		if (!$this->mini_data_definition) {
			
			$fields =  array(
				array('label'=>'', 'field_key'=>'id', 'type'=>'hidden'),
				array('label'=>$L[$this->module_name_lower . 'DropdownValueName'], 'field_key'=>$this->dropdown_config['field'], 'type'=>'text', 'notnull' => 1),
				array('label'=>$L['DisplayOrder'], 'field_key'=>'priority', 'type'=>'integer', 'min' => 0, 'max' => 999, 'default_value' => 100, 'notnull' => 1)
			);
			
		} else {
			$fields = $this->mini_data_definition;
		}
		
		foreach ($fields as $k => $v) {
			if (isset($item[$v['field_key']]) && $item[$v['field_key']]) {
				$fields[$k]['value'] = $item[$v['field_key']];
			} else {
				$fields[$k]['value'] = $fields[$k]['default_value'];
			}
		}
		return $fields;
	}
	
	/**
	 * returns the Drop Down Object
	 *
	 * @return object the dropdown object. Exits on error.
	 */
	function getDropDownObject() {
		if (is_array($this->dropdown_config)) {
			$dropdown = getWebMATObject(
				'WebMATDropdown', 
				$this->dropdown_config['module'], // module
				array(
					'module' => $this->dropdown_config['module'], 
					'table' => $this->dropdown_config['table'], 
					'field_name' => $this->dropdown_config['field'], 
					'where' => false
				)
			); // new WebMATDropDown( $this->dropdown_config['module'], $this->dropdown_config['table'], $this->dropdown_config['field']);
			return $dropdown;
		} else {
			quickPage('Invalid table');
			exit();
		}
	}
	
	/**
	 * Displays a tabular list of data for a dropdown, and exits.
	 */
	function main() {
		$content = array();
		// ---------------
		$content['beginMain']        = $this->getBeginMainContent();
		// Title (and links)
		$content['title']            = $this->getTitleContent();
		$content['newMenu']         .= $this->getNewMenuContent();  //new 2005-02-07
		$content['beginDataCell']    = $this->getBeginDataCell();
		$content['bodyTable']	     = $this->output->getBeginTable('viewTbl');
		$content['pre_data']         = $this->getBeforeDataContent();
		$content['dataContent']      = $this->getNestedTable(); //TODO rename function
		$content['post_data']         = $this->getAfterDataContent();
		//$content[]                 = $this->output->getEndTable();
		$content['endBodyTable']     = $this->output->getEndTable();
		$content['endDataCell']      = $this->getEndDataCell();
		$content['endMain']          = $this->getEndMainContent();
		// Bosco 20050803 | Changed to add New Value in pop up	
		$content[]                  .= $this->getProcessClose();	
		
		return $this->display($content);
	}
	
	function getBeforeDataContent() {
		return false;
	}
	function getAfterDataContent() {
		return false;
	}
	
	/**
	 * returns the title row array
	 *
	 * @param string $contact_id the contact id if required
	 * @return string a string containing the menu items
	 */
	function getTitleRowLinksList() {
		$title_links = array(
			'help' => $this->getHelpLink(), 
		//	'new' => $this->getNewLink()
		);
		return $title_links;
	}
	
	/**
	 * @author Bosco 20050803
	 * @return string a string containing close window link
	 */	
	function getProcessClose() {
		global $L;
		$content = '';
		if (!empty($this->form_values['pop_up']) && !empty($this->form_values['new_id']) && !empty($this->form_values['name']) && !empty($this->form_values['element_name'])) {
			$js = '<script type="text/javascript" language="javascript">
                			function processOnLoad() {
                       				window.opener.reloadDropdown(' . $this->form_values["new_id"] . ',\'' . $this->form_values["name"] . '\',\'' . $this->form_values["element_name"] . '\');
					}
	                       </script>';
        	        $this->output->addToHead($js);
			$this->output->addInitJS('processOnLoad()');// onload=""');
			
			if ($this->form_values['pop_up']) {
				// Phil 20060223 Added table (for formatting). 
				// Could replace with a centered <p> tag and appropriate style for text	
			 	$content .= "
				<table><tr><td style='text-align: center; padding: 5'>
				" . $this->output->getLinkHref(
					array(     
						'tag_extras' =>' onClick="javascript:window.close(); " ',       
						'href' => 'javascript:;',                     
						'text' => $L['labelCloseWindow'] 
					)
				) . 
				"</td></tr></table>";
				return $content;
			}
		}
		return $content;
	}

	/**
	 * returns the 'new' menu row
	 * 
	 * @return string the 'new' menu row
	 */
	function getNewMenuContent() {
		foreach ($this->new_menu_links as $k => $v) {
			if (!$v) {
				bcrm_unset($this->new_menu_links, $k);
			}
		}
		if (count($this->new_menu_links)) {
			if ($this->toolbar_link_mode) {
				// print_r($this->new_menu_links); print '<hr>';
				$this->output->setActionLinksMenu($this->new_menu_links);
			} else {
				return $this->output->getMenuRow( implode($this->getMenuSeparator(), $this->new_menu_links) );
			}
		}
	}
	
	function getTitleContent() {
		global $L, $SERVICES;
		return $this->output->getTitleRow(
			$L[$SERVICES->config->lookup('modules.' . $this->dropdown_config['module'] . '.label')] . ' ' . $L[$this->dropdown_config['label']],
			$this->getTitleRowLinksList()
		);
	}
	
	/**
	 * Builds the nested table for the list view
	 *
	 * @return string the table rows for the nested table
	 */
	function getNestedTable(){
		$table_rows = ''; // Phil - This needs to be initialsed if we are going to return it.
		$values = $this->dropdown->getValues();
		if (is_array($values)) {
			$items = array();
			foreach ($values as $k => $v) {
				$items[] = Array( 'id' => $k, $this->dropdown_config['field'] => $v);
			}
			foreach ($items as $item) {
				$table_row = array();
				foreach ($this->getFormFields($item) as $field) {
					if ($field['field_key'] == $this->dropdown_config['field']) {
						// $table_row[] = $this->output->getStringField($field); // Phil
						$table_row[] = array (
							'content' => $this->output->getStringField($field),
							'tag' => 'th'
						);
					}
				}
				$table_row = $this->getEditDeleteLinks($table_row,$item['id']);
				$table_rows .= $this->output->getTableRow($table_row); //Phil - This wasnt initialised, so cannot concatenate
			}
		}
		return $table_rows; // Phil - This needs to be here - the function must return a string
	}
	
	/**
	 * Returns the new link
	 * 
	 * @return string the new link
	 */
	function getNewLink() {
		global $L, $SERVICES;
		$php_self = $SERVICES->getGlobal('php_self');
		$new_link = $php_self . '?id=-1';
		if (!empty($this->form_values['pop_up']) && !empty($this->form_values['element_name'])) {
			$new_link .= '&pop_up=' . $this->form_values['pop_up'] . '&element_name=' . $this->form_values['element_name'];
		} 
		
		if ($this->toolbar_link_mode) {
			return array(
				'href' => $new_link, 
				'title' => $L['LinkNew'], 
				'text' => $L['LinkNew'], 
				'icon' => 'page_add.png'
			);	
		}
		
		return $this->output->getLinkHref( array('href' => $new_link, 'text' =>$L['LinkNew']) );
	}
	
	/**
	 * @deprecated version 4.0 - {@link getItemLinks()}
	 *
	 * @param array $link_array the current row_array
	 * @param string $id the current item id
	 * @return array the row_array with the links appended
	 */
	function getEditDeleteLinks($row_array, $id) {
		return $this->getItemLinks($row_array, $id);
	}
	
	/**
	 * Adds item links (edit and delete) to the given row array
	 * 
	 * @param array $row_array The current row (to add links to)
	 * @param integer $id The item id
	 * @return array The current row with links appended to it
	 * @author Phil Calder
	 */
	function getItemLinks($row_array, $id) {
		global $L, $SERVICES;
		if (!is_array($row_array)) {
			$row_array = (array)$row_array;	
		}
		$php_self = $SERVICES->getGlobal('php_self');
		$row_array['edit'] = $this->output->getLinkHref( array('href' => $php_self . '?id=' . $id, 'text' =>$L['LinkEdit']) );
		$row_array['delete'] = $this->output->getLinkDelete($php_self . '?delete=' . $id);
		return $row_array;
	}
	
	/**
	 * Process an edit/delete and updates the database
	 */
	function doSubmit() {
		global $SERVICES, $L;
		$php_self = $SERVICES->getGlobal('php_self');
		$submit = $_POST['submit'];
		$item = $_POST['item'];
		$delete = $_GET['delete'];
		
		if ($submit) {
			//server-side data validation goes here
			$fields = $this->getFormFields($item);	//get field definitions
			$this->output->validateFields($item, $fields, $error_msg); //validate $item based on the definitions in $fields; $item values will get updated
			if (is_array($error_msg)) {		//if server-side validation errors, return to form
				$this->edit( $item['id'], $item, $error_msg);
				exit();
			}
			//
			$this->beforeDBWrite($item);
			$new_id = $this->dropdown->setValue($item);
			$this->afterDBWrite($item, $new_id);
			// Bosco 20050803 | Changed to add New Value	
			header('Location:' . $php_self . ($this->form_values['pop_up'] ? '?pop_up=' . $this->form_values["pop_up"] . '&new_id=' . $new_id . '&name=' . $item['name'] . '&element_name=' . $this->form_values["element_name"] : ''));
			exit();
			
		} else if ($delete) {
			$this->dropdown->deleteValue($delete);
			header('Location:' . $php_self);
			exit();
		}
	}
	
	/**
	 * Process an edit/delete and updates the database
	 *
	 * @param integer $id the id for the item for edit
	 * @param array $item the item information for error handling
	 * @param array $error_msg a list of error messages for error handling
	 */
	function edit($id = '', $item = '', $error_msg = '') {
		global $SERVICES, $L;
		$php_self = $SERVICES->getGlobal('php_self');
		if (!$id) {
			$id = $this->form_values['id'];
		}
		if ( !$item ) { //item could be passed back from server-side data validation failure
			if ($id == -1) {
				unset( $id ); //this is just a flag for 'new'
				 $item = $this->getExtraDefaults($item);
			}
			if ($id) {
				$item = $this->dropdown->getValue($id);
			}
		}
		
		$this->beforeForm($id, $item, $error_msg);
		
		$content = array();
		// ---------------
		$content['beginMain']        = $this->getBeginMainContent();
		// Title (and links)
		$content['title']            = $this->getTitleContent();
		$content['newMenu']          = $this->getNewMenuContent();  //new 2005-02-07
		
		$content['beginDataCell']    = $this->getBeginDataCell();
		
		$content['bodyTable']		 = $this->output->getBeginTable('viewTbl');
		
		$content['editForm']		 = $this->getEditForm($item);
		
		$content['endBodyTable']     = $this->output->getEndTable();
		
		$content['endDataCell']      = $this->getEndDataCell();
		
		$content['endMain']          = $this->getEndMainContent();
		
		// Bosco 20050803 | Changed to add New Value in pop up	
		$content[]                   = $this->getProcessClose();	
		
		return $this->display($content);
	}
	
	function getEditForm($item) {
		global $SERVICES;
		$php_self = $SERVICES->getGlobal('php_self');
		$content = '';
		$content .= $this->output->getBeginForm($php_self . ($this->form_values['pop_up'] ? '?pop_up=' . $this->form_values['pop_up'] . '&element_name=' . $this->form_values["element_name"] : ''));
		$fields = $this->getFormFields($item);
		foreach ($fields as $v) {
			$content .= $this->output->getHtmlFieldRow($v, $error_msg[$v['field_key']]);
		}
		$content .= $this->output->getSubmitRow();
		$content .= $this->output->getEndForm();
		
		return $content;
	}
	
	/**
	 * returns the title row array
	 *
	 * @param string $contact_id the contact id if required
	 * @return string a string containing the menu items
	 */
	function getTitleRowLinksEdit() {
		$title_links = array(
			'bookmark' => 'off',
			'help' => $this->getHelpLink()
		);
		return $title_links;
	}
	
	function getHelpLink() {
		return '';
	}
	
	function getExtraDefaults($item) {
		return $item;
	}
	
	function beforeForm(&$id, &$item, &$error_msg) {}
	
	function beforeDBWrite(& $item) {}
	
	function afterDBWrite($item, $id = false) {}
}
?>