<?php
/**
 * Handles the database functions for detail tables in WebMAT
 *
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('database_table.inc');

/**
 * Handles the database functions for detail tables in WebMAT
 * 
 * An extended version of WebMATDatabaseTable to handle the detail table 
 * database functions
 * 
 * @author Matthew Corich
 * @abstract
 */
 class WebMATStandardDatabaseDetailTable extends WebMATDatabaseTable {
	 /**
	 * @var string the matser table name
	 */
	 var $master_table = false;
	 /**
	 * @var mixed the master id/s as either an integer or an array of integers
	 */
	 var $master_ids = false;
	 /**
	  * @var object the parent database_functions object
	  */
	 var $parent_db_functions = false;
	 /**
	  * @var array containg the parent item/s
	  */
	  var $parent_items = false;
	
	/**
	 * The constructor class for WebMATDatabaseDetailTable
	 * 
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function WebMATStandardDatabaseDetailTable($module_name, $data_definition) {
		
		if (!$this->table_name) {
			// quickPage('<strong>WebMATDatabaseDetailTable ERROR:</strong> $this->table_name is not set in the constructor.');
			global $L;
			quickPage('<strong>' . $L['errDatabaseDetailTable'] . ':</strong> ' . $L['errNoTableNameSet'] . '.');
			exit();
		}
		
		if (!$this->master_table) {
			$this->master_table = 'item';//defaults to item if no value it set
		}
		
		$this->WebMATDatabaseTable($module_name, $data_definition);
		
	}
	
	/**
	 * The constructor class for WebMATDatabaseDetailTable
	 * 
	 * @param mixed $ids the master id/s as either an integer or an array of integers
	 */
	function setMasterIds($ids){
		$this->master_ids = $ids;
	}
	
	/**
	 * Creates the where string
	 * 
	 * Overwritten to add the check for the master table id
	 * 
	 * @param array $where the current where statement
	 * @return array $where, the current where statement with the new where statement added to the end
	 */
	function getFixedWhere($where) {
		$where[] = 'active=' . $this->active;
		if ($this->master_ids) {
			//$key_field = $this->master_table . '_id';
			$key_field = 'item_id';
			if (is_array($this->master_ids)) {
				$where[] = $key_field . ' IN (' . implode(",", $this->master_ids) . ') ';
			} else {
				$where[] = $key_field . '=' . $this->master_ids . ' ';
			}
		}
		return $where;
	}
	
	
	/**
	 * returns the parent database_functions object
	 * 
	 * @return object the parent database_functions object
	 */
	function getParentDbFunctions() {
		if (!$this->parent_db_functions) {
			global $SERVICES;
			$SERVICES->getService('Modules');
			$parent_mod =& $SERVICES->modules->getModule($this->module_name);
			$this->parent_db_functions = $parent_mod->getDatabaseTable($this->master_table);
		}
		return $this->parent_db_functions;
	}
	
	/**
	 * returns the perent item or items
	 * 
	 * @return array the parent item/s
	 */
	function getParentItems() {
		if (!$this->parent_items) {
			if (!$this->master_ids) {
				$item = $this->getItem($_GET['id']);
				$parent_id_field = $this->master_table . '_id';
				if (isset($item[$parent_id_field])) {
					$this->setMasterIds($item[$parent_id_field]);
				} else {
					return array();
				}
			}
			
			$parent_db_functions =& $this->getParentDbFunctions();
			$items = array();
			if (is_array($this->master_ids)) {
				foreach ($this->master_ids as $k => $v) {
					$this->parent_items[$v] = $parent_db_functions->getItem($v);
				}
			} else {
				$this->parent_items = $parent_db_functions->getItem($this->master_ids);
			}
		}
		return $this->parent_items;
	}
	
	/**
	 * updates a parent item
	 * 
	 * @param array $item the parent item array
	 * @return mixed the item_id on success or FALSE on failure
	 */
	function updateParentItem($item){
		if ($item['id'] || !is_array($this->master_ids) ) {
			if (!$item['id']) {
				$item['id'] = $this->master_ids;
			}
			$parent_db_functions =& $this->getParentDbFunctions();
			$parent_db_functions->setItem($item);
			$this->parent_items = false; //reset the parent_items class variable
			return $item['id'];
		} else {
			return false;
		}
	}
	
	/**
	 * gets a array of parent ids that are editable
	 * 
	 * @return array a array of parent ids that are editable
	 */
	function getEditableParents() {
		$editable_parents = array();
		$parent_db_functions =& $this->getParentDbFunctions();
		$parent_items = $this->getParentItems();
		if (isset($parent_items['id'])) {
			if ($parent_db_functions->isEditable($parent_items['id'])) {
				$editable_parents[] = $parent_items['id'];
			}
		} else {
			if (is_array($parent_items)) {
				foreach ($parent_items as $k => $v) {
					$editable_parents[] = $parent_db_functions->isEditable($v['id']);
				}
			}
		}
		return $editable_parents;
	}

}
?>