<?php
/**
 * Class for displaying desktop content
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */
 
includeWebMAT('desktop_pieces_view.inc');

/**
 * @package WebMAT
 * @author Phil Calder
 * @abstract
 */
class WebMATStandardDesktop extends WebMATDesktopPieces {
	
	/**
	 * @access private
	 */
	var $begin_table_arg = 'searchPieceTable'; // Phil 20050728, added to skins.config

	/**
	 * @access private
	 */
	var $has_db_functions = false;
	
	/**
	 * @access private
	 */
	var $piece_filter_type = false;

	// WebMATList functions
	function showCheckboxMenuRow(){ return false; }
	function showSearchFilterForm() { return false; }
	function getDescriptionContent() { return ''; }
	function getPagingRow() { return ''; }
	function beginSelectItemForm() { return ''; }
	function getCheckboxMenuRow() { return ''; }
	function getTitleRowLinks() { return ''; }
	function getSecondMenuRow() { return ''; }
	function showConfigFilters() { return false; }
	function showFilteredOnRow() { return false; }
	function getItemCheckbox() { return array(); }
	function getNewMenuContent() { return ''; }

	/**
	 * Need to call setShowContent to show by default
	 */
	function init($module, $data_option, $post= false, $get = false, $table_name = false, $language_overwrite = false) {
		if (is_array($post) && isset($post['piece_filter_type'])) { // Phil 20061024
			// print '<div>piece_filter_type: ' . $post['piece_filter_type'] . '</div>';
			$this->setPieceFilterType($post['piece_filter_type']);
		}
		// print '<div>data_option: ' . $data_option . '</div>';
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);

		$this->setSortField(); // Phil 20050510 - Consistent with Dashboard. ets default order by (created DESC)

		$this->setShowContent(true);

		// Phil - Could change this depending on isStandardDisplay (?)
		$this->setTagHeaderRow('cpHeaderRow');  //css class for contact piece header row
	}
	
	function setPieceFilterType($type = false) {
		$this->piece_filter_type = $type;
	}
	
	function getPieceFilterType() {
		return $this->piece_filter_type;
	}

	/**
	 * Sets a search's "where" filter
	 * @param mixed $arg Optional, default false. Can supply a string 'type' here
	 * @return bool If any fields were actually searched.
	 */
	function setPieceFilter($arg = false) {
		global $USER;
		// print '<div>original arg: ' .$arg . '</div>';
		$arg = $this->getPieceFilterType();
		// print '<div>' . get_class($this) . '::setPieceFilter  [module: ' . $this->getModuleName() . '] arg: ' . $arg . '</div>'; 
		if (isset($this->db_functions) || (isset($this->mod) && is_object($this->mod))) {
			if (!isset($this->db_functions)) {
				$this->db_functions =& $this->mod->getDatabaseTable('item');
			}
			
			// reset
			$this->db_functions->resetDesignTimeFilters();
			
			// Phil 20060224 Will override or base on config settings at a later date
			$dtf = false;
			switch ($this->getModuleName()) {
				case 'Contact': 
					global $SERVICES, $L, $PHP_SELF;
					$this->summary_content = '';
					if (isset($this->form_values['ContactDesktopFilter'])) {
						$type = $this->form_values['ContactDesktopFilter'];
					} else {
						$prefs =& $SERVICES->getService('Preferences');
						$type = $prefs->getUserPref('contactDesktopFilter');
					}
					
					$contact_desktop_filter_links = array(
						'alert' => array(
							'href' => $PHP_SELF . $this->output->updateQueryString(array('ContactDesktopFilter' => 'alert')), 
							'text' => $L['labelQfilterAlert']
						), 
						'unseen' => array(
							'href' => $PHP_SELF . $this->output->updateQueryString(array('ContactDesktopFilter' => 'unseen')), 
							'text' => $L['labelQfilterUnseen']
						), 
						'modified' => array(
							'href' => $PHP_SELF . $this->output->updateQueryString(array('ContactDesktopFilter' => 'modified')), 
							'text' => $L['labelQfilterModified']
						), 
						'recent' => array(
							'href' => $PHP_SELF . $this->output->updateQueryString(array('ContactDesktopFilter' => 'recent')), 
							'text' => $L['LabelLastViewed']
						), 
						'duplicates' => array(
							'href' => $PHP_SELF . $this->output->updateQueryString(array('ContactDesktopFilter' => 'duplicates')), 
							'text' => $L['contactContactDuplicates']
						)
					);
					
					// print '<div>contactDesktopFilter: ' . $type . '</div>'; print_r($this->form_values); print '<hr>';
					switch ($type) {
					case 'alert':
						// $this->summary_content = $L['labelQfilterAlert'];
						$contact_desktop_filter_links['alert']['href'] = false;
						
						$dtf = array(
							'__special__' => array('where' => $this->db_functions->getAlertWhere())
						);
						if ($arg == 'mine') {
							$dtf['owner_group_id'] = array('where' => 'owner_group_id IN (' . $USER['group_string'] . ')');
						}
						// print_r($dtf); print '<hr>';
						break;
					case 'unseen':
						// $this->summary_content = $L['labelQfilterUnseen'];
						$contact_desktop_filter_links['unseen']['href'] = false;
						$dtf = array(
							'__special__' => array(
								'where' => " (seen IS NULL) "
							)
						);
						if ($arg == 'mine') {
							$dtf['owner_group_id'] = array('where' => 'owner_group_id IN (' . $USER['group_string'] . ')');
						}
						break;
					case 'modified':
						// $this->summary_content = $L['labelQfilterModified'];
						$contact_desktop_filter_links['modified']['href'] = false;
						$dtf = array(
							'__special__' => array(
								'where' => " (seen <> 1 OR seen IS NULL) "
							)// , 
							// 'owner_group_id' => array(
								// 'where' => 'owner_group_id IN (' . $USER['group_string'] . ')'
							// )
						);
						if ($arg == 'mine') {
							$dtf['owner_group_id'] = array('where' => 'owner_group_id IN (' . $USER['group_string'] . ')');
						}
						break;
					case 'duplicates':
						// $this->summary_content = $L['contactContactDuplicates'];
						$contact_desktop_filter_links['duplicates']['href'] = false;
						$dtf = array(
							'__special__' => array(
								'where' => " (is_duplicate_checked = 2) "
							)// , 
							// 'owner_group_id' => array(
								// 'where' => 'owner_group_id IN (' . $USER['group_string'] . ')'
							// )
						);
						if ($arg == 'mine') {
							$dtf['owner_group_id'] = array('where' => 'owner_group_id IN (' . $USER['group_string'] . ')');
						}
						break;
					case 'recent':
					default:
						// $this->summary_content = $L['LabelLastViewed'];
						$contact_desktop_filter_links['recent']['href'] = false;
						// Recent Contacts
						// Get ids for recent contacts
						$this->db_functions->show_last_viewed_for_module = 'Contact'; // Restricts query to this module only
						$last_viewed = $this->db_functions->getLastViewedPages();
						unset($this->db_functions->show_last_viewed_for_module);
						// print '<div>last viewed contacts</div>'; print_r($last_viewed); print '<hr>';
						$ids = array();
						if (is_array($last_viewed)) {
							foreach ($last_viewed as $k => $v) {
								if (isset($v['item_id']) && $v['item_id']) {
									$ids[] = $v['item_id'];
								}
							}
						}
						if (is_array($ids) && count($ids)) {
							$ids_string = implode(',', $ids);
							$dtf = array(
								'id' => array(
									'where' => "id IN ($ids_string) "
								)
							);
						} else { // Dont display any contacts
							$dtf = array(
								'id' => array(
									'where' => " 0 "
								)
							);
						}
						break;
					}
					
					foreach ($contact_desktop_filter_links as $k => $v) {
						if (isset($v['href']) && $v['href']) {
							$this->summary_content .= $this->output->getLinkHref($v) . ' ';
						} else if (isset($v['text']) && $v['text']) {
							$this->summary_content .= $v['text'] . ' ';
						}
					}
					
				break;
				case 'Calendar': // Current Appointments
					global $SERVICES;
					$cur_time =time();
					$current_date = $SERVICES->offsetTimeZoneForWrite(mktime(0, 0, 0, date('m', $cur_time), date('d', $cur_time), date('Y', $cur_time)));
			
					$dtf = array('date_start' => array('where' => '(Calendar__item.date_start >= ' . $current_date . ' OR Calendar__item.date_start+Calendar__item.duration >= ' . $current_date . ')'));
					if ($arg == 'mine') {
						// Phil - This should really replace the Calendar getDispalyByGroupWhere functionality
						$dtf['__special__'] = array('where' => '( Calendar__item.creator_id = "' . $USER['user_id'] .'" OR Calendar__item_user.user_id = "' . $USER['user_id'] .'" ) ');
					}
				break;
				case 'Tasks': // Open Tasks
					
					$dtf = array('is_closed' => '0');
					if ($arg == 'mine') {
						$dtf['assigned_to_group_id'] = array('where' => 'assigned_to_group_id IN (' . $USER['group_string'] . ')');
					}
				break;
				case 'Opportunities': // Open Opportunities ie not closed ('won' or 'lost')
					$dtf = array(
						'stage_id' => array(
							'where' => " stage_id NOT IN (6,7) "
						)
					);
					if ($arg == 'mine') {
						$dtf['owner_group_id'] = array('where' => 'owner_group_id IN (' . $USER['group_string'] . ')');
					}
				break;
				case 'Mail': // Inbox action required, not hidden, not trash
					$seen_where = " (seen <> 3 OR seen IS NULL)";
					$dtf = array(
						'__special__' => array(
							'where' => "Mail__item.sent_status = 'mailInbox' AND Mail__item.action_required = 1 AND Mail__item.is_trash = 0 AND ". $seen_where
						)
					);
					if ($arg == 'mine') {
						$dtf['owner_group_id'] = array('where' => 'owner_group_id IN (' . $USER['group_string'] . ')');
					}
				break;
				case 'Memo':
					global $USER;
					
					// Not written by current user, (seen status filter set elsewhere: getFixedWhere (not deleted or archived) - ie Inbox)
					// $dtf = array(
						// 'creator_id' => array(
							// 'where' => "Memo__item.creator_id <> '" . $USER['user_id'] . "'"
						// )
					// );
					// Not written by current user, not deleted or archived - ie Inbox
					$dtf = array(
						'creator_id' => array(
							'where' => "Memo__item.creator_id <> '" . $USER['user_id'] . "'"
						), 
						'__special__'  => array(
							'where' => "Memo__item_seen.seen < 3"
						)
					);
				break;
				case 'Jobs': // Jobs Todo
					$dtf = array('status' => '1');
					if ($arg == 'mine') {
						$dtf['owner_group_id'] = array('where' => 'owner_group_id IN (' . $USER['group_string'] . ')');
					}
				break;
				case 'Support': // Open Support items
					$dtf = array('is_closed' => '0');
					if ($arg == 'mine') {
						$dtf['owner_group_id'] = array('where' => 'owner_group_id IN (' . $USER['group_string'] . ')');
					}
				break;
				default:
				break;
			}
			
			if ($dtf !== false) {
				// print '<div>setting dtf for  ' . $this->getModuleName() . '</div>'; print_r($dtf); print '<hr>';
				$this->db_functions->setDesignTimeFilters($dtf);
			}
			
		}

		return true;
	}
	
	/**
	 * Overridden to return summary content labels
	 */
	function getSummaryContent() {
		$content = '';
		if (isset($this->summary_content) && $this->summary_content) {
			$padding = true;
			$content .= $this->output->getBeginRow('') . '<td colspan="2" ' . ($padding ? 'style="padding: 10px"' : '') . '>';
			$content .= $this->summary_content;
			$content .= '</td>' . $this->output->getEndRow();
		}
		return $content;
	}
	
	/**
	 * Overridden - functionality disabled in contact view
	 * 
	 * @return string an empty string
	 * @author Phil Calder
	 * 20050517
	 */
	function getBookmarkLinkLink() {
		return '';
	}

	/**
	 * For use in a MySQL 'IN' query, eg IN ( id1 [, id2, id3])
	 *
	 * @param boolean $include_parent Whether to include the current contact in the returned string. Defaults to false.
	 * @return mixed false or a comma separated string of ids (possibly blank)
	 * @author Phil Calder
	 */
	function getSubContactIdsString($include_parent = false) {
		$sub_contacts = $this->getSubContactIds();
		if (is_array($sub_contacts)) {
			if ($include_parent) {
				$sub_contacts = $this->arrayMergeReplace((array)$this->getContactId(), $sub_contacts);	
			}
			return implode(', ', $sub_contacts);
		}
		return false;
	}
	
	/**
	 * Applies only to companies
	 *
	 * @return mixed false or an array of ids
	 * @author Phil Calder
	 */
	function getSubContactIds() {
		global $SERVICES;
		$SERVICES->getService('Modules');
		$contact =& $SERVICES->modules->getModule('Contact');
		$contact_id = $this->getContactId();
		return $contact->getSubContacts($contact_id);
	}
	
	/**
	 * @return array with 'field' and 'order' keys, used to build an order by clause
	 * @author Phil Calder
	 */
	function getDefaultSort() {
		if (isset($this->sort_field) && is_array($this->sort_field)) {
			return $this->sort_field;
		}
		return array(
			'field' => 'id', 
			'order' => 'DESC'
		);
	}

	/**
	 * Set to 'true'. Can be overridden.
	 */
	function showViewLinkOnly() {
		return false;
	}

	/**
	 * Instanciates the database functions object.
	 * This is over-rides the WebMATStandard version, because we don't want to print out
	 * an error. If there is no database table, we don't include the module, and keep
	 * moving along, no errors.
	 *
	 * @param boolean $refresh Optional (default false) parameter to force a new db object to be created
	 * @author Brenton Bartel
	 */
	function setDbFunctions($refresh = false) {
		if (!is_object($this->mod)) {
			$this->setModuleInfo($this->module_name);
		} 

		if (is_object($this->mod) && method_exists($this->mod, 'getDatabaseTable')) {
			$this->db_functions =& $this->mod->getDatabaseTable($this->table_name, $refresh);
			$this->has_db_functions = true;
		} else {
			$this->has_db_functions = false;
		}
	}

	/**
	 * returns an array of standard links
	 * 
	 * Will only show (at most) view link based on {@link showViewLinkOnly}
	 * @param integer $id the current item id
	 * @param array the item array
	 * @return array the array of standard links
	 */
	function getStandardLinks($id, $item) {
		if ($this->showViewLinkOnly()) {
			global $SERVICES;
			$links = array();
			
			if ($this->getFileSetHref('view')) {
				$links['view'] = $this->getViewLink($id);
			}
			
		} else {
			$links = parent::getStandardLinks($id, $item);
		}

		return $links;
	}	
}
?>