<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 **/

includeWebMAT('list.inc');

/**
 * Class for displaying an expandable list
 * @see WebMATDashboard
 * @see WebMATContactView
 * @abstract
 * @package WebMAT
 * @author Phil Calder
 */
class WebMATStandardExpandableList  extends WebMATList {
	
	/**
	 * @access private
	 */
	var $output = '';
	/**
	 * @access private
	 */
	var $config = '';
	/**
	 * @access private
	 */
	var $display_limit = '';
	/**
	 * @access private
	 */
	var $title_text = '';
	/**
	 * @access private
	 */
	var $menu_separator = '';
	/**
	 * @access private
	 */
	var $edit_link = '';
	/**
	 * @access private
	 */
	var $delete_link = '';
	/**
	 * @access private
	 */
	var $view_link = '';
	/**
	 * @access private
	 */
	var $list_link = '';
	/**
	 * @access private
	 */
	var $module_path = '';
	/**
	 * @access private
	 */
	var $contact_id = '';
	/**
	 * @access private
	 */
	var $show_no_items_message = '';
	/**
	 * @access private
	 */
	var $items = '';
	/**
	 * @access private
	 */
	var $content_field_trim = '';
	/**
	 * @access private
	 */
	var $sort_field = '';
	/**
	 * @access private
	 */
	var $limit = '';
	/**
	 * @access private
	 */
	var $piece_id = '';
	/**
	 * @access private
	 */
	var $disable_webmat_init = false;
	/**
	 * @access private
	 */
	var $tag_header_row = '';
	/**
	 * @access private
	 */
	var $alert_count = -1;
	/**
	 * @access private
	 */
	var $mine_count = -1;
	/**
	 * @access private
	 */
	var $unseen_count = -1;
	/**
	 * @access private
	 */
	var $item_count = -1;
	/**
	 * @access private
	 */
	var $no_items_flag = false;
	/**
	 * @access private
	 */
	var $begin_table_arg = 'viewTbl' ; //'pieceTable';
	
	/**
	 * @access private
	 */
	var $do_count_items = true;
	/**
	 * @access private
	 */
	var $do_count_alerts = true;
	/**
	 * @access private
	 */
	var $do_show_items = true;
	
	/**
	 * Sets piece related flags
	 *
	 * Will only call parent init if disable_webmat_init is no set or not true
	 */
	function init($module, $data_option, $post = false, $get = false) {
		if ($this->disable_webmat_init) {
			$this->setOutput();
			$this->contact_id = (isset($this->form_values['contact_id']) ? $this->form_values['contact_id'] : $this->form_values['id'] );
		} else {
			parent::init($module, $data_option, $post, $get);
		}
		
		$this->output->setRowFlag(); // resets the row flag to its default value
		$this->output->req_config_js = false; // Overrides WebMATList setting
		
		// Phil 20060130 - see predend.inc::getWebMATObject
		if (isset($get['piece_identifier']) && $get['piece_identifier']) {
			$this->setPieceId($get['piece_identifier']);
		} else {
			$this->setPieceId($module);
		}
		
		$this->setModulePath();
		
		$this->setMenuSeparator('&nbsp;|&nbsp;');
		
		$this->view_link = 'view';
		$this->edit_link = 'edit';
		$this->delete_link = 'edit_delete';
		$this->list_link = 'list';
		
		$this->setTitleText();
		
		$this->setContentFieldTrim();
		
		$this->setPrefName(strtolower($module) . 'ContactView'); // Will have 'ListSize' appended
		
		$this->setShowNoItemsMessage(false);
		
		// $this->header_colspans = array();
		
		$this->setContentDisplayMode();
		
		if ($this->getContentDisplayMode() != 2) {
			$this->setLimit();
			$this->setShowContent();
		}
		
		// print '<div>' . get_class($this) . ' ' . $this->getModuleName() . ' - expandable list init calling setPieceFilter</div>';
		$this->setPieceFilter();
	}
	
	/**
	 * Builds the expandable header row and content piece (list page).
	 *
	 * The content piece (list page) is often empty, unless overridden.
	 * Alternatively, call {@link getExpandableHeader} and {@link getDisplayContent} which both return content. 
	 * You can format this content your own way.
	 *
	 * @return mixed Either string content, or sends content to Output and exits.
	 */
	function main($return_string = true) {
		$this->setReturnString($return_string);
		
		$content = $display_content = $header = '';
		
		$content .= $this->output->getBeginTable($this->begin_table_arg);
		
		// --
		$header = $this->getExpandableHeader(true); // true = as a row
		if ($this->do_show_items) {
			// Phil 20070426 We used to only display the content table if there were items to show 
			// but Matt now wants the empty table displayed
			// Note: still checks $this->do_show_items (eg contact view setup)
			// if ($this->item_count) {
			$display_content .= $this->getDisplayContent();
			// }
		} else {
			// For a contact view piece, this displays a 'Settings' link.
			$display_content .= $this->output->getBeginRow() . $this->output->getBeginCell(2); 
			$display_content .= $this->getShowItemsLink();
			$display_content .= $this->output->getEndCell() . $this->output->getEndRow(); 
		}
		// --
		
		$content .= $header;
		
		// Call this after getDisplayContent (even though we can display it before the content)
		$content .= $this->getSummaryContent(); 
		
		$content .= $display_content;
		
		
		$content .= $this->output->getEndTable();
		
		return $this->display($content);
	}
	
	/**
	 * Disabled
	 *
	 * Override to implement
	 *
	 * @return string An empty string
	 */
	function getSummaryContent() {
		// eg: 
		// $content = '';
		// $content .= $this->output->getBeginRow() . $this->output->getBeginCell(2); 
		// $content .= 'summary content here'
		// $content .= $this->output->getEndCell() . $this->output->getEndRow(); 
		return '';
	} 
	
	/**
	 * Disabled
	 *
	 * Override to implement
	 *
	 * @return string An empty string
	 */
	function getDisplayContent() {
		return '';
	} 
	
	/**
	 * Disabled
	 *
	 * Override to implement
	 *
	 * @return string An empty string
	 */
	function getShowItemsLink() {
		return '';
	}
	
	/**
	 * Disabled
	 *
	 * @return string An empty string
	 */
	function getPieceContent() {
		return '';
	}
	
	/**
	 * @param string The piece id
	 */
	function setPieceId($id) {
		$this->piece_id = $id;
	}
	
	/**
	 * @return string The piece id
	 */
	function getPieceId() {
		return $this->piece_id;
	}
	
	/**
	 * @param string The title text (for the header row)
	 */
	function setTitleText($text = '') {
		if (empty($text) && is_object($this->mod)) {
			global $SERVICES, $L;
			$SERVICES->getService('Language');
			$SERVICES->language->getLanguage($this->module_name);
			$text = $L[$this->mod->getModuleNameLower() . 'EventNamePlural'];
		}	
		$this->title_text = $text;
	}
	
	function setHeaderLinks($links) {
		$this->header_links = $links;
	}
	
	function setContentLinks($links) {
		$this->header_links = $links;
	}
	
	function setItems($items) {
		$this->items = $items;
	}
	
	function getTitleLink($text) {
		if (($link = $this->getFileSetHref($this->list_link)) && ($this->getContentDisplayMode() == 2)) {
			$href = $this->getModulePath() . str_replace('%id%', '', (str_replace('%contact_id%', $this->getContactId() ,$link)) );
		} else {
			$href = '#';
		}
		
		$title_text = $this->output->getLinkHref(
			array(
				'href' => $href, 
				'text' => $text
			)
		);			
		return $title_text; 
	}

	
	function getTitleText() {
		return $this->title_text;
	}

	function getHeaderLinks() {
		if (isset($this->header_links) && $this->header_links) {
			$content = $this->header_links;
		} else {
			$content = array();
			if ($bookmark = $this->getBookmarkLinkLink()) {
				$content[] = $bookmark;
			}
			//mcr 2005-05-18 //phil re-added - will remove this from dashboard only
			if ($all = $this->getAllLink()) {
				$content[] = $all;
			}
			
			if ($new = $this->getNewLink()) {
				$content[] = $new;
			}
		}
		
		if (is_array($content)) {
			//return implode($this->getMenuSeparator(), $content);
			return implode('', $content);
		} 
		
		return $content;
	}
	
	/**
	 * @return string String of links, separated by menu separator.
	 */
	function getContentLinks($id = '') {
		
		if (isset($this->content_links) && $this->content_links) {
			$content = $this->content_links;
		} else {
		
			$content = array();
			if ($view = $this->getViewLink($id)) {
				$content[] = $view;
			}
			if ($all = $this->getEditLink($id)) {
				$content[] = $all;
			}
			
		}
		
		if (is_array($content)) {
			return implode($this->getMenuSeparator(), $content);
		} 
		return $content;
	}
	
	/**
	 * Wrapper for database tables getItems
	 */
	function getItems($sort, $limit) {
		return $this->db_functions->getItems($sort, $limit);
	}
	
	/** 
	 * Empty. This can be overridden.
	 */
	function calculatePiecesValues(& $item) {}	
	
	function getTagHeaderRow() {
		return $this->tag_header_row;
	}
	
	function setTagHeaderRow($str) {
		$this->tag_header_row = $str;
	}
	
	function getExpandableHeader($as_row = true) {
		$content = '';
		
		$alert_tag = '';
		if ($this->do_count_alerts) {
			$alert_count = $this->countAlerts();
			if ($alert_count) {
				$alert_tag = '-alert';
			}
		} else {
			$alert_count = false;
		}
		// print '<div>do_count_alerts: ' . ($this->do_count_alerts ? 'yes' : 'no') . ' alert_count: ' . $alert_count . ' getHeaderRow: ' . $this->getTagHeaderRow() . '</div>';
		
		if ($as_row) {
			$content .= $this->output->getBeginRow($this->getTagHeaderRow() . $alert_tag);
		}
		
		
		$tools = '<span class="tools">' . $this->getHeaderLinks() . '</span>'; 
		
		$counts = $this->getItemCounts();
		// print '<div>getExpandableHeader called getItemCounts</div>'; print_r($counts); print '<hr>';
		$title = $this->getTitleText();
		// print '<div>$this->getContentDisplayMode(): ' . $this->getContentDisplayMode() . '</div>';	
		if ($this->getContentDisplayMode() != 2 && $this->item_count) {  //build expand link
			// This is the only link in the page that will always target itself
			$current_target = $this->output->getDefaultLinkTarget();
			$this->output->setDefaultLinkTarget('');
			//$title_text = $this->getTitleLink($title) . ' ' . $counts;
			
			//$title_text = $title . ' ' . $counts;
			// restore target to the contentframe
			$this->output->setDefaultLinkTarget($current_target);
		} else { //no content, link goes to list view
			// $title_text = $this->getTitleLink($title) . ' ' . $counts;
			
			//$title_text = $title . ' ' . $counts;
		}
		// $content .= $this->output->getTableHeaderCell($this->getTitleImage(), 'class="icon"');  //alert image
		// $content .= $this->getMenuCellContent(); 
		// Phil 20050523 - replaced above with this:
		// $content .= $this->output->getTableHeaderCell($this->getTitleImage()  . '' . $this->getMenuLink(), 'class="icon"');  //alert image
		// $content .= $this->getTitleImageCell();
		
		$label_flag = '';
		if ($this->getContentDisplayMode() == 1) {
			$label_flag = 'labelCurrent';	
		} else { 
			$label_flag = 'label';
		}
		
		$content .= $this->output->getTableHeaderCell($title_text . $tools, 'class="' . $label_flag . '" id="' . strtolower($this->getPieceId()) . 'PieceLabel"' . $this->getEventHandlerShowContent());
		
		if ($as_row) {
			$content .= $this->output->getEndRow();
		}
		// print '<div>getExpandableHeader returning:<br>---<br>' . $content . '<br>---</div>';
		return $content;
	}
	
	/**
	 * Gets the title image cell
	 *
	 * Override {@link getMenuLink} in WebMATDashboard to include (optional) menu link 
	 * @return string Table cell containing the title image (default is an alert image)
	 * @author Phil Calder
	 */
	function getTitleImageCell() {
		return $this->output->getTableHeaderCell($this->getTitleImage()  . '' . $this->getMenuLink(), 'class="icon" style="valign:top;" width="100"');  //alert image
	}
	
	function getMenuCellContent() {
		//TODO
		// return '<th class="menu_icon"><a href="" target="contentframe">' . $this->output->getImageTag('icons_dash/drop_menu.gif', false, 'align="absmiddle"') . '</a></th>';
		// Phil, created getMenuLink (because I want to include this link somewhere else - ie not in its own cell)
		return '<th class="menu_icon">' . $this->getMenuLink() . '</th>';
	}
	
	/**
	 * @return string An empty string
	 */
	function getMenuLink() {
		// Phil - took this link out of getMenuCellContent
		// Would return something like this in a 'Dashboard' piece:
		// return '<a href="" target="contentframe">' . $this->output->getImageTag('icons_dash/drop_menu.gif', false, 'align="absmiddle"') . '</a>';
		return '';
	}
	
	function getTableRows($items) {
		$content = '';
		foreach($items as $k => $v) {
			$content .= $this->getTableRow($v, $k);
		}
		return $content;
	}
	
	function beforeTableRow(&$item, &$style_flag) {}
	
	function setContentCellsFormat($format_array, $new_row_fields = '') {
		$this->content_cells_format = $format_array;
		$this->new_row_content_fields = $new_row_fields;
		// $this->content_cells_format = array(
			// 'label' => '',
			// 'content' => 'colspan="2"', 
			// 'links' => 'style="text-align:right"'
		// );
	}
	
	function getContentCellsFormat() {
		return $this->content_cells_format;
		// $this->content_cells_format = array(
			// 'label' => '',
			// 'content' => 'colspan="2"', 
			// 'links' => 'style="text-align:right"'
		// );
	}
	
	function setContentFormat($type = '') {
		$this->content_format = $type;
	}
	
	/**
	 * Gets the alert link to the current modules list page
	 *
	 * @param string $link_text The text to display for the link
	 * @return string an a href for the a new link
	 */
	function getAlertLink($link_text = '') {
		if ($this->getFileSetHref($this->list_link)) {
			global $SERVICES, $L;
			
			if ($this->useFilters()) {
				$contact_id = $this->filters->getContactFilter();
			} else {
				$contact_id = $this->form_values['contact_id'];
			}
			
			$SERVICES->getService('Config');
			//if ( ($this->isContactRequired() == 0) || ($contact_id != '') && ($this->isDataOption('edit')) ) {
			if ($this->isDataOption('list')) {
				$link_href = str_replace('%id%', '', (str_replace('%contact_id%', $contact_id ,$this->getFileSetHref($this->list_link))) );
				
				if (empty($link_text)) {
					$link_text = $L['LinkAlerts'];
				}
				
				return $this->output->getLinkHref( array('href' => $link_href, 'text' => $link_text) );
			} else {
				return '';
			}
		}
	}
	
	function getNewLink() {
		if ($this->getFileSetHref($this->edit_link)) {
			global $L; // $SERVICES, 
			// $SERVICES->getService('Config');
			$newText = $this->output->getImageTag('img/page_add.png', false, 'width=16 height=16') . $L['LinkNew'];
			$contact_id = $this->getContactId();
			if ($this->isDataOption('edit')) {
				$link_href = $this->getModulePath() . str_replace('%id%', '', (str_replace('%contact_id%', $contact_id ,$this->getFileSetHref($this->edit_link))) );
				return $this->output->getLinkHref(
					array(
						'href' => $link_href,
						'text' => $newText,
						'title' => $L['LinkNew']
					)
				);
			}
		}
		
		if ($link = $this->getFileSetHref($this->edit_link)) {
			$href = $this->getModulePath() . str_replace('%id%', '', (str_replace('%contact_id%', $this->getContactId(), $link)) );
			global $L;
			
			if (!$title) {
				// $title = $L['LinkNew'];
				$title = $this->output->getImageTag('img/page_add.png', false, 'width=16 height=16') . $L['LinkNew'];
			}
			$linkText = $title;
			return $this->output->getLinkHref(array('href'=>$href, 'text'=>$linkText, 'title' => $title), false);
			//return $this->output->getLinkNew($link_href);
		}
		return false;
	}
	
	/**
	 * returns a link to the first bookmark in the module
	 * 
	 * @return string the link
	 */
	function getBookmarkLinkLink() {
		global $SERVICES;
		$system_bookmarks_mod =& $SERVICES->modules->getModule('SystemBookmarks');
		if ($link = $system_bookmarks_mod->getFirstBookmarkForModule($this->module_name)) {
			//$linkText = $this->output->getImageTag('icons_dash/bookmark.gif', $link['text'], 'width="16" height="16"');
			$linkText = 'Bookmark';
			return $this->output->getLinkHref(array('href'=>$link['href'], 'text'=>$linkText, 'title' => $link['text']), false);
		}
	}
	
	function getAllLink() {
		global $L;
		if ($link = $this->getFileSetHref($this->list_link)) {
			$href = $this->getModulePath() . str_replace('%id%', '', (str_replace('%contact_id%', $this->getContactId() ,$link)) );
			if (strstr($href, '?')) {
				$href .= '&';
			} else {
				$href .= '?';
			}
			$href .= 'search_filter=off';
			
			global $L;
			$title = '';
			
			// $title .= $L['ListView'];
			$title = $this->output->getImageTag('img/table.png', false, 'width=16 height=16') . $L['ListView'];
			$linkText = $title;
		
			return $this->output->getLinkHref(array('href'=>$href, 'text'=>$linkText, 'title' => $L['ListView']), false);
			//return $this->output->getLinkAll($link_href);
		}
		return false;
	}
	
	function getViewLink($id = '') {
		if ( ($link_href = $this->getFileSetHref('view')) && ($link = parent::getViewLink($id)) ) {
			$search = array('href="' , '%id%', '%contact_id%');
			$replace = array('href="' . $this->getModulePath() , $id, $this->getContactId());
			return str_replace($search, $replace, $link);
			//$link_href = $this->getModulePath() . str_replace('%id%', $id, (str_replace('%contact_id%', $this->getContactId(), $link)) );
			//return $this->output->getLinkView($link_href);
		}
		return false;
	}
	
	function getDeleteLink($id = '') {
		if ($link = $this->getFileSetHref($this->delete_link)) {
			$link_href = $this->getModulePath() . str_replace('%id%', $id, (str_replace('%contact_id%', $this->getContactId(), $link)) );
			return $this->output->getLinkDelete($link_href);
		}
		return false;
	}
	
	/**
	 * Will have 'ListSize' appended when used
	 */
	function setPrefName($key) {
		$this->pref_name = $key;
	}
	
	/**
	 * Returns the paging list size
	 */
	function getPrefName() {
		return $this->pref_name;
	}
	
	function getContactId() {
		return $this->contact_id;
	}
	
	function setContactId($id) {
		$this->contact_id = $id;
	}
	
	function setPagingOverwrite($limit = '', $sort_field = '', $sort_order = '') {
		if (!is_array($this->paging_overwrite)) {
			$this->paging_overwrite = array();
		}
		
		if (!empty($limit)) {
			$this->paging_overwrite['limit'] = $limit;
		}
		
		if (!is_array($this->paging_overwrite['sort'])) {
			$this->paging_overwrite['sort'] = array();
		}
		
		if (!empty($sort_field)) {
			$this->paging_overwrite['sort']['field'] = $sort_field;
		}
		
		if (!empty($sort_order)) {
			$this->paging_overwrite['sort']['order'] = $sort_order;
		}
	}
	
	function getShowNoItemsMessage() {
		return $this->show_no_items_message;
	}
	
	function setShowNoItemsMessage($boolean_flag) {
		$this->show_no_items_message = $boolean_flag;
	}
	
	function getNoItemsMessage() {
		if (!isset($this->no_items_message)) {
			global $L;
			$this->no_items_message .= $L['msgNoItemsForView'];
			// global $SERVICES, $L;
			// $SERVICES->getService('Language');
			// $SERVICES->language->getLanguage($this->mod->getModuleName());
			// 
			// $this->no_items_message = $L[$this->mod->getModuleNameLower() . 'NoItems'];
		}
		return $this->no_items_message;	
	}
	
	function setNoItemsMessage($msg = '') {
		$this->no_items_message = $msg;
	}
	
	/**
	 * As of version 5, always returns false
	 */
	function getEventHandlerShowContent() {
		// print '<div>' . get_class($this) . ' module: ' . $this->getModuleName() . '</div>';
		if ($this->getContentDisplayMode() == 2 || !$this->item_count) {
			return false;
		}
		return false;
		// return 'onClick=\'showSection("' . $this->getPieceId() . '")\'';
		// return 'onClick=\'openPieceFromPiece("' . strtolower($this->getPieceId()) . 'Piece");\'';
		// return 'onClick=\'togglePiece("' . strtolower($this->getPieceId()) . 'Piece", "' . strtolower($this->getPieceId()) . 'PieceLabel");\'';
	}
	
	/**
	 * @deprecated
	 */
	function getBeginShowContent() {
		if ($this->getContentDisplayMode() == 2) {
			return false;
		}
		$show = '' . ($this->isShowContent() ? 'show' : 'none');
		return '<div id="' . $this->getPieceId() . '" style="display:' . $show . ';">';  // this->module is not unique!!
	}
	
	/**
	 * @deprecated
	 */
	function getEndShowContent() {
		if ($this->getContentDisplayMode() == 2) {
			return false;
		}
		return '</div>';
	}
	
	function setShowContent($value = false) {
		if ($value) {
			$this->is_show_content = true;
		} else {
			if (isset($this->form_values['show']) && isset($this->form_values['show'][$this->getPieceId()]) && $this->form_values['show'][$this->getPieceId()]) {
				$this->is_show_content = true;
			} else {
				//$this->is_show_content = false; //removed to load from user pref
				$this->is_show_content = $this->getContentDisplayMode();
			}
		}
	}
	
	function isShowContent() {
		return $this->is_show_content;
	}
	
	/**
	 * Do not trim fields that are pre-formatted urls
	 */
	function setContentFieldTrim($value = '') {
		if (empty($value)) {
			$value = $this->getDefaultContentFieldTrim();
		}
		$this->content_field_trim = $value;
	}
	
	function getDefaultContentFieldTrim($value = '') {
		global $SERVICES;
		$SERVICES->getService('Preferences');
		$length = $SERVICES->preferences->getUserPref('frontDashboardContentLength', 50);
		return $length;
	}
	
	function getContentFieldTrim() {
		return $this->content_field_trim;
	}
	
	function getTrimmedContentField($value) {
		if ($length = $this->getContentFieldTrim()) {
			if (strlen($value) >= $length) {
				$value = trim(substr($value, 0, $length)) . '...';
			}
		}
		return $value;
	}

	function getTitleImage() {
		global $L;
		if ($alerts = $this->countAlerts()) {
			$alert_flag = 'alert';
			$text = $L['Alerts'] . ': ' . $alerts;
		} else {
			$alert_flag = 'normal';
			$text = $L['NoAlert'];
		}
		
		if ($href = $this->getFileSetHref('list')) {
			$href = $this->getModulePath() . $href;
		} else {
			$href = $this->getModulePath();
		}
		if ($query_start = strpos($href, '?')) {
			$href = substr($href, 0, $query_start);
		}
		// $href .= '?qfilter=alert'; // Phil 20050714 @see getQtypeFilterQueryString
		$href .= $this->getQtypeFilterQueryString('alert', true);
		
		$link = array(
			'href' => $href,
			'title' => $text,
			'icon' => 'icons_dash/' . strtolower($this->getPieceId()) . '_' . $alert_flag . '.gif', 
			'if_exists' => true // Phil 20050427
		);
		
		// return $this->output->getLinkDashIcon($link);
		// Phil 20050427 - Will return default icon if there isnt one for this piece
		$default_icon_name = 'other';
		if ($retval = $this->output->getLinkDashIcon($link)) {
			return $retval;
		}
		$link = array(
			'href' => $href,
			'title' => $text,
			'icon' => 'icons_dash/' . $default_icon_name . '_' . $alert_flag . '.gif',
		);
		return $this->output->getLinkDashIcon($link);
	}
	
	/**
	 * @param string type - The query type (eg alert)
	 * @param boolean incl_questionmark Whether to prepend the return value with a questionmark
	 * @return string containing the necessary qtype filter to use in a query string (eg ?qfilter=alert)
	 */
	function getQtypeFilterQueryString($type = 'alert', $incl_questionmark = false) {
		if (empty($type)) {
			$type = 'alert';	
		}
		// Phil 20051208 Added contact filter to alert query string
		$contact_filter = '';
		if (isset($this->form_values['contact_id']) && $this->form_values['contact_id']) {
			$contact_filter = '&contact_id=' . $this->form_values['contact_id'];
		}
		return ($incl_questionmark ? '?' : '') . 'qfilter=' . $type . $contact_filter; //Syed 20050714 Changed the query string name qtype to qfilter
	}
	
	/**
	 * @todo Phil - remove duplication between this function and {@link getTitleText}
	 */
	function getItemCounts() {
		global $L;
		
		if ($this->getShowCountAlerts()) {
			$alerts = $this->countAlerts(); // Phil 20050822
		}
		// $mine = $this->countMine();
		// Phil 20051031 may return false for some sub classes (pending new UI)
		// see getShowCountItems
		if ($this->getShowCountItems()) {
			$items = $this->countItems(); 
		}
		// $unseen = $this->countUnseen();
		
		if (empty($alerts)) {
			unset($alerts);
		}
		// if (empty($mine)) {
			// unset($mine);
		// }
		if (empty($items) && $this->getShowCountItems()) {
			$items = 0;
			//unset($items);
		}
		// if (empty($unseen)) {
			// unset($unseen);
		// }
		$prefix = $middle = $suffix = '';
		
		if (isset($alerts) || isset($items)) {
			if (isset($alerts) && isset($items)) {
				$middle = $this->getMenuSeparator(); // ' | ';
			}
			
			$prefix = '(';
			$suffix = ')';
			
			global $L;
			if (isset($alerts)) {
				if ($alerts) {
					$text = $L['Alerts'] . ': ' . $alerts;
				} else {
					$text = $L['NoAlert'];
				}
				if ($href = $this->getFileSetHref('list')) {
					$href = $this->getModulePath() . $href;
				} else {
					$href = $this->getModulePath();
				}
				if ($query_start = strpos($href, '?')) {
					$href = substr($href, 0, $query_start);
				}
				
				// $href .= '?qfilter=alert';
				$href .= $this->getQtypeFilterQueryString('alert', true);// Phil 20051109
				
				$alerts = $this->output->getLinkHref(
					array(
						'href' => $href,
						'title' => $text, 
						'text' => $alerts
					)
				);
			}
			
			if (isset($items)) {
				if ($href = $this->getFileSetHref('list')) {
					$href = $this->getModulePath() . str_replace('%id%', '', (str_replace('%contact_id%', $this->getContactId() ,$href)) ); // duplicated from getAllLink
				} else {
					$href = $this->getModulePath();
				}
				if ($items) {
					$items = $this->output->getLinkHref(
						array(
							'href' => $href, 
							'text' => $items,
							'title' => $L['LinkAll'] . '...',
						)
					);
				}
			}
		}
		return $prefix . $alerts . $middle . $items . $suffix;
	}
	
	function countAlerts() {
		// print '<div>' . get_class($this) . ' ' . $this->getModuleName() . ' - expandable countAlerts init calling setPieceFilter</div>';
		if ($this->alert_count == -1) { // means we havent calculated this value yet
			$this->alert_count = false;
			if (isset($this->db_functions) && is_object($this->db_functions)) {
				$this->setPieceFilter('alerts');
				$this->alert_count = $this->db_functions->countAlerts();
			} 
		}
		return $this->alert_count;
	}
	
	/**
	 * Item counts can  be disabled here (or in a sub class)
	 * Can override per installation using webmat_ext directory
	 * by overriding getShowCountItems
	 * @return false
	 * @access private
	 */
	function getShowCountItems() {
		if ($this->do_count_items) {
			return true;
		}
		return false;
	}
	
	/**
	 * Item counts can  be disabled here (or in a sub class)
	 * Can override per installation using webmat_ext directory
	 * by overriding getShowCountItems
	 * @return false
	 * @access private
	 */
	function getShowCountAlerts() {
		if ($this->do_count_alerts) {
			return true;
		}
		return false;
	}
	
	function countItems() {
		// print '<div>' . get_class($this) . ' ' . $this->getModuleName() . ' - expandable countItems init calling setPieceFilter</div>';
		// Phil 20051031 removed pending new UI
		// Can override per installation using webmat_ext directory
		// by overriding getShowCountItems
		if (!$this->getShowCountItems()) { 
			return false; 
		}
		
		if ($this->item_count == -1) { // Phil 20050824 - means we havent calculated this value yet
			$this->item_count = false;
			if (isset($this->db_functions) && is_object($this->db_functions)) {
				// print '<div>[exp] ' . get_class($this) . '::countItems [mod: ' . $this->getModuleName() . ']</div>';
				$this->setPieceFilter('items');
				$this->item_count = $this->db_functions->countItems();
			}
		}
		// print '<div>count: ' . $this->item_count . '</div>';
		return $this->item_count;	
	}
	
	function countMine() {
		// print '<div>' . get_class($this) . ' ' . $this->getModuleName() . ' - expandable countMine init calling setPieceFilter</div>';
		if ($this->mine_count == -1) { // Phil 20050824 - means we havent calculated this value yet
			$this->mine_count = false;
			if (isset($this->db_functions) && is_object($this->db_functions)) {
				// print '<div>[exp] ' . get_class($this) . '::countMine [mod: ' . $this->getModuleName() . ']</div>';
				$this->setPieceFilter('mine');
				$this->item_count = $this->db_functions->countMine();
			}
		}
		return $this->mine_count;	
	}
	
	function countUnseen() {
		// print '<div>' . get_class($this) . ' ' . $this->getModuleName() . ' - expandable countUnseen init calling setPieceFilter</div>';
		if ($this->unseen_count == -1) { // Phil 20050824 - means we havent calculated this value yet
			$this->unseen_count = false;
			if (isset($this->db_functions) && is_object($this->db_functions)) {
				// print '<div>[exp] ' . get_class($this) . '::countUnseen [mod: ' . $this->getModuleName() . ']</div>';
				$this->setPieceFilter('unseen');
				$this->unseen_count = $this->db_functions->countUnseen();
			}
		}
		return $this->unseen_count;	
	}
	
	/**
	 * sets the content display mode
	 * 
	 * @param string $mode the display mode see below:
	 * 	- 0 = display content in hidden div
	 * 	- 1 = display content in shown div
	 * 	- 2 = do not display content
	 */
	function setContentDisplayMode($mode = false) {
		if ($mode) {
			$this->content_display_mode = $mode;
		} else {
			global $SERVICES;
			$this->content_display_mode = $SERVICES->preferences->getUserPref($this->getPrefName() . 'ContentDisplay', 0);
		}
	}
	
	function getContentDisplayMode() {
		//return 1; //temp this delivers all open pieces;
		//remember that dash is currently mode 2 (no content)
		//eventually do a bitwise & here with no_items_flag & userpref ?? 
		if (!$this->no_items_flag && ($this->content_display_mode != 2) ) {
			return 1;	
		}
		// if ($this->content_display_mode) {
			// $this->setContentDisplayMode();
		// }
		return $this->content_display_mode;
	}
	
	function setLimit($number = 0, $start = 0) {
		// Note: overridden in WebMATContactView
		// print '<div>setLimit for module ' . $this->getModuleName() . '</div>';
		if (isset($this->form_values['limit'][$this->getPieceId()]) && ($this->form_values['limit'][$this->getPieceId()])) {
			$this->limit = array('number'=>$this->output->paging_list_size);
			return;
		}
		
		if (empty($number)) {
			global $SERVICES;
			$SERVICES->getService('Preferences');
			$number = $SERVICES->preferences->getUserPref($this->getPrefName() . 'ListSize');
			if (!$number) {
				$number = $SERVICES->preferences->getUserPref('frontDashboardListSize');
			}
		}
		
		$this->limit = array();
		$this->limit['number'] = $number;
		$this->limit['start'] = $start;
	}
	
	function getLimit() {
		if (!is_array($this->limit)) {
			$this->setLimit();
		}
		
		if (isset($this->limit['start']) && isset($this->limit['number'])) {
			return 'LIMIT ' . $this->limit['start'] . ', ' . $this->limit['number'];
		}
		return ''; // return $this->limit;
	}
	
	function getLimitNumber() {
		if (!is_array($this->limit)) {
			$this->setLimit();
		}
		return $this->limit['number'];
	}
	
	function setSortField($field = 'created', $order = 'DESC') {
		$this->sort_field = array();
		$this->sort_field['field'] = $field;
		$this->sort_field['order'] = $order;
	}
	
	function getSortField() {
		return $this->sort_field;
	}
	
	function setPieceFilter() {
	}
}

?>