<?php
/**
 *  This class handles the filters used in the list class
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('webmat.inc');

/**
 * This class handles the filters used in the list class
 *
 * @author Matthew Corich, Phil Calder, Brenton LeTrabb
 */
class WebMATStandardFilters extends WebMAT {
	/**
	 * @var mixed $search_filter_items the search filter items (array), or false
	 * @access private
	 */
	var $search_filter_items = false;
	/**
	 * @var array $design_time_filters the current design time filters
	 * @access private
	 */
	var $design_time_filters = array();
	/**
	 * @var array $extra_filters_disable_row extra filters to be disabled
	 * @access private
	 */
	var $extra_filters_disable_row = array();
	/**
	 * @var boolean $show_qfilters
	 * @access private
	 */
	var $show_qfilters = true;
	
	function WebMATStandardFilters($module, &$form_values, $table, &$dd, &$db_functions) {
		global $SERVICES;
		// $this->printDebug('WebMAT constructor for : ' . get_class($this) . '');
		$SERVICES->getService('Preferences');
		$this->init($module, $form_values, $table, $dd, $db_functions);
	}
	
	function init($module, &$form_values, $table, &$dd, &$db_functions) {
		$this->setModuleName($module);
		$this->setOutput();
		$this->setTableName($table);
		$this->setDataDefinition($dd);
		$this->setDbFunctions($db_functions);
		// print '<div>init form_values</div>'; print_r($form_values); print '<hr>';
		$this->setFormValues($form_values);
		$this->setPrefPrefix();
		//=============================
		$this->setContactFilter();
		$this->setSearchFilterActive();
		$this->setConfigFilterInfo();
		$this->setDbFunctionsConfigFilterInfo();
		$this->setSearchFilter();
		$this->setIdsFilter();
		$this->setSearchFilterFormVisible();
	}
	
	/**
	 * sets the $module_name variable
	 * 
	 * @param string the module name
	 */
	function setModuleName($module) {
		if (is_string($module)) {
			$this->module_name = $module;
		}
	}
	
	/**
	 * returns the $module_name variable
	 * 
	 * @return string the module name
	 */
	function getModuleName() {
		if ($this->module_name) {
			return $this->module_name;
		}
		return false;
	}
	
	/**
	 * sets the $output object
	 */
	function setOutput() {
		global $SERVICES;
		$this->output =& $SERVICES->getService('Output');
	}
	
	/**
	 * returns the $output object
	 * 
	 * @return string the output object
	 */
	function &getOutput() {
		return $this->output;
	}
	
	/**
	 * sets the $table_name variable
	 * 
	 * @param string the table name
	 */
	function setTableName($table) {
		if (is_string($table)) {
			$this->table_name = $table;
		}
	}
	
	/**
	 * returns the $table_name variable
	 * 
	 * @return string the table name
	 */
	function getTableName() {
		if ($this->table_name) {
			return $this->table_name;
		}
		return false;
	}
	
	/**
	 * sets the $db_functions object
	 * 
	 * @param object the db_functions object
	 */
	function setDbFunctions(&$db_functions) {
		if (is_object($db_functions)) {
			$this->db_functions =& $db_functions;
		} else {
			$this->db_functions = false;
		}
	}
	
	/**
	 * returns the $db_functions object
	 * 
	 * @return object the db_functions object
	 */
	function &getDbFunctions() {
		return $this->db_functions;
	}
	
	/**
	 * sets the data definition object
	 * 
	 * @param object the data definition object
	 */
	function setDataDefinition(&$dd) {
		if (is_object($dd)) {
			$this->dd =& $dd;
		} else {
			$this->dd = false;
		}
	}
	
	/**
	 * returns the $db_functions object
	 * 
	 * @return object the db_functions object
	 */
	function &getDataDefinition() {
		return $this->dd;
	}
	
	/**
	 * sets the $pref_prefix variable
	 */
	function setPrefPrefix() {
		global $SERVICES;
		$php_self = $SERVICES->getGlobal('php_self');
		$module = $this->getModuleName();
		$form_values = $this->getFormValues();
		$this->pref_prefix = $module . '.';
		$this->pref_prefix .= substr(basename($php_self), 0, strpos(basename($php_self), '.')) . '.';
	}
	
	/**
	 * returns the $pref_prefix variable with the $filter string added to the end
	 * 
	 * @param string $filter the filter name (optional)
	 * @return the perference name
	 */
	function getPrefPrefix($filter = false) {
		if ($filter) {
			return $this->pref_prefix . $filter;
		}
		return $this->pref_prefix;
	}
	
	/**
	 * sets the $form_values variable
	 * 
	 * @param array the form values array
	 */
	function setFormValues(&$form_values) {
		if (is_array($form_values)) {
			// print '<div>setFormValues</div>'; print_r($form_values); print '<hr>';
			$this->form_values =& $form_values;
		}
	}
	
	/**
	 * returns the $form_values variable
	 * 
	 * @return array the form values array
	 */
	function &getFormValues() {
		if ($this->form_values) {
			return $this->form_values;
		}
		return array();
	}
	
	/**
	 * sets the search filter form visible
	 * @author Syed
	 */
	function setSearchFilterFormVisible() {
		if (!$this->getSearchFilterActive()) {
			$this->output->show_filter_form= false;
		}
	}
	
	function getFilteredFields() {
		$keys = array_keys($this->design_time_filters);
		return $keys;
	}
	
	/**
	 * sets the design time filters for each filter
	 * 
	 * @param array a design time filter
	 */
	function setDesignTimeFilter($design_time_filter) {
		// print '<p>setDesignTimeFilter</div>';
		$db_functions =& $this->getDbFunctions();
		$db_functions->setDesignTimeFilters($design_time_filter);
		$this->design_time_filters = bcrm_array_merge($design_time_filter, $this->design_time_filters);
		// print '<div>filters: setDesignTimeFilter</div>'; print_r($design_time_filter); print '<hr>';
		// print '<div>design_time_filters</div>'; print_r($this->design_time_filters); print '<hr>';
	}
	
	/**
	 * returns the design time filters used
	 * 
	 * @return array a design time filters array
	 */
	function getDesignTimeFilter() {
		if ($this->design_time_filters) {
			return $this->design_time_filters;
		}
		return array();
	}
	
	/**
	 * Sets the contact filter if a contact id is in the query string
	 *
	 * Includes sub contacts based on user preference
	 *
	 @author Phil Calder
	 */
	function setContactFilter() {
		// print '<div>' . get_class($this) . ' &&&&&&&&&&&&&&&&&&&&&& module: ' . $this->getModuleName() . ' columns:</div>'; print_r($this->columns); print '<hr>';
		 if ( isset($this->form_values['contact_id']) && $this->form_values['contact_id']) {
			$dd =& $this->getDataDefinition();
			if (isset($this->form_values['contact_depth'])) { 
				$contact_depth = $this->form_values['contact_depth'];
			} else {
				global $SERVICES;
				$prefs =& $SERVICES->getService('Preferences');
				$contact_depth = $prefs->getUserPref('contactShowCompanyDepth', '0');
			}
			if ($contact_depth) {
				if ($contact_ids_string = $this->getSubContactIdsString(true)) {
					$this->db_functions->setDesignTimeFilters( 
						array(
							'contact_id' => array(
								'where' => 'contact_id IN (' . $contact_ids_string . ')'
							)
						) 
					);
					return;
				}
			}
			$this->setDesignTimeFilter( array('contact_id' => $this->form_values['contact_id']) );
		}
	}
	
	// Phil -- see WebMATContactView
	/**
	 * For use in a MySQL 'IN' query, eg IN ( id1 [, id2, id3])
	 *
	 * @param boolean $include_parent Whether to include the current contact in the returned string. Defaults to false.
	 * @return mixed false or a comma separated string of ids (possibly blank)
	 * @author Phil Calder
	 */
	function getSubContactIdsString($include_parent = false) {
		$sub_contacts = $this->getSubContactIds();
		if (is_array($sub_contacts)) {
			if ($include_parent) {
				$sub_contacts = $this->arrayMergeReplace((array)$this->getContactId(), $sub_contacts);	
			}
			return implode(', ', $sub_contacts);
		}
		return false;
	}
	
	/**
	 * Applies only to companies
	 *
	 * @return mixed false or an array of ids
	 * @author Phil Calder
	 */
	function getSubContactIds() {
		global $SERVICES;
		$SERVICES->getService('Modules');
		$contact =& $SERVICES->modules->getModule('Contact');
		$contact_id = $this->form_values['contact_id'];
		return $contact->getSubContacts($contact_id);
	}
	
	/**
	 * returns the contact id for the current contact filter
	 * 
	 * @param integer the current contact id from the contact filter
	 */
	function getContactFilter() {
		if ( isset($this->form_values['contact_id']) && $this->form_values['contact_id']) {
			return $this->form_values['contact_id'];
		} else {
			return false;
		}
	}
	
	/**
	 * sets the $search_filter_active variable
	 */
	function setSearchFilterActive() {
		global $SERVICES;
		$form_values =& $this->getFormValues();
		if ($form_values['search_filter']) {
			switch ($form_values['search_filter']) {
				case 'on':
					$SERVICES->preferences->setUserPref($this->getPrefPrefix('search_filter'), 1);
					$this->search_filter_active = true;
				break;
				case 'off':
					$SERVICES->preferences->setUserPref($this->getPrefPrefix('search_filter'), 0);
					$this->search_filter_active = false;
				break;
				default:
					//do nothing
				break;
			}
		} else {
			if ( $SERVICES->preferences->getUserPref($this->getPrefPrefix('search_filter'), 0) == 1 ) {
				$this->search_filter_active = true;
			} else if (!empty($form_values['submit'])) {
					$SERVICES->preferences->setUserPref($this->getPrefPrefix('search_filter'), 1);
					$this->search_filter_active = true;
			} else {
				$this->search_filter_active = false;
			}
		}
	}
	
	/**
	 * returns the $search_filter_active variable
	 * 
	 * @return bool the $search_filter_active variable
	 */
	function getSearchFilterActive() {
		return $this->search_filter_active;
	}
	
	/**
	 * Checks if the the fields on the pages are searchable
	 * 
	 * @return  bool TRUE if filterable; FALSE otherwise
	 */
	function hasFilterableFields(){
		$dd =& $this->getDataDefinition();
		foreach ($dd->getDataDictionary() as $k => $v) {
			if ($v['calculated'] != 1 && $v['type'] != 'hidden' && 'hidden' != 1) {
				return true;
			}
		}
		return false;
	}
	
	
	/**
	 * sets the filter info variable
	 */
	function setConfigFilterInfo() {
		global $SERVICES;
		$SERVICES->getService('Config');
		$module_name = $this->getModuleName();
		$table_name = $this->getTableName();
		
		$filter_config = $SERVICES->config->lookup('modules.' .  $module_name . '.file_set.' . str_replace($module_name . '__', '', $table_name) . '.filter');
		
		//print '<div>filter_config</div>'; print_r($filter_config); print '<hr>';
		
		if ($filter_config) {
			$filter_info = array();
			if (isset($filter_config['display_no_filter_link'])) {
				if ($filter_config['display_no_filter_link'] == 1) {
					$filter_info['no_filter_link'] = $this->output->updateQueryString(array('filter' => 'fnone'));
				} else {
					$filter_info['no_filter_link'] = false;
				}
				bcrm_unset($filter_config, 'display_no_filter_link');
			}
			
			if (isset($filter_config['select_menu'])) {
				if ($filter_config['select_menu'] == 1) {
					$filter_info['use_select'] = true;
				} else {
					$filter_info['use_select'] = false;
				}
				bcrm_unset($filter_config, 'select_menu');
			}
			
			// Phil 20051025 Config filter can contain false (false, 0, '') values (ie settings which have been disabled)
			if (is_array($filter_config[0])  || is_array(reset($filter_config)) || (reset($filter_config) == false) ) {
				if (sizeof($filter_config)) {
					// Phil 20060701 filter_config links can contain URLs. 
					// We may need to override this (eg for a Contact selector)
					// Actually, the only place we really need the full URL is the application Menu (or any direct config lookup)
					$use_self = true;
					if ($use_self && ($php_self = $SERVICES->getGlobal('php_self')) ) {
						$ext = '.php';
						// Replace everything in link up to .php with $php_self
						// print '<div>$filter_config</div>'; print_r($filter_config); print '<hr>';
						// print '<div>php_self: ' . $php_self . '</div>';
						foreach ($filter_config as $k => $v) {
							if (is_array($v) && $v['link']) {
								if ( ($pos = strpos($v['link'], $ext)) !== false) {
									// print '<div>SUBSTR: ' . substr($v['link'], ($pos + strlen($ext))) . '</div>';
									$filter_config[$k]['link'] = $php_self . substr($v['link'], ($pos + strlen($ext)));
								}
							}
						}	
						// print '<div>$filter_config</div>'; print_r($filter_config); print '<hr>';
					}
					$filter_info['type'] = 'sql';
					$filter_info['options'] = $filter_config;
				}
				
			} elseif (strstr($filter_config[0], '__table(')) {
				$filter_info['type'] = 'dropdown';
				$temp =  explode('.', str_replace(array('__table(', ')__'), '', $filter_config[0]) );
				if (is_array($temp) && count($temp) > 1) {
					$filter_info['module'] = $temp[0];
					$filter_info['table'] = $temp[1];
					$filter_info['item_field'] = ( (isset($filter_config['item_field']) && $filter_config['item_field']) ? $filter_config['item_field'] : false);
					if (!$temp[2]) {
						$filter_info['name'] = 'name';
					} else {
						$filter_info['name'] = $temp[2];
					}
					$dd =& $this->getDataDefinition();
					
					
					$filter_info['in_data_option'] = $dd->fieldExists($filter_info['table'] . '_id');
					
					
					$dropdown = getWebMATObject(
						'WebMATDropdown', 
						$filter_info['module'], // module
						array(
							'module' => $filter_info['module'], 
							'table' => $filter_info['table'], 
							'field_name' =>$filter_info['name']
						)
					);
					$dropdown_values = $dropdown->getValues(' > ');
					foreach ($dropdown_values as $k => $v) {
						$filter_info['options'][$k] = array(
						'label' => $v, 
						'link'  => $this->output->updateQueryString(array('filter' => 'f' . $k))
						);
					}
					unset($dropdown);
				}
			}
			
			$form_values =& $this->getFormValues();
			if ($form_values['filter']) {
				$SERVICES->preferences->setUserPref($this->getPrefPrefix('Filter'), $form_values['filter']);
				$filter_value = $form_values['filter'];
			} else {
				// Phil - 20050209 - Optimised if block - only query prefs once
				if (!($filter_value = $SERVICES->preferences->getUserPref($this->getPrefPrefix('Filter'), 0))) {
					$filter_value = 'fnone';
				}
			}
			$filter_info['value'] = ltrim($filter_value, 'f');
			
			$this->filter_info = $filter_info;
			// Phil 20060925 Added replacePlaceholders
			if (is_array($this->filter_info['options'])) {
				foreach ($this->filter_info['options'] as $k => $v) {
					if (isset($v['where'])) {
						$this->filter_info['options'][$k]['where'] = $this->db_functions->replacePlaceholders($v['where']);
					}
				}
			}
		} else {
			$this->filter_info = false;
		}
	}
	
	/**
	 * return the filter info array
	 * 
	 * @return array an array of information for the filter
	 */
	function getConfigFilterInfo($key = false) {
		if ($key) {
			return $this->filter_info[$key];
		}
		return $this->filter_info;
	}
	
	/**
	 * sets the filter
	 */
	function setDbFunctionsConfigFilterInfo() {
		$db_functions =& $this->getDbFunctions();
		$filter_info = $this->getConfigFilterInfo();
		$db_functions->setConfigFilterInfo($filter_info);
		$this->filter = $filter_info['value'];
	}
	
	/**
	 * returns the filter value
	 * 
	 * @return integer the filter value
	 */
	function getFilter() {
		return $this->filter;
	}
	
	/**
	 * sets the search filter 
	 */
	function setSearchFilter() {
		global $SERVICES;
		$filter_info = $this->getConfigFilterInfo();
		$form_values =& $this->getFormValues();		
		// print '<div>setSearchFilter form_values:</div>'; print_r($form_values); print '<hr>';
		if ($this->getSearchFilterActive()) {
			if ($form_values['item'] || $form_values['clear']) {
				if ($form_values['clear']) {
					$SERVICES->preferences->setUserPref($this->getPrefPrefix('search_filter_values'), serialize(array()));
				} else {
					$SERVICES->preferences->setUserPref($this->getPrefPrefix('search_filter_values'), serialize($form_values['item']));
				}
			} else {
				$item = $SERVICES->preferences->getUserPref($this->getPrefPrefix('search_filter_values'), 0);
				// print '<div>setSearchFilter: item:</div>'; print_r($item); print '<hr>';
				if (!$item && ($filter_info['type'] == 'dropdown') && $filter_info['in_data_option']) {
					if ($filter_info['value'] != 'none' && !$form_values['item']) {
						$form_values['item']['_options_' . $filter_info['table'] . '_id'] = 'dropdown';
						$form_values['item'][$filter_info['table'] . '_id'] = array($filter_info['value']);
						$form_values['item'][$filter_info['table'] . '_id'] = $form_values['item'][$filter_info['table'] . '_id'];
					}
					$SERVICES->preferences->setUserPref($this->getPrefPrefix('search_filter_values'), serialize($form_values['item']));
				}
			}
			
			if ($filter_info['type'] == 'dropdown' && $filter_info['in_data_option']) {
				$db_functions =& $this->getDbFunctions();
				$db_functions->disableFilter(true);
			}
			
			$search = unserialize($SERVICES->preferences->getUserPref($this->getPrefPrefix('search_filter_values')));
			// print '<div>setSearchFilter search (unserialized):</div>'; print_r($search); print '<hr>';
			$form_values['item'] = $search;
			// print '<div>setSearchFilter this->form_values:</div>'; print_r($this->form_values); print '<hr>';
			// $this->form_values['item'] = $this->arrayMergeReplace((array)$this->form_values['item'], $search); // Phil 20060927
			if (is_array($search)) {
				$output =& $this->getOutput();
				$dd =& $this->getDataDefinition();
				
				$fields = $dd->getDataStructureWithData($search);
				// print '<div>setSearchFilter fields:</div>'; print_r($fields); print '<hr>';
				$fields_old = $fields;
				foreach ($fields as $k => $v) {
					if ($v['type'] == 'date_time') { //set the date time field so that 
						$fields[$k]['type'] = 'integer_date';
						// this is a rough validation of our own for the date, just
						// quickly check if what we have is remoetly close to a date
						// MCR 20060505 added  && $fields[$k]['value']   to prevent warning message
						// Phil 20060927 added is_int check
						if (isset($fields[$k]['value']) && $fields[$k]['value'] && (strtotime($fields[$k]['value']) < 0)) {
							// print '<div>unsetting ' . $fields[$k]['field_key'] . ' value: ' . $fields[$k]['value'] . '</div>';
							bcrm_unset($fields[$k], 'value');
						}
					}
					
					// if we're a checkbox, and our value is blank '', the validateFields
					// function will resolve it to 0, which we do not want, so here we
					// explicitly set it to -1, this will tie in with the stuff happening
					// down near line 570
					if (($v['type'] == 'checkbox') || ($v['type'] == 'dropdown')){
						if (is_array($v['value'])) {
							foreach ($v['value'] as $kk => $vv) {
								if ($vv == '') {
									$fields[$k]['value'][$kk] = -1;
								}
							}
						} else if ($v['value'] == ''){
							$fields[$k]['value'] = -1;
						}
					}
					
					if ($v['notnull'] == 1) {
						$fields[$k]['notnull'] = 0;
					}
				}

				$output->validateFields($search, $fields, $error_msg);
				$form_values['item'] = $search; //added to set the value to the correct option
				$this->form_values['item'] = $search; // Phil 20060927
				// print '<div>form_values set to search:</div>'; print_r($search); print '<hr>';
				foreach ($fields_old as $k => $v) {
				// foreach ($fields as $k => $v) {
					if ($v['type'] == 'date_time') {
						// print '<div>setSearchFilter field: ' . $k . ' [' . $form_values['item'][$v['field_key']] . ']</div>'; print_r($v); print '<hr>';
						global $SERVICES;
						$SERVICES->getService('DateLib');
						if ($form_values['item'][$v['field_key']] && is_int($form_values['item'][$v['field_key']])) { // Phil 20060927
							$form_values['item'][$v['field_key']] = $SERVICES->datelib->getMysqlDateFromTimestamp($form_values['item'][$v['field_key']]);
							// print '<div>changed date field to mysql format ' . $form_values['item'][$v['field_key']] . '</div>';
						}
					}
					if ($v['type'] == 'checkbox' && is_array($form_values['item'][$v['field_key']]) ) {
						$form_values['item'][$v['field_key']] = implode(",", $form_values['item'][$v['field_key']]);
					}
					if ($v['type'] == 'percentage') {
						$form_values['item'][$v['field_key']] = $form_values['item'][$v['field_key']]*100;
					}
					// if ($v['type'] == 'date') {
						// print '<div>1)setSearchFilter field: ' . $k . '</div>'; print_r($v); print '<hr>';
					// }
				}
				// tmp
				// $form_values['item']['created'] = '2006-11-09';
				$this->form_values['item'] = $form_values['item']; // Phil 20060927
				// $SERVICES->preferences->setUserPref($this->getPrefPrefix('search_filter_values'), serialize($form_values['item']));// Phil 20060927
				// print '<div>setSearchFilter form_values[item]:</div>'; print_r($form_values['item']); print '<hr>';
				// print '<div>$$ setSearchFilter search:</div>'; print_r($search); print '<hr>';
				
				foreach ($search as $k => $option) {
					if (substr($k, 0, 9) == '_options_' && $option != 'no_search') {
						$search_dtf = array();
						$field = substr($k, 9);
						//Syed 20050628 check for a string that contains codes 
						if (!is_array($search[$field]) && !get_magic_quotes_gpc()) {
							$search_value = addslashes($search[$field]);
						} else {
							$search_value = $search[$field];
						}
						
						if ($search_value) {
							// TODO: Phil 20060615 remove?
							// if (is_array($search_value)) {
								// // print '<div>## search_value:</div>'; print_r($search_value); print '<hr>';
								// $before_search_value = $search_value['b']; // TODO: just testing
								// $after_search_value = $search_value['a'];	
							// } else {
								// $before_search_value = $search_value;
							// }
							
							switch ($option) {
								case '>':
								case '<':
									$search_dtf = array($field => array('where' => $field . ' ' . $option . ' ' . $search_value));
								break;
								case 'LIKE':
									$search_dtf = array($field => array('where' => $field . ' LIKE "' . $search_value . '"'));
								break;
								case 'CONTAINS':
									$search_dtf = array($field => array('where' => $field . ' LIKE "%' . $search_value . '%"'));
								break;
								// case 'BETWEEN': // TODO: Phil 20060615
									// print '<div>search_value: ' . $search_value . '</div>';
									// print '<div>before_search_value: ' . $before_search_value . '</div>';
									// Before and After
									// print '<div>before: ' . $before_search_value . ', after: ' . $after_search_value . '</div>';
									// $search_dtf = array($field => array('where' => '((' . $field . ' < "' . $search_value . '") AND (' . $field . ' > "' . $search_value . '"))'); 
									// **** // $search_dtf = array($field => array('where' => '( (' . $field . ' < "' . $before_search_value . '") AND (' . $field . ' > "' . $after_search_value . '") )'));
									// $search_dtf = array($field => array('where' => $field . ' > "' . $search_value . '"')); // After
								// break;
								case 'BEFORE':
									$search_dtf = array($field => array('where' => $field . ' < "' . $search_value . '"'));
								break;
								case 'AFTER':
									$search_dtf = array($field => array('where' => $field . ' > "' . $search_value . '"'));
								break;
								
								case 'checkbox':
								case 'dropdown':
									// this ties in with stuff up near line 505
									// k, here we need to do some checks, if the array has a key who's
									// value is just an empty string or -1, then we unset the key.
									// because it's value is blank (obviously), and as such we don't 
									// want to actually search on it. *NB* only works if there's no
									// actual valid key with -1 (theoretically should never happen)
									if (is_array($search_value)) {
										foreach ($search_value as $kk => $vv) {
											if (($vv == '') || ($vv == -1)) {
												bcrm_unset($search_value, $kk);
											}
										}
										// after all that, if the array is empty, just delete it
										if (count($search_value) <= 0) {
											unset($search_value);
										}
									}

									// added quotes in for search_value for string keys.
									if (is_array($search_value) && ($search_text = "'". implode("','", $search_value) ."'")) {
										$search_dtf = array($field => array('where' => $field . ' IN (' . $search_text . ') '));
									}
								break;
								case 'multi_select':
									if ($search_value) {
										foreach (explode(',', $search_value) as $kk => $vv) {
											 $temp_values[] = $field . ' LIKE "%' . $vv . '%"';
										}
										$search_dtf = array($field => array('where' => '('. implode(" OR ", $temp_values) . ')'));
									}
								break;
								default :
									if ($fields_old[$field]['type'] == 'date_time') {
										// Phil 20060427 was not showing correct date when user if different timezone to server
										$search_value = $SERVICES->offsetTimeZoneForWrite($search_value);
										$where = $field . ' > ' . $search_value . ' AND ' . $field . ' < ' . ($search_value + 86400);
										$search_dtf = array($field => array('where' => $where));
									} else {
										$search_dtf = array($field => $search_value);
									}
								break;
							}
							if (is_array($search_dtf)) {
								$this->setDesignTimeFilter($search_dtf);
							}
						}
					}
				}
			}
			$this->search_filter_items = $search;
		} else {
			$this->search_filter_items = false;
		}
	}
	
	/**
	 * returns the search filter items
	 * 
	 * @return array the array of search filter items
	 */
	function getSearchFilter() {
		return $this->search_filter_items;
	}
	
	/**
	 * sets and enables the ids filter
	 */
	function setIdsFilter() {
		global $SERVICES;
		$form_values =& $this->getFormValues();
		if (is_array($form_values['ids_session'])) {
			$session_ids = $form_values['ids_session'];
		} else {
			if ($form_values['ids_session'] == 'off') {
				$session_ids = 'off';
			} else {
				$session_ids = $_SESSION[$form_values['ids_session']];
				bcrm_unset($_SESSION, $form_values['ids_session']);
			}
		}
		if ($session_ids) {
			$SERVICES->preferences->setUserPref($this->getPrefPrefix('select_ids'), serialize($session_ids));
		}
		$this->ids_filter = unserialize( $SERVICES->preferences->getUserPref($this->getPrefPrefix('select_ids')) );
		if (is_array($this->ids_filter) && count($this->ids_filter)) {
			$this->setDesignTimeFilter(
				array(
					'id' => array(
						'where' => $this->getTableName() . '.id IN (' . implode(',', $this->ids_filter) . ')'
					)
				)
			);
		}
	}
	
	/**
	 * returns an array of ids
	 * 
	 * @return array an array of ids or an blank array
	 */
	function getIdsFilter() {
		if (is_array($this->ids_filter)) {
			return $this->ids_filter;
		}
		return array();
	}
	
	/**
	 * adds the values from the search filter to the header row.
	 * 
	 * @return string the filter row
	 */
	function getSearchFilterTableLabels($table_labels = array()) {
		if ($this->getSearchFilterActive()) {
			$form_values =& $this->getFormValues();
			// print '<div>getSearchFilterTableLabels form_values:</div>'; print_r($form_values); print '<hr>';
			if ($item = $form_values['item']) {
				$output =& $this->getOutput();
				$dd =& $this->getDataDefinition();
				$fields = $dd->getDataStructureWithData($item);
				//$output->validateFields($item, $fields, $error_msg);
				foreach ($table_labels as $k => $v) {
					if ($item[$v['field_key']]) {
						$table_labels[$k]['value'] = $item[$v['field_key']];
						$table_labels[$k]['_search_option_value'] = $form_values['item']['_options_' . $v['field_key']];
					}
				}
			}
		}
		// print '<pre>' . print_r($table_labels, true) . '</pre>';
		return $table_labels;
	}
	
	/**
	 * Returns the search filter on/off Link
	 * 
	 * @param boolean $incl_adv Include the Advanced search link if it exists
	 * @return string the search filter on/off link
	 */
	function getSearchFilterLink($incl_adv = true) {
		// Phil 20060701 Added $incl_adv param
		if ($this->hasFilterableFields()) {
			global $L, $SERVICES;
			$php_self = $SERVICES->getGlobal('php_self');
			$output =& $this->getOutput();
			//Syed 20050725  | modified the search filter link 
			if ($this->getSearchFilterActive()) {
				// Phil 20060602 Turn off search filter, adv filter, bookmark, custom_page
				$new_query_str = $output->updateQueryString(
					array(
						'search_filter' => 'off', 
						'adv_filter' => '', 
						'bookmark' => '', 
						'custom_page' => '0', 
						'clear' => 0
					)
				);
				
				$link .= '<strong>' . $L['labelSearchFilterOn'] . '</strong>' . 
					$this->getMenuSeparatorOpen() . 
					$output->getLinkHref( 
						array(
							'href' => $php_self . $new_query_str,
							'text' => $L['labelSearchFilterTurnOff']
						) 
					) . 
					$this->getMenuSeparatorClose();	
			} else {
				
				$link .= '<strong>' . $L['labelSearchFilterOff'] . '</strong>' . 
					$this->getMenuSeparatorOpen() . 
					$output->getLinkHref(
						array(
							'href' => $php_self . $output->updateQueryString(array('search_filter' => 'on')) ,
							'text' => $L['labelSearchFilterTurnOn']
						) 
					) .  
					$this->getMenuSeparatorClose();
			}
			
			if ($incl_adv && ($advanced_filter_link = $SERVICES->config->lookup($this->db_functions->getFilesetConfigPath() . 'functions.adv_filter')) ) {
				$link .= ' ' . $this->output->getLinkHref( array('href' => $advanced_filter_link, 'text' => $L['labelAdvancedSearchFilter']) ) ;
			}
			
			return $link;
		}
		return false;
	}
	
	/**
	 * returns the show/hide link
	 * @author Syed
	 */
	function getSearchFilterShowHideLink(){
		global $L;
		
		$link = $this->getMenuSeparator();
		if ($this->output->show_filter_form && $this->getSearchFilterActive()) {
			$link .= $this->output->getLinkHref(array(
				'tag_extras' =>' onClick="javascript: showSearchForm(this,\''.$L['labelSearchFilterShow'].'\',\''.$L['labelSearchFilterHide'].'\');" ',
				'href' => 'javascript: ;',
				'text' => $L['labelSearchFilterHide']));
		} else {
				$link .= $this->output->getLinkHref(array(
				'tag_extras' =>' onClick="javascript: showSearchForm(this,\''.$L['labelSearchFilterShow'].'\',\''.$L['labelSearchFilterHide'].'\');" ',
				'href' => 'javascript: ;',
				'text' => $L['labelSearchFilterShow']))  ;
		}
		return $link;
	}
	
	/**
	 * Gets the config filters
	 *
	 * @param string $filter the value passed from the filter
	 * @return array an array of config filters
	 */
	function getConfigFilters($filter = false) {
		// print '<div>getConfigFilters</div>'; print_r($filter); print '<hr>';
		$config_filters = array();
		if ($filter_info = $this->getConfigFilterInfo()) {
			$config_filters = array(
				'options' => $filter_info['options'], 
				'module' => $this->module_name, 
				'value' => $filter_info['value'], 
				'no_filter_link' => $filter_info['no_filter_link'], 
				'use_select' => $filter_info['use_select']
			);
			if (isset($config_filters['options']) && is_array($config_filters['options'])) {
				global $L, $SERVICES;
				$lang =& $SERVICES->getService('Language');
				foreach ($config_filters['options'] as $k => $v) {
					if (isset($config_filters['options'][$k]['label'])) {
						$config_filters['options'][$k]['label'] = $lang->getLanguageText($config_filters['options'][$k]['label'], $this->module_name);
					}
				}
			}
		}
		return $config_filters;
	}
	
	/**
	 * @author Phil Calder
	 */
	function getContactFilterContent() {
		$content = array();
		if ($contact_id = $this->getContactFilter()) {
			global $L, $SERVICES;
			$this_page = $SERVICES->getGlobal('php_self');
			$output =& $this->getOutput();
			$content['label'] = $L['labelFilterContact'];
			$content['contactLink']  =  $output->getLinkContact($contact_id);
			$content['cancelLink']  =  $output->getLinkHref(
				array(
					'text' => $L['labelCancel'] , 
					'href' => $this_page . $output->updateQueryString(array('contact_id' => '')), 
					'tag' => 'contactActionLink'
				)
			);
			// Phil 20060117 - TODO - make this a contact selector
			// $content['changeLink']  =  $output->getLinkHref(
				// array(
					// 'text' => $L['labelChange'] , 
					// 'href' => $this_page . $output->updateQueryString(array('contact_id' => '')), 
					// 'tag' => 'contactActionLink'
				// )
			// );
			
			if ($this->isCompany($this->getContactFilter())) {
				
				if (isset($this->form_values['contact_depth'])) { 
					$contact_depth = $this->form_values['contact_depth'];
				} else {
					global $SERVICES;
					$prefs =& $SERVICES->getService('Preferences');
					$contact_depth = $prefs->getUserPref('contactShowCompanyDepth', '0');
				}
				
				if ($contact_depth) {
					$content['depth'] = $output->getLinkHref(
						array(
							'text' => $L['contactToggleDepthCompanyOnly'] , 
							'href' => $php_self . $output->updateQueryString(array('contact_depth' => '0')), 
							'tag' => 'contactActionLink'
						)
					);
				} else {
					$content['depth'] = $output->getLinkHref(
						array(
							'text' => $L['contactToggleDepthCompanyDepth'] , 
							'href' => $php_self . $output->updateQueryString(array('contact_depth' => '1')), 
							'tag' => 'contactActionLink'
						)
					);
				}
			} 
		}
		return $content;
	}
	
	/**
	 * @author Phil Calder
	 */
	function isCompany($contact_id) {
		global $SERVICES;
		if ($contact_id) {
			$SERVICES->getService('Modules');
			$contact_mod =& $SERVICES->modules->getModule('Contact');
			return $contact_mod->isCompany($contact_id);
		}
		return false;
	}
	
	/**
	 * @return array An array of available qfilters.
	 * @author Phil Calder
	 */
	function getQFilterContent() {
		if ($this->show_qfilters) {
			$qlinks = array();
			$qlinks = $this->getQfilters();
			if (count($qlinks)) {
				global $L;
				$qfilters = array(
					'label' => $L['labelFilterConstraints'], 
					'options' => $qlinks
				);
				return $qfilters;
			}
		}
		return false;
	}
	
	/**
	 * @see getConfigFilters
	 * @author Phil Calder
	 */
	function getConfigFiltersContent($filter) {
		return $this->getConfigFilters($filter);
	}
	
	/**
	 *
	 * @author Phil Calder
	 */
	function getIdsFilterContent() {
		$content = array();
		if ($id_count = count($this->getIdsFilter())) {
			$output =& $this->getOutput();
			
			//Phil 20060123 Create a seperate instance of db_functions 
			// so as not to affect the other filters (ie dont use =& )
			$temp_db_functions = $this->getDbFunctions(); 
			$temp_db_functions->resetDesignTimeFilters();
			$temp_db_functions->setDesignTimeFilters(
				array(
					'id' => array( // Phil 20060123
						'where' => $this->getTableName() . '.id IN (' . implode(',', $this->ids_filter) . ')'
					)
				)
			);
			
			$invisible = '';
			if ( ($invisible_count = $id_count - count($temp_db_functions->getItems())) ) {
				$invisible = ' (' . $L['labelFilterInVisable'] . ' ' . $invisible_count .')';
			} 
			
			global $SERVICES, $L;
			$php_self = $SERVICES->getGlobal('php_self');
			
			$content['label']          = $L['labelFilterIds'];
			$content['idsCount']       =  $id_count;
			$content['invisibleCount'] = $invisible;
			// Phil 20060117 - todo - make this a contact selector
			$content['cancelLink']     =  $output->getLinkHref(
				array(
					'text' => $L['labelCancel'] , 
					'href' => $php_self . $output->updateQueryString(array('ids_session' => 'off')), 
					// 'tag' => 'contactActionLink'
				)
			);
		}
		return $content;
	}
	
	/**
	 * @author Phil Calder
	 */
	function getQfilterLabel() {
		$type = $this->form_values['qfilter'];
		global $L;
		switch ($type) {
			case 'mine':
				return $L['labelQfilterMine'];
			break;
			case 'unseen':
				return $L['labelQfilterUnseen'];
			break;
			case 'alert':
				return $L['labelQfilterAlert'];
			break;
			default:
				return '';
			break;
		}
		return '';
	}
	
	/**
	 * @return array An array of qfilters (label, link, selected)
	 * 
	 * @author Phil Calder
	 */
	function getQfilters() {
		global $SERVICES,$L;
		$qfilters = array();
		$href = $SERVICES->getGlobal('php_self');
		// None - Add a Cancel/None option
		$new_query = $this->output->updateQueryString(array('qfilter' => ''));
		$qfilters['_none_'] = array(
			'label' => '', 
			'href'  => $href . $new_query
		);
		if ($this->form_values['qfilter'] == '') {
			$qfilters['_none_']['selected'] = true;
		}
		
		// alert
		if ($SERVICES->config->lookup($this->db_functions->getFilesetConfigPath() . 'alert')) {
			$new_query = $this->output->updateQueryString(array('qfilter' => 'alert'));
			$qfilters['alert'] = array(
				'label' => $L['labelQfilterAlert'], 
				'href'  => $href . $new_query
			);
			if ($this->form_values['qfilter'] == 'alert') { // $qfilter = $this->db_functions->getQueryType();
				$qfilters['alert']['selected'] = true;
			}	
		}
		
		// mine
		if ($SERVICES->config->lookup($this->db_functions->getFilesetConfigPath() . 'mine')) {
			$new_query = $this->output->updateQueryString(array('qfilter' => 'mine'));
			$qfilters['mine'] = array(
				'label' => $L['labelQfilterMine'], 
				'href'  => $href . $new_query
			);
			if ($this->form_values['qfilter'] == 'mine') {
				$qfilters['mine']['selected'] = true;
			}	
		}
		
		// unseen
		if ($SERVICES->config->lookup($this->db_functions->getFilesetConfigPath() . 'seen_table')) {
			$new_query = $this->output->updateQueryString(array('qfilter' => 'unseen'));
			$qfilters['unseen'] = array(
				'label' => $L['labelQfilterUnseen'], 
				'href'  => $href . $new_query
			);
			if ($this->form_values['qfilter'] == 'unseen') {
				$qfilters['unseen']['selected'] = true;
			}	
			
			//--
			// modified - [added Phil 20070521]
			// If we have a seen table, we can have a modified qfilter
			$new_query = $this->output->updateQueryString(array('qfilter' => 'modified'));
			$qfilters['modified'] = array(
				'label' => $L['labelQfilterModified'], 
				'href'  => $href . $new_query
			);
			if ($this->form_values['qfilter'] == 'modified') {
				$qfilters['modified']['selected'] = true;
			}
			
			//--
			
		}
		
		// None - Remove the blank option if there are no qfilters
		if (!count($qfilters)) {
			bcrm_unset($qfilters, '_none_');
		}
		
		return $qfilters;
	}
	
	/**
	 * @see getSearchFilterShowHideLink
	 */
	function getMenuSeparator() {
		return ' | ';
	}
	
	/**
	 * @see getSearchFilterLink
	 */
	function getMenuSeparatorOpen() {
		return ' [';
	}
	
	/**
	 * @see getSearchFilterLink
	 */
	function getMenuSeparatorClose() {
		return '] ';
	}
	
	// ===== methods deprecated below =====
	
	/**
	 * @deprecated 20060119
	 * @see getConfigFilterInfo
	 */
	function getFilterInfo($key = false) {
		return $this->getConfigFilterInfo($key);
	}
	
	/**
	 * @deprecated 20060119
	 * @see setDbFunctionsConfigFilterInfo
	 */
	function setFilter() {
		$this->setDbFunctionsConfigFilterInfo();
	}
	
	/**
	 * gets the filter list links
	 *
	 * @param string $filter the value passed from the filter
	 * @return string a string containing the filter menu items
	 * @deprecated 20050117
	 * @see getConfigFilters
	 */
	function getFilterLinks($filter = false) {
		return $this->getConfigFilters($filter);
	}
	
	/**
	 * returns the filter row
	 * 
	 * @param string $filter the filter info
	 * @return string the table row with the filter links
	 * @deprecated 20060119
	 */
	function getFilterRow($filter) {
		global $L;
		$output =& $this->getOutput();
		$filter_info = $this->getConfigFilterInfo();
		$content = '';
		if ($this->getSearchFilterActive() && $filter_info['in_data_option']) {
			$content .= $L['labelNewSearchFilter'] . " [" ;
			$content .= $this->getSearchFilterLink();
		} else {
			$content .= $this->getConfigFilterLinks($filter);
			if ($content) {
				$content .= '&nbsp; &nbsp; &nbsp; &nbsp;';
			} 
			$content .= $L['labelNewSearchFilter'] . " [" . $this->getSearchFilterLink();
			
		}
		$content .=  $this->getSearchFilterShowHideLink();
		$content .= "]";
		return $output->getMenuRow($content);  //MCR, 3
	} // deprecated
	
	
	/**
	 * Adds any extra filters that can be disabled, and show up in getFilterDisableRow
	 *
	 * @param string $label label to be used
	 * @param string $value value of the label
	 * @param array $cancelQuery href query string of what the 'cancel' will have (less php_self)
	 * @deprecated 20050117
	 */
	function addExtraFilterToDisableRow($label, $value, $cancelQuery){
		global $SERVICES, $L;
		$php_self = $SERVICES->getGlobal('php_self');
		$output =& $this->getOutput();
		
		// just a guess fix, do *not* rely on this.
		if (!is_array($cancelQuery)) {
			$cancelQuery = array($cancelQuery => '');
		}
		
		$href = $php_self . $output->updateQueryString($cancelQuery);
		$text = $label;
		// need a colon
		if (substr($label, -1, 1) != ':') {
			$text .= ':';
		}
		$text .= ' '. $value;
		$text .= '&nbsp;&nbsp;&nbsp;' . $output->getLinkHref(array('text' => $L['labelCancel'] , 'href' => $href, 'tag' => 'contactActionLink'));

		$this->extra_filters_disable_row[] = $text;
	} // deprecated
	
	/**
	 * gets the the filter select row
	 * @deprecated 20050117
	 */
	function getFilterDisableRow() {
		global $SERVICES, $L;
		$php_self = $SERVICES->getGlobal('php_self');
		$output =& $this->getOutput();
		// see if we already have some (we're assuming it's an array)
		if (count($this->extra_filters_disable_row) > 0) {
			$links = $this->extra_filters_disable_row;
		} else {
			$links = array();
		}
		if ($contact_id = $this->getContactFilter()) {
			$text = $L['labelFilterContact'] . ' ';
			$text .= $output->getLinkContact($contact_id);
			$href = $php_self . $output->updateQueryString(array('contact_id' => ''));
			$text .= '&nbsp;&nbsp;&nbsp;' . $output->getLinkHref(array('text' => $L['labelCancel'] , 'href' => $href, 'tag' => 'contactActionLink'));
			$links[] = $text;
		}
		if ($id_count = count($this->getIdsFilter())) {
			//Phil 20051124 Used to create a seperate instance of db_functions 
			// so as not to affect the other filters
			// Now we use the same object
			$temp_db_functions =& $this->getDbFunctions(); 
			$temp_db_functions->resetDesignTimeFilters();
			$temp_db_functions->setDesignTimeFilters(array('__special__' => array('where' => $this->getTableName() . '.id IN (' . implode(',', $this->ids_filter) . ')')));
			if ( ($invisible_count = $id_count - count($temp_db_functions->getItems())) ) {
				$invisible = ' (' . $L['labelFilterInVisable'] . ' ' . $invisible_count .')';
			} else {
				$invisible = '';
			}
			$href = $php_self . $output->updateQueryString(array('ids_session' => 'off'));
			$text = $L['labelFilterIds'] . ' ';
			$text .= $id_count;
			$text .= $invisible;  //mcr 20060130 was $invisable;
			$text .= '&nbsp;&nbsp;&nbsp;' . $output->getLinkHref(array('text' => $L['labelCancel'] , 'href' => $href, 'tag' => 'contactActionLink'));
			$links[] = $text;
		}
		
		//Syed 20050706 enables qfilter
		$qfilter = $this->db_functions->getQueryType();
		if ($qfilter) { // Phil 20050816 - extending the work Syed started
			$text = '';
			switch ($qfilter) {
				case 'alert': 
					$text = $L['labelFilterAlert'];
					break;
				case 'mine': 
					$text = $L['labelFilterMine'];
					break;
				case 'unseen': 
					$text = $L['labelFilterUnseen'];
					break;
				case 'modified': 
					$text = $L['labelFilterModified'];
					break;
				default:
				break;
			}
			if (strlen($text)) {
				$href = $php_self . $output->updateQueryString(array('qfilter' => ''));
				$text .= ' &nbsp;&nbsp;&nbsp;' . $output->getLinkHref(array('text' => $L['labelCancel'] , 'href' => $href, 'tag' => 'contactActionLink'));
				$links[] = $text;
			}
		}
		//syed 20050822 | modified to display cancel  and  qfilters links
		if ($link_count = count($links)) {
			if ($link_count < 2) {
				//$align = ' align="left" ';
				$content = $L['labelFilteredOn'] . ' ' . implode(', ', $links) . '&nbsp;&nbsp;&nbsp;';
			} else {
				$content = $L['labelFilteredOn'] .':';
				foreach ($links AS $filter_link) {
					$content .= '<br> &nbsp;&nbsp;&nbsp; ' . $filter_link . '&nbsp;&nbsp;&nbsp;';
				}
			}
		}
		if ($this->show_qfilters) {
			$qlinks = array();
			$qlinks = $this->getQfilterLinks();
			$content .= implode($this->getMenuSeparator(),$qlinks);
		}
		return $this->output->getTextRow($content, 2, 'lMainMenuCell');
	} // deprecated
	
	/**
	 * returns the qfilter row
	 * @return string the table row with the qfilter links
	 * @author Syed
	 * @deprecated 20060117
	 */
	function getQfilterLinks(){
		global $SERVICES,$L;
		$links = array();
		$href = $SERVICES->getGlobal('php_self');
		if ($SERVICES->config->lookup($this->db_functions->getFilesetConfigPath() . 'alert')) {
			if ($this->form_values['qfilter'] == 'alert') {
				$links[] = $L['labelQfilterAlert'];
			} else {
				$new_query = $this->output->updateQueryString(array('qfilter' => 'alert'));
				$links[] = $this->output->getLinkHref(array('text' => $L['labelQfilterAlert'] , 'href' => $href . $new_query));
			}
		}
		if ($SERVICES->config->lookup($this->db_functions->getFilesetConfigPath() . 'mine')) {
			if ($this->form_values['qfilter'] == 'mine') {
				$links[] = $L['labelQfilterMine'];
			} else {
				$new_query = $this->output->updateQueryString(array('qfilter' => 'mine'));
				$links[] = $this->output->getLinkHref(array('text' => $L['labelQfilterMine'] , 'href' => $href . $new_query));
			}	
		}
		if ($SERVICES->config->lookup($this->db_functions->getFilesetConfigPath() . 'seen_table')) {
			if ($this->form_values['qfilter'] == 'unseen') {
				$links[] = $L['labelQfilterUnseen'];
			} else {
				$new_query = $this->output->updateQueryString(array('qfilter' => 'unseen'));
				$links[] = $this->output->getLinkHref(array('text' => $L['labelQfilterUnseen'] , 'href' => $href . $new_query));
			}
		}
		return $links;
	} // deprecated
}	
?>