<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
*/

includeWebMAT('edit.inc');

/**
 * Mass-edit class (Bulk Edit)
 * 
 * A extended version of the edit page that enables you to edit more then one
 * record at once.
 *
 * @package WebMAT
 * @author Matthew Corich
 */
class WebMATStandardMassEdit extends WebMATEdit {
	
	/**
	 * init function sets all the class variables
	 * 
	 * changed to add the function "setIdsVariable"
	 * 
	 * @param  string $module  the current module name
	 * @param  string $data_option  the current data option used for DataDictionary
	 * @param  array $post the post array
	 * @param  array $get the get array
	 * @param  string $table_name the name of the table the page is using
	 * @param  array $language_overwrite has the keys title and description
	 */
	function init($module, $data_option, $post, $get, $table_name = false, $language_overwrite = false) {
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);

		$this->setIdsVariable();
		
	}
	
	/**
	 * returns the WebMat type
	 */
	function getWebmatType() {
		return 'mass_edit';
	}
	
	/**
	 * Checks if for a delete or update runs the submit function
	 * 
	 * Sets the log action accordingly
	 */
	function run() {
	
		if (isset($this->form_values['submit']) || isset($this->form_values['delete'])) {
			if (isset($this->form_values['submit'])) {
				$this->db_functions->setLogAction('mass_update');
			} else {
				$this->db_functions->setLogAction('mass_delete');
			}
			$this->doSubmit();
		}
	}
	
	/**
	 * returns the title row array
	 * 
	 * Changed to remove some links
	 * 
	 * @return string a string containing the menu items
	 */
	function getTitleRowLinks() {
		$title_links = parent::getTitleRowLinks();
		$title_links['bookmark'] = 'off';
		return $title_links;
	}
	
	/** 
	 * sets the $ids class variable
	 */
	function setIdsVariable() {
		if ($this->form_values['ids']) {
			$ids_session = $this->form_values['ids'];
		} else {
			$ids_session = $this->form_values['delete'];
		}
		
		$this->ids = $_SESSION[$ids_session];
	}
	
	/** 
	 * returns the $ids class variable
	 */
	function getIdsVariable() {
		return $this->ids;
	} 
	
	/**
	 * removes the ids that are not editable
	 */
	function doSubmitPermissions() {
		global $L, $SERVICES;
		if (
			is_array($this->ids) 
			&& ($this->getDisplayByGroupFlag() == 1) // Phil 20051005
		) { 
			foreach ($this->ids as $k => $id) {
				if (!$this->db_functions->isEditable($id)) {
					bcrm_unset($this->ids, $k);
				}
			}
		}
	}
	
	/**
	 * deletes the item.
	 */
	function doSubmitDelete() {
		foreach ($this->ids as $id) {
			$this->delete = $id;
			parent::doSubmitDelete();
		}
	}
	
	
	/**
	 * Overridden to handle checkboxes with a false value
	 */
	function doSubmitBegin() {
		$item = $this->item;

		//Syed 20050831 | includes fields having value as false of type checkbox 
		foreach ($this->dd->data_dictionary as $k => $v) {
			if (isset($item['_changed_' . $k]) && !isset($item[$k])) {
				$this->item[$k] = 0;
			}	
		}

		foreach ($this->item as $k => $v) {
			if (!$item['_changed_' . $k]) {
				bcrm_unset($this->item, $k);
			}
		}
	}
	
	/**
	 * Sets data dictionary include fields based on the item parameter keys
	 * 
	 * @param array $item the item array
	 * @return array containing the item and the error message if an error message is present
	 */
	function validateFields(& $item) {
		$this->dd->setDictionaryIncludeFields(array_keys($item));
		return parent::validateFields($item);
	}
	
	
	/**
	 * processes the item and returns the id
	 * 
	 */
	function doSubmitProcess() {
		
		if (is_array($this->ids)) {
			foreach ($this->ids as $id) {
				$this->item['id'] = $id;
				parent::doSubmitProcess();
			}
		}
	}
	
	/**
	 * gets the form fields
	 * 
	 * @param array $item the item array
	 * @param array $error_msg the error message array
	 * @return string and table row for the form
	 */
	function getFieldRows($item, $error_msg) {
		$dd_item = $this->dd->getDataStructureWithData($item, $this->getPriorityEdit());
		$dd_item = $this->handleMetaFields($dd_item, $item['id']);
		$dd_item = $this->handleUserFields($dd_item, $item['id']);
		$dd_item = $this->handleOtherFields($dd_item, $item['id']);
		$content = '';
		if ($error_msg) {
			$checked = 1;
		} else {
			$checked = 0;
		}
		if (is_array($dd_item)) {
			foreach ($dd_item as $k => $v) {
				$field_row = false;
				$checkbox = false;
				if ($v['field_key'] != 'id') {
					bcrm_unset($v, 'notnull');
					$this->output->setUncheckIngore(true);
					$v['tag_extras'] = ' onChange="javascript:uncheckIgnore(\'_changed_' . $v['field_key'] . '\')" ';
					// $field_row = $this->output->getHtmlFieldRow($v, $error_msg[$v['field_key']]); // Phil 20050915 - using wrapper function now (below)
					$checkbox = $this->output->getHtmlField(
						array('field_key' => '_changed_' . $v['field_key'], 'type' => 'checkbox', 'value' => $checked)
					);
					$field_row = $this->getFieldRow($v, $error_msg[$v['field_key']], $checkbox);
					
					// print '<div>*** ' . $field_row . ' ===</div>';
					// Phil 20060223 removed str_replace - checkbox added above
					// $content .= str_replace('nowrap><b>', 'nowrap>' . $checkbox . '<b>', $field_row);
					$content .= $field_row;
				}
			}
		}
		return $content;
	}
	
	/**
	 * Wrapper function for {@link Output::getHtmlFieldRow}
	 * @param mixed $checkbox Optional, default false. Can be a string checkbox, passed to {@link Output::getHtmlFieldRow}
	 * @author Phil Calder
	 */
	function getFieldRow($field, $error_msg, $checkbox = false) {
		return $this->output->getHtmlFieldRow($field, $error_msg, $checkbox);
	}
	
	function getDescriptionContent() {
		global $L;
		$ids = $this->getIdsVariable();
		$description = $L['labelMassEditItems'] . ' ' . count($ids);
		
		// $content = $this->output->getTextRow($description_text);
		// $content .= $this->output->getSpacerRow();
		// Phil 20060223
		$padding = true;
		$content = $this->output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>';
		$content .= $description;
		$content .= '</td>' . $this->output->getEndRow(true);
		return $content;
	}
	
	/**
	 * builds the title row for the page
	 * 
	 * @param string $prefix what to prefix the file name with (optional)
	 * @return string with the title
	 */
	function getTitleText($prefix = 'Title_') {
		return WebMATDisplay::getTitleText($prefix);
	}
	
	/**
	 * Overridden to remove next page row, permissions controls, and to return the mass edit submit
	 *
	 * @return string The mass edit submit row
	 * @see Output::getSubmitRowMassEdit
	 * @author Phil Calder
	 */
	function getSubmitRow() {
		// Phil 20060223 - There is no 'next page' row
		// Phil 20060223 - There are no permissions controls
		return $this->output->getSubmitRowMassEdit();
	}
	
	/**
	 * Disabled in this class
	 *
	 * @author Phil Calder, 20060223
	 */
	function addPermissionControls(&$id, &$item, &$error_msg) {
	}
	
	/**
	 * Disabled in this class
	 *
	 * @author Phil Calder, 20060223
	 */
	function getNextPageRow() {
		return false;
	}
	
	/**
	 * Disabled in this class
	 *
	 * @author Phil Calder
	 */
	function getExtraItemLinks() {
		return false;
	}
	
	/**
	 * Disabled in this class
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return boolean false
	 * @author Phil Calder
	 */
	function getDuplicateLink($id = false) {
		return false;
	}
	
	/**
	 * Disabled in this class
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return boolean false
	 * @author Phil Calder
	 */
	function getHideUnhideLink($id = false) {
		return false;
	}
	
	/**
	 * Disabled in this class
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return boolean false
	 * @author Phil Calder
	 */
	function getDeleteLink($id = false) {
		return false;
	}
	
	/**
	 * Disabled in this class
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return boolean false
	 * @author Phil Calder
	 */
	function getViewLink($id = false) {
		return false;
	}
	
	/**
	 * Uses this modules list page as the return page
	 *
	 * @param string $page The relative page (Unused in this class)
	 * @author Phil Calder
	 */
	function setReturnPage($page = false) {
		global $SERVICES;
		$SERVICES->getService('Config');
		$_SESSION['return_page'] = $SERVICES->config->lookup('modules.Front.modulelink.' . $this->module_name . '.link');
	}
	
	/**
	 * @deprecated version 4.150
	 * @see addPermissionControls
	 * @author Phil Calder, 20050712
	 */
	function addMakePermissionsPrivate(&$id, &$item, &$error_msg) {
		return false;
	}
	
	/**
	 * @deprecated version 4.150 see {@link getFieldRows}
	 */
	function getHtmlFieldRows($item, $error_msg) {
		return $this->getFieldRows($item, $error_msg);
	}
	
}
?>