<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('webmat.inc');

/**
 * Calls the correct set of database functions
 *
 * @author Phil Calder, Matthew Corich
 * @abstract
 * @package WebMAT
 */
class WebMATStandardModule extends WebMAT{
	/**
	 * @var string the module name
	 */
	var $module_name = '';
	/**
	 * @var array of the different database table objects
	 */
	var $table_objects_array = array();
	/**
	 * @var string the module name with a lower case first letter.
	 */
	var $module_name_lower = false;
	/** 
	 * @access private
	 */
	var $data_option_flag = '';
	
	/**
	* Constructor function sets all the class variables
	*/
	function WebMATStandardModule() {
		// $this->printDebug('WebMAT constructor for : ' . get_class($this) . '');
		global $data_definition;
		if (!$this->module_name) {
			// quickPage('<strong>WebMATModule ERROR:</strong> No module name set.');
			global $L;
			quickPage('<strong>' . $L['errModule']. ':</strong> ' . $L['errNoModuleName'] . '.');
			exit();
		}
	}
	
	
	/**
	* Gets a database table objects
	*
	* @param string $table_name the table name
	* @param boolean $refresh Requires the database table object to be recreated (Optional, default false)
	
	* @return object The database table object
	*/
	function &getDatabaseTable($table_name, $refresh = false) {
		global $SERVICES;
		// $this->printDebug(get_class($this) . '::getDatabaseTable ' . $table_name . ' for module ' . $this->module_name);
		if (!is_array($this->table_objects_array)) {
			$this->table_objects_array = array();
		}
		
		$table_name = str_replace($this->module_name . '__', '', $table_name);
		
		if (($refresh !== false) || !array_key_exists($table_name, $this->table_objects_array)) {
			global $SERVICES, $L;
			// $this->printDebug(get_class($this) . '::getDatabaseTable loading data definition...');
			// -- The data definition file can be used as well as storing the definition in the database
			// -- To do this, load the data_definition from the database inside the data definition file.
			$data_definition = array();
			
			$SERVICES->getService('Language');
			$SERVICES->language->getLanguage($this->module_name);
			
			$txt_data_def_loaded = false;
			if ($this->hasTextDataDefinition($table_name) ) { // Phil 20051027, 20060626
				// $this->printDebug(get_class($this) . '::getDatabaseTable looking for text data definition file(s)');
				if ( $data_definition_file = $this->getBeforeDatabaseTablesFileName() ) {
					if (file_exists($data_definition_file) && is_readable($data_definition_file)) {
						@require($data_definition_file);
						$txt_data_def_loaded = true;
						// $this->printDebug(get_class($this) . '::getDatabaseTable loaded before db tables data_definition_file: ' . $data_definition_file);
					}
				} else if (getExtDirExists('modules_custom')) {
					$custom_data_definition_file = $SERVICES->config->lookup('modules.custom_basedir') . '/' . $this->module_name . '/' . $table_name . '_data_definition.inc';
					if ($custom_data_definition_file && file_exists($custom_data_definition_file) && is_readable($custom_data_definition_file)) {
						@require($custom_data_definition_file);
						$txt_data_def_loaded = true;
						// $this->printDebug(get_class($this) . '::getDatabaseTable loaded custom_data_definition_file: ' . $custom_data_definition_file);
					}
				}
				
				//MCR
				if (!$txt_data_def_loaded) {
					$data_definition_file = $SERVICES->config->lookup('modules.basedir') . '/' . $this->module_name . '/' . $table_name . '_data_definition.inc';
					if ($data_definition_file && file_exists($data_definition_file) && is_readable($data_definition_file)) {
						@require($data_definition_file);
						$txt_data_def_loaded = true;
						// $this->printDebug(get_class($this) . '::getDatabaseTable loaded standard txt data_definition_file: ' . $data_definition_file);
					}
				}
				
				if (!$txt_data_def_loaded) {
					//MCR TODO print/log error message?
				}
			}
			
			if (!$txt_data_def_loaded) {
				$SERVICES->getService('DataDefinition');
				$data_definition = $SERVICES->datadefinition->getDataDefinition($this->module_name . '__' . $table_name);
				// $this->printDebug(get_class($this) . '::getDatabaseTable loaded data_definition from database');
			}
			
			// Phil 20050812
			$data_definition = (is_array($new_data_definition = $this->beforeDatabaseTables($data_definition)) ? $new_data_definition : $data_definition);
			// $this->printDebug(get_class($this) . '::getDatabaseTable after calling beforeDatabaseTables');
			
			// Phil 20070904 Added abaility to extend database class in modules_ext/[ModuleName]/inc/database_tables.inc
			// Extended class should be called:
			// Custom[TableName][ModuleName]DatabaseTable
			// Ideally this should extend the [TableName][ModuleName]DatabaseTable class
			$class_name = $table_name . $this->module_name .'DatabaseTable';
			$ext_class_name = 'Custom' . $class_name;
			
			if (class_exists($ext_class_name)) {
				$this->table_objects_array[$table_name] = new $ext_class_name($this->module_name, $data_definition);
			} else if (class_exists($class_name)) {
				$this->table_objects_array[$table_name] = new $class_name($this->module_name, $data_definition);
			} else {
				// $this->printDebug(get_class($this) . '::getDatabaseTable returning false (class does not exist)');
				return false;
			}
		}
		// $this->printDebug(get_class($this) . '::getDatabaseTable returning object ' . get_class($this->table_objects_array[$table_name]));
		return $this->table_objects_array[$table_name];
	}
	
	/**
	 * Empty function which can be overridden.
	 *
	 * Will be called just prior to creating the current database table class
	 *
	 * @param array $data_definition The data definition array from text file or database.
	 * @return mixed Either a (modified) data_definition array, or false
	 * @author Phil Calder
	 */
	function beforeDatabaseTables($data_definition) {
		return false;
	}
	
	/**
	 * Empty function which can be overridden.
	 * Can be used in {@link beforeDatabaseTables}
	 * @return mixed Either an absolute filename, or false
	 * @author Phil Calder
	 */
	function getBeforeDatabaseTablesFileName() {
		// print '<div>get before_database_tables_filename: ' . $this->before_database_tables_filename . '</div><hr>';
		return ( (isset($this->before_database_tables_filename) && $this->before_database_tables_filename) ? $this->before_database_tables_filename : false);
	}
	
	/**
	 * Empty function which can be overridden.
	 * Can be used in {@link beforeDatabaseTables}
	 * @return mixed Either an absolute filename, or false
	 * @author Phil Calder
	 */
	function setBeforeDatabaseTablesFileName($filename) {
		// print '<div>set before_database_tables_filename: ' . $filename . '</div><hr>';
		if ($filename && file_exists($filename) && is_readable($filename)) {
			$this->before_database_tables_filename = $filename;
		} else {
			$this->before_database_tables_filename = false;
		}
	}
	
	/**
	 * @param string $table The table (suffix) / file_set name (eg 'item')
	 * @return boolean true or false, whether this modules table has a text data definition
	 * @author Phil Calder
	 */
	function hasTextDataDefinition($table = 'item') {
		global $SERVICES;
		return $SERVICES->config->lookup('modules.' . $this->getModuleName() . '.file_set.' . $table . '.text_data_definition');
	}
	
	/**
	 * Cascading delete
	 * 
	 * Add tables to the cascading delete in the module.config
	 * 
	 * @param integer $contact_id the contact id 
	 * @deprecated version 4.130 see {@link contactCascadingDelete}
	 */
	function crmCascadingDelete($contact_id) {
		$this->contactCascadingDelete($contact_id);
	}
	
	/**
	 * Cascading delete
	 * 
	 * Add tables to the cascading delete in the module.config
	 * 
	 * @param integer $contact_id the contact id 
	 */
	function contactCascadingDelete($contact_id) {
		global $SERVICES;
		$cascading_delete_tables = $SERVICES->config->lookup('modules.Contact.cascading_delete.' . $this->module_name . '.tables');
		
		if ( is_array($cascading_delete_tables) ) {
			foreach ($cascading_delete_tables as $k => $v) {
				if ($db_functions =& $this->getDatabaseTable($v)) {
					if (method_exists($db_functions, 'contactCascadingDelete')) {
						$db_functions->contactCascadingDelete($contact_id);
					}
				}
			}
		}
	}
	
	/**
	 * Reassigns items to a different contact
	 * 
	 * @param integer $contact_id the old contact id
	 * @param integer $new_contact_id the new contact id
	 */
	function reassignItemForcontact($contact_id, $new_contact_id) {
		global $SERVICES;
		$reassign_tables = array();
		$reassign_tables = $SERVICES->config->lookup('modules.Contact.reassign.' . $this->module_name . '.tables');
		if ( is_array($reassign_tables) ) {
			foreach ($reassign_tables as $k => $v) {
				$db_functions = $this->getDatabaseTable($v);
				if (method_exists($db_functions, 'reassignItemForcontact')) {
					$db_functions->reassignItemForcontact($contact_id, $new_contact_id);
				}
			}
		}
	}
	
	
	/**
	 * Override for per module data cleanup functionality
	 * @author Phil Calder
	 */
	function cleanupData() {}
	
	/**
	 * Purges (soft) deleted items from the specified modules
	 *
	 * @param string $type The purge type to run. Options are: count, purge, report
	 * @return mixed Either false or an array with table names as keys and either a count, 
	 * number of deleted items, or boolean depending on $type parameter.
	 * @author Phil Calder
	 */
	function purgeDeleted($type = 'count') {
		if (!is_object($this->config)) {
			global $SERVICES;
			$this->config =& $SERVICES->getService('Config');
		}
		if (!is_array($tables = $this->config->lookup('modules.Admin.hard_delete.' . $this->getModuleName() . ''))) {	
			return false;
		}
		
		switch ($type) {
			case 'purge':
				$method_name = 'hardDeleteItems';
			break;
			case 'report':
				$method_name = 'hardDeleteItems';
			break;
			
			case 'count':
			default:
				$method_name = 'countDeleted';
			break;
		}
		
		if (!is_array($tables)) {
			return false;	
		}
		
		$exclude_tables = array('item_seen');; // Phil - functionality for seen tables not implemented yet.
		
		$results = array();
		foreach ($tables as $k => $v) {
			if (!in_array($v, $exclude_tables) && is_object($tbl_db_functions =& $this->getDatabaseTable($v))) {
				if (method_exists($tbl_db_functions, $method_name)) {
					$results[$v] = $tbl_db_functions->$method_name();
					// print '<div>' . $this->getModuleName() . ' ' . $v . ' ' . $method_name . ' returned ' . $retval . '</div>';
				}
			}
		}
		return $results;
	}
	
	/**
	 * sets the database functions object
	 */
	function setDbFunctions($refresh = false) {
		$this->db_functions =& $this->getDatabaseTable($this->table_name, $refresh);
	}
	
	/**
	 * Overrides parent functionality (because this object *IS* a module, it does not have a module)
	 * 
	 * @param string $module the module name
	 */
	function setModuleInfo($module) {
		global $SERVICES;
		$php_self = $SERVICES->getGlobal('php_self');
		$this->module_name_lower = $this->getModuleNameLower(); //gets the module name with a lower case starting letter
		$this->module_name = $module; //sets the module name
		$this->pref_name = $this->getModuleNameLower() . str_replace(array(dirname($php_self), '.php', '/'), '', $php_self);
	}
	
	/**
	 * Returns the module name
	 * 
	 * @return string $this->module_name
	 */
	function getModuleNameLower() {
		if (!$this->module_name_lower) {
			$this->module_name_lower = strtolower( substr($this->module_name, 0, 1) ) . substr($this->module_name, 1);
		}
		return $this->module_name_lower;
	}
	
	/**
	 * Returns the module name
	 * 
	 * @return string $this->module_name
	 */
	function getModuleName() {
		return $this->module_name;
	}
	
	/**
	 * @param string $data_option Usually the data option for the current page
	 * @author Phil Calder
	 */
	function setDataOptionFlag($data_option) {
		$this->data_option_flag = $data_option;
	}
	
	/**
	 * @author Phil Calder
	 * @return string The data option flag (Usually the data option for the current page)
	 * @see beforeDatabaseTables
	 */
	function getDataOptionFlag() {
		return $this->data_option_flag ;
	}
	
}
?>