<?php
/**
 * Class for displaying contact view content
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('search_pieces_view.inc');

/**
 * @package WebMAT
 * @author Phil Calder
 * @abstract
 */
class WebMATStandardSearch  extends WebMATSearchPieces {

	var $begin_table_arg = 'searchPieceTable'; // Phil 20050728, added to skins.config

	var $has_db_functions = false;

	// WebMATList functions
	function showCheckboxMenuRow(){ return false; }
	function showSearchFilterForm() { return false; }
	function getDescriptionContent() { return ''; }
	function getPagingRow() { return ''; }
	function beginSelectItemForm() { return ''; }
	function getCheckboxMenuRow() { return ''; }
	function getTitleRowLinks() { return ''; }
	function getSecondMenuRow() { return ''; }
	function showConfigFilters() { return false; }
	function showFilteredOnRow() { return false; }
	function getItemCheckbox() { return array(); }
	function getNewMenuContent() { return ''; }

	/**
	 * Need to call setShowContent to show by default
	 */
	function init($module, $data_option) {

		parent::init($module, $data_option);

		$this->setSortField(); // Phil 20050510 - Consistent with Dashboard. ets default order by (created DESC)

		$this->setShowContent(true);

		// Phil - Could change this depending on isStandardDisplay (?)
		$this->setTagHeaderRow('cpHeaderRow');  //css class for contact piece header row

		//$this->setContentDisplayMode(1);

	//	$this->addContactField();
	}

	/**
	 * Adds the contact field to the data definition (if applicable)
	 *
	 * Only for modules which have a contact field and when contact company depth is on
	 * @author Phil Calder
	 */
	function addContactField() {
		global $SERVICES;

		if (isset($this->db_functions)) {
			$prefs =& $SERVICES->getService('Preferences');
	
			if ($prefs->getUserPref('contactShowCompanyDepth', '0')) {
				$dd =& $this->getDataDefinition();
				$fields = $this->db_functions->getDataDefinitionFields();
				$contact_id_field = $this->db_functions->getContactIdField();
				if (isset($fields[$contact_id_field]) && $fields[$contact_id_field] ) { 
					$dd->setDictionaryIncludeField($fields[$contact_id_field], true);
				}
			}		
		}
	}

	/**
	 * Sets a search's "where" filter
	 * @param string $search_keyword The string that we are searching.
	 * @return bool If any fields were actually searched.
	 */
	function setPieceFilter($type = false, $search_keywords = '', $search_type = '') {
		// print '<div>' . get_class($this) . '::setPieceFilter<br>keywords: ' . $search_keywords . '<br>type: ' . $search_type . '<br>module: [' . $this->getModuleName() . ']</div>';
		// if no search words, no search
		if ($search_keywords == '') {
			return false;
		}

		// We only search text & long_text, so if a module has all ints, but none of either of these,
		// we need to know to not display anything, otherwise it'll be everything
		$search_fields = false;

		if (isset($this->db_functions) || (isset($this->mod) && is_object($this->mod))) {
			if (!isset($this->db_functions)) {
				$this->db_functions =& $this->mod->getDatabaseTable('item');
			}
			
			// Phil 20061109 Whether to include custom fields in where clause
			global $SERVICES;
			$prefs =& $SERVICES->getService('Preferences');
			$include_custom = $prefs->getUserPref('searchIncludeCustomFields');
			
			// reset
			$this->db_functions->resetDesignTimeFilters();

			// get all the fields
			$field_list = $this->db_functions->getDataDefinitionFields();

			$where = array();

			// now loop through and set a filter for the where clause.
			foreach ($field_list as $k => $field) {
				if (!$include_custom && $field['field_is_custom'] == 1) {
					// Phil 20061109 dont process this field
					// print '<div>SKIP field: ' . $field['field_key'] . '</div>'; print_r($field); print '<hr>';
				} else {
					// we can only search if it's a text, or long_text
					if (($field['type'] == 'text') || ($field['type'] == 'long_text') || ($field['type'] == 'email')) {
	
						if ($search_type == 'initial') {
							$query = array('where' => $field['field_key'] . " LIKE '" . $search_keywords . "%'");
						}
						else {
							$query = array('where' => $field['field_key'] . " LIKE '%" . $search_keywords . "%'");
						}
	
						// if this is a column of the given table, need to tack on the table name in front
						if ( in_array($k, $this->db_functions->columns) ) {
							// this is slightly somewhat duplicating stuff in 'getDesignTimeFiltersWhere' in webmat_database_table.inc
							// but it belongs nicely here, so that's the way it is. (only duplicated in spirit, not actual code)
							if (isset($field['field_is_custom']) && ($field['field_is_custom'] == 1)) { 
								$where[] = $this->db_functions->processDesignTimeFilter($field['field_key'], $query, "_custom");	
							} else {
								$where[] = $this->db_functions->processDesignTimeFilter($field['field_key'], $query);
							}
						}
						else if (substr($k, 0, 1) == '_') { // foreign fields
							$where[] = $query['where'];
						}
	
						$search_fields = true;
					}
				}
			}

			$where = '(' . implode(' OR ', $where) . ')';

			// only use a filter if we've got fields
			if ($search_fields) {
				$this->db_functions->setDesignTimeFilters( 
					array(
						'__special__' => array(
							'where' => $where
						)
					) 
				);
			}
		}
		// print '<div>where: ' . $where . '</div><hr>';
		return $search_fields;
	}


	/**
	 * Overridden - functionality disabled in contact view
	 * 
	 * @return string an empty string
	 * @author Phil Calder
	 * 20050517
	 */
	function getBookmarkLinkLink() {
		return '';
	}

	/**
	 * For use in a MySQL 'IN' query, eg IN ( id1 [, id2, id3])
	 *
	 * @param boolean $include_parent Whether to include the current contact in the returned string. Defaults to false.
	 * @return mixed false or a comma separated string of ids (possibly blank)
	 * @author Phil Calder
	 */
	function getSubContactIdsString($include_parent = false) {
		$sub_contacts = $this->getSubContactIds();
		if (is_array($sub_contacts)) {
			if ($include_parent) {
				$sub_contacts = $this->arrayMergeReplace((array)$this->getContactId(), $sub_contacts);	
			}
			return implode(', ', $sub_contacts);
		}
		return false;
	}
	
	/**
	 * Applies only to companies
	 *
	 * @return mixed false or an array of ids
	 * @author Phil Calder
	 */
	function getSubContactIds() {
		global $SERVICES;
		$SERVICES->getService('Modules');
		$contact =& $SERVICES->modules->getModule('Contact');
		$contact_id = $this->getContactId();
		return $contact->getSubContacts($contact_id);
	}
	
	/**
	 * @return array with 'field' and 'order' keys, used to build an order by clause
	 * @author Phil Calder
	 */
	function getDefaultSort() {
		if (isset($this->sort_field) && is_array($this->sort_field)) {
			return $this->sort_field;
		}
		return array(
			'field' => 'id', 
			'order' => 'DESC'
		);
	}

	/**
	 * Set to 'true'. Can be overridden.
	 */
	function showViewLinkOnly() {
		return true;
	}

	/**
	 * Instanciates the database functions object.
	 * This is over-rides the WebMATStandard version, because we don't want to print out
	 * an error. If there is no database table, we don't search the module, and keep
	 * moving along, no errors.
	 *
	 * @param boolean $refresh Optional (default false) parameter to force a new db object to be created
	 * @author Brenton Bartel
	 */
	function setDbFunctions($refresh = false) {
		if (!is_object($this->mod)) {
			$this->setModuleInfo($this->module_name);
		} 

		if (is_object($this->mod) && method_exists($this->mod, 'getDatabaseTable')) {
			$this->db_functions =& $this->mod->getDatabaseTable($this->table_name, $refresh);
			$this->has_db_functions = true;
		} else {
			$this->has_db_functions = false;
		}
	}

	/**
	 * returns an array of standard links
	 * 
	 * Will only show (at most) view link based on {@link showViewLinkOnly}
	 * @param integer $id the current item id
	 * @param array the item array
	 * @return array the array of standard links
	 */
	function getStandardLinks($id, $item) {
		if ($this->showViewLinkOnly()) {
			global $SERVICES;
			$links = array();
			
			if ($this->getFileSetHref('view')) {
				$links['view'] = $this->getViewLink($id);
			}
			
		} else {
			$links = parent::getStandardLinks($id, $item);
		}

		return $links;
	}
}

?>