<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Services
 */

/**
 * Output - Contains the code to handle client validation as well as form field display and validation
 *
 * If you are looking to write new BrowserCRM pages, you should look at the Template module files
 * for best exmaples of usage of this class.
 * The class now contains the code to handle client validation as well as
 * form field display and validation. The new methods are described below
 *
 *	1) {@link getHtmlFieldRow()}
 *		draws a table row with the $field['label'] in the first cell and the
 *		html field in the second. error_msg the variable to contain
 *		error messages you on unsucessful server validation
 *
 *	2) {@link getHtmlField()}
 *		Returns a form field based on the given array
 *		field_key, value, type (validation - can be
 *		text, long_text, hidden, integer, float, date, time, dropdown),
 *		other (see the documentation in the function header), width (in pixels - sets the style width),
 *		tag_extras (text to go verbatim inside the form field tag eg. 'onChange="do_stuff();"'),
 *
 *	3) {@link getStringField()}
 *		Returns the field based on the given array for displaying
 *
 *	4) {@link _setClientSideCheck()}
 *		Sets values in the property client_side_checks used for client validation. client_side_checks is
 *		indexed by field_key. The label is used in validation error messages. The set of values
 *		are of a submit error type currently non_empty, max, min and format. non_empty test
 *		against an empty string, format matches the value against a regular expression - test.
 *		max and min compares the value against the value 'test'. Each possible error may now have
 *		its own error message however non_empty error message is generated withing the
 *		_getClientValidationScript() function.
 *	5) {@link _getClientValidationScript()}
 *		This function is called as the page is drawn to include the javascript
 *		necessary to perform the client validation on your form. The script is generated based
 *		on the values sent to the client_side_checks property using the setClientSideCheck() function.
 *	6) {@link validateField()}
 *		TO DO. returns an error message
 *
 * @author Matthew Ross, Phil Calder, Peter Sharman
 * @package Services
 */
class Output extends Service {

	/**
	 * @access private
	 */
	var $head_block = '';
	
	/**
	 * @access private
	 */
	var $body_block = '';
	
	/**
	 * @access private
	 */
	var $title = '';
	
	/**
	 * @access private
	 */
	var $messages = '';
	
	/**
	 * @access private
	 */
	var $init_js = '';
	
	/**
	 * @access private
	 */
	var $page_type = '';
	
	/**
	 * @access private
	 */
	var $stylesheet = '';
	
	/**
	 * @access private
	 */
	var $row_flag = 0;
	
	/**
	 * @access private
	 */
	var $skin_config = array();
	
	/**
	 * @access private
	 */
	var $tag_content = array();

	/**
	 * @access private
	 */
	var $js_validate = 0;

	/**
	 * adds pieces.js file
	 * @access private
	 */
	var $js_pieces = false;
	
	/**
	 * @access private
	 */
	var $field_defaults;
	
	/**
	 * flag for whether to add xajax stuff
	 * @access private
	 */
	var $has_xajax = false;
	
	/**
	 * flag for whether to add ext stuff
	 * @access private
	 */
	var $has_ext = false;
	
	/**
	 * flag for whether to add tabs stuff
	 * @access private
	 */
	var $has_tabs = false;
	var $tabsets = array();
	var $tab_max_title_length = 20;
	var $ext_theme = 'gray'; //just default
	
	/**
	 * flag for whether to add tabs stuff
	 * @access private
	 */
	var $has_toolbar = false;
	var $toolbar_linkset = array();
	var $toolbar_linkset_right = array();  //icons eg back, refresh (will eventually be right)
	/**
	 * @var new flag to set style of link generated - string if false, array if true for ext toolbar
	 */
	var $toolbar_link_mode = false;
	/**
	 * @var $toolbar_link_increment Keep track of each toolbar item by assigning a unique number to it.
	 */
	var $toolbar_link_increment = 0;
	
	/**
	 * @var $ext_init_script  Script to add to the ext onReady function
	 */
	var $ext_onready_script = '';

	
	/**
	 * @access private
	 */
	var $has_calendar_button = 0;
	
	/**
	 * @access private
	 */
	var $has_pref_button = 0;
	
	/**
	 * @access private
	 */
	var $has_help_button = 0;
	
	/**
	 * @access private
	 */
	var $req_main_js = false;
	
	/**
	 * @access private
	 */
	var $req_list_js = false;
	
	/**
	 * @access private
	 */
	var $req_viewFrame_js = false;
	
	/**
	 * @access private
	 */
	var $req_config_js = false;
	
	/**
	 * @access private
	 */
	var $req_popup_menu_js = false;
	
	/**
	 * @access private
	 */
	var $is_child_iframe = false;
	
	/**
	 * set in setPagingLimits
	 * @access private
	 */
	var $paging_list_size = 20;
	
	/**
	 * total number of items, set in setPagingLimits
	 * @access private
	 */
	var $paging_items_count = 1;
	
	/**
	 * used for 'view_all'
	 * @access private
	 */
	var $paging_max_list_size = 1000;
	
	/**
	 * eg [1 2 3 4 5 6 7 8 9 10]
	 * @access private
	 */
	var $paging_num_page_links = 10;
	
	/**
	 * start at item index 0
	 * @access private
	 */
	var $paging_start_num = 0;
	
	/**
	 * The user pref for the paging row
	 * @access private
	 */
	var $paging_pref_name = '';      
	
	/**
	 * used for the multiedit for unchecking the ingore boxs
	 * @access private
	 */
	var $uncheck_ingore_js = false;
	
	/**
	 * array of errors to check for (currently only format and nonull)
	 * indexed by field name using javascript client valadiation
	 * @access private
	 */
	var $client_side_checks;
	
	/**
	 * @access private
	 */
	var $charset = 'iso-8859-1';
	
	/**
	 * @access private
	 */
	var $message = ''; //added 20040803 MCR
	
	/**
	 * @access private
	 */
	var $footer_block_switch = true;
	
	/**
	 * @access private
	 */
	var $footer_timer_switch = true;
	
	/**
	 * @access private
	 */
	var $footer_info_switch = false;
	
	/**
	 * @access private
	 */
	var $footer_notice = '';
	
	/**
	 * @access private
	 * @todo Investigate setting this to true for shipping
	 */
	var $whitespace_compression_switch = false;
	
	/**
	 * @access private
	 */
	var $padded_table_switch = true;
	
	/**
	 * @access private
	 */
	var $base_skin_file_path = '';  //as seen by the file system: path to /skins/_base
	
	/**
	 * @access private
	 */
	var $skin_file_path = '';  //as seen by the file system
	
	/**
	 * @access private
	 */
	var $skin_web_path = '';  //as seen by the browser
	
	/**
	 * @access private
	 */
	var $js_web_path = '';  //as seen by the browser
	
	/**
	 * @access private
	 */
	var $users_list = 0;
	
	/**
	 * @access private
	 */
	var $groups_list = 0;
	
	/**
	 * @access private
	 */
	var $default_link_target = ''; // see getLinkHref
	
	/**
	 * @access private
	 */
	var $show_contact_selector = true;
	
	/**
	 * @access private
	 */
	var $show_contact_popup = true;
	
	/**
	 * @access private
	 */
	var $no_trim = true;
	
	/**
	 * @access private
	 */
	var $no_trim_types;
	
	/**
	 * @access private
	 */
	var $master_cb_counter = 0;
	
	/**
	 * Holds if a module specific template exists (ex: using smarty).
	 * When returning variables, if $has_template is true, it will return it
	 * as a simply string, whereas if it's false, it'll be wrapped in table tr & td
	 * If there are multiparts, it will be return as an array, with proper keys
	 * @access private
	 */
	var $has_template = false;
	
	/**
	 * @access private
	 */
	var $tree_view = false;
	
	/**
	 * @var $search_filter_defaults array stores the search filter defaults
	 * MSC 20041115
	 * @access private
	 */
	var $search_filter_defaults = false;

	var $show_filter_form= true ; //Syed 20050728 show/hide the search filter form

	var $calendar_setup_js = '';

	/**
	 * Class Functions:
	 */

	/**
	 * Passes parameter to {@link init()}
	 *
	 * @param string $page_type can be 'main' or 'dashboard'
	 * @todo Parameter $page_type is being deprecated.
	 */
	function Output($page_type = 'main') {
		$this->init($page_type);
	}

	/**
	 * @param string $page_type can be 'main' or 'dashboard'
	 * @todo Will possibly add 'iframed', and other page types for pda, mobile phones etc
	 */
	function init($page_type = '') {
		global $SERVICES;
		// $this->printDebug('Output::init.begin');
		$SERVICES->getService('Preferences');
		$SERVICES->getService('Language');
		
		$this->setPagingListSize();
		$this->setPrintView(); // Phil 20070510
		$this->setSkinFilePath();
		$this->setSkinWebPath();
		$this->setSkinConfig();
		$this->setJsWebPath();
		$this->initTagContents();
		$this->setFieldDefaults();
		$this->setCharset();
		$this->setShowContactPopup(); // uses default value
		
		//xajax
		if ($this->has_xajax) {
			$this->initXajax();
		}
		
		if ($this->has_ext) {
			$this->initExt();
		}
		
		//default to have toolbar (implies has ext)
		$this->initToolbar();
		
		
		// The 'TemplateEngine' service *should* only be include if a template exists.
		// So if we have the service, we have a template, so use the variable to keep track.
		if ($SERVICES->hasService('TemplateEngine')) {
			$this->has_template = true;
		}
		$this->setNoTrimTypes(array('contact', 'email', 'url'));
		// $this->printDebug('Output::init.end');
	}

	function initXajax() {
		global $SERVICES;
		require_once($SERVICES->config->lookup('site.root') . '/external/xajax/xajax_core/xajax.inc.php');
		if (!isset($this->xajax) || !$this->xajax) {
			$this->xajax = new xajax();
			$this->has_xajax = true;
		}
	}
	
	function initExt() {
		global $SERVICES;
		$this->has_ext = true;
	}
	
	function initTabs($tabset) {
		$this->initExt();
		$this->has_tabs = true;
		$this->tabsets[$tabset] = array();
	}
	
	function initToolbar() {
		$this->initExt();
		$this->has_toolbar = true;
	}
	
	function addTab($tab, $tabset) {
		$this->tabsets[$tabset][] = $tab;
	}
	
	function setFirstTab($tab, $tabset) {
		$this->tabsets_active_tab[$tabset] = $tab;
	}
	
	function addOnReadyScript($script) {
		$this->ext_onready_script .= $script;
	}
	
	/**
	 * Sets the paging list size based on a user preference (defaults to 20)
	 * @author Phil Calder
	 */
	function setPagingListSize() {
		global $SERVICES;
		$pref_size = $SERVICES->preferences->getUserPref('frontDefaultListSize', 20);
		if ($pref_size) {
			if (strtolower($pref_size) == strtolower('All')) {
				$pref_size = intVal($this->paging_max_list_size);
			}
			$this->paging_list_size = $pref_size;
		}
	}

	/**
	 * Format a name
	 *
	 * @param string $first_name Optional
	 * @param string $last_name Optional
	 * @param string $company Optional
	 * @param string $title Optional
	 * @return string The formatted name
	 * @deprecated version 5
	 */
	function formatName($first_name='', $last_name='', $company='', $title='') {
		return false;
	}

	/**
	 * @param string $input The email to validate
	 * @return boolean If the input is a valid email.
	 * @author Phil Calder
	 */
	function isValidEmail($input) {
		if ($input) {
			$exp = "/^[a-zA-Z0-9_+\.\-]+@((?:\w+\.)*\w[\w-\.]{0,66})\.([a-zA-Z\-]{2,12}(?:\.[a-zA-Z\-]{2})?)$/i";
			if(preg_match($exp, $input)==1){
				return true;
			}
		}
		return false;
	}

	/**
	 * @return string The default URL prefix (eg 'http://')
	 * @author Phil Calder
	 */
	function getDefaultUrlPrefix() {
		return 'http://';
	}

	/** 
	 * @param string $input to check for url or relativeurl
	 * @return string
	 * @author syed 20050622
	 */
	function getUrl($input){
		if($this->isValidUrl($input) || $this->isRelativeUrl($input)){
			 return $input;
		}
		if($this->isValidUrl($this->getDefaultUrlPrefix() . '' . $input)){
			 return $this->getDefaultUrlPrefix() . '' . $input;
		}
		return $input;
	}

	/** 
	 * @param string $input The URL to validate
	 * @return boolean If the input is a valid URL.
	 * @author Phil Calder 20070416
	 */
	function isValidUrl($input){
		// syed 20050622
		// validates all possible protocols
		// $exp='/^([a-z0-9\.]+):\/\/[a-z0-9-]+(\.[a-z0-9-]+)*(:[0-9]+)?(\/.*)?$/i';
		// return preg_match($exp,$input);
		
		// Phil 20070416
		$url_regex = '/^';
		// $url_regex .='(([a-z0-9\.]+):\/\/)?([a-z0-9-]+(\.[a-z0-9-]+)+)?';
		$url_regex .='(([a-z0-9\.]+):\/\/)?([a-z0-9-]+(\.[a-z0-9-]+(.*))+)?';
		$url_regex .='$/i';
		
		return preg_match($url_regex,$input);
		
		
	}

	/**
	 * @param string $input The RelativeUrl to validate
	 * @return true If the input is a valid Relativeurl.
	 * @author syed 20050622
	 */
	function isRelativeUrl($input) {
		//it checks both files and directories for existance
		return file_exists($input);
	}

	/**
	 * Returns a list of users
	 *
	 * Results of this function are cached
	 * @return boolean If the input is a valid email.
	 * @see Users::getUsersList()
	 * @author Phil Calder
	 */
	function getUsersList($show_all = false, $limit = false, $incl_deleted = false, $incl_resources = false) { //Used to get the users
		// Users service caches common users lists
		global $SERVICES;
		$ua =& $SERVICES->getService('Authentication');
		$this->users_list = $ua->users->getUsersList($show_all, $limit, $incl_deleted, $incl_resources);
		
		return $this->users_list;
	}

	/**
	 * Get a list of groups
	 *
	 * @param string $type Optional, default is 'all'.
	 * Can be any group type accepted by {@link Groups::getGroupList}
	 * @return array  List of groups.
	 * @author Phil Calder
	 */
	function getGroupList($type = 'all') { //Used to get the user groups
		if (!is_array($this->groups_list)) {
			$this->groups_list = array();
		}
		if (!is_array($this->groups_list[$type])) {
			if (!is_array($this->groups_list[$type])) {
				global $SERVICES;
				$ua =& $SERVICES->getService('Authentication');
				$this->groups_list[$type] = $ua->groups->getGroupList($type);
			}
		}
		return $this->groups_list[$type];
	}

	/**
	 *
	 */
	function addToHead($str) {
		$this->head_block .= $str;
	}
	
	/**
	 * Appends the given content to the body block
	 *
	 * @param mixed Either an array or a string
	 */
	function addToBody($content) {
		if (is_array($content)) {
			$content = implode(' ', $content);
		}
		$this->body_block .= $content;
	}
	
	/**
	 * Required for the template, we need to assign the variables.
	 *
	 * @param array An array of the variables to be used in the template
	 * @author Brenton Bartel
	 */
	function assignToTemplate($vars){
		global $SERVICES;
		$template =& $SERVICES->getService('TemplateEngine');
		// here we'll only loop through the first level of arrays
		// since they're already split up into subsectons anyway
		foreach($vars as $k => $v){
			$template->assign($k, $v);
		}
	}
	
	/**
	 * @param sting $str The title
	 * @return string $str The title trimmed to a set length
	 * @author Phil Calder
	 */
	function trimTitle($str) {
		$str = addslashes($str);
		$str = (strlen($str) > $this->tab_max_title_length ? substr($str, 0, ($this->tab_max_title_length-2)) . '..' : $str);
		return $str;
	}
	
	function setTitle($str) {
		$this->title = $this->trimTitle($str);
	}
	
	function getTitle() {
		if (isset($this->title) && $this->title) {
			return $this->title;
		}
		return false;
	}
	
	function setCharset() {
		global $SERVICES;
		$SERVICES->getService('Config');
		$this->charset = $SERVICES->config->lookup('charset');
		@ini_set('default_charset', $this->charset);
		//@ini_set('default_charset', 'iso-8859-1');
	}

	/**
	 * @deprecated since version 4
	 * @see addTagContent
	 * @see substitutetagContent
	 */
	function setOuterTableWidth($str) {
		$this->printDebug('Please replace the call to setOuterTableWidth. Try addTagContent(\'outerTable\', \'width\', \'xxx\'); or substitutetagContent(\'outerTable\', \'newClass\')');
	}

	/**
	 * Adds the given string alert (javascript) to the message list
	 * @param string $str The message to alert
	 */
	function addMessage($str) {
		$this->messages .= '<script type="text/javascript">alert("' . addslashes($str) . '");</script>';
	}

	/**
	 * Adds the given string (javascript) to the init js
	 * @param string $str The javascript to call on 'onLoad'
	 */
	function addInitJS($str) {
		$this->init_js .= $str . ';';
	}

	/**
	 * @deprecated since version 4
	 */
	function addButton($str) {
	}

	/**
	 * Adds the uncheck ingore js function to the page header
	 *
	 * @author Matthew Corich
	 * @return string the script block
	 */
	function setUncheckIngore($bulk_edit = false) {
		$this->uncheck_ingore_js = '
			function uncheckIgnore(id) {';
		if ($bulk_edit) {
			$this->uncheck_ingore_js .= '
				document.forms["mainform"].elements["item[" + id + "]"].checked = true;';
		} else {
			$this->uncheck_ingore_js .= '
				document.forms["mainform"].elements["items[" + id + "][_ignore_row]"].checked = false;';
		}
		$this->uncheck_ingore_js .= '
			}
		';
	}

	/**
	 * Adds the a JS popup function to the page header
	 *
	 * @author Matthew Corich
	 *
	 * @param string $name the name of the function
	 * @param array $popup_options an array for popup options, array keys listed below:
	 	'vars' => array, 'url' => string, 'window_name' => string, 'height' => integer,
		'width' => integer,	'top' => integer, 'left' => integer, 'toolbar' => bool,
		'status' => bool, 'menubar' => bool, 'scrollbars' => bool, 'resizable' => bool,
		'fullscreen' => bool, 'controls' => bool
		);
	 */
	function addJSPopup($name, $popup_options) {
		global $SERVICES;
		if (is_array($popup_options['vars'])) {
			$vars =  implode(', ', $popup_options['vars']);
			foreach ($popup_options['vars'] as $k => $v) {
				$search_vars['search'][] = '%' . $v . '%';
				$search_vars['replace'][] = '"+' . $v . '+"';
			}
		} else {
			$vars = '';
		}
		$window_options = '';
		if ($popup_options['url']) {
			$window_options .= '"'. str_replace($search_vars['search'], $search_vars['replace'], $popup_options['url']) . '"';
		} else {
			$window_options .= '"' . $SERVICES->getGlobal('php_self') . '"';
		}
		if ($popup_options['window_name']) {
			$window_options .= ', "' . $popup_options['window_name'] .'"';
		} else {
			$window_options .= ', ""';
		}
		$window_options .= ', "width=';
		if ($popup_options['width']) {
			 $window_options .= $popup_options['width'];
		} else {
			$window_options .= '600';
		}
		$window_options .= ', height=';
		if ($popup_options['height']) {
			 $window_options .= $popup_options['height'];
		} else {
			$window_options .= '400';
		}
		if ($popup_options['left']) {
			 $window_options .= ', left=' . $popup_options['left'];
		}
		if ($popup_options['top']) {
			 $window_options .= ', top=' . $popup_options['top'];
		}
		if ($popup_options['status']) {
			$window_options .= ', status=yes';
		} else {
			$window_options .= ', status=no';
		}
		if ($popup_options['menubar']) {
			$window_options .= ', menubar=yes';
		} else {
			$window_options .= ', menubar=no';
		}
		if ($popup_options['scrollbars']) {
			$window_options .= ', scrollbars=yes';
		} else {
			$window_options .= ', scrollbars=no';
		}
		if ($popup_options['resizable']) {
			$window_options .= ', resizable=yes';
		} else {
			$window_options .= ', resizable=no';
		}
		if ($popup_options['fullscreen']) {
			$window_options .= ', fullscreen=yes';
		} else {
			$window_options .= ', fullscreen=no';
		}
		if ($popup_options['controls']) {
			$window_options .= ', controls=yes';
		} else {
			$window_options .= ', controls=no';
		}
		$window_options .= '"';
		
		// Bosco 20051129 | Added $focus to bring the focus to Window
		if ($popup_options['save_to']) {
			$save_to = 'var ' . $popup_options['save_to'] . '=';
			if ($popup_options['focus']) {
				$focus = $popup_options['save_to'] . '.focus();';
			}
		} else {
			$save_to = 'return ';
		}

		$this->js_pop_ups[$name] = '
			function  ' . $name . '(' . $vars . ') {
				' . $save_to . 'window.open(' . $window_options . ');' .
				$focus . '
			}
			';
	}

	/**
	 * Setup jscalendar config
	 *
	 * @author Matthew Ross
	 * @param array $setup the params for the jscalendar setup
	 */
	function setCalendarSetup($setup) {
		$this->calendar_setup_js .= '
		Calendar.setup(
			{
			';
		foreach ($setup as $k => $v) {
			$this->calendar_setup_js .= $k . ' : ' . '"' . $v . '",
			';
		}
		// Bosco 20060328 | To Remove trailing comma and newline
		$this->calendar_setup_js = rtrim(rtrim($this->calendar_setup_js), ',');
		$this->calendar_setup_js .= '			}
		);
		';
	}
	
	/**
	 * Return jscalendar setup
	 *
	 * @author Matthew Ross
	 */
	function getCalendarSetup() {
		$content = '<script type="text/javascript">
		' . $this->calendar_setup_js . '
		</script>';
		$this->calendar_setup_js = ''; //reset
		return $content;
	}
	
	function setJsPieces($str) {
		$this->js_pieces = $str;
	}
	
	/**
	 * generates JS
	 */
	function getJS() {
		global $USER, $L, $SERVICES;
		$content = '';
		
		if ($this->isPrintView()) {
			// TODO: Add print call to a separate function. Call function through addInitJs
			// $print_str = '
			// function printPage() {	
			// }
			// ';
			// $print = 'window.print(); window.close();';
			$print = ' self.setTimeout(\'window.print()\', 5000); ';
			// $this->body_block .= $this->getJsTags('', $print);
			$this->addInitJs($print);
		}
		
		if ($this->is_child_iframe) {
			$content .= $this->getJsInclude('child.js');
		}
		
		// JACK 20060127
		if (($this->req_main_js) || ($this->req_list_js) || ($this->req_viewFrame_js) || ($this->req_config_js) || ($this->req_contact_js) || ($this->req_popup_menu_js) ) {
			
			$content .= $this->getJsInclude('browser.js');
			
			if (($this->req_viewFrame_js) || ($this->req_contactMenu_js)) {
				$content .= $this->getJsInclude('iframes.js');
			}

			if ($this->req_main_js) {
				$content .= $this->getJsInclude('main.js');
			}

			if ($this->req_list_js) {
				$content .= '<script language="Javascript" type="text/javascript" src="' . $this->getSkinWebPath() . 'js/variables.js"></script>';
			}

			if ($this->req_list_js) {
				$content .= $this->getJsInclude('data.js');
			}

			if ($this->has_toolbar) {
				$this->body_block = '
				<div id="toolbar"></div>
				' . $this->body_block;
			}

			if ($this->req_viewFrame_js) {
				$this->body_block = '
				<div id="ppWn" class="popup"> 
				 <table align="center" border="0" cellpadding="0" cellspacing="0" id="mainTble">
				  <tr id="ppWnBar"> 
				   <td id="commands"> <a href="javascript:openWinBg();">Open in New Window</a> <a href="javascript:hM(\'ppWn\');">Close X</a> </td>
				  </tr>
				  <tr> 
				   <td align="CENTER" valign="MIDDLE" height="100%"> <iframe id="ppWnIframe"> 
					<p>Due to your browser not supporting iFrames you will not be able to see 
					 the progress of the file that you are uploading.</p>
					</iframe> </td>
				  </tr>
				 </table>
				</div>
				' . $this->body_block;
			}

			if ($this->req_contactMenu_js) {
				$this->body_block = '
				<div id="contactMen" class="menHide" onMouseOut="hM(\'contactMen\')" onMouseOver="kM(\'contactMen\');" >
				</div>
				' . $this->body_block;
			}

			if ($this->req_config_js) {
				
				$content .= $this->getJsInclude('config.js');
				$this->body_block = '
				<div id="configMenu"> 
				 <div class="head">' . $L['LabelDisplayConfig'] . ':</div>
				 <table cellpadding="0" cellspacing="0" border="0" id="configTbl">
				  <tr> 
				   <td> <input type="checkbox" name="wrap" id="wrap" value="checkbox">
					' . $L['LabelNoWrap'] . '<br>
					<input type="checkbox" name="trim" id="trim" value="checkbox">
				
					' . $L['LabelTrim'] . '<br>
					<input type="checkbox" name="colrng" id="colrng" value="checkbox">
					' . $L['LabelNoColouring'] . '</td>
				   <td><!-- <input type="hidden" id="pagingDef" value="25"> <select name="paging" id="paging" onChange="chkNAlert();">
					 <option>10</option>
					 <option selected>25</option>
				
					 <option>50</option>
					 <option>100</option>
					 <option>all</option>
					</select>
					Paging*<br>-->
					' . $L['LabelShowRecentAndBookmarks'] . '<br>
					<select name="recent" id="recent">
				
					 <option value="-2">' . $L['LabelNever'] . '</option>
					 <!--<option value="0">' . $L['LabelDependant'] . '</option>-->
					 <option value="-1" selected>' . $L['LabelAlways'] . '</option>
					</select> <!--<br> <input type="checkbox" name="checkbox" value="checkbox">
					Use Popup Frames --></td>
				  </tr>
				
				  <tr> 
				   <td colspan="2"><!--Save Options: 
					<select name="save">
					 <option selected>Do not Save</option>
					 <option>Save for this session</option>
					 <option>Save for current Display Settings</option>
					 <option>Save for all Display Settings</option>
				
					</select><br />-->
					<input type="button" onClick="closeConfig(1);" value="' . $L['LabelOk'] . '"> 
					<input type="button" onClick="closeConfig(0);" value="' . $L['LabelCancel'] . '"> 
					<div id="note">* ' . $L['LabelRequiresRefresh'] . '</div></td>
				  </tr>
				 </table>
				</div>
				' . $this->body_block;
			}

			if ($this->req_contact_js) {
				$content .= $this->getJsInclude('contact.js');
			}

			if (($this->req_popup_menu_js) || ($this->req_contactMenu_js)) {
				$content .= $this->getJsInclude('popupMenu.js');
			}
		}


		//xajax
		if ($this->has_xajax) {
			$content .= $this->xajax->getJavascript('../../external/xajax/');
		}

		//ext
		if ($this->has_ext) {
			//$content .= $this->xajax->getJavascript('../../external/xajax/');
			$content .= '<style type="text/css">@import url(' . $this->getJsWebPath() . '../external/ext/resources/css/ext-all.css);</style>' . "\n";
			//$content .= '<style type="text/css">@import url(' . $this->getJsWebPath() . '../external/ext/resources/css/bcrm-extras.css);</style>' . "\n";
			
			if ($this->ext_theme) {  //could be '' 
				$content .= '<style type="text/css">@import url(' . $this->getJsWebPath() . '../external/ext/resources/css/xtheme-' . $this->ext_theme . '.css);</style>' . "\n";
			}
			
			
			//$content .= $this->getJsInclude('ext/resources/css/ext-bcrm.css', $this->getJsWebPath() . '../external/');
			
			// $content .= $this->getJsInclude('ext/adapter/jquery/jquery.js', $this->getJsWebPath() . '../external/');
			// $content .= $this->getJsInclude('ext/adapter/jquery/jquery-plugins.js', $this->getJsWebPath() . '../external/');
			// $content .= $this->getJsInclude('ext/adapter/jquery/ext-jquery-adapter.js', $this->getJsWebPath() . '../external/');
			
			//Ext is now using the Ext native ext-base.js
			$content .= $this->getJsInclude('ext/adapter/ext/ext-base.js', $this->getJsWebPath() . '../external/');
			
			//main Ext js file
			$content .= $this->getJsInclude('ext/ext-all.js', $this->getJsWebPath() . '../external/');
		
			//still add in jquery for our add hoc javascript
			$content .= $this->getJsInclude('jquery/jquery.js', $this->getJsWebPath() . '../external/');
			
			$content .= '
			<script type="text/javascript">
			
			 Ext.onReady(function(){
				
				'
				. $this->ext_onready_script
				. 
				'';
			
		}
		//start Ext onReady section
		
		

		
		// //tabs
		if ($this->has_tabs && !$this->isPrintView()) { // Phil - dont use tabs in print view
			foreach ($this->tabsets as $k => $tabset) {
				$content .= '
				var ' . $k . ' = {
					init : function(){
					
						var tabs = new Ext.TabPanel(\'' . $k . '\');
						';
						
						foreach($tabset as $tab) {
							$tab['title'] = $this->trimTitle($tab['title']);
							
							if (isset($tab['is_alert']) && $tab['is_alert']) {
								$tab['title'] = '* ' . $tab['title'];
							}
							
							$content .= 'tabs.addTab(\'' . $tab['id'] . '\', "' . $tab['title'] . '");
							';
							// Phil 20070501 - tabs can be disabled (eg if there is no content to display)
							if (isset($tab['disable']) && $tab['disable']) {
								$content .= 'tabs.disableTab(\'' . $tab['id'] . '\');';
							}
							$tmp_id = $tab['id'];
						}
				
				$content .= '
						tabs.activate(\'' . $this->tabsets_active_tab[$k] . '\');
					}
				}
				' . $k . '.init();
				
				';
				
				// print $tmp_id . '<hr>';
				//Ext.EventManager.onDocumentReady(' . $k . '.init, ' . $k . ', true);
				
			}
			
		}
		
		// toolbar
		if ($this->has_toolbar && !$this->isPrintView()) {
			
			$content .= "
				
				var tb = new Ext.Toolbar('toolbar');
				
			";
			
			// print '<div>toolbar_linkset</div>'; print_r($this->toolbar_linkset); print '<hr>';
		    
			
			$toolbar_linkset_content = $this->processToolbarLinkset($this->toolbar_linkset);
			$has_previous_content = (strlen($toolbar_linkset_content) ? true : false);
			$toolbar_linkset_right_content = $this->processToolbarLinkset($this->toolbar_linkset_right, $has_previous_content, true); //3rd param is position right
			
			$content .= $toolbar_linkset_content;
			// if (strlen($toolbar_linkset_content) && strlen($toolbar_linkset_right_content)) {
				// $content .= '
				// tb.add(new Ext.Toolbar.Separator());
				// ';
			// }
			$content .= $toolbar_linkset_right_content;
			
		}
		//end toolbar

		//end  Ext onReady section
		
		if ($this->has_ext) {
			$content .= '
				
				Ext.QuickTips.init();
			 });
			
			</script>
			';
		}
			
		if ($this->has_calendar_button) {
			$content .= '<style type="text/css">@import url(' . $this->getSkinWebPath() . 'css/cal.css);</style>' . "\n";
			$content .= $this->getJsInclude('jscal/calendar.js', $this->getJsWebPath() . '../external/' );
			
			//en lang
			$content .= $this->getJsInclude('jscal/lang/calendar-en.js', $this->getJsWebPath() . '../external/');
			
			//other langs over the top MCR 20060503
			$SERVICES->getService('Preferences');
			$lang = $SERVICES->preferences->getUserPref('cfgLanguage');
			if (file_exists(getExternalDir() . '/jscal/lang/calendar-' . $lang . '.js')) {
				$content .= $this->getJsInclude('jscal/lang/calendar-' . $lang . '.js', $this->getJsWebPath() . '../external/');
			}
			
			$content .= $this->getJsInclude('jscal/calendar-setup.js', $this->getJsWebPath() . '../external/');
		}
		
		if (($this->has_pref_button) || ($this->has_calendar_button) || ($this->has_help_button) ||
			($this->js_redirect) || ($this->js_pieces) || ($this->js_pop_ups) || ($this->uncheck_ingore_js) || ($this->req_search_reset_js) ) {
			
			$content .= '
			<script type="text/javascript">
			var aModes = new Array("normal", "down");
			var aPushBtn = new Array();
			';
			
			if ($this->has_help_button)  {
				$SERVICES->getService('Config');
				$help_link = $SERVICES->config->lookup('help.url');

				$help_options = array(
					'window_name' => 'helpw',
					'save_to' => 'helpw',
					'vars' => array('helpid', 'ref'),
					'url' => $help_link . '?id=%helpid%&r=%ref%',
					'window_name' => 'help',
					'height' => 500,
					'width' => 880,
					'top' => 80,
					'left' => 80,
					'menubar' => true,
					'scrollbars' => true,
					'resizable' => true,
					'controls' => true
				);
				$this->addJSPopup('loadhelp', $help_options);
			}

			if ($this->has_delete_link) {
				$content .= 'function getDeleteMessage() {
					return confirm(\'' . $L['DeleteAreYouSure'] . '\');
				}';
				// $content .= 'function getDeleteMessage() {
					// return Ext.Msg.confirm(\'' . $L['DeleteAreYouSure'] . '\');
				// }';
			}

			if ($this->has_refresh_button) {
				$content .= 'function refresh() {
					window.location.replace("' . $SERVICES->getGlobal('php_self') . '?' . $SERVICES->getGlobal('query_string') . '");
				}
				';
			}

			if ($this->master_checkbox) {
				$content .= $this->master_checkbox;
			}
			
			if ($this->has_calendar_button) {
				$content .= '
			';
			}
			
			if ($this->js_redirect) {
			$content .= '
				function redirect(link) {
					if (link != "") {
						location=link;
					}
				}
			';
			}

			if ($this->js_pop_ups) {
				$content .= '
			' . implode('', $this->js_pop_ups) . '
			';
			}

			if ($this->uncheck_ingore_js) {
				$content .= '
			' . $this->uncheck_ingore_js . '
			';
			}
			
			// Phil 20060201
			if ($this->req_search_reset_js) {
				$content .= $this->getResetJs();
			}

			//20070314
			//set Ext panel title
			//@todo check if in frameset
			// parent frame...
			// if ($this->title) {
				
			$content .= "
			try {
				parent.window.Layout.updateContentPanelTitle('" . $this->title . "', window.name);
			} catch(exception) {
				//if not framed
			}
			";
			
			// $content .= "
			// try {
				// parent.window.Layout.updateContentPanelTitle('" . $this->title . "');
			// } catch(exception) {
				// //if not framed
			// }
			// \$('#title_tbody').find('#title_span').val('');
			// ";
			
			
			// "Ext.onReady(function(){ layout.getRegion('center').setTitle('" . $this->title . "') } );";
			// }
			
			
			$content .= '
			</script>
			';
		}
		
		if (!empty($content)) {
			$lang_vars = $this->getJsLangVars();
			$content = $lang_vars . $content;
		}
		
		return $content;
	}
	
	/**
	 * Process toolbar linksets
	 *
	 * toolbar linksets contain an array on 'linksets'
	 * each non-empty linkset is separated by a toolbar separator
	 * each linkset contains an array on 'links'
	 * a 'link' should be an array used to build the toolbar item but 
	 * can also be a string that we process to turn it into an array
	 * a 'link' can be defined as:
	 * array(
	 * 	'href'     => [url - when this is defined, we will use the function 'tbButtonClickGo' as our handler]
	 *  'handler1' => [name of a predefined function that must return true before processing the handler]
	 * 	'title'    => [optional text]
	 *  'text'     => [optional text]
	 *  'icon'     => [optional icon]
	 *  'script'   => [optional javascript to execute instead of assigning a handler for this item] 
	 *  'menu'     => [item can have a sub menu as well as / instead of a handler]
	 * )
	 *
	 * @param array $linksets An array of linksets. Each linkset contains an array of links.
	 * @return string Javascript content.
	 *
	 * @see getJs
	 * @author Phil Calder, Mathew Ross
	 * @access private
	 */
	function processToolbarLinkset($linksets = array(), $has_previous_content = false, $position_right = false) {
		global $L;
		$content = '';
		
		if (!isset($this->toolbar_link_increment)) {
			$this->toolbar_link_increment = 0;
		}
		$i = $this->toolbar_link_increment;
		foreach($linksets as $linkset) {
			if ($position_right) {
				$content .= '
				tb.addFill();
				';
			}
			elseif ($has_previous_content && !empty($linkset)) {
				$content .= '
				tb.add(new Ext.Toolbar.Separator());
				';
			}
			$has_previous_content = false;
			foreach($linkset as $link) {
				$i++;
				
				// print '<div>Output -- toolbar link</div>'; print_r($link); print '<hr>';
				// Phil 20070507 Attempt to handle old style links
				if (!is_array($link)) {
					if (strlen($link)) {
						
						$regexp = '<a\s[^>]*href=(\"??)([^\" >]*?)\\1[^>]*>(.*)<\/a>'; 
						if(preg_match_all("/$regexp/siU", trim($link), $matches)) {
							// print_r($matches);
							$link = array(
								'href' => current($matches[2]), 
								'title' => current($matches[3]), 
								'text' => current($matches[3])
								//,'icon' => 'page_misc.png'
								
							);
							// Phil - use this for debugging
							// $error_msg = 'NOTICE - Output - invalid toolbar link: [' . $link['text'] . '] parsed';
							// $this->logError($error_msg, 'Output');
							// $error_msg = '';
						} else {
							$error_msg = 'WARNING - Output - unable to parse invalid toolbar link: [' . $link . ']';
							// print '<div>---- ' . $error_msg . ' ------</div>';
							$this->logError($error_msg, 'Output');
							$error_msg = '';
							continue;
						}
					} else {
						continue;
					}
				}
				if (empty($link)) {
					continue; // Ignore empty links	
				}
				$has_previous_content = true;
				
				// Phil 20070508
				// Look for 'href', 'script', 'handler1' to determine how to create this button.
				
				if (isset($link['handler1']) && $link['handler1']) {
					$content .= '
			function tbButtonClick' . $i . '(btn){ if (' . $link['handler1'] . ') window.location.href = \'' . $link['href'] . '\'; }
				';
				} else if (isset($link['href']) && $link['href'] ) {
				$content .= '
			function tbButtonClick' . $i . '(btn){ window.location.href = \'' . $link['href'] . '\'; }
				';
				} else if (isset($link['script']) && $link['script'] ) {
					// print '<div>link with script:</div>'; print_r($link); print '<hr>';
					// TODO probably extend this to handle complex scripts
					// currently script needs to be a complete function call (including parentheses and args), eg alert('Hello World')
					$content .= '
			function tbButtonClick' . $i . '(btn){ ' . $link['script'] . '; }
				';
				}
				//icon should be in format 'refresh.gif'
				
				$menu_name = ''; // set by reference
				$content .= $this->processToolbarLinksetSubmenu($link, $i, $menu_name);
				
				// if (!$link['title'] && !$link['text']) { $link['text'] = ''; }
				// print '<div>adding standard item for ' . ($link['title'] ? $link['title'] : $link['text']) . ' . [' . addslashes($link['text']) . ']</div>';
				
				$content .= '
			tb.add(
				new Ext.Toolbar.' . ($menu_name ? 'Menu' : '') . 'Button({
				' . (isset($link['text']) ? 'text: \'' . $link['text'] . '\',' : '') . '
				handler: tbButtonClick' . $i . ',
				cls: \'x-btn' . (isset($link['text']) ? '-text' : '') . '' . (isset($link['icon']) ? '-icon' : '') . '\',
				' . (isset($link['icon']) ? 'icon: \'' . $this->getSkinWebPath() . 'img/' . $link['icon'] . '\',' : '') . '
				' . (isset($link['menu']) ? 'menu: \'' . $menu_name . '\',' : '') . 
				//'tooltip: \'<b>' . $L['LabelQuickTip'] . '</b><br/>' . (isset($link['title']) ? $link['title'] : $link['text']) . '\'
				'tooltip: \'' . (isset($link['title']) ? '<b>' . $link['title'] . '</b>' : $link['text']) . '\'
				
				})
			);
			';
			
			// <br><pre>' . $link['href'] . '</pre>
			
			}
		}
		$this->toolbar_link_increment = $i;
		return $content;
	}
	
	function processToolbarLinksetSubmenu($link, $i, &$menu_name) {
		
		$content = '';
		$menu_name = false;
		if (isset($link['menu']) && is_array($link['menu'])) {
			// print '<div>processToolbarLinksetSubmenu</div>>'; print_r($link); print '<hr>';
			$menu_functions = '';
			$menu_name = 'tbMenu' . $i;
			// print '<div>adding menu var for ' . $menu_name . '</div>';
			$content .= "
			var " . $menu_name . " = new Ext.menu.Menu({
				id: '" . $menu_name . "', 
				items: [
			";
			$first = true;
			$j = 0;
			foreach ($link['menu'] as $menulink) {
				$j++;
				if (!$first) {
					$content .= ", 
				";
				} else {
					$first = false;
				}
					
				if (isset($menulink['href']) && $menulink['href'] ) {
					$menu_functions .= '
			function tbButtonClick' . $i . '_' . $j .'(btn){ window.location.href = \'' . $menulink['href'] . '\'; }
				';
				}
				
				$sub_menu_name = '';
				if (isset($menulink['menu']) && is_array($menulink['menu'])) {
					$menu_functions .= $this->processToolbarLinksetSubmenu($menulink, $i . '_' . $j, $sub_menu_name); 
				}
					
				$content .= "
					{
						text: '" . $menulink['text'] . "',
						" . (isset($link['icon']) ? "icon: '" . $this->getSkinWebPath() . "img/" . $link['icon'] . "'," : "") . "
						" . ( (isset($menulink['menu']) && $sub_menu_name) ? "menu: '" . $sub_menu_name . "'," : "") . "
						handler: tbButtonClick" . $i . "_" . $j . "
					}";
			}
				$content .= "
				]
			});
				";
				
				// Add menu function / sub menu definitions before the menu content
				$content = $menu_functions . $content;
				
			}
				
		return $content;
	}
	
	
	function getJsLangVars() {
		global $SERVICES, $L;
		if ($lang =& $SERVICES->getService('Language')) {
			$lang->getLanguage('_Global');
		}
		// Pre-load required language variables
		//BAD W3C	<script language="javascript">		
		$lang_vars = '
		<script type="text/javascript">
		var jsLangIframesProblem = "' . $L['jsLangIframesProblem'] . '";
		var jsLangIframesLoadingMenu = "' . $L['jsLangIframesLoadingMenu'] . '";
		var jsLangMenuSearch = "' . $L['jsLangMenuSearch'] . '";
		var jsLangMenuAlerts = "' . $L['jsLangMenuAlerts'] . '";
		var jsLangMenuNoAlerts = " ' . $L['jsLangMenuNoAlerts'] . '";
		</script>
		';
		return $lang_vars;
	}

	function getJsInclude($script, $dir = false) {
		return $this->getJsTags('src="' . ($dir ? $dir : $this->getJsWebPath()) . $script . '"');
	}
  
	function getJsTags($tags = '', $code = '') {
		return '<script type="text/javascript" ' . $tags . '>' . $code . '</script>';
	}
  
	/**
	 * Puts all the pieces of the page together.
	 *
	 * @see display()
	 * @param array $custom Added 20040609, Phil,
	 * 'custom' can be an array used to override certain values here, such as (keys) 'css' and 'title
	 * @return string The page content. Use {@link display()} to print a page to screen.
	 * @todo in future we could add 'charset', meta-tags, and individual style settings
	 */
	function buildPage($custom = array()) {

		if ($this->has_template) {
			$this->addToHead($this->_getClientValidationScript());

			return array(
				'clientValidationJS' => $this->_getClientValidationScript(),
				'initJS' => $this->init_js,
				'message' => $this->getMessage(),
				'headBlock' => $this->getHeadBlock(),
				'bodyBlock' => $this->getBodyBlock()
				);
		} else {
			global $USER;
			
			//todo: test if files exist before adding
			
			if (!$this->isPrintView()) {
				$this->base_stylesheet = $this->getBaseSkinWebPath() . 'css/main.css'; // default
				$this->stylesheet = $this->getSkinWebPath() . 'css/main.css'; 
			} else {
				$this->base_stylesheet = $this->getBaseSkinWebPath() . '_print/css/main.css'; // default
				$this->stylesheet = $this->getSkinWebPath() . '_print/css/main.css';
			}
			
			
			if (isset($custom['css'])) { // override stylesheet
				if (is_array($custom['css'])) {
					$this->stylesheet = array();
					foreach ($custom['css'] as $k => $v) {
						$this->stylesheet[] = $this->getBaseSkinWebPath() . ($this->isPrintView() ? '_print/' : '') . $v;
						$this->stylesheet[] = $this->getSkinWebPath() . ($this->isPrintView() ? '_print/' : '') . $v;
					}
				} else {
					$this->stylesheet = $this->getBaseSkinWebPath() . ($this->isPrintView() ? '_print/' : '') . $custom['css'];
					$this->stylesheet = $this->getSkinWebPath() . ($this->isPrintView() ? '_print/' : '') . $custom['css'];
				}
			}

			if (isset($custom['title'])) { // override title
				$this->title = $custom['title'];
			}

			$this->addToHead($this->_getClientValidationScript());

			$content = array();
			$content[] = '<!DOCTYPE HTML PUBLIC "-//W3C//Dtd HTML 4.01 Transitional//EN">';
			$content[] = $this->getBeginHtmlTag();
			$content[] = $this->getHeadBlock();
			$content[] = $this->getBeginBodyTag();
			$content[] = $this->getMessage();
			$content[] = $this->getXajaxSpace();
			//$content[] = $this->getGraphicBlock();
			$content[] = $this->getBodyBlock();
			return implode("\n", $content);
		}
	}

	/**
	 * Builds the page, adds the footer and prints to screen.
	 *
	 * It is up to the calling function to exit (if desired)
	 * @param array $custom This is passed to {@link buildPage}
	 */
	function display($custom = array()) {
		if ($this->has_template) {
			$this->assignToTemplate($this->buildPage($custom));
			$this->assignToTemplate(array('footerBlock' => $this->getFooterBlock()));
			global $SERVICES;
			$template =& $SERVICES->getService('TemplateEngine');
			$template->display();
		} else {
			$content = $this->buildPage($custom);
			$content = $this->stripWhitespace($content);
			$content .= $this->getFooterBlock() . $this->getEndBodyTag() . $this->getEndHtmlTag();
			echo $content;
			
		}
	}

	/**
	 * @access private
	 */
	function stripWhitespace($content) {
		//this is pretty quick, so worth doing
		// Tasks list view with 18 rows (compression time: 0.0006s)
		if ($this->getWhitespaceCompressionSwitch()) {
			if (!strpos($content, '<textarea')) {
				//$time_start = $this->getPageExecutionTime();
				$size = strlen($content);
				$content = str_replace(array("\t", "\r\n", "\n"), '', $content);
				$content = str_replace('  ', ' ', $content); //double space with single
			}
		}
		return $content;
	}

	/**
	 * @access private
	 */
	function setFooterNotice($str) {
		$this->footer_notice = $str;
	}

	/**
	 * @access private
	 */
	function addFooterNotice($str) {
		$this->footer_notice .= '<br>' . $str;
	}

	/**
	 * @access private
	 */
	function getFooterNotice() {
		return $this->footer_notice;
	}

	/**
	 * @access private
	 */
	function setWhitespaceCompressionSwitch($switch = true) {
		$this->whitespace_compression_switch = $switch;
	}

	/**
	 * @access private
	 */
	function getWhitespaceCompressionSwitch() {
		return $this->whitespace_compression_switch;
	}

	/**
	 * Returns html tag to begin a page.
	 */
	function getBeginHtmlTag() {
		return '<html>';
	}

	/**
	 * Builds the page head block, including title, charset, javascript, and stylesheets
	 *
	 * @param array $data This parameter is deprecated since version 4
	 */
	function getHeadBlock($data = array()) {
		global $L;
		if ($this->has_template) {
			return array(
				'title' => $L['ApplicationTitle'] . ($this->title ? ' -' . $this->title : '' ),
				'headBlock' => $this->head_block,
				'JS' => $this->getJS()
				);
		} else {
			$content = '';
			$content .= '<head>' . "\n";
			$content .= '<title>' . $L['ApplicationTitle'] . ($this->title ? ' - ' . $this->title : '' ) . '</title>' . "\n";
			$content .= '<meta http-equiv="Content-Type" content="text/html; charset=' . $this->charset . '">' . "\n";

			$content .= '<link rel="stylesheet" type="text/css" href="' . $this->base_stylesheet . '">' . "\n";
			
			if (is_array($this->stylesheet)) {
				foreach ($this->stylesheet as $k => $v) {
					$content .= '<link rel="stylesheet" type="text/css" href="' . $v . '">' . "\n";
				}
			} else {
				//test if file exists
				$content .= '<link rel="stylesheet" type="text/css" href="' . $this->stylesheet . '">' . "\n";
			}
			$content .= $this->head_block . $this->getJS() . "\n";
			$content .= '</head>' . "\n\n";
			return $content;
		}
	}

	/**
	 * Returns html tag to begin a the body block
	 *
	 * Includes javascript onLoad tag, if any
	 * @see getTagContent
	 * @see addInitJS
	 * @return string The begin body tag.
	 */
	function getBeginBodyTag() {
		return '<body' . ($this->init_js ? ' onload="' . $this->init_js . '"' : '') . '>' . "\n";
		//idea: class="xtheme-gray" 
	}

	/**
	 * @return string The page message variable
	 */
	function getMessage() {
		return $this->message;
	}
	
	function getXajaxSpace() {
		if ($this->has_xajax) {
		
				//@todo could have another loader message div later
				return '
			<div id="xajax_space_div" class="xajax_space_div">
			<table border="0" width="100%" align="center" class=""><tbody>
				<tr><th valign="center" align="left"><img style="cursor: pointer; height: 16px;" title="Close Window" onclick="javascript:document.getElementById(\'xajax_space_div\').style.visibility=\'hidden\';return false;" src="' . $this->getSkinWebPath() . 'img/close.png"/></th></tr>
				<tr><td>'
				//. ' <span id="xajax_view_title">View</span>'
				. '<div id="xajax_view_content" name="xajax_view_content"></div>
				</td></tr></tbody>
			</table></div>';
				
		}
	}

	/**
	 * Returns the formatted body block for the page
	 * @see buildPage
	 * @return string Page body (everything inside the body tabs)
	 */
	function getBodyBlock() {
		if ($this->has_template) {
			return $this->body_block;
		}
		$content = $this->body_block;
		return $content;
	}
	
	function setPaddedTableSwitch($switch = true) {
		$this->padded_table_switch = $switch;
	}
	
	function getPaddedTableSwitch() {
		return $this->padded_table_switch;
	}
	
	// Phil 20070503 - removed - this is defined in parent class
	// function getPageExecutionTime() {
		// global $BCRM_TIME_START, $L;
		// $time_end = microtimeFloat();
		// $time = $time_end - $BCRM_TIME_START;
		// return $time;
	// }

	/**
	 * @access private
	 */
	function setFooterBlockSwitch($switch = true) {
		$this->footer_block_switch = $switch;
	}

	/**
	 * @access private
	 */
	function getFooterBlockSwitch() {
		return $this->footer_block_switch;
	}

	/**
	 * @access private
	 */
	function setFooterTimerSwitch($switch = true) {
		$this->footer_timer_switch = $switch;
	}

	/**
	 * @access private
	 */
	function getFooterTimerSwitch() {
		return $this->footer_timer_switch;
	}

	/**
	 * @access private
	 */
	function setFooterInfoSwitch($switch = true) {
		$this->footer_info_switch = $switch;
	}

	/**
	 * @access private
	 */
	function getFooterInfoSwitch() {
		return $this->footer_info_switch;
	}

	/**
	 * Empty function. Override to add custom code to the page footer.
	 * @access private
	 * @todo Implement this (by calling this method in eg {@link getFooterBlock})
	 */
	function setFooterBlock() {}

	/**
	 * @access private
	 */
	function getFooterBlock() {
		global $L, $USER, $SERVICES;
		if ($this->getFooterBlockSwitch()) {
			$footer_lines = array();
			if ($this->getFooterTimerSwitch()) {
				$time = $this->getPageExecutionTime();
				//compact html here since it is after compression mechanism
				if ($time) {
					$footer_lines[] = $L['PageExecution'] . ' ' . round($time, 3) . ' ' . $L['SecondsFull'] . '<br>';
				}
			}

			if ($this->getFooterInfoSwitch()) {
				//compact html here since it is after compression mechanism
				$items[] = $L['frontVersion'] . ': 5.000.01';
				$items[] = $L['frontLicenceID'] . ': 17de344cb9c7cabb4aae3f7e85f5d612';
				$items[] = $L['frontCurrentServerTime'] . ': ' . date("Y-m-d H:i");
				$items[] = $L['frontUser'] . ': ' . $USER['username'];
				if (date("Y-m-d H:i") != date("Y-m-d H:i", $SERVICES->offsetTimeZoneForRead(time()))) {
					$items[] = $L['frontCurrentUserTimeInTimezone'] . ': ' . date("Y-m-d H:i", $SERVICES->offsetTimeZoneForRead(time()));
				}
				$footer_lines[] = implode('<br>', $items);
				//$items[] = $L['frontLicensedInfo'];
			}

			$footer_notice = $this->getFooterNotice();
			if ($footer_notice) {
				$footer_lines[] = $footer_notice;
			}

			if (sizeof($footer_lines)) {
				$content = implode('<br>', $footer_lines);

				if (!$this->has_template) {
					//$content = $this->getSpacerRow() . "\n" . '<table' . $this->getTagContent('zeroTable') . '><tr><td' . $this->getTagContent('footerCell') . '>' . $content . '<br>&nbsp;</td></tr></table>';
					$content = '<div' . $this->getTagContent('footerCell') . '>' . $content . '</div>';
				}
				return $content;
			} else {
				return'';
			}
		}
	}

	/**
	 * @access private
	 */
	function setModuleEditorInfo($module_name, $table_name = false) {
		$this->module_editor_info['module_name'] = $module_name;
		$this->module_editor_info['table_name'] = $table_name;
	}

	/**
	 * @access private
	 */
	function getModuleEditorLink($module_name = false, $table_name = false) {
		// See WebMATList (calls Output::setModuleEditorInfo)
		global $SERVICES, $L, $USER;
		$ua =& $SERVICES->getService('Authentication');
		
		if (!$module_name) {
			$module_name = ( (isset($this->module_editor_info) && $this->module_editor_info['module_name']) ? $this->module_editor_info['module_name'] : false);
			$table_name = ( (isset($this->module_editor_info) && $this->module_editor_info['table_name']) ? $this->module_editor_info['table_name'] : false);
		}
		if ($ua->adminAccess($USER['user_id'], 'Admin')) {
			if ($module_name && $table_name) {
				$full_table_name = $this->module_editor_info['module_name'] . '__' . str_replace($this->module_editor_info['module_name'] . '__', '', $this->module_editor_info['table_name']);
				// Phil - updated url to handle modules_custom dir ie ../../modules/ instead of ../
				return $this->getLinkHref(
					array(
						'href' => '../../modules/Admin/admin_module_table_edit.php?table=' . $full_table_name. '',
						'text' => $L['LinkModuleEditor']
					)
				);
			} else if ($module_name) {
				return $this->getLinkHref(
					array(
						'href' => '../../modules/Admin/admin_module_index.php?module=' . $module_name. '',
						'text' => $L['LinkModuleEditor']
					)
				);
			}
		}
		return false;
	}

	/**
	 * @access private
	 */
	function setGraphicBlockSwitch($switch = true) {
		$this->graphic_block_switch = $switch;
	}

	/**
	 * @access private
	 */
	function getGraphicBlockSwitch() {
		return $this->graphic_block_switch;
	}

	/**
	 * @access private
	 */
	function setGraphicBlock($str = false) {
		if ($str) {
			$this->graphic_block = $str;
		} else {
			$this->graphic_block = $this->getBeginTable('zeroTable')
				. $this->getBeginRow() . $this->getBeginCell(1, 'graphicCell') . '&nbsp;'
				. $this->getEndCell() . $this->getEndRow(). $this->getEndTable();
		}
	}

	/**
	 * @access private
	 */
	function getGraphicBlock() {
		global $L;
		if ($this->getGraphicBlockSwitch()) {
			return $this->graphic_block;
		}
	}

	/**
	 * @return string Html body end tag
	 */
	function getEndBodyTag() {
		return  "\n" . '</body>';
	}

	/**
	 * @return string Html end tag
	 */
	function getEndHtmlTag() {
		return  "\n" . '</html>';
	}

	/**
	 * @return string html for start of container
	 */
	function getBeginContainer($container_name) {
		return '
		
		<div class="' . $container_name. '">
		 	
			';
			// <div class="x-box-tl"><div class="x-box-tr"><div class="x-box-tc"></div></div></div>
			// <div class="x-box-ml"><div class="x-box-mr"><div class="x-box-mc">
			
	}
	
	function getEndContainer() {
		// </div></div></div>
		// <div class="x-box-bl"><div class="x-box-br"><div class="x-box-bc"></div></div></div>
		 
		return '
			
		</div>
		';
	}

	
	function startDataBlock() {
		return '<tbody id="data">';
	}

	function endDataBlock() {
		return '</tbody>';
	}

	// -------------------------------------------------------------------------

	/**
	 * Sets type for next table to be drawn
	 *
	 * @author Matthew Ross
	 */
	function setInitTableType($str = 'contentTable') {
		$this->init_table_type = $str;
	}

	/**
	 * Gets type for next table to be drawn
	 *
	 * @author Matthew Ross
	 */
	function getInitTableType() {
		return $this->init_table_type;
	}

	/**
	 * @return The begin table tag, with optional tag content (defaults to contentTbl)
	 * @see getTagContent
	 */
	function getBeginTable($tag_name = 'contentTbl', $border = false) {
		if (is_array($tag_name)) {
			extract($tag_name);
		}
		$content =  "\n" . '<table' . $this->getTagContent($tag_name) . (isset($tag_extras) ? $tag_extras : '') . '>';
		if ($border) {
			$content .= $this->getTableTopBorder(); 
		}
		return $content;
	}
	
	/**
	 * @return The top table row for graphical border
	 * @author Matthew Ross
	 */
	function getTableTopBorder() {
		if ($this->getPaddedTableSwitch()) {
			//return '<tr><td width="5" height="5"><img src="' . $this->getSkinWebPath() . 'img/crnr_tl.gif" width="5" height="5" alt=""></td><td class="border"></td><td width="5"><img src="' . $this->getSkinWebPath() . 'img/crnr_tr.gif" width="5" height="5" alt=""></td></tr>';
		}
	}
		
	/**
	 * @return string  Html table end tag
	 */
	function getEndTable($border = false) {
		$content = '';
		if ($border) {
			$content .= $this->getTableBottomBorder(); 
		}
		$content .=  "\n" . '</table>';
		return $content;
	}

	/**
	 * @return The bottom table row for graphical border
	 * @author Matthew Ross
	 */
	function getTableBottomBorder() {
		if ($this->getPaddedTableSwitch()) {
			//return '<tr><td width="5" height="5"><img src="' . $this->getSkinWebPath() . 'img/crnr_bl.gif" width="5" height="5" alt=""></td><td class="border"></td><td width="5"><img src="' . $this->getSkinWebPath() . 'img/crnr_br.gif" width="5" height="5" alt=""></td></tr>';
		}
	}
	
	function getTitleRow($title, $menu = array(), $extra_title = '') {
		// $this->title = $title;
		$this->setTitle($title); // Phil 20070618
		$title = '<span id="title_span">' . $title . '</span>';
		if (!empty($extra_title)) {
			$title .= ' ' . $extra_title;
		}
		//new way, expecting array with keys 'help', 'bookmark',
		$bm_title = $title;
		if (is_array($menu)) { // Phil 20040922
			if (array_key_exists('print_view', $menu)) {
				$print_view = $menu['print_view'];
				bcrm_unset($menu, 'print_view');
			}
			if (array_key_exists('back', $menu)) {
				$back = $menu['back'];
				bcrm_unset($menu, 'back');
			}
			if (array_key_exists('help', $menu)) {
				$help_id = $menu['help'];
				bcrm_unset($menu, 'help');
			}
			if (array_key_exists('bookmark', $menu)) {
				$bookmark = $menu['bookmark'];
				$bm_title = false;
				bcrm_unset($menu, 'bookmark');
			}
			if (array_key_exists('refresh', $menu)) {
				$refresh = $menu['refresh'];
				bcrm_unset($menu, 'refresh');
			}
			// Phil 20060131
			if (array_key_exists('settings', $menu)) {
				$settings = $menu['settings'];
				bcrm_unset($menu, 'settings');
			}
			
			if (array_key_exists('addtab', $menu)) {
				$addtab = $menu['addtab'];
				bcrm_unset($menu, 'addtab');
			}
			
			$menu = $this->getLinkMenu($menu); // this transforms it from an array, to a string!
		}
		
		// Phil 20060615 - Can disable title row links
		if ($menu !== false) {
			$this->toolbar_link_mode = true;
			
			$linkset = array();
			
			
			// Phil 20070522 - removed back link. Inconsistant results due to AJAX implementation.
			// Phil 20070726 - restored back link, see getLinkBack for changes
			if ($back != 'off') $linkset[] = $this->getLinkBack($back);
			
			if ($addtab != 'off') $linkset[] = $this->getLinkAddTab(); //
			
			if ($bookmark != 'off') $linkset[] = $this->getAddBookmarkLink($bookmark, $bm_title); //
			
			if ($refresh != 'off') $linkset[] = $this->getLinkRefresh($back); //
			
			if ($print_view != 'off') $linkset[] = $this->getLinkPrintView($back); //
			if ($settings != 'off') $linkset[] = $this->getLinkSettings($settings);
			
			if ($help_id != '-1') $linkset[] = $this->getLinkHelp($help_id);
			
			$this->toolbar_linkset_right[] = $linkset;
			$this->toolbar_link_mode = false;
		}
		
		//20070427 TODO convert above links to
		//array elements in $this->toolbar
		
		
		// if it has a template, return the values in an array
		// otherwise we return a string
		if ($this->has_template) {
			return array (
				'title' => $title,
				'primaryMenu' => $primaryMenu,
				'secondaryMenu' => $menu
				);                                            
		} else {
			if ($this->has_toolbar) {
				//20070509 not returning anymore, toolbar is added by Output to top of <body>
				//return '<tr><td><div id="toolbar"></div></td></tr>';
				
			}
			// return $this->getBeginRow('', true) . '<td id="commands"><div>' . $temp_menu . $primaryMenu . '</div></td>' . $this->getEndRow(true) . 
				// '<tbody id="title_tbody" style="display:none;">' . $this->getBeginRow('', true) . '<td id="title">' . $title . '</td>' . $this->getEndRow(true) . '</tbody>' . 
				// $this->getMenuRow($menu, 2, 'lNewMenuRow');
		}
	}
	
	/**
	 * returns the given last viewed items as a list
	 * @author Phil Calder
	 */
	function getLastViewedMenuList($menu) {
		global $L;
		if (is_array($menu)) {
			foreach ($menu as $k => $v) {
				if (is_string($v)) {
					//$menu[$k] = '<li style="margin-left: 5px;">' . $v . '</li>';
					$menu[$k] = $v;
				}
			}
		}
		// $content = '<strong>' . $L['LabelLastViewed'] . ':</strong> ';
		$content .= $this->getLinkMenu($menu);
		//return '<ul>' . $content . '</ul>';
		$content = str_replace('<a ', '<a style="line-height: 22px;color: #7C7C7C;padding-right: 8px;" ', $content);
		return $content;
	}
	
	/**
	 * returns the last viewed row
	 * @author Syed
	 */
	function getLastViewedMenu($menu, $colspan=2, $tag='') {
		global $L;
		if (is_array($menu)) {
			foreach ($menu as $k => $v) {
				if (is_string($v)) {
					$menu[$k] = '<span style="margin-left: 5px;">' . $v . '</span>';
				}
			}
		}
		$content = '<strong>' . $L['LabelLastViewed'] . ':</strong> ';
		$content .= $this->getLinkMenu($menu);
		return '<tr><td' . $this->getTagContent($tag) . (($colspan > 1) ? ' colspan="' . $colspan . '"' : '') . '>' . $content . '</td></tr>';
	}
	
	/**
	 * @returns string the action links menu (table)
	 * @author Phil Calder
	 */
	function getActionLinksMenu($content1 = false, $content2 = false, $tag = '') {
		
		$content1_str = $content2_str = '';
		if (is_array($content1) && count($content1)) {
			$content1_str = $this->getLinkMenu($content1);
		}
		if (is_array($content2) && count($content2)) {
			$content2_str = $this->getLinkMenu($content2);
		}
		if ($content1_str) {
			$content1_str = '<td' . $this->getTagContent($tag) . ' ' . $this->getTagContent('newActions') . ' nowrap>' . $content1_str . '</td>';
		}
		if ($content2_str) {
			$content2_str = '<td' . $this->getTagContent($tag) . ' ' . $this->getTagContent('otherActions') . '>' . $content2_str . '</td>';
		}
		if ($content1_str || $content2_str) {
			return '<table cellpadding=0 cellspacing=0><tr>' . $content1_str . $content2_str . '</tr></table>';
		}
		return false;
		
	}
	
	/**
	 * adds action links to toolbar
	 * @author Phil Calder
	 */
	function setActionLinksMenu($content1 = false, $content2 = false, $tag = '') {
		
		//print_r($content1);die();
		if (is_array($content1) && count($content1)) {
			$this->toolbar_linkset[] = $content1;
			//$content1_str = $this->getLinkMenu($content1);
		}
		if (is_array($content2) && count($content2)) {
			$this->toolbar_linkset[] = $content2;
			//$content1_str = $this->getLinkMenu($content1);
		}
	}
	
	/**
	 * Returns the bookmarks menu
	 *
	 * @param array $menu An array of bookmark items.
	 * @param integer $colspan. Optional, default 2
	 * @param string $tag. Optional style tag, defualt ''. See {@link getTagContent}
	 *
	 * @author Phil Calder
	 */
	function getBookmarkMenu($menu, $colspan = 2, $tag = '', $module_label = false) {
		global $L;
		if ($module_label) {
			$label = $L['LabelModuleBookmarks'];	
		} else {
			$label = $L['LabelBookmarks'];	
		}
		$content = '<strong>' . $label . ':</strong> ';
		$links = array();
		if (is_array($menu)) {
			foreach ($menu as $k => $v) {
				if (!isset($v['selected']) || !$v['selected']) {
				$links[] = $this->getLinkHref($v);
				} else {
					$links[] = $v['text'];
				}
			}
			$content .= $this->getLinkMenu($links);
			return '<tr><td' . $this->getTagContent($tag) . (($colspan > 1) ? ' colspan="' . $colspan . '"' : '') . '>' . $content . '</td></tr>';
		}
		return false;
	}
	
	/**
	 * @author Jack Farley
	 */
	function setFilterStatus($status = false) {
		global $filterStatus;
		if ($status) {
			$filterStatus = 1;
		} else {
			$filterStatus = 0;
		}
	}
	
	function getFilterStatus() {
	  	global $filterStatus;
		$filterClasses = array('lFilterRow-grey', 'lFilterRow');
		return $filterClasses[$filterStatus];
	}
	
	/**
	 * Processes given filters data and generates filters display content
	 * 
	 * @param array filters.
	 * @param $title_text (by reference) populates this variable with a config filter or bookmark title
	 * @return string The filters content (for a list page)
	 * @see WebMATFilters
	 * @author Phil Calder, Jack Farley
	 */
	function getFiltersContent($filters, &$title_text) {
		global $L;
		$content = '';
		extract($filters);
		
		if (strstr($showFilters, 'search_filter=off')) {
			$this->setFilterStatus(true);
		} else {
			$this->setFilterStatus();
		}
		
		// contact filter
		if (( is_array($contact) ? !empty($contact) : false )) {
			$contactPart .= '<strong>' .$contact['label'] . '</strong>';
			// Phil 20060221 Added (optional) depth link
			$contactPart .= '
			' . $contact['contactLink'] . ' [ ' . (isset($contact['depth']) && $contact['depth'] ? $contact['depth'] . ' | ' : '') . ' ' . (isset($contact['changeLink']) && $contact['changeLink'] ? $contact['changeLink'] . ' | ' : '') . $contact['cancelLink'] . ' ]
			';
		}
		
		// id filter
		if (( is_array($idsFilters) ? !empty($idsFilters) : false )) {
			$idsPart .= '<strong>' .$idsFilters['label'] . '</strong>';
			$idsPart .= '
			' . $idsFilters['idsCount'] . ' (' . (!empty($idsFilters['invisibleCount']) ? $idsFilters['invisibleCount'] : '0') . ') ' /*. '[Full Depth]' */. ' [ ' . $idsFilters['cancelLink'] . ' ]
			';
		} // else {
			// TODO	$content .= 'None';
		// }
		
		// filters and constraints row
		$filterRow .= '
		<tbody id="filterConstr"' . (!$this->show_filter_form ? ' style="display:none;"' : '') . '>
		<tr> 
			<!--td style="width: 100px ! important;">&nbsp;</td-->
			<td>
		';
		
		// Config filters
		if (( is_array($configFilters) ? !empty($configFilters) : false )) {
			// $filterRow .= $configFilters['module'] . ' Label?: ';
			// Phil 20060719 Need to retain contact_id, qfilter, search_filter, and field (selector)
			global $SERVICES;
			$query_string = $SERVICES->getGlobal('query_string');
			$query_extra = '';
			if (strlen($query_string)) {
				$query_arr = array();
				parse_str($query_string, $query_arr);
				if (is_array($query_arr) && count($query_arr)) {
					if (isset($query_arr['contact_id']) && $query_arr['contact_id']) {
						$query_extra .= '&contact_id=' . $query_arr['contact_id'];
					}
					if (isset($query_arr['qfilter']) && $query_arr['qfilter']) {
						$query_extra .= '&qfilter=' . $query_arr['qfilter'];
					}
					if (isset($query_arr['search_filter']) && $query_arr['search_filter']) {
						$query_extra .= '&search_filter=' . $query_arr['search_filter'];
					}
					if (isset($query_arr['field']) && $query_arr['field']) {
						$query_extra .= '&field=' . $query_arr['field'];
					}
				}
			}
			
			$filterRow .= $L['labelFilterConfig']; // Filters:
			$selOpts = '';
			foreach ($configFilters['options'] as $k => $v) {
				// print_r($v); print '<hr>';
				if (is_array($v)) {
					$selOpts .= '<option value="' . $v['link'] . $query_extra . '"' . ( ($k == $configFilters['value']) ? ' selected' : '') . '>' . $v['label'] . '</option>';
					if ( ($k == $configFilters['value']) && ($configFilters['value'] != 'none')) { // Phil 20060307 added 2nd condition
						// $configPart = $configFilters['module'] . ' Label?: ' . $v['label'];
						$configPart = '<strong>' . $L['labelFilterConfig'] . '</strong> ' . $v['label'];
						$title_text .= $v['label'];
					}
				}
			}
			if (strlen($selOpts)) {
				// Phil 20060119 - Adding no filter link
				if ($configFilters['no_filter_link']) {
					$selOpts .= '<option value="' . $configFilters['no_filter_link'] . '"' . ( ($configFilters['value'] == 'none') ? ' selected' : '') . '>' . $L['LinkAll'] . '</option>';
				}
				$filterRow .= '<select name="configFilters" onChange="window.location=this.value;">' . $selOpts . '</select>';
			} else {
				$filterRow .= $L['LabelNone']; // 'None';
			}
			$filterRow .= '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
		}
		
		// qFilters (Constraints)
		if (( is_array($qFilters) ? !empty($qFilters) : false )) {
			
			if ( isset($qFilters['options']) && (sizeof($qFilters['options']) == 1) && isset($qFilters['options']['_none_']) ) {
				// The only contstraint is the 'All' link (filter none). Display display contaraints.
			} else {
				$filterRow .= $qFilters['label'] . ': ';
				$qOpts = '';
				foreach ($qFilters['options'] as $k => $v) {
					// print_r($v); print '<hr>';
					if (is_array($v)) {
						$qOpts .= '<option value="' . $v['href'] . '"' . ( (isset($v['selected']) && $v['selected']) ? ' selected' : '') . '>' . $v['label'] . '</option>';
						if (isset($v['selected']) && $v['selected'] && ($v['label'] != '')) {
							$qPart = '<strong>' . $L['labelFilterConstraint'] . ':</strong> ' . $v['label'];
						}
					}
				}
				
				if (strlen($qOpts)) {
					$filterRow .= '<select name="qFilters" onChange="window.location=this.value;">' . $qOpts . '</select>';
				} else {
					$filterRow .= $L['LabelNone']; // 'None';
				}
			
			}
		}
		$filterRow .= '
			</td>
		</tr>
		';
		$onRowFilter = $contactPart . $idsPart;
		if (!empty($onRowFilter)) {
			// filter on row
			$filterRow .= '
			<tr>
				<td>
			' . $contactPart . ' ' . $idsPart . '
				</td>
			</tr>
			';
		}
		$filterRow .= '
			</tbody>
		';
		
		$content .= '<table width="100%" border="0" cellpadding="0" cellspacing="0" class="' . $this->getFilterStatus() . '">';
	 
		// activate button and contact/id filter row
		$content .= '
		<tr> 
		<td nowrap><div onClick="showFilter(this);" class="' . (!$this->show_filter_form ? 'shower' : 'hider' ) . '"></div>
		';
		
		// build filter status
		$labels = array();
		if (is_array($bookmark)) {
			$labels[] = $bookmark['label'];
			$title_text = $bookmark['label'];
		} else {
			$labels[] = $configPart;
			$labels[] = $qPart;
			$labels[] = $contactPart;
			$labels[] = $idsPart;
			
		}
		$labels[] = '<strong>' . $L['labelNewSearchFilter'] . '</strong> ' . $showFilters;
		
		$labels_str = '';
		if (is_array($labels)) {
			foreach($labels as $k => $v) {
				if (strlen($v)) {
					$labels_str .= '<span style="margin-left: 8px;">' . $v . '</span>';
				}
			}
		}
		
		$content .= $labels_str;
		$content .= '</td></tr>';
		
		$content .= $filterRow;
		
		$content .= '</table>';
		return $content;
	}
	
	/**
	 *
	 */
	function getMenuRow($menu, $colspan = 2, $tag = 'actions') {
		if (is_array($menu)) {
			$menu = $this->getLinkMenu($menu);
		}

		// if it has a template, just return the menu with no table stuff
		// otherwise we return a string with table tr & td tags
		if ($this->has_template) {
			return $menu;
		} else{
			//<td' . $this->getTagContent($tag) . (($colspan > 1) ? ' colspan="' . $colspan . '"' : '') . '>' . $menu . '</td>
				
			//TODO fix temp code
			$tag = 'actions';
			
			if ($menu) {
				return $this->getBeginRow('', true) . '<td' . $this->getTagContent($tag) . '>' . $menu . '</td>' . $this->getEndRow(true);
			}
		}
	}

	/**
	 *
	 */
	function getTextRow($str, $colspan = 2, $old_param = '-1', $brdr_row = false) {
		//todo become function getTextRow($str, $colspan = 2, $tag = 'lTextCell') {
		//once we have found any breakages with 3 params from before we can reinstate the changed param as
		//function getTextRow($str, $colspan = 2, $tag = 'lTextCell') {	
		if ($str) {
			return $this->getBeginRow('', $brdr_row) . '<td' . $this->getTagContent($old_param) . (($colspan > 1) ? ' colspan="' . $colspan . '"' : '') . '>' . $str . '</td>' . $this->getEndRow($brdr_row);
		}
	}

	/**
	 * @deprecated
	 */
	function getSpacerRow() {
		//return '<tr class="spacer"><td colspan="2"><div>&nbsp;</div></td></tr>';
	}

	/**
	 * @author MCR
	 * specific to item data table
	 */
	function getFieldSpacerRow($colspan = 2) {
		//return '<tr><td>&nbsp;</td></tr>';
		return '<tr class="spacer"><td colspan="' . $colspan . '"><div>&nbsp;</div></td></tr>';
	}

	/**
	 * str to display in first cell,
	 * user pref name to create pref link in second cell,
	 * first colspan number,
	 * target - used by getLinkPref
	 */
	function getUserPrefRow($str, $user_pref, $colspan = 2, $target = '') {
		if ($colspan == '' || $colspan == null || $colspan < 1) {
			$colspan = 2;
		}
		return '
		<tr>
			<td align="right"' . (($colspan > 1) ? ' colspan="' . $colspan . '"' : '') . '>' . $str .
			'&nbsp;&nbsp;&nbsp;' . $this->getLinkPref($user_pref, $target) . '</td>
		</tr>';
	}

	/**
	 * returns a row similar to getUserPref, but with paging info and links.
	 * all variables required for this function should already have been set when setPagingLimits called.
	 *
	 * @author Phil Calder
	 * @param string $mode 'lPagingCell' is default for when locate at end of list.
	 * 				'itemPagingCell' is for item paging 
	 */
	function getPagingRow($mode = 'lPagingCell') {
		// Phil 20060504 We no longer display prefs in the paging row
		$start_num = $this->getPagingItemStartNumber($this->paging_list_size);
		$items_count = $this->paging_items_count;
		$page_num = (isset($_GET['page_num']) && $_GET['page_num'] ? $_GET['page_num'] : 1);
		$n1 = $start_num+1; // we index from 0
		$n2 = ($items_count > ($start_num+$this->paging_list_size) ?  ($start_num+$this->paging_list_size) : $items_count);
		$pref = '';
		if ($items_count > 1) {
			$content = $this->buildPagingRow(
				$this->getPagingLinks($items_count, $page_num, $this->paging_list_size),
				$this->getPagingListShowing($n1, $n2, $items_count),
				$this->getPagingLinkViewAll(($start_num+$this->paging_list_size), $items_count),
				$pref,
				1,
				$mode
			);

		} else {
			$content = '';
		}
		return $content;
	}
	
	function setPagingRowsSwitch($switch = true) {
		$this->paging_rows_swtich = $switch;
	}
	
	function getPagingRowsSwitch() {
		return $this->paging_rows_swtich;
	}

	/**
	 * str1, str2, and str3 should be the output of the paging functions:
	 * getPagingLinks, getPagingListShowing, getPagingLinkViewAll (see below)
	 * in the order you want them displayed
	 * user_pref is for number of items to display per page
	 *
	 * @author Phil Calder
	 */
	function buildPagingRow($str_paging = '', $str2='', $str3='', $user_pref = '', $colspan = 1, $mode = 'lPagingCell') {
		$spacer = '&nbsp;&nbsp;&nbsp;&nbsp;';
		//MCR 20060130 
		//$str_showing_all_pref = $str2 . $spacer . $str3 . '&nbsp;&nbsp;&nbsp;' . $pref;
		$str_showing_extras = $str2 . $spacer . $str3;

		$rows = array();
		if ($this->getPagingRowsSwitch()) {
			$rows[] = $str_paging;
			$rows[] = $str_showing_extras;
		} else {
			$rows[] = $str_paging . ($str_paging ? $spacer : '') . $str_showing_extras;
		}

		//return $this->getNowrapRow($str,  $pref, $colspan);
		$content = '';
		foreach ($rows as $k => $row) {
			$content .= $this->getTextRow($row, $colspan, $mode, true);
		}

		return $content;
	}

	/**
	 * gets the header row for a subtable
	 *
	 * @param array an array containing the title row information similar to a data definition
	 * @param array the query string sort values ie. $_GET['sort']
	 */
	function getHeaderRow($items, $sort = '', $search_filter = false, $extra_action_query = false) {
		// print '<div>Output::getHeaderRow search_filter: ' . ($search_filter ? 'true' : 'false') . '</div>'; print_r($items); print '<hr>';
		$header = ''; // The header content (our return value)
		global $SERVICES;
		if ($search_filter) {
			// if ($_GET['contact_id'] || $_GET['id'] || $_GET['item_id']) {
			if ($_GET['contact_id'] || $_GET['id'] || $_GET['item_id'] || $_GET['field']) { // Phil 20050511 added field
				if ($_GET['id']) {
					$extra_query[] = 'id=' . $_GET['id'];
				}
				if ($_GET['item_id']) {
					$extra_query[] = 'item_id=' . $_GET['item_id'];
				}
				if ($_GET['contact_id']) {
					$extra_query[] = 'contact_id=' . $_GET['contact_id'];
				}
				if ($_GET['field']) {
					$extra_query[] = 'field=' . $_GET['field'];
				}
				$query = '?' . implode('&', $extra_query);
			} else {
				$query = '';
			}
			if (strlen($extra_action_query)) {
				$query = (strlen($query) ? '&' : '?') . $extra_action_query;
			}
			$action = $SERVICES->getGlobal('php_self');
			$begin_form = $this->getBeginForm($action . $query);
		} else {
			$begin_form = '';
		}
		
		global $USER;

		$content = '';
		$search_filter_row = array();
		
		foreach($items as $k => $v) {
			// print '<div>k=' . $k . '</div>'; print_r($v); print '<br>';
			$cell_align = '';
			$col_span_txt = '';
			if ( (strlen($k) == strlen('__cb')) && ($k == '__cb') ) { // Phil 20060227 Prevent $items[0]
				$col_name = $v;
				$cell_align = '';
				$class = 'minWidth headbutt_l';
			} else if ($k == '__links') {
				$col_name = $v;
				$cell_align = '';
				$col_span_txt = ' colspan="2"';
				$class = 'minWidth headbutt_r';
			} else if ($k == '__flags') {
				$col_name = $v;
				$class = 'minWidth headbutt';
			} else {
				$type = $v['type'];
				
				// Phil 20051111 see WebMATList excludeCalculated, excludeForeign, excludeCustom functions
				// (they set $items[$k]['no_sort'] = 1 for any non-sortable fields)
				// see also further down in this function when calling "getHeaderSortLink"
				
				$col_name = ($v['list_label'] ? $v['list_label'] : $v['label']);
				switch ($type) {
					case 'integer':
					case 'float':
					case 'currency':
					case 'percentage':
						$cell_align = ' align="right"';
						break;
					default:
						$cell_align = '';
						break;
				}
				$class = 'headbutt';
			}
			if ($search_filter) {
				// Phil 20060201 see _getClientValidationScript
				$this->ignore_notnull_clientside_check = 1;
				// Phil 20060118
				// if (!in_array($k, array('__cb', '__links', '__flags'))) {
				// print '<p>k=' . $k . ', v=' . $v . ', col_name=' . $col_name . '</p>';
				$search_filter_row[$k]['tagExtra'] = ' id="filterCell_' . $k . '" ';
				if (isset($v['value']) && $v['value'] != -1) {
					$search_filter_row[$k]['tagExtra'] .= 'class="fActv" ';
				}
				$search_filter_row[$k]['content'] = $this->getSearchFilterCell($k, $v, $col_name);
			}
			if ($col_name == '') {
				$col_name = '&nbsp;';
			}
			if ($sort != '') {
				// print '<div>k=' . $k . ' v=</div>'; print_r($v); print '<hr>';
				if (!isset($v['no_sort']) || !$v['no_sort']) { // Phil 20051111
					// print '<div>Output calling getHeaderSortLink</div>'; print_r($v); print '<hr>';
					if ($k == '__links') {// if ($k == '__flags') {
						$col_name = $this->getHeaderSortLink($k, $col_name, $sort);
					} else {
						$col_name = $this->getHeaderSortLink($v['field_key'], $col_name, $sort);
					}
					// $col_name = $this->getHeaderSortLink($k, $col_name, $sort);
					// echo $col_name;
				} else if ($k != '__cb') {
					$class .= ' headerCellTitle';
					//$col_name = '<span' . $this->getTagContent('headerCellTitle') . '>' . $col_name . '</span>';
				}
			} else {
				$class .= ' headerCellTitle';
				//$col_name = '<span' . $this->getTagContent('headerCellTitle') . '>' . $col_name . '</span>';
				//$col_name = '<table' . $this->getTagContent('headerCellTbl') . '><tr><th></th></tr></table>';
			}
			// print '<div>calling getTableHeaderCell(' . $col_name . ')</div>';
			$content .= $this->getTableHeaderCell($col_name, $cell_align . ' class="' . $class . '"' . $col_span_txt);
			
		}
		
		// Main table header (labels, sorting)
		$main_header = '
		<tr' . $this->getTagContent('lHeaderRow') . '>
		' . $content . '
		</tr>
		';
		
		// search filter form
		if ($search_filter) {
			$clear_tag_extras = 'onClick="resetSearchFields(); return false;"'; // Phil: might need to add js to deal with multi select [unrestricted]
			$reset_tag_extras = 'onClick="resetFilterBgs();"';
			$this->req_search_reset_js = true; // see getJs - loads reset function in document head
			global $L;
			$submit = '
				<div align="center">' . 
				$this->getHtmlSubmit($L['labelSearchFilter'], 'submit') . 
				'<br />' . $this->getHtmlSubmit($L['labelSearchFilterClear'], 'clear', $clear_tag_extras) . 
				'<br />' . $this->getHtmlSubmit($L['labelSearchFilterReset'], 'reset', $reset_tag_extras, 'reset') . 
				'</div>';
				
			
			
			// Jack 20060120 Expands _links to fit 3 cols
			$search_filter_row['__links'] = array('content' => $submit, 'tagExtra' => ' colspan="4"', 'tag' => 'th');
			// $search_filter_row['__link'] = array('content' => $submit, 'tagExtra' => ' colspan="4"', 'tag' => 'th');
			unset($search_filter_row['__cb']);
			unset($search_filter_row['__flags']);
			// unset($search_filter_row['__menu']);
			
			// $header .= $this->getTableRow($search_filter_row, 'lFilterRow', $extra);
			// Phil 20060118 - Search row comes before main table header
			// echo nl2br(print_r($search_filter_row, 1));
			$search_header = $this->getTableRow($search_filter_row, 'lFilterRow', $extra);
			
			//Syed 20050728 | added to show/hide the filter search form
			$jsearch_filter="<script type='text/javascript'> function showSearchForm(obj,show,hide) {

						if (obj.innerHTML == show) {
							obj.innerHTML = hide;
							status = 'visible';
							row_status='';

						} else if( obj.innerHTML == hide ) {
							obj.innerHTML = show;
							status = 'hidden';
							row_status='none';

						}

						if (document.getElementById) { // DOM3 = IE5, NS6
							document.getElementById('searchform').style.visibility=status;
							document.getElementById('search_form_row').style.display=row_status;

						} else if (document.layers){ // Netscape 4
							document.searchform.visibility = status;
						} else { // IE 4
							document.all.searchform.style.visibility = status;
						}

					}</script>";
			// $this->addToHead($jsearch_filter);
			$end_form .= $this->getEndForm($SERVICES->getGlobal('php_self'), false);
		}
		$header = $begin_form . $search_header . $main_header . $end_form;
		
		return $header;
	}
	
	function getTableHeaderCell($item, $tag_extras = '') {
		return '
		<th nowrap ' . ($tag_extras ? ' ' . $tag_extras : ''). '>' . $item . '</th>
		';
	}
	
	function getTableRow($item, $class = array(), $cell_format = array(), $data_row_counter = '') {
		if (!is_array($item)) {
			$item = array($item, '', ''); // blank entries for type and detail
		}

		if ($class && !is_array($class)) {
			$class = array($class);
		}

		$row_class = 'r' . $this->row_flag;

		if (is_array($class)) {
			//get images
			$images = '';
			
			//MCR 20060629 alert first reqd for checkbox js
			if (in_array('rAlert', $class)) {
				$images = $this->getFlagAlert() . $images;  //alert is first
				$this->addTagContent($row_class, 'class', 'rAlert');
			}

			if (in_array('rUnseen', $class)) {
				$images .= $this->getFlagUnseen();
				$this->addTagContent($row_class, 'class', 'rUnseen');
			} else if (in_array('rEdited', $class)) {
				$images .= $this->getFlagEdited();
				$this->addTagContent($row_class, 'class', 'rEdited');
			} else if (in_array('rHighLight', $class)) { // Bosco 20050823 | Changed 'rHighlight' to 'rHighLight'	
				$images .= $this->getFlagHighlight();
				// Bosco 20060328 | Changed 'rHighlight' to 'rHighLight'
				$this->addTagContent($row_class, 'class', 'rHighLight');
			} else if (in_array('lFilterRow', $class)) {
			  	$filterClass = $this->getFilterStatus();
				$this->setTagContent($row_class, 'class', $filterClass); //'set' removes r0, r1 etc
			}
			
			if (in_array('rForumPost', $class)) { // Bosco 20051114 | Added rForumPost Class
				$this->setTagContent($row_class, 'class', 'rForumPost');
			}
			
			// Phil 20060117
			if (array_key_exists('__flags', $item)) {
				$item['__flags'] = $images . $item['__flags'];
			}
			
		}

		$content = '';

		if (is_array($item)) {

			foreach($item as $k => $v){
				
				if (empty($v) || $v == ' ') {
					$v = '&nbsp;';
				}
				$td_format = (isset($cell_format[$k]) && $cell_format[$k] ? ' ' . $cell_format[$k] : '');

				
				//always, for now, have nowrap on first 2 cells in row
				if (strstr($v, '<img ')) {
					$td_format .= /*' style="padding: 0px ! important;"*/ ' nowrap';
				}
				if (is_array($v)) {
					if (!isset($v['tag'])) $v['tag'] = 'td';
					if (empty($v['content']) || $v['content'] == ' ') {
						$v['content'] = '&nbsp;';
					}
				  $content .= '<' . $v['tag'] . $v['tagExtra'] . '>' . $v['content'] . '</' . $v['tag'] . '>';
				} else {
					// if (($k == '__menu') || ($k == '__links')) { //(strstr($k, '__') && $k != '__flags') {
					// Phil 20060614 - key can be 0
					if ( (!empty($k)) && (($k == '__menu') || ($k == '__links')) ) {
						$content .= '<th>' . $v . '</th>';
					} else {
						$content .= '<td' . $td_format . '>' . $v . '</td>';
					}
				}
			}
		}

		if (!empty($class) && in_array('lFilterRow',$class) ) {
			if (!$this->show_filter_form) {
				$row_display = 'display: none;';
				$div_visible='hidden';
			}
			$content = '<tbody id="searchform" style="' . $row_display . '">'. 
				'<tr ' . $this->getTagContent($row_class) . ' id="search_form_row">'.
				$content . '</tr></tbody>';
			//$content = '<tr ' . $this->getTagContent($row_class) . ' id="search_form_row" style="display:'.$row_display.'">'.
			//	'<div id="searchform" style="visibility:' . $div_visible . ';">'
			//	.  $content . '</div></td></tr>';
		} else {
			$content = '<tr' . (!empty($data_row_counter) ? $this->getTagContent($row_class) . ' id="row' . $data_row_counter . '" onClick="chgCB(\'' . $data_row_counter . '\');" onMouseOver="rwE(\'' . $data_row_counter . '\', 1);" onMouseOut="rwE(\'' . $data_row_counter . '\', 0);"' : '') . '>' . $content . '</tr>';
		}

		$this->resetTagContent($row_class); //remove special classes etc
		$this->row_flag = 1 - $this->row_flag; //toggle row flag
		return $content;
	}
	
	function getHrRow() {	//draws hr; also keeps the table row striping
		$content = '
		<tr height="6" class="row' . $this->row_flag . '">
			<td colspan="99" style="padding:1px;"><hr width="98%"></td>
		</tr>
		';
		$this->row_flag = 1 - $this->row_flag; //toggle row flag
		return $content;
	}

	/**
	 * added tag_extras - any text to go in input tag (eg onClick="")
	 */
	function getHtmlSubmit($label = 'default', $name = 'submit', $tag_extras = '', $type="submit") {
		global $L;
		return '<input class="buttonBox" type="' . $type . '" name="' . $name . '" value="' . $label . '"'.($tag_extras != '' ? ' ' . $tag_extras : '').'>';
	}

	/**
	 * added tag_extras - any extra text to go in input tag (ie onClick="")
	 */
	function getSubmitRow($label = 'default', $old_blank_cells_before_button = 2, $name = 'submit', $tag_extras = '') {
		global $L;
		if ($label == 'default') {
			$label = $L['ButtonSubmit'];
		}

		// condition if we are using a template
		if ($this->has_template) {
			return $this->getHtmlSubmit($label, $name, $tag_extras);
		}
		
		$content = '<tr>
			<th>&nbsp;</th>
			<td align="left">' . $this->getHtmlSubmit($label, $name, $tag_extras) . '</td>
		</tr>';
		
		return $content;
	}
	
	/**
	 * Similar to {@link getSubmitRow} but with an optional 'extra cell' placeholder for bulk edit checkboxes
	 * @author Phil Calder
	 */
	function getSubmitRowMassEdit($label = 'default', $old_blank_cells_before_button = 2, $name = 'submit', $tag_extras = '') {
		global $L;
		if ($label == 'default') {
			$label = $L['ButtonSubmit'];
		}

		// condition if we are using a template
		// if ($this->has_template) {
			// return $this->getHtmlSubmit($label, $name, $tag_extras);
		// }
		
		$content = '<tr>
			<th>&nbsp;</th>
			<th>&nbsp</th>
			<td align="left">' . $this->getHtmlSubmit($label, $name, $tag_extras) . '</td>
		</tr>';
		
		return $content;
	}

	/**
	 * returns the dropdown with various redirect options
	 * @param string $default_selected
	 * @author Syed, Bosco
	 */
	function getEditRedirectOptions($default_selected = '') {
		// Bosco 20051129 | Added parameter 
	 	global $L;
	 	$links = array(
			'View' => $L['EditRedirectView'],
			'List' => $L['EditRedirectList'],
			'Add' => $L['EditRedirectAdd']
		);
		
		// Phil 20051208 - Adds a View Contact or View Company option if we know the contact or parent company
		if (isset($_GET['contact_id']) && $_GET['contact_id']) {
			$links['Contact'] = $L['labelViewContact'];
		} else if (isset($_GET['item']['contact_company']) && $_GET['item']['contact_company']) {
			$links['Contact'] = $L['labelViewCompany'];
		}
		
		$content = '';
		//MCR converted to radio btns 20060201
		//$content = '<select name="redirect">';
		// Phil 20051208 Removed 'Back to previous page' option
		// if (isset($_SESSION['return_page']) && $_SESSION['return_page']) {
			// $content .= '<option selected>' . $L['EditRedirectBack'] . '</option>';
		// }
		foreach ($links as $k => $v) {
			if ($k == $_POST['redirect']) {
				$selected = "checked";
			} else if ($k == $default_selected){ // if Passed
				$selected = 'checked'; 
			} else {
				$selected = "";
			}
			$content .= '<td><input type="radio" name="redirect" value="' . $k .'"' . $selected . '></td><td>' . $v . '&nbsp;&nbsp;</td>';
		}
		//$content .= '</select>';
		$content = '<table cellpadding="0" cellspacing="0" class="nextPageTbl"><tr>' . $content . '</tr></table>';
		return $content;
	}

	/**                     
	 * gets the next page row
	 * moved from webmat_edit.inc
	 * @param string $default_selected
	 * @return string the next page row for the form
	 * @author Syed Bosco
	 * Bosco 20051129 | Added parameter 
	 */
	function getNextPageRow($default_selected = '') {
		global $L;
		$redirectOptions = $this->getEditRedirectOptions($default_selected);
		if ($this->has_template) {
			return array(
				'label' => $L['NextPage'],
				'input' => $redirectOptions
				);
		} else {
			$content = '<tr class="spacer"><td colspan="2">&nbsp;</td></tr><tr class="buttons"><th>' . $L['NextPage'] . '</th><td>' . $redirectOptions . '</td></tr><tr class="spacer"><td colspan="2">&nbsp;</td></tr>';
			return $content;
		}
	}

	/**
	 * Gets the multiedit submit row with extra add rows
	 */
	function getSubmitRowMultiEdit($show_extra = false, $label = 'default', $blank_cells_before_button = 1, $name = 'submit') {
		if (!$show_extra) {
			return $this->getSubmitRow($label, $blank_cells_before_button, $name, $tag_extras);
		} else {
			global $L;
			if ($label == 'default') {
				$label = $L['ButtonSubmit'];
			}

			$input = array('field_key' => '_extra_rows', 'type'=>'integer', 'min'=>0, 'max'=>100, 'value' => 10, 'width' => 25);
			$content = '
			<tr>
				<td>' . $this->getHtmlField($input, false, $L['labelExtraRows']) . '&nbsp;' . $this->getHtmlSubmit('Add rows', 'add_rows') . '</td>
				'. str_repeat('<td>&nbsp;</td>', $blank_cells_before_button-1) . '
				<td align="right">' . $this->getHtmlSubmit($label, $name, $tag_extras) . '</td>
			</tr>
			';
			return $content;
		}
	}
	
	/**
	 *
	 */
	function getHtmlMasterCheckbox($field_key, $tag_extras = '', $form_name = 'mainform') {
		$this->master_cb_counter++;
		return '<input type="checkbox" class="cb" id="mstrCB' . $this->master_cb_counter . '" name="' . $field_key . '[]" onClick="chkAll(this)"' . ($tag_extras != '' ? ' ' . $tag_extras : ''). '>';
	}

	/**
	 * @var integer $js_validate sets javascript state for the form
	 * 0 is off
	 * 1 is on (means add js call to form tag AND autogenerate field validation)
	 * 2 is partial on; means just add js call to form tag
	 */
	function getBeginForm($action, $js_validate = 1, $encode_type = '', $method = false, $name = 'mainform', $target = false) {
		if ($encode_type) {
			$enctype = ' enctype="' . $encode_type . '" ';
		}
		if (!$method) {
			$method = 'post';
		}
		if ($js_validate) {
			$js_validate_string = ' onSubmit="return validateForm();"';
			if ($js_validate == 1) {
				//set class variable for autogeneration of validateForm function
				$this->js_validate = 1;
			}
		}
		
		// Phil 20060703, updated 20070125
		// $disable_autocomplete = $this->getFormAutoCompleteOff();
		
		return '
		<form name="' . $name . '" ' . (isset($enctype) ? $enctype : '') . ' action="' . $action . '" method="' . $method . '"' . $js_validate_string . '' . $this->getFormAutoCompleteOff('string') . ' ' . ($target ? ' target="' . $target . '"': '') . ' >
		';   //accept-charset="' . $this->charset . '"

	}
	
	function getEndForm() {
		return '
		</form>
		' . $this->getCalendarSetup();
	}
	
	/**
	 * @author Phil Calder
	 */
	function setFormAutoCompleteOff() {
		$this->form_disable_autocomplete = true;
	}
	
	/**
	 * @author Phil Calder
	 */
	function setFormAutoCompleteOn() {
		$this->form_disable_autocomplete = false;
	}
	
	/**
	 * @author Phil Calder
	 */
	function getFormAutoCompleteOff($return_string = false) {
		// return ( (isset($this->form_disable_autocomplete) && $this->form_disable_autocomplete) ? true : false);
		if (isset($this->form_disable_autocomplete) && $this->form_disable_autocomplete) {
			if ($return_string) {
				return ' autocomplete="off"';
			}
		}
		return false;
	}
	
	function getBeginDataCell($padding = true) {
		//return $this->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? ' style="padding: 10px"' : '') . '>';
		return $this->getBeginRow('', true) . '<td id="bodyCell">';
	}
	
	function getEndDataCell() {
		return '</td>' . $this->getEndRow(true);
	}
	
	function getBeginRow($tag_name = '', $brdr_cell = false) {
		$content = '';
		$content .= '<tr' . ($brdr_cell ? ' class="border"' : '') . $this->getTagContent($tag_name) . '>';
		if ($brdr_cell && $this->getPaddedTableSwitch()) {
			//$content .= '<td></td>';
		}
		return $content;
	}
	
	function getEndRow($brdr_cell = false) {
		$content = '';
		if ($brdr_cell && $this->getPaddedTableSwitch()) {
			//$content .= '<td></td>';
		}
		$content .= '</tr>';
		
		return $content;
	}
	
	function getBeginCell($colspan = 1, $tag_name = '', $tag = 'td') {  //MCR 20050110 //$tag_extras = '') { // tag_extras - Phil 20040726
		return '
		<' . $tag . ($colspan > 1 ? ' colspan="' . $colspan . '"' : '') . $this->getTagContent($tag_name) . '>
		';
	}
	
	function getEndCell($tag = 'td') {
		return '
		</' . $tag . '>
		';
	}
	
	function getBeginNestedEditTable() {
		return $this->getBeginNestedTable();
	}
	
	function getBeginNestedViewTable() {
		return $this->getBeginNestedTable();
	}
	
	function getBeginNestedListTable() {
		return $this->getBeginNestedTable('contentTbl');
	}
	

	/**
	 * functions to easily set up our standard nested edit table which sits inside the standard 2 coluun outer table
	 */
	function getBeginNestedTable($table_type = 'nestedTable') {
		return $this->getBeginRow() . $this->getBeginCell(2) . $this->getBeginTable($table_type) . $this->startDataBlock();
	}
	
	function getEndNestedTable() {
		return $this->endDataBlock() . $this->getEndTable() . $this->getEndCell() . $this->getEndRow();
	}

	/**
	 * Generate a 'default' help id based on the current page ($SERVICES->getGlobal('php_self'))
	 */
	function getHelpId() {
		global $SERVICES;
		//NH - 200408130930 - Fix to help system, enables default help link icons to work when browsercrm is installed below the www root
		$dir = str_replace('\\', '/', ($SERVICES->getGlobal('php_self')));
		$dir = str_replace('//', '/', ($dir));
		$dir = trim($dir, '/');
		$parts = explode('/', $dir);
		$size_of = sizeof($parts);
		$dir = $parts[$size_of-2];
		$dir = strtolower($dir);
		$file = ucfirst(str_replace('.php', '', $parts[$size_of-1]));
		$help_id = $dir.$file;
		//module name in lowercase plus page name, ucfirst, without .php, eg mailIndex, tasksEdit
		return $help_id;

	}

	/**
	 * returns a string with a sort link
	 *
	 * @param string $field_key the field key for current item
	 * @param string $col_name the column name
	 * @param array $sort the sort array
	 * @return string the link for sort
	 * @updated Jack Farley 20060120
	 */
	function getHeaderSortLink($field_key, $col_name, $sort) {
		global $SERVICES, $L;
		// print '<div>Output::getHeaderSortLink field_key: ' . $field_key . ', col_name: ' . $col_name . ' sort</div>'; print_r($sort); print '<hr>';
		//  dn_f.gif dn_t.gif up_f.gif up_t.gif
		//  Descending is up arrow
		$sort_type = '';
		$arrow = '';
		
		// image titles
		$desc_title = $L['desc_title'];
		$asc_title = $L['asce_title'];
		
		// images
		$desc_t = 'img/sort_asc_t.gif';
		$desc_f = 'img/sort_asc_f.gif';
		$asc_t = 'img/sort_desc_t.gif';
		$asc_f = 'img/sort_desc_f.gif';
		
		if ($field_key && !is_numeric($field_key)) {
			if ($field_key == $sort['field']) {
				if ($sort['order'] == 'desc') {
					$sort_type = 'asc';
					$arrow = 'desc';
					$image_name_up = $desc_t;
					$image_name_dn = $asc_f;

				} else {
					$sort_type = 'desc';
					$arrow = 'asc';
					$image_name_up = $desc_f;
					$image_name_dn = $asc_t;

				}
			} else { //MCR 20050118
				$arrow = 'off';
				$sort_type = 'asc';
				$image_name_up = $desc_f;
				$image_name_dn = $asc_f;
			}

			// print '<div>sort [sort_table: ' . $sort['sort_table'] . ']</div>'; print_r($sort); print '<hr>';

			$keys = array();
			if (isset($sort['sort_table']) && $sort['sort_table']) {
				$keys['field'] = $sort['sort_table'] . '[sort][field]';
				$keys['order'] = $sort['sort_table'] . '[sort][order]';
			} else {
				$keys['field'] = 'sort[field]';
				$keys['order'] = 'sort[order]';
			}

			// TODO: Replace with below (once fixed)
			$new_query_desc = $this->updateQueryString(
				array(
					'sort[field]' => $field_key,
					'sort[order]' => 'desc',
					'page_num' => ''
				)
			);
			$new_query_asc = $this->updateQueryString(
				array(
					'sort[field]' => $field_key,
					'sort[order]' => 'asc',
					'page_num' => ''
				)
			);
			$new_query = $this->updateQueryString(
				array(
					'sort[field]' => $field_key,
					'sort[order]' => $sort_type,
					'page_num' => ''
				)
			);
			// Phil - 20050506 - temporarily removed (back to above) for now. Needs more work
			// see WebMATList::setSortTable(), WebMATList::sortTable()
			// $new_query = $this->updateQueryString(
				// array(
					// $keys['field'] => $field_key,
					// $keys['order'] => $sort_type,
					// 'page_num' => ''
				// )
			// );

			$link = array('href' => $SERVICES->getGlobal('php_self') . $new_query, 'text' => $col_name);
			$links = $this->getLinkHref($link, false);
			if ($arrow) {
				// Phil 20060418
				// Commented the following 3 if statements. 
				// The images dont align correctly if only 1 of them is a link
				// TODO: Fix this in css and uncomment the if statements
				
			    // Descending arrow (up)
				$image_up = $this->getImageTag($image_name_up, false, ' border="0" width="9" height="5" vspace="1" hspace="2"', $desc_t);
				// if (!($sort['order'] == 'desc' && $field_key == $sort['field'])) {
					$link_up = array('href' => $SERVICES->getGlobal('php_self') . $new_query_desc, 'text' => $image_up, 'title' => $desc_title);
					$image_up = $this->getLinkHref($link_up, false);
				// }
				
				// Ascending arrow (down)
				$image_dn = $this->getImageTag($image_name_dn, false, ' border="0" width="9" height="5" vspace="1"  hspace="2"', $asc_t);
				// if (!($sort['order'] == 'asc' && $field_key == $sort['field'])) {
					$link_dn = array('href' => $SERVICES->getGlobal('php_self') . $new_query_asc, 'text' => $image_dn, 'title' => $asc_title);
					$image_dn = $this->getLinkHref($link_dn, false);
				// }
				
				// Phil 20060418
				// Added a reset sorting option for the flags header cell
				// Phil 20060503 changing to links cell and add icon
				// if ($field_key == '__flags') {
				if ($field_key == '__links') {
					$image_sort = $this->getImageTag(
						'img/rm_f.gif', 
						$L['LabelImgNoSortText'], 
						' border="0" width="13" height="20" ', 
						'img/rm_t.gif'
					);
					
					$no_sort_query = $this->updateQueryString(
						array(
							'sort[field]' => '',
							'sort[order]' => '',
							'page_num' => ''
						)
					);
					
					$reset_sort_link_parameters = array(
						'href' => $SERVICES->getGlobal('php_self') . $no_sort_query,  
						'text' => $image_sort,
						'title' => $asc_title
					);
					$reset_sort_link = $this->getLinkHref($reset_sort_link_parameters, $false);
					return '<table' . $this->getTagContent('headerCellTbl') . '><tr><th' . $this->getTagContent('headerCellTitle') . '>' . $reset_sort_link . '</th></tr></table>';
				}
			}
			//return '<div class="sortArrows">' . $image_up . '<br />' . $image_dn . '</div>' . '<span style="margin-right: 14px">' . $links . '</span>';
			// JACK 210206 - change to table
			return '<table' . $this->getTagContent('headerCellTbl') . '><tr><th' . $this->getTagContent('headerCellTitle') . '>' . $links . '</th><td>' . $image_up . '<br />' . $image_dn . '</td></tr></table>';
		}
		return $col_name;
		//return 'dfd' . $col_name;
	}

	/**
	 * returns a string with a sort link
	 *
	 * @author Matthew Corich
	 * @param string $field_key the field key for current item
	 * @param array $field the field array
	 * @param string $col_name the column name
	 * @return string the link for sort
	 */
	function getSearchFilterCell($field_key, $field, $col_name) {
		// USED FOR ONCHANGE JAVASCRIPT FUNCTION CALL TO CHANGE BACKGROUND COLOUR OF CELL
		//$tag_extras = ' onChange="filterChg(\'' . $field_key . '\')" ';
		$tag_extras = '';
		
		if (is_array($field)) {
			$field_defaults = $this->getSearchFilterDefault($field['type']);

			if ( ($field_defaults['search_options'] && !$field['calculated']) &&
			( ( $field['type'] == 'multi_select' && $field['csv'] ) ||
			($field['type'] != 'dropdown' || $field['type'] != 'multi_select')) &&
			!($field['disable_filter'] == '1') ) {
				// fixups - Brenton Bartel 20050411
				// since we're dealing with filters, we don't require that
				// something be read only, *however*, we do require that
				// a dropdown list have a blank entry, as such, we make up
				// an index called 'is_in_filter' so later on, it will know
				// how to deal with it, (this could be done globally, but meh)
				bcrm_unset($field, 'read_only');
				$field['is_in_filter'] = 1;
				
				$field['width'] = $field_defaults['size']; 
				// Phil 20060616 Can disable date options (before, after, etc)
				if (!isset($field['no_options']) || !$field['no_options']) {
					if (is_array($field_defaults['search_options'])) {
						// $field_defaults['search_options']['no_search'] = '';
						$search_options = array(
							'label' => '',
							'field_key' => '_options_' . $field['field_key'],
							'type' =>'dropdown',
							'dropdown_values' => $field_defaults['search_options'],
							'notnull' => 1,
							'value' => 'no_search',
						);
						if ($field['_search_option_value']) {
							$search_options['value'] = $field['_search_option_value'];
						}
						switch ($field['type']) {
							case 'date':
							case 'date_time':
							case 'integer_date':
							case 'time':
							case 'integer_time':
								$search_options['width'] = $field['width'] + 45;
								$field['hide_mask'] = 1;
								break;
							default:
								$search_options['width'] = $field['width'];
							break;
						}
					} else {
						$search_options = array(
							'label'=>'',
							'field_key'=> '_options_' . $field['field_key'],
							'type'=>'hidden',
							'value' => $field_defaults['search_options']
						);
					}
					$search_options['tag_extras'] .= $tag_extras . ' onChange="filterChg(\'' . $field_key . '\', false)" ';
					$content = $this->getHtmlField($search_options);
					if ($search_options['type'] == 'dropdown') {
						$content .= '<br>';
					}
				}
				switch ($field['type']) {
					case 'long_text':
					case 'medium_text':
						$field['type'] = 'text';
					break;
					case 'date_time':
						$field['type'] = 'date';
					break;
					case 'checkbox':
						if ($field['list_values']) {
							$field['dropdown_values'] = $field['list_values'];
						} else {
							global $L;
							$field['dropdown_values'] = array(0 => $L['False'], 1 => $L['True']);
						}
						$field['csv'] = 1;
					case 'dropdown':
					case 'radio_v':
					case 'radio_h':
						$field['type'] = 'multi_select';
					case 'multi_select':
						$field['size'] = 5;
						$field['blank_label_for_null'] = 0;
						unset($field['width']);
						$tag_extras .= ' onFocus="focusF(this, 1);" onBlur="focusF(this, 0);" class="Norm" ';
					break;
				}
				$field['notnull'] = 0;
				$field['tag_extras'] .= $tag_extras . ' onChange="filterChg(\'' . $field_key . '\', true)" ';
				
				// TODO: Phil 20060615
				// $date_field_types = array('date');
				// if ($field['type'] && in_array($field['type'], $date_field_types)) {
					// $field['search_multiple'] = true;
					// // TODO: handle reset js for multiple search items
				// }
				$content .= $this->getHtmlField($field);
				// Phil 20060201
				// print '<div>addToResetJs(' . $field_key . ', ' . $field['field_key'] . ', ' . $field['type'] . ')</div>';
				$this->addToResetJs($field_key, $search_options['field_key'], $field['type']);
				return $content;
			}
		} else {
			return '';
		}
	}
	
	/**
	 * @author Phil Calder
	 */
	function addToResetJs($field_key, $option_key, $type) {
		$this->reset_js_fields[] = array(
			'field_key' => $field_key, 
			'option_key' => $option_key, 
			'type'      => $type
		);
	}
	
	/**
	 * @author Phil Calder
	 */
	function getResetJs() {
		// The standard html reset button sets form fields to "0" - we need to reset to "" in some cases (eg group dropdowns have unrestricted as "0")
		$form_name = 'document.forms["mainform"]';
		$content = '';
		$tmp = '';
		if (isset($this->reset_js_fields) && is_array($this->reset_js_fields) && count($this->reset_js_fields) ) {
			$content .= '
			function resetSearchFields() {
				var tmp_fields = "";
				var tmp_options = "";
			';
			foreach ($this->reset_js_fields as $k => $v) {
				switch ($v['type']) {
					// case 'dropdown':
					case 'multi_select':
						// print '<div>++ is a multi select ++</div>';
						$content .= '
						if (' . $form_name . '["item[' . $v['field_key'] . '][]"]) {
							' . $form_name . '["item[' . $v['field_key'] . '][]"].selectedIndex = ""; 
						}
						';
					break;
					default:
						$content .= '
						if (' . $form_name . '["item[' . $v['field_key'] . ']"]) {
							' . $form_name . '["item[' . $v['field_key'] . ']"].selectedValue = ""; 
							' . $form_name . '["item[' . $v['field_key'] . ']"].checked = "false";
							' . $form_name . '["item[' . $v['field_key'] . ']"].value = "";
						}
						';
						$content .= '
						if (' . $form_name . '["item[' . $v['option_key'] . ']"]) {
							' . $form_name . '["item[' . $v['option_key'] . ']"].selectedIndex = ""; 
							' . $form_name . '["item[' . $v['option_key'] . ']"].checked = "false";
							' . $form_name . '["item[' . $v['option_key'] . ']"].value = "";
						}
						';
					break;
				}
			}
			// $content .= '
				// alert("fields: " + tmp_fields + " options: " + tmp_options);
				//';
				
			$content .= '
			}
			';
			
			// ADD FUNCTION FOR RESET BUTTON
			$content .= '
			function resetFilterBgs() {
			';
			foreach ($this->reset_js_fields as $k => $v) {
			 $content .= '
				resetFilterChg(\'' . $v['field_key'] . '\');
				 ';
			}
			$content .= '
			}
			';
		}	
		return $content;
	}

	/**
	 * Returns the default settings for a search filter field
	 *
	 * @param string $field_type the field type
	 * @return array with the keys size and search_options
	 */
	function getSearchFilterDefault($field_type) {
		if (!$this->search_filter_defaults) {
			global $L;
			$number_options = array(
				'=' => $L['LabelEqualTo'], 
				'>' => $L['LabelGreaterThan'], 
				'<' => $L['LabelLessThan']
			);
			$text_options = array(
				'CONTAINS' => $L['LabelContains'], 
				'=' => $L['LabelIs'], 
				'LIKE' => $L['LabelLike']
			);
			$date_options = array(
				'=' => $L['LabelEqualTo'], 
				'BEFORE' => $L['LabelBefore'], 
				'AFTER' => $L['LabelAfter'], 
				// 'BETWEEN' => $L['LabelBetween'] // TODO: Phil 20060615
			);
			//text fields
			$this->search_filter_defaults['text'] = array ('size' => 80, 'search_options' => $text_options);
			$this->search_filter_defaults['medium_text'] = array ('size' => 120, 'search_options' => $text_options);
			$this->search_filter_defaults['long_text'] = array ('size' => 120, 'search_options' => $text_options);
			$this->search_filter_defaults['email'] = array ('size' => 120, 'search_options' => $text_options);
			$this->search_filter_defaults['url'] = array ('size' => 120, 'search_options' => $text_options);//Syed 20050630
			// number fields
			$this->search_filter_defaults['integer'] = array ('size' => 80, 'search_options' => $number_options);	//MCR 20051128 80
			$this->search_filter_defaults['float'] = array ('size' => 80, 'search_options' => $number_options);//MCR 20051128 80
			$this->search_filter_defaults['currency'] = array ('size' => 80, 'search_options' => $number_options);//MCR 20051128 80
			$this->search_filter_defaults['percentage'] = array ('size' => 80, 'search_options' => $number_options);//MCR 20051128 80
			// misc fields
			$this->search_filter_defaults['dropdown'] = array ('size' => 100, 'search_options' => 'dropdown');	//MCR 20051128 100
			$this->search_filter_defaults['radio_v'] = array ('size' => 100, 'search_options' => 'dropdown');	//MCR 20060201
			$this->search_filter_defaults['radio_h'] = array ('size' => 100, 'search_options' => 'dropdown');	//MCR 20060201
			$this->search_filter_defaults['multi_select'] = array ('size' => 100, 'search_options' => 'multi_select');  //MCR 20051128 100
			$this->search_filter_defaults['checkbox'] = array ('size' => 80, 'search_options' => 'checkbox');
			//date fields
			$this->search_filter_defaults['date'] = array ('size' => 80, 'search_options' => $date_options);
			$this->search_filter_defaults['date_time'] = array ('size' => 80, 'search_options' => $date_options);
			$this->search_filter_defaults['integer_date'] = array ('size' => 80, 'search_options' => $date_options);
			$this->search_filter_defaults['time'] = array ('size' => 80, 'search_options' => $date_options);
			$this->search_filter_defaults['integer_time'] = array ('size' => 80, 'search_options' => $date_options);
			//non-searchable fields
			$this->search_filter_defaults['contact'] = false;
			$this->search_filter_defaults['password'] = false;
			$this->search_filter_defaults['hidden'] = false;
			$this->search_filter_defaults['file_upload'] = false;
		}
		return $this->search_filter_defaults[$field_type];
	}

	/**
	 * Displays an image
	 *
	 * @param string $image the path to the image (if in skins only the filename)
	 * @param string $title the text to be used for title and the alt tag (optional)
	 * @param string $tag_extras extra tag information (optional)
	 * @param boolean $in_skins_folder used to indicate if the image is in the
	 * skins folder (optional)
	 * @return string the img tag or '&nbsp;' if the image dosen't exist
	 */
	function getImageTag($image, $title = false, $tag_extras = false, $overImage = false) { //, $in_skins_folder = false
		// Jack 20060120; MCR 20060131 added .=
		if ($overImage) {
			$tag_extras .= ' onMouseOut="swapImage(this, \'' . $this->getSkinWebPath() . $image . '\')" onMouseOver="swapImage(this,\'' . $this->getSkinWebPath() . $overImage . '\')" ';
		}
		return '<img src="' . $this->getSkinWebPath() . $image . '" alt="' . $title . '" ' . $tag_extras . ' title="' . $title . '">';
	}
	
	// commonly used links ----------------------------------
	
	function getDefaultLinkTarget() {
		return $this->default_link_target;
	}
	
	function setDefaultLinkTarget($target) {
		$this->default_link_target = $target;
	}

	/**
	 * getLinkHref
	 *
	 * @param $link an array or string link (with the keys 'href', 'text' , 'title'
	 * @param boolean check_language - Whether to lookup up language keys - Optional, default true
	 * @return an html a tag with href, text, title, tag_extras
	 * Phil 20040922
	 */
	function getLinkHref($link, $check_language = true) {
		$text_link = '';
		if (is_array($link)) {
			$href = $link['href'];
			if (isset($link['text']) && $link['text']) {
				if ($check_language) {
					global $SERVICES;
					$SERVICES->getService('Language');
					$text = $SERVICES->language->getLanguageText($link['text']);
					$title = (isset($link['title']) && $link['title'] ? $link['title'] : $text);
				} else {
					$text = $link['text'];
				}
			} elseif ($link['img']) {
				$text = $link['img'];
				$title = '';
			} else {
				$text = $link['href'];
				$title = '';
			}
			if (isset($link['target']) && $link['target']) {
				$target = ' target="'. $link['target'] . '" ';
			} else if ($this->getDefaultLinkTarget()) {
				$target = ' target="'. $this->getDefaultLinkTarget() . '" ';
			}
			$tag_extras = (isset($link['tag_extras']) ? $link['tag_extras'] : '');

		} else if (is_string($link)) {
			// Log this
			$this->logError('NOTICE: getLinkHref was sent a string link [' . $link . ']', 'Output');
			$href = $link;
			$text = $link;
			$tag_extras = '';
			$title = '';
		}
		
		if (is_array($link) && isset($link['text_only']) && $link['text_only']) {
			$text_link = $text;
		} else {
			$text_link .= '<a href="' . $href . '"' . $target . ($title ? ' title="' . $title . '"' : '')
			. ($link['tag'] ? ' ' . $this->getTagContent($link['tag']) : '')
			. ($tag_extras ? ' ' . $tag_extras : '')
			. '>' . $text . '</a>';
		}
		///. '>' . $img . $text . '</a>';
		return $text_link;
	}

	/**
	 * getLinkAll
	 *
	 * @param $action The complete url to proceed to.
	 * @return link including href and text
	 * @author Phil Calder
	 */
	function getLinkAll($action, $text = '') {
		global $L;
		$text = (!empty($text) ? '' . $text : $L['ListAll']);
		$img = $this->getImageTag('img/table.png', false, 'width=16 height=16 align="absmiddle"') . ' ' . $text;
		return $this->getLinkHref(array('href'=>$action, 'text'=>$img, 'title' => $text), false);
	}

	/**
	 * getLinkNew
	 *
	 * @param $action The complete url to.
	 * @return link including href and text
	 * @author Phil Calder
	 */
	function getLinkNew($action, $text = '') {
		global $L;
		$text = (!empty($text) ? '' . $text : $L['LinkNew']);
		$img = $this->getImageTag('img/page_add.png', false, 'width=16 height=16 align="absmiddle"') . ' ' . $text;
		return $this->getLinkHref(array('href'=>$action, 'text'=>$img, 'title' => $text), false);
	}

	/**
	 * getLinkDelete
	 *
	 * @param string $action The complete url to proceed to once the user has confirmed.
	 * @param string $text Optional, defaults to the 'LinkDelete' language item.
	 * @param boolean $as_array Optional, defaults to false. When true, returns link as an array for a toolbar link.
	 * @return link including href, text, and javascript confirmation
	 * @author Phil Calder
	 */
	function getLinkDelete($action, $text = '', $as_array = false) {
		global $L;
		
		$text = (!empty($text) ? '' . $text : $L['LinkDelete']);
		$this->has_delete_link = true;
		if (!$as_array) {
			return $this->getLinkHref(array('href'=>$action, 'text'=>$text, 'tag_extras'=>'onclick="return getDeleteMessage()"'), false);
		}
		return array(
			'href' => $action,
			'handler1' => 'getDeleteMessage()',
			'text' => $text,
			'icon' => 'page_delete.png',
			'title' => $text
		);
	}

	/**
	 * getLinkView
	 *
	 * @param $action The complete url.
	 * @return link including href, and text.
	 * @author Phil Calder
	 */
	function getLinkView($action, $text = '') {
		global $L;
		$text = (!empty($text) ? '' . $text : $L['LabelView']);
		return $this->getLinkHref(array('href'=>$action, 'text'=>$text), false);
	}

	/**
	 * getLinkHide
	 *
	 * @param $action The complete url.
	 * @return link including href, and text.
	 * Brenton 20050331
	 */
	function getLinkHide($action, $text = '') {
		global $L;
		$text = (!empty($text) ? '' . $text : $L['LabelHide']);
		return $this->getLinkHref(array('href'=>$action, 'text'=>$text), false);
	}

	/**
	 * getLinkUnhide
	 *
	 * @param $action The complete url.
	 * @return link including href, and text.
	 * Brenton 20050331
	 */
	function getLinkUnhide($action, $text = '') {
		global $L;
		$text = (!empty($text) ? '' . $text : $L['LabelUnhide']);
		return $this->getLinkHref(array('href'=>$action, 'text'=>$text), false);
	}

	/**
	 * getLinkEdit
	 *
	 * @param $action The complete url.
	 * @return link including href, and text.
	 * @author Phil Calder
	 */
	function getLinkEdit($action, $text = '') {
		global $L;
		$text = (!empty($text) ? '' . $text : $L['LinkEdit']);
		return $this->getLinkHref(array('href'=>$action, 'text'=>$text), false);
	}

	/**
	 * getLinkList
	 *
	 * @param $action The complete url.
	 * @return link including href, and text.
	 */
	function getLinkList($action, $text = '') {
		global $L;
		$text = (!empty($text) ? '' . $text : $L['ListView']);   //MCR 20060125; was ListAll
		return $this->getLinkHref(array('href'=>$action, 'text'=>$text), false);
	}

	/**
	 * getLinkConfirm
	 *
	 * @param $action The complete url to proceed to once the user has confirmed.
	 * @param $text Optional, defaults to 'Delete'.
	 * @param $confirm_text Optional, defaults to 'Are you sure you want to delete'.
	 * @return link including text, and javascript confirmation and javascript redirect
	 * see also Page::getLinkDelete
	 * +++ Use getLinkDelete +++
	 * @ author Phil Calder
	 */
	function getLinkConfirm($action, $text = '', $confirm_text = '', $toolbar = false) {
		global $L;
		if (empty($confirm_text)) {
			$confirm_text = $L['DeleteAreYouSure']; // We could do better here
		}

		$text = (!empty($text) ? '' . $text : $L['LinkDelete']);
		
		if ($toolbar || $this->toolbar_link_mode) {
			return array(
				'href' =>$action,
				'handler1' => 'confirm(\'' . $confirm_text . '\')',
				// 'script' => $url, 
				'title' => $text, 
				'text' => $text, 
				'icon' => 'page_misc.png'
			);
		}

		return $this->getLinkHref(
			array(
				'href'=>'javascript:if(confirm(\'' . $confirm_text . '\')){window.location.replace(\''. $action . '\');}',
				'text'=>$text
			),
			false
		);
	}


	/**
	 * getLinkDashIcon
	 *
	 * @param $action The complete url to.
	 * @return mixed - string link including href and text, or boolean false if the image does not exist
	 * @author Phil Calder
	 * Phil 20041109
	 * Phil 20050427 - If param link[if_exists] is true, will return false if the icon does not exist
	 */
	function getLinkDashIcon($link, $tag = 'icon') {
		global $L;
		$if_exists = ( (isset($link['if_exists']) && $link['if_exists']) ? true : false);
		bcrm_unset($link, 'if_exists');

		$link['img'] = $this->getImageTag($link['icon'], $link['title'], $this->getTagContent($tag));
		// print '<div>img: ' . $link['icon']  . '</div>';

		if ($if_exists && !file_exists($this->getSkinFilePath() . $link['icon'])) {
			return false;
		}

		bcrm_unset($link, 'icon');

		return $this->getLinkHref($link, false);
	}

	/**
	 * gets the filter list links and sets the filter user pref
	 *
	 * @param array $filters_options the list for possable filter values
	 * @param string $module_name the current module name
	 * @param string $filter the value passed from the filter
	 * @param string $pref_name the name for the user prefer you want to store the filter info to
	 * @return string a string containing the filter menu items
	 *
	 * MSC 20040815
	 * Phil 20040922, 20051025
	 */
	function getLinkFilter($filter_options, $module_name, $filter = false, $filter_off = true , $use_select = false) {
		global $SERVICES;
		$SERVICES->getService('Preferences');

		$filter = 'f' . $filter;

		$filter_links = '';
		$filter_links = array();
		if ($filter_off) {
			$filter_options['none'] = array('label' => 'msgNoFilters');
		}

		if (is_array($filter_options)) { // Phil 20050722 - added 'if'
			foreach ($filter_options as $k => $v) {
				// print '<div>k=' . $k . '</div>'; print_r($v); print '<hr>';
				if (is_array($v)) { // Phil 20051025
					
					$new_query = $this->updateQueryString(array('filter' => 'f' . $k, 'page_num' => ''));
					$label_text = $SERVICES->language->getLanguageText($v['label'], $module_name);
	
					if ($use_select) {
						$filter_link = array(
							'href' => $SERVICES->getGlobal('php_self') . $new_query,
							'text' => $label_text,
							'selected' => ($filter == 'f' . $k ? true : false )
						);
						$filter_links[] = $filter_link;
					} else {
						if ($filter == 'f' . $k) {
							$filter_links[] = $label_text;
						} else {
							 $filter_links[] = $this->getLinkHref(array('href'=>$SERVICES->getGlobal('php_self') . $new_query, 'text'=>$label_text), false);
						}
	
					}
				}
			}
		}

		if ($use_select) {
			return $this->getRedirectSelect($filter_links);
		} else {
			return $this->getLinkMenu($filter_links);
		}
	}
	
	/**
	 * @return array The list table links, values: first_link and optional other links
	 * @author Phil Calder
	 */
	function getListTableLinks($table_links, $identifier, $default = 'edit') {
		// print '<div>default: ' . $default . '</div>'; print_r($table_links); print '<hr>';
		$first_link = '&nbsp;';
		$links = '&nbsp;';
		if (is_array($table_links)) {
			if ($default && array_key_exists($default, $table_links) && strlen($table_links[$default]) ) {
				$first_link = $table_links[$default];
			} else if (array_key_exists('edit', $table_links) && strlen($table_links['edit']) ) {
				$first_link = $table_links['edit'];
			} else {
				if (count($table_links)) {
					// reset($table_links);
					// $first_link = current($table_links);
					// Phil 20060203 Make sure the first link has a value
					$first_is_set = false;
					foreach ($table_links as $k => $v) {
						if (!$first_is_set && $v && ($v != '&nbsp;')) {
							$first_link = $v;
							$first_is_set = true;
						}
						if (!$v || ($v == '&nbsp;')) {
							bcrm_unset($table_links, $k);
						}
					}
				}
			}
			// $links = $first_link; 
			if (count($table_links) > 1) {
				$links = '
				<a href="javascript:sM(\'' . $identifier . '\');"><img src="' . $this->getSkinWebPath() . 'img/menu.gif" width="11" height="11" id="butt' . $identifier .'" style="vertical-align: top;" alt="" /></a>
				<div id="men' . $identifier .'" onMouseOut="hM(\'men' . $identifier . '\')" onMouseOver="kM(\'men' . $identifier . '\');" class="menHide">' . implode('', $table_links) . '</div>';
			}
		}
		return array($first_link, $links);
	}

	/**
	 * Creates a select redirect menu
	 *
	 * @param array $links an array of link (with the keys 'href', 'text' and
	 * 'selected'
	 * @param booluan
	 * @return string the menu
	 */
	function getRedirectSelect($links, $on_change = false) {
		if (count($links)) {
			if ($on_change) {
				$select = ' onchange="redirect(this.form.selectmenu.value)"';
				$button = '';
			} else {
				$select = '';
				$button = "\n" . '<button name="go" type="button"  onclick="redirect(this.form.selectmenu.value)">Go</button>' . "\n";
			}

			$select_menu = '
				<form>
					<select name="selectmenu"' . $select . '>';
			foreach ($links as $k => $v){
				$selected = '';
				if ($v['selected']) {
					$selected = 'selected="selected"';
				}
				if ($v['href'] == "#") {
					$v['href'] = "";
				}
				$select_menu .= '<option ' . $selected . ' value="' . $v['href'] . '">' . $v['text'] . '</option>';
			}
			$select_menu .= '
					</select>' . $button . '
				</form>';
			$this->js_redirect = true;
			return $select_menu;
		}
		return '';
	}


	/**
	 * returns a list of breadcrumb links
	 *
	 * @param array
	 * @return sting the breadcrumb links
	 */
	function getLinkBreadcrumbs($links) {
		$breadcrumbs = '';
		$separator = ' > ';
		if ($crumb_count = count($links)) {
			foreach ($links as $k => $v) {
				if ($k != ($crumb_count-1)) {
					$breadcrumbs .= $this->getLinkHref($v) . $separator;
				} else {
					$breadcrumbs .= $v['text'];
				}
			}
		}
		return $breadcrumbs;
	}
	
	/**
	 * @param mixed $contact Either an integer contact_id, or an array containing contact id (id) and name (display_name)
	 * @author Phil Calder
	 */
	function getContactName($contact) {
		// print '<div>getContactName(' . $contact . ')</div>'; print_r($contact); print '<hr>';
		global $SERVICES, $L, $USER;
		if (!$contact || (is_array($contact) && !$contact['id']) ) { // Phil - 20050204 - saves processing if we return here
			// $this->printDebug('Output::getLinkContact no id specified');
			return '';
		}

		$ua =& $SERVICES->getService('Authentication');

		if (!$ua->readAccess($USER['user_id'], 'Contact')) {
			// $this->printDebug('Output::getLinkContact read access denied');
			return '';
		}
		
		// print '<div>Output::getContactName</div>'; print_r($contact); print '<hr>';
		
		$SERVICES->getService('Modules');
		$contact_module =& $SERVICES->modules->getModule('Contact');
		if (is_object($contact_module)) {
			if ($contact) {
				if (is_array($contact)) {
					if ($contact['id']) {
						if ($contact['display_name']) {
							$contact_name = trim($contact['display_name']);
						} else {
							$contact_name = trim($contact_module->getName($contact['id']));
						}
						$contact_id = $contact['id'];
						$id = $contact['id'];
					}
				} else {
					if ($contact) {
						$contact_name = $contact_module->getName($contact);
						// print '<div>contact_name: ' . $contact_name . '</div>'; print '<hr>';
						$contact_id = $contact;
					}
				}
				if (!$contact_name) {
					$contact_name = $L['Contact'] . ':&nbsp;' . $contact_id;
				}
				return $contact_name;
			}
			// $this->printDebug('Output::getContactName no id specified');
			return '';
		}		
	}

	/**
	 * Creates a link to a contact
	 *
	 * @param mixed $contact Either the integer contact id, oor an array containing id and (optional) display_name keys
	 * @param string $target frame - (optional)
	 * @param boolean $popup_only - (optional)
	 */
	function getLinkContact($contact, $target = '', $popup_only = 0, $tag_extras = '') {	//, $show_name = true
		
		if ( ! ($contact_name = $this->getContactName($contact)) ) { // Phil 20050909
			return '';
		}
		
		$contact_id = ( (is_array($contact) && $contact['id']) ? $contact['id'] : (int)$contact);
		
		global $SERVICES;

		// if ($target) {
			// $target_str = 'target=' . $target . '';
		// } else {
			// $target_str = '';
		// }

		$SERVICES->getService('Config');
		$SERVICES->getService('Modules');
		
		$contact_module =& $SERVICES->modules->getModule('Contact');
		
		if (is_object($contact_module)) {
			
			if ($contact_id) {
				
				$link = $this->getLinkHref(
					array(
						'href' => $SERVICES->config->getModuleHref('Contact', 'view') . '?id=' .  $contact_id,
						'text' => $contact_name, 
						'target' => $target, 
						'tag_extras' => $tag_extras
					),
					false // no language text check
				);

				if (1) {
					// popup only? unset the link
					if (isset($popup_only) && ($popup_only == 1)) {
						$link = $contact_name;
					}

					if ($this->getShowContactPopup()) { // Phil 20050908
						
						$this->req_contactMenu_js = true;
						
						// Phil 20060504 Not using the 'Open in a new window' popup
						// $selector_options = array(
							// 'vars' => array('contact_id'),
							// 'url' => $SERVICES->config->getModuleHref('Contact', 'view') . '?id=%contact_id%',
							// 'window_name' => 'contact_view',
							// 'resizable' => true,
							// 'scrollbars' => true,
							// 'width' => 760,
							// 'height' => 580,
						// );
						// $this->addJSPopup('contactView', $selector_options);
						
						// Phil 200603?? Contact menu enabled 
						// [todo: replace with $this->getShowContactLinkMenu(), similar to getShowContactPopup]
						if ($popup_only < 2) { 
							//todo
							$this->contact_menu_counter++;
							// Contact menu (dropdown/iframe)
							// Bosco 20060327 | Removed margin space after contact menu image
							$contact_menu_link = $this->getLinkHref(
								array(
									'href' => 'javascript:contactMenu(\'' . 'cMen' . $this->contact_menu_counter . '\',\'' . $contact_id . '\');',
									'img' => $this->getFlagMenu('cMen' . $this->contact_menu_counter),
									//'tag_extras' => ' onMouseOut="//hM(\'contactMen\')"'
									'tag_extras' => ' onMouseOut="//hM(\'contactMen\')" '
								),
								false // no language text check
							);
							
							// Phil 20060504 Added space between links
							$link = $contact_menu_link . ' ' . $link;
						}
					}
				}
				return $link;
			}
		}
	}
	
	/**
	 * @author Phil Calder
	 * @param boolean Whether to display contact links with a popup (new window) option
	 */
	function setShowContactPopup($status = true) {
		$this->show_contact_popup = ($status ? true : false);
	}
	
	/**
	 * @author Phil Calder
	 * @return boolean Whether to display contact links with a popup (new window) option
	 */
	function getShowContactPopup() {
		return $this->show_contact_popup;
	}

	/**
	 *
	 * @param string show_selector - (optional)
	 * @param string $selector_field - (optional)
	 $show_selector = true, $selector_field = 'contact_id'
	*/
	function setShowContactSelector($val = true) {
		$this->show_contact_selector = $val;
	}

	/**
	 *
	 */
	function getShowContactSelector() {
		return $this->show_contact_selector;
	}
	
	/**
	 * @param array $field Optional, uses keys contact_company (0 or 1), is_person (0 or 1), search_filter (on or off)
	 * @access private
	 */
	function loadContactSelectorJs($field = array()) {
		global $SERVICES;
		
		$selector_url = $SERVICES->config->getModuleHref('Contact', 'contact_selector');
		// Phil 20051220 Pass contact_company or is_person to selector
		// Selector will filter on company, or only display companies
		// see Company edit class for an example
		if (isset($field['contact_company']) && $field['contact_company']) {
			$selector_url .= '&contact_company=' . $field['contact_company'];
		} else if (isset($field['is_person']) && $field['is_person']) {
			$selector_url .= '&is_person=' . $field['value'];
		}
		
		// Phil 20060428
		if (isset($field['search_filter']) && $field['search_filter']) {
			$selector_url .= '&search_filter=' . $field['search_filter'];
		}
		
		// Bosco 20051129 | Added save_to and focus
		$selector_options = array(
			'vars' => array('field'),
			'url' => $selector_url,
			'window_name' => 'contact_selector',
			'resizable' => true,
			'scrollbars' => true,
			'width' => 380,
			'height' => 580,
			'save_to' => 'newWin',
			'focus' => true
		);
		$this->addJSPopup('contactSelector', $selector_options);
	}

	/**
	 * Creates a link to a contact for Edit mode
	 *
	 * Handles display only mode (no_link)
	 *
	 * @param string $id the contact id
	 * @param string $target frame - (optional)
	 * @author Phil Calder
	 */
	function getEditLinkContact($field, $id_item_key) {
		global $SERVICES, $L;
		// print '<div>id_item_key: ' . $id_item_key . '</div>'; print_r($field); print '<hr>';

		if (!$this->getShowContactSelector() || (isset($field['no_link']) && $field['no_link']) || (isset($field['read_only']) && $field['read_only'])) {
			if (is_array($field['value'])) { // && $field['value']['id']) {
				$contact_id = $field['value']['id'];
			} else {
				$contact_id = $field['value'];
			}
			// print '<p>returning getLinkContact(' . $contact_id .')</p>';
			$id_field = '<input type="hidden" name="item' . $id_item_key . '[' .	$field['field_key'] . ']" value="' . $field['value'] . '">';
			return $id_field . ( (isset($field['no_link']) && $field['no_link']) ? $this->getContactName($contact_id) : $this->getLinkContact($contact_id) );
		}
		
		// Phil 20060703 - added this as a separate function
		$this->loadContactSelectorJs($field);

		$select_link = $this->getLinkHref(array(
			'tag_extras' =>' class="contactChg" onClick="javascript:contactSelector(\'item' . $id_item_key . '[' . $field['field_key'] . ']\');" ',
			'href' => 'javascript:;',
			'text' => $L['labelContactSelector'],
			'tag' => 'contactActionLink'
			));

		$html_id = '_' . $field['field_key'] . '_text';

		$id_field = '<input type="hidden" name="item' . $id_item_key . '[' .	$field['field_key'] . ']" value="' . $field['value'] . '">';


		if ($field['value']) {
			$SERVICES->getService('Modules');
			// $SERVICES->service->printDebug('-- Output 1 --');
			$SERVICES->getService('Config');
			$contact_link = $SERVICES->config->getModuleHref('Contact', 'view') . '?id=' . $field['value'];
			$contact_module =& $SERVICES->modules->getModule('Contact');
			if (is_array($field['value'])) {
				$contact_id = $field['value']['id'];
			} else {
				$contact_id = $field['value'];
			}
			$contact_name = $contact_module->getName($contact_id);
		} else {
			$contact_name = $L['NoContactLink'];
			$contact_link = '';
		}

		if (!$contact_name && $field['value']) {
			$contact_name = $L['Contact'] . ': ' . $field['value'];
		}

		if (strstr($field['field_key'], '][') ) {
			$text_label = substr(rtrim($field['field_key'], ']'), (strrpos($field['field_key'], '[') +1));
		} else {
			$text_label = $field['field_key'];
		}
		$contact_link = $this->getLinkHref(array(
			//'tag_extras' =>'  id="_' . $field['field_key'] . '_text" ',
			'tag_extras' =>'  id="_' . $text_label . '_text" ',
			'href' => $contact_link,
			'text' => $contact_name,
			'tag' => 'eContactLink'
			));

		return $id_field . $contact_link . ' &nbsp; ' . $select_link;
	}

	/**
	 * @author Phil Calder
	 */
	function getLinkUser($id, $target = '') {
		global $L, $USER;
		if ($target) {
			$target_str = 'target=' . $target . '';
		} else {
			$target_str = '';
		}

		if ($id) {
			$users=$this->getUsersList();
			return $users[$id];
		} else {
			//return $L['NoContactLink'];
			return '';
		}
	}

	/**
	 * returns a link to an email address
	 *
	 * @param string $email recipients email address
	 * @param string $message a url friendly string to be included in the body of the new message window (Phil 20040811)
	 */
	function getLinkEmail($address = '', $text = '') {
		if ($address) {
			$url = '../../modules/Mail/edit.php?address=%address%';
			$url = str_replace('%address%', $address, $url);
			
			$text = ( (isset($text) && $text) ? $text : $address);

			$link = $this->getLinkHref(
				array(
					//'tag_extras' =>' onClick="emailCompose(\'' . $address_id. '\', \'' . $address . '\');" ',
					'href' => $url,
					'text' => $text
				),
				false // no language text check
			);
			return $link;
		} 
		return '';
	}

	/**
	 * pref can be a string, or an array of strings containing user pref names.
	 * target is the return page to load in the referring window (a url).
	 * if target is the word 'parent', the referring pages parent frame will be reloaded.
	 * This function brings up the user preference dialog, unless companyPrefs = true
	 */
	function getLinkPref($pref, $target = '', $companyPrefs = false) {
		global $USER, $L;

		$height = 150; // height for 1 pref
		if ($companyPrefs) {
			$height += 80; // more text in company pref dialog
		}

		$increment = 25; // incremet height this much if more prefs are to be displayed

		// Might be more than one pref
		if (is_array($pref)) {

			$pref_arr = array();
			foreach ($pref as $k=>$val) {
				$pref_arr[$k] = 'prefs_inc[' . $k . ']=' . $val;
			}

			$pref = implode('&', $pref_arr);
			$height = $height + ($increment * (sizeof($pref_arr)-1));

		} else {
			$pref = 'prefs_inc[0]=' . $pref;
		}

		$this->has_pref_button = true;

		if ($target) {
			$target = base64_encode($target);
			$target = '&target=' . $target;
		}

		$content = '
		<a href="../' . ($companyPrefs ? 'UserAdmin' : 'Front') . '/preferences.php?' . $pref  . $target . '" target="_blank"
		onClick="javascript: window.open(this.href, this.target, \'scrollbars=no, menubar=no, resizable=yes, width=550, height=' . $height . ' \'); return false;">' .
		$this->getImagePref() . $L['PrefsLink'] . '</a>';

		return $content;
	}
	
	function getImagePref() {
		return ''; //MCR 20060413 no more buttons
	}
	
	function getLinkBack($url = '', $title = '') {
		global $SERVICES;
		$type = 0; // script
		
		if ($referrer = $SERVICES->getGlobal('HTTP_REFERER')) {
			if (strpos($referrer, 'modules')) {
				$url = $referrer;
				$type = 1; // href
			}
		}

		if (empty($title)) {
			global $L;
			$title = $L['Back'];
		}

		if ($this->toolbar_link_mode) {
			
			// Phil 20070515 - Will need to resolve AJAX history.go problem, 
			// possibly by reimplementing the return_page code from WebMAT
			// eg: 
			// if (isset($_SESSION['return_page']) && $_SESSION['return_page']) {
				// $url = $_SESSION['return_page'];
				// return array(
					// 'href' =>$url,
					// 'title' => $title,
					// 'icon' => 'back.png'
				// );
			// }
			
			$url = str_replace('javascript:', '', $url);
			
			$return_arr = array(
				// 'href' =>$url,
				//'script' => $url, 
				'title' => $title,
				'icon' => 'back.png'
			);
			
			// If we dont have a URL, we dont have a back link
			if (!$url) {
				return false;
			}
			// Use either 'script' or 'href' depending on the type of url we are returning
			if ($type) {
				$return_arr['href'] = $url;
			} else {
				$return_arr['script'] = $url;
			}
			// print_r($return_arr); print '<hr>';
			return $return_arr;
			
		} else {
			return $this->getButtonHref(
				array(
					'href' =>$url,
					'button' => 'back',
					'title' => $title
				)
			);
		}
	}

	/**
	 * @param string $module_name The name of the module to display settings for
	 * @return A link to the given modules 'settings' page
	 * @author Phil Calder
	 */
	function getLinkSettings($module_name) {
		global $L;
		
		if ($module_name) {
			$href = '../../modules/Front/modules.php?module=' . $module_name;
			$label = $L['LinkSettings']; //'Settings';
			if ($this->toolbar_link_mode)
			{
				return array(
						'href' =>$href,
						'title' => $label,
						'icon' => 'settings.png'
						);
			} else {
				return $this->getLinkHref(
					array(
						'href' => $href, 
						'text' => $label
					)
				);
			}
		}
		return false;
	}
	
	function getLinkHelp($id = '', $target = '') {
		global $USER, $L, $SERVICES;

		if (empty($id)) {
			$id = $this->getHelpId();
		}

		$ref = base64_encode($SERVICES->getGlobal('http_host') . $SERVICES->getGlobal('php_self'));
		$this->has_help_button = true;
		
		if ($this->toolbar_link_mode)
		{
			return array(
					'script' => 'javascript:loadhelp(\'' . $id . '\',\'' . $ref . '\');',
					'title' => $L['Help'],
					'icon' => 'help.png'
					);
		} else {
			$content = '
			<a href="javascript:loadhelp(\'' . $id . '\',\'' . $ref . '\');">' . $L['Help'] . '</a>';
		}

		return $content;
	}
	
	function getLinkPrintView() {
		global $USER, $L, $SERVICES;
		
		$php_self = $SERVICES->getGlobal('php_self');
		
		// Phil 20060427
		$selector_options = array(
			'url' => $php_self . $this->updateQueryString(array('print' => 'true')), 
			'window_name' => 'print_page',
			'resizable' => true,
			'scrollbars' => true,
			'width' => 600,
			'height' => 530, 
		);
		$this->addJSPopup('showPrintView', $selector_options);
		
		
		if ($this->toolbar_link_mode) {
			return array(
				'script' => 'showPrintView();', 
				'icon' => 'print.png', 
				'title' => $L['LinkPrintView']
			);
		}

		return $this->getButtonHref(
			array(
				'href' => 'javascript:;',
				'on_click' =>'javascript:showPrintView();',
				'button' => 'print',
				'title' => $L['LinkPrintView']
			)
		);
	}
	
	function getLinkAddTab() {
		global $SERVICES;
		if ($this->toolbar_link_mode) {
			$tab_title = $this->trimTitle($this->title);
			return array(
				'href' =>'javascript:parent.Layout.addTab("' . $SERVICES->getGlobal('php_self') . '?' . $SERVICES->getGlobal('query_string') . '", "' . $tab_title .'");',
				//'href' =>'javascript:parent.Layout.showHelp();',
				'title' => $L['LinkAddTab'],
				'icon' => 'tab_add.png'
			);
		}
	}
	
	function getLinkRefresh() {
		global $USER, $L;
		//$this->has_refresh_button = true;
		$this->req_main_js = true; // Phil 20060207 Ensure the relevant javascript is included

		if ($this->toolbar_link_mode) {
			return array(
				'href' =>'javascript:refresh();',
				'title' => $L['LinkRefresh'],
				'icon' => 'refresh.png'
			);
		}
		
		return $this->getButtonHref(
			array(
				'href' =>'javascript:refresh();',
				'button' => 'refresh',
				'title' => $L['LinkRefresh']
			)
		);
	}
	
	function getButtonHref($link) {
		if ($link['on_click']) {
			$on_click = ' onClick="' . $link['on_click'] . '" ';
		} else {
			$on_click = '';
		}
		if ($link['target']) {
			$target = ' target="' . $target . '" ';
		} else {
			$target = '';
		}
		
		$content = '
		<a href="' . $link['href'] . '"' . $on_click . '' . $target . '>' . $link['title'] . '</a>';

		return $content;
	}

	/**
	 * returns the add bookmark link
	 *
	 * @return string the add bookmark link
	 */
	function getAddBookmarkLink($session_name = false, $title = false) {
		// print '<div><b>session_name</b>: ' .  $session_name . ' <b>title</b>: ' . $title . '</div>';
		global $SERVICES, $USER;
		$ua =& $SERVICES->getService('Authentication');
		if (!$ua->writeAccess($USER['user_id'], 'SystemBookmarks')) {
			return '';
		}
		if ($session_name != 'off') {
			global $L;
			if ($title) {
				$file = basename($SERVICES->getGlobal('php_self'));
				$session_name = 'bookmark';
				$bookmark['name'] = $title;
				//$bookmark['module'] = dirname($SERVICES->getGlobal('php_self')); 
				$bookmark['module'] = basename(dirname($SERVICES->getGlobal('php_self'))); //Syed 20050809
				$bookmark['filename'] = $file;
				$bookmark['querystring'] = $SERVICES->getGlobal('query_string');
				//$bookmark['page_type'] = 'url';
				$bookmark['page_type'] = 'view';//Syed 20050809
				$_SESSION[$session_name] = $bookmark;
			}

			$SERVICES->getService('Config');

			$new_href = $SERVICES->config->getModuleHref('SystemBookmarks', 'new', 0, 'item');

			$selector_options = array(
				'url' => str_replace('%bookmark%' , $session_name, $new_href),
				'window_name' => 'add_bookmark',
				'resizable' => true,
				'scrollbars' => true,
				'width' => 600,
				'height' => 530,  //Syed 20050726 | changed the height
			);
			$this->addJSPopup('addBookmark', $selector_options);
			
			if ($this->toolbar_link_mode) {
				return array(
					'script' => 'addBookmark()',
					'icon' => 'bookmark_add.png', // 'bookmark.gif', // TODO
					'title' => $L['labelAddBookmark'], 
					//'text' => $L['labelAddBookmark']
				);
			}
			
			return $this->getButtonHref(
				array(
					'href' => 'javascript:;',
					'on_click' =>'javascript:addBookmark();',
					'button' => 'bookmark',
					'title' => $L['labelAddBookmark']
				)
			);

		}
	}

	/**
	 * getLinkMenu
	 *
	 * @param $menu an array of links
	 * @return a string of links
	 * Phil 20040922
	 */
	function getLinkMenu($menu) {
		if (is_array($menu)) {
			//return implode(' | ', $menu);
			return implode(' ', $menu);
		}
		return rtrim($menu, ' | ');
	}
	
	function getFlagNew() {
		return $this->getFlagUnseen();
	}
	
	function getFlagUnseen() {
		global $USER, $L;
		return $this->getImageTag('img/flag_unseen.png', $L['ImageIsUnseen'], 'width="16" height="16" border="0" hspace="1"');
	}

	function getFlagUnseenByOther() {
		global $USER, $L;
		return $this->getImageTag('img/flag_unread.png', $L['ImageIsUnseen'], 'width="16" height="16" border="0" hspace="1"');
	}
	
	function getFlagEdited() {
		global $USER, $L;
		return $this->getImageTag('img/flag_edited.png', $L['ImageIsEdited'], 'width="16" height="16" border="0" hspace="1"');
	}
	
	function getFlagAlert($count = false) {
		global $USER, $L;
		return $this->getImageTag('img/flag_alert.png', $L['ImageIsAlert'] . ($count ? ' ' . $count : ''), 'width="16" height="16" border="0" hspace="1"');
	}
	
	function getFlagHighlight() {
		global $USER, $L;
		return $this->getImageTag('img/flag_hilite.png', $L['ImageIsHighlight'], 'width="16" height="16" border="0" hspace="1"');
	}
	
	function getFlagMenu($id = false) {
		global $USER, $L;
		return $this->getImageTag('img/menu.gif', $L['ImageMenu'], 'width="11" height="11" border="0"' . ($id ? ' id="' . $id . '"' : ''));
	}
	
	function getIconXajaxPopup($id = false) {
		//icon source http://www.matterform.com/mac_software/free_web_icons/legend.html
		global $USER, $L;
		return $this->getImageTag('img/xajax_popup.gif', $L['ImageXajaxPopup'], 'width="13" height="11" border="0"' . ($id ? ' id="' . $id . '"' : ''));
	}
	
	function getValidationWarning($val) {
		global $USER, $L;
		$content = '';
		if ($val) {
			$content = '<img src="'.$this->getSkinWebPath().'img/invalid.gif" vspace="2" hspace="4" align="absmiddle" title="' . $L['ImageIsInvalid'] . '" width="22" height="22" alt=""><span class="eInvalidSpan">&nbsp;';
			if (is_array($val) ) {
				foreach($val as $k=>$v) {
					$content .= $v;
				}
			} else {
				$content .= $val;
			}
			$content .= '</span>';
		}
		return $content;
	}

	/**
	 * draws standard field row with label in one cell and field in the other
	 */
	function getHtmlFieldRow($field, $error_msg = '', $extra_cell = false) {
		global $SERVICES;
		// Phil 20060223 Added param $extra_cell (see bulk edit WebMAtMassEdit)
		if ($extra_cell) {
			$extra_cell = '<th style="text-align: center">' . $extra_cell . '</th>';	
		}
		$extra_cell_placeholder = '<th>&nbsp;</th>';
		
		// Phil - there are two ways to specify a hidden field
		// field[type] = hidden, or field[type] = whatever and field[hidden] = 1
		// This allows us to differentiate between different types of hidden fields
		if (isset($field['hidden']) && $field['hidden']) {
			$field['type'] = 'hidden';
			bcrm_unset($field, 'hidden');
		}

		//special block to split date_time into two fields
		if ($field['type'] == 'date_time') {
			// print '<p>field_key: ' . $field['field_key'] . ' - value: ' . $field['value'] . ' - time: ' .time() . '</p>';
			$html_field = '';
			if ($field['field_key'] == 'last_update') { // Phil 20070308
				$tmp_field = array(
					'field_key' => $field['field_key'], 
					'value' => $field['value'], 
					'type' => 'integer', 
					'hidden' => 1
				);
				$html_field .=  $this->getHtmlField($tmp_field);
				$field['field_key'] = '__tmp_' . $field['field_key'];
				// $field['read_only'] = 1;
			}
			
			//offset by timezone for display
			$field['value'] = $SERVICES->offsetTimeZoneForRead($field['value']);
			
			$original_field_key = $field['field_key'];
			$original_field_label = $field['label'];

			$field['field_key'] = $original_field_key . '][date';
			$field['label'] = $original_field_label . ' (date)';
			$field['type'] = 'integer_date';

			$html_field .= $this->getHtmlField($field);
			$html_field = str_replace('</tr></table>', '', $html_field);  //remove the end table tags
			
			$field['field_key'] = $original_field_key . '][time';
			$field['label'] = $original_field_label . ' (time)';
			$field['type'] = 'integer_time';

			$html_field .= '<td nowrap>&nbsp;' . $this->getHtmlField($field) .'</td></tr></table>';
			
			$nowrap = 1;

			$field['label'] = $original_field_label;

		} else {
			$html_field = $this->getHtmlField($field);
			
			// Phil 20070718 (see contact item_edit.inc address fields for an example)
			if (isset($field['extra_field']) && $field['extra_field']) {
				$html_field .= ' ' . $field['extra_field'];
			}
		}

		if ($this->has_template) {
			// different type of array being returned
			if ($field['label'] || $field['type'] != 'hidden' || $error_msg) {
				$retVal = array (
					'label' => $field['label'],
					'notnull' => (isset($field['notnull'] ) && $field['notnull'] ? '<b>*</b>' : ''),
					'htmlField' => $html_field,
					'errorMessage' => ($error_msg ? '<br>' . $this->getValidationWarning($error_msg) : ''),
					'explanation' => $field['explanation']
					);
			} else {
				$retVal = array (
					'htmlField' => $html_field,
					'explanation' => $field['explanation']
					);
			}

			return $retVal;

		} else {
			//if ($field['label'] || $field['type'] != 'hidden' || $error_msg) {
			if ($field['label'] || ($field['type'] != 'hidden') || $error_msg) {
				
				//<td valign="top" align="left" valign="top" nowrap><b>' . $field['label'] . '</b>&nbsp;</td>'
				// print '<div>1 - <b>field_key:</b> ' .$field['field_key'] . ' <b>type:</b> ' .$field['type'] . ' <b>value:</b> ' . $field['value'] . '</div>' . "\n";
				if ($field['type'] == 'long_text') {
					if (!isset($field['no_resize']) || !$field['no_resize']) { // Phil 20070306
						$content = '
						<tr class="spacer">' . ($extra_cell ? $extra_cell_placeholder : '') . '<td colspan="2"><div>&nbsp;</div></td></tr>' .
						'<tr class="double">' . $extra_cell . '<th>' . $field['label'] . (isset($field['notnull'] ) && $field['notnull'] ? '*' : '') .
						'</th><td align="right">&nbsp; &nbsp;<a href="javascript:change(\'fld_' . $field['field_key'] . '\',\'normal\');">--</a> &nbsp; <a href="javascript:change(\'fld_' . $field['field_key'] . '\',\'big\');">&gt;&gt;</a>&nbsp;
						 </td>';
						 //<a href="javascript:change(\'fld_' . $field['field_key'] . '\',\'huge\');">&gt;&gt;</a>

					}
						
					$content .=  '<tr>' . ($extra_cell ? $extra_cell_placeholder : '') . '<td colspan="2">' . $html_field . '</td></tr>' .
						($error_msg ? '<tr>' . ($extra_cell ? $extra_cell_placeholder : '') . '<td colspan="2">' . $this->getValidationWarning($error_msg) . '</td></tr>' : '');
					
					if (isset($field['explanation']) && $field['explanation']) {
						//MCR 20060707 $content .= '<tr class="double">' . ($extra_cell ? '<th>&nbsp;</th>' : '') . '<th>&nbsp;</th><td><table' . $this->getTagContent('explanationTbl') . '><tr>' . ($extra_cell ? $extra_cell_placeholder : '') . '<td>' . $field['explanation'] . '</td></tr></table></td></tr>';
						$content .= '<tr class="double">' . ($extra_cell ? '<th>&nbsp;</th>' : '') . '<th>&nbsp;</th><td><div' . $this->getTagContent('explanation') . '>' . $field['explanation'] . '</div></td></tr>';
					}
					
					
				} else {
					$content = '<tr>' . $extra_cell . '<th>' . $field['label'] . (isset($field['notnull'] ) && $field['notnull'] ? '*' : '') . '</th>' .
						'<td' . (isset($nowrap) && $nowrap ? ' nowrap' : '')
						. '>' . $html_field . ($error_msg ? '<br>' . $this->getValidationWarning($error_msg) : '') . '</td></tr>';
					
					if (isset($field['explanation']) && $field['explanation']) {
						// = is correct here
						//$content = '<tr class="double"><th>&nbsp;</th><td>' . $field['explanation'] . '</td></tr>' . $content;
						//MCR 20060707 $content = '<tr class="double">' . ($extra_cell ? $extra_cell_placeholder : '') . '<th>&nbsp;</th><td><table' . $this->getTagContent('explanationTbl') . '><tr><td>' . $field['explanation'] . '</td></tr></table></td></tr>' . $content;
						
						//$content = '<tr class="double">' . ($extra_cell ? $extra_cell_placeholder : '') . '<th>&nbsp;</th><td><div' . $this->getTagContent('explanation') . '>' . $field['explanation'] . '</div></td></tr>' . $content;
						$content .= '<tr class="double">' . ($extra_cell ? $extra_cell_placeholder : '') . '<th>&nbsp;</th><td><div' . $this->getTagContent('explanation') . '>' . $field['explanation'] . '</div></td></tr>';
					}
				}
				
			} else {
				$content = $html_field;
			}
			
			// Phil 20060202 Added key to field: additional_output. 
			// This is for html to be added BEFORE and/or AFTER this field row.
		
			// Phil 20060202
			if (is_array($field['additional_output'])) { 
				if (isset($field['additional_output']['before'])) {
					$content = $field['additional_output']['before'] . $content;
				}
				if (isset($field['additional_output']['after'])) {
					$content = $content . $field['additional_output']['after'];
				}
			}
			
			return $content;
		}
	}

	/**
	 * draws standard view page field row with label in one cell and string field in the other
	 */
	function getStringFieldRow($field) {
		global $L;
		if ($this->has_template) {
			return array(
				'label' => ($field['label'] ? $field['label'] : $field['list_label']),
				'value' => $this->getStringField($field)
			);
		} else {
			bcrm_unset($field, 'list_detail_length');
			
			$label = $field['label'] ? $field['label'] : $field['list_label'];
			
			$string_field = $this->getStringField($field);
			// print '<div><b>field_key:</b> ' .$field['field_key'] . ' <b>type:</b> ' .$field['type'] . ' <b>value:</b> ' . $field['value'] . ' <b>string_field:</b> ' . $string_field . '</div>' . "\n";
		
			if (empty($string_field) || $string_field == ' ') {
					$string_field = '&nbsp;';
			}
			
			if (($field['type'] == 'iframe') || ($field['type'] == 'long_text')) {
				$iframe_link = '<a href="' . $field['value'] . '" target="_blank">' . $L['LinkIframeContentPopup'] . '</a>';
				// Phil 20070618 added no_resize option for long_text fields
				if (isset($field['no_resize']) && $field['no_resize']) {
					$resize_link = ' ';
				} else {
					$resize_link = '<a href="javascript:change(\'fld_' . $field['field_key'] . '\',\'normal\');">--</a> &nbsp; <a href="javascript:change(\'fld_' . $field['field_key'] . '\',\'big\');">&gt;</a>&nbsp; <a href="javascript:change(\'fld_' . $field['field_key'] . '\',\'huge\');">&gt;&gt;</a>';
				}
				
				// Phil 20070618 added no_spacer option for long_text fields
				if (isset($field['no_spacer']) && $field['no_spacer']) {
					$spacer_row = '';
				} else {
					$spacer_row = '<tr class="spacer"><td colspan="2"><div>&nbsp;</div></td></tr>';
				}
				
				$content = '
				' . $spacer_row . '
				<tr class="double"><th>' . $label . '</th><td align="right">&nbsp; &nbsp;' .
					( ($field['type'] == 'iframe') ? $iframe_link : $resize_link ).
					'</td></tr>
				';
				$content .=  '<tr><td colspan="2">' . $string_field .
					'<div class="h-prop"></div>' .
					'</td></tr>' .
					$spacer_row . '
					';
				
				
			} else {
				$content = '
				<tr><th>' . $label . '</th><td>' . $string_field . '</td></tr>
				';
			}
			
			return $content;
		}
	}
	
	function getStringFieldContent($field) {
		return $this->getStringFieldRow($field);
		// Duplicate functionality - for now - with getStringFieldRow
		// Currently unused. May possibly provide a table-less view / edit page option in future
		global $L;
		
		if ($this->has_template) {
			return array(
				'label' => ($field['label'] ? $field['label'] : $field['list_label']),
				'value' => $this->getStringField($field)
			);
		} else {
			bcrm_unset($field, 'list_detail_length');
			
			$label = $field['label'] ? $field['label'] : $field['list_label'];
			
			$string_field = $this->getStringField($field);
			
			if (empty($string_field) || $string_field == ' ') {
					$string_field = '&nbsp;';
			}
			
			if (($field['type'] == 'iframe') || ($field['type'] == 'long_text')) {
				
				/*
				$content = '<tr class="spacer"><td colspan="2"><div>&nbsp;</div></td></tr>' .
					'<tr class="double"><th>' . $label .
					'</th><td align="right">&nbsp; &nbsp;' .
					(($field['type'] == 'iframe') ? 
					'<a href="' . $field['value'] . '" target="_blank">' . $L['LinkIframeContentPopup'] . '</a>' :					
					'<a href="javascript:change(\'fld_' . $field['field_key'] . '\',\'normal\');">--</a> &nbsp; <a href="javascript:change(\'fld_' . $field['field_key'] . '\',\'big\');">&gt;</a>&nbsp; <a href="javascript:change(\'fld_' . $field['field_key'] . '\',\'huge\');">&gt;&gt;</a>' 
					).
					'</td>';
				$content .=  '<tr><td colspan="2">' . $string_field . '</td></tr>' .
					'<tr class="spacer"><td colspan="2"><div>&nbsp;</div></td></tr>';
				*/
				$content = '
				<div class="x-form-item">
					<label for="x-form-el-' . $field['field_key'] . '" style="width: 100px;">' . $label . ' ' .
					(($field['type'] == 'iframe') ? 
					'<a href="' . $field['value'] . '" target="_blank">' . $L['LinkIframeContentPopup'] . '</a>' :					
					'<a href="javascript:change(\'fld_' . $field['field_key'] . '\',\'normal\');">--</a> &nbsp; <a href="javascript:change(\'fld_' . $field['field_key'] . '\',\'big\');">&gt;</a>&nbsp; <a href="javascript:change(\'fld_' . $field['field_key'] . '\',\'huge\');">&gt;&gt;</a>' 
					).
					'</label>
					
					<div class="x-form-textarea x-form-field" id="x-form-el-' . $field['field_key'] . '">
					<br />' . $string_field . '
					</div>
				</div>
				';
				
				
			} else {
				// $content = '<tr><th>' . $label . '</th><td>' . $string_field . '</td></tr>';
				// print_r($field); print '<hr>';
				$content = '
				<div class="x-form-item">
					<label for="' . $field['field_key'] . '" style="width: 100px;">' . $label . '</label>
					<div class="x-form-element" id="x-form-el-' . $field['field_key'] . '">
					' . $string_field . '
					</div>
				</div>
				';
				
			}
			
			return $content;
		}
	}
		
	/**
	 * function getHtmlFieldRow($html_field, $error_msg = '') {
	 * draws standard field row with label in one cell and field in the other
	 */
	function getMultiHtmlFieldRow($id, $fields, $error_msgs = '', $row_number = false, $extra_cell = false) {
		global $SERVICES, $L;

		$error_text = $L['labelRow'] . ' ' . $row_number . ': ';
		$content = '';

		if (is_array($fields)) {
			if ($row_number) {
				$content .= '<td>' . $row_number . '</td>';
			}

			if ($extra_cell) {
				if (is_array($extra_cell)) {
					$fields = $extra_cell + $fields;
				} else {
					$content .= '<td>' . $extra_cell . '</td>';
				}
			}

			if (isset($fields['_ignore_row'])) {
				$this->setUncheckIngore();
				$tag_extras = ' onChange="javascript:uncheckIgnore(\'' . $fields['id']['value'] . '\')';
			} else {
				$tag_extras = false;
			}

			foreach ($fields as $k => $field) {
				if ( ($tag_extras) && ($k != '_ignore_row') ) {
					if ($index = strpos($field['tag_extras'], 'onChange=')) {
						// Adding to the onChange JS
						$tag = substr($field['tag_extras'], $index+strlen('onChange='));
						$tag = str_replace('"', '', $tag);
						$field['tag_extras'] = $tag_extras . '; ' . $tag . '"';
					} else {
						$field['tag_extras'] = $tag_extras . '"' . $field['tag_extras'];
					}
				}

				if ($field['multi_edit_width']) {
					$field['width'] = $field['multi_edit_width'];
					bcrm_unset($field, 'multi_edit_width');
				} else {
					$defaults = $this->getSearchFilterDefault($field['type']);
					$field['width'] = $defaults['size'];
				}

				//if ( ($blank_row) && ($field['type'] != 'dropdown') ){
				if ( ($blank_row) && ($field['type'] != 'dropdown') && ($field['type'] != 'radio_v') && ($field['type'] != 'radio_h') ){
					bcrm_unset($field, 'notnull');
				}

				$error_msg = $error_msgs[$field['field_key']];

				//special block to split date_time into two fields
				if ($field['type'] == 'date_time') {
					//offset by timezone for display
					$field['value'] = $SERVICES->offsetTimeZoneForRead($field['value']);

					$original_field_key = $field['field_key'];
					$original_field_label = $field['label'];

					$field['field_key'] = $original_field_key . '][date';
					$field['label'] = $original_field_label . ' (date)';
					$field['type'] = 'integer_date';

					$html_field = $this->getHtmlField($field, $id, $error_text);
					$html_field = str_replace('</tr></table>', '', $html_field);  //remove the end table tags
					//$html_field = str_replace('</td></tr></table>', '', $html_field);  //remove the end table tags

					$field['field_key'] = $original_field_key . '][time';
					$field['label'] = $original_field_label . ' (time)';
					$field['type'] = 'integer_time';

					$html_field .= '<td nowrap>&nbsp;' . $this->getHtmlField($field, $id, $error_text) .'</td></tr></table>';  //&nbsp; MCR 20060627
					//$html_field .= '<br>hello' . $this->getHtmlField($field, $id, $error_text) .'</td></tr></table>';  //&nbsp; MCR 20060627
					$nowrap = 1;
					$field['label'] = $original_field_label;

				} else {
					
					// Phil 20060405 - Currently cannot have a contact selector 
					// (ie change link) on a multi edit form
					if ($field['type'] == 'contact') {
						$field['no_link'] = 1;	
					}

					// Phil 20060223 Disable config_key for dropdown
					$current_config_key_value = $this->setAllowDropdownConfigKey(false);
					$html_field = $this->getHtmlField($field, $id, $error_text);
					$this->setAllowDropdownConfigKey($current_config_key_value);
				}

				if ( ($field['label']) || ($field['type'] != 'hidden') || ($error_msg) ) {
					$content .= '<td valign="top" align="left"' . ($nowrap ? ' nowrap' : '') . '>' . $html_field . ($error_msg ? '<br>' . $this->getValidationWarning($error_msg) : ''). '</td>';
				} else {
					$content .= $html_field;
				}
			}
		}

		return '
		<tr>
		' . $content . '
		</tr>
		';
	}

	/**
	 *
	 */
	function getHtmlField($field, $id = false, $extra_text = false) {
		global $SERVICES, $L, $USER;
		// Phil - there are two ways to specify a hidden field
		// field[type] = hidden, or field[type] = whatever and field[hidden] = 1
		// This allows us to differentiate between different types of hidden fields
		if (isset($field['hidden']) && $field['hidden']) {
			$field['type'] = 'hidden';
			bcrm_unset($field, 'hidden');
		}

		//*****required array elements are:
		//label				the field label (also used jfor js messages)
		//field_key			gets wrapped in item[...] to become html form element
		//type				our data types
		//value

		//optional array elements are:
		// hidden
		// width
		// tag_extras
		// text 			plain text to add after the field (mostly for use with a hidden field, or as free text with 'type' = free)
		// regex
		// dropdown_values	an array of option values for a dropdown type indexed by field value mapping to text.
		// dropdown_text  	an array of option text for a dropdown type. Must map 1=>1 to
		//					dropdown_values and may be identical
		// read_only  		if the field is read only
		// notnull 			add javascript for client side validation
		// maxlength		for maxlength tag for shorttext
		// rows 			number of rows to display in a long_text field
		// max 				the highest number (float or integer) allowed
		// min 				the lowest number allowed
		// decimals 		the number of decimal points to display in a float number
		// checked 			allows override of default 0 /1 functionlity for checkboxes; see case 'checkbox'


		//check for old code
		if (array_key_exists('field_name', $field)) {
			$this->printDebug('The field definition has an element field_name. This is wrong; old code. It probably should be field_key.');
			return '';
		}

		if ($id) {
			$id_item_key = 's[' . $id . ']';
		} else {
			$id_item_key = '';
		}

		if ($field['read_only']) {
			$read_only = ' readonly ';
			// $read_only_class = ' eReadOnly';
			$read_only_class = ' ' . $this->getTagContent('eReadOnly', 'class');
			// print '<div>field: ' . $field['field_key'] . ' read only class: [' . $read_only_class . ']</div>';
		} else {
			$read_only = '';
			$read_only_class = '';
		}

		$defaults = $this->getFieldDefaults();

		if (!isset($field['width']) || !$field['width']) {
			if (($field['type'] == 'date') || ($field['type'] == 'integer_date')) {
				$field['width'] = 75;
			} elseif (($field['type'] == 'time') || ($field['type'] == 'integer_time')) {
				$field['width'] = 40;
			} else {
				$field['width'] = 180;  //This was commented out
				//$field['width'] = 0;  //This was in
			}
		}

		switch ($field['type']) {
			case 'free':   //allows free text or html to be submitted in the field loop // edited 15/1/05
				$content = $field['value']; //'';
				break;

			// if $field['maxlength'] is set make it the maxlength of the field otherwise it's 255
			case 'url':
				$field['value'] = urldecode($field['value']);
				//Syed 20050622 adding default value
				 if ((empty($field['value']) && !$field['is_in_filter'])) {
					 //20071119 
					 //$field['value']= $this->getDefaultUrlPrefix();
				 }
			case 'email':
			case 'text':
				if (!isset($field['maxlength']) || !$field['maxlength']) {
					 $field['maxlength'] = $defaults[$field['type']]['maxlength'];
				}
				if ($field['type'] != 'url') {
					$field['value'] = bcrm_htmlspecialchars($field['value'], ENT_QUOTES, $this->charset);
					// print '<div>new value: ' . $field['value'] . '</div>';
				}
				$content = '<input type="text" ' . ($field['width'] ? 'style="width: ' . $field['width'] . 'px" ' : '') . 'class="' . $read_only_class . '"'. $read_only .' name="item' . $id_item_key . '[' . $field['field_key'] . ']"
					' . ($field['maxlength'] ? 'maxlength="' . $field['maxlength'] . '"' : '') . ' ' . (isset($field['tag_extras']) ? $field['tag_extras'] : '') . ' value="' . $field['value'] . '"' . $this->getFormAutoCompleteOff('string') . '>';
				break;

			case 'iframe':
			if (!isset($field['maxlength']) || !$field['maxlength']) {
				 $field['maxlength'] = $defaults[$field['type']]['maxlength'];
			}
			$content = '<iframe width="' . $field['width'] . '" height="' . $field['rows'] . '" src="' . $field['value'] . '"></iframe>';
			//$field['value'] = bcrm_htmlspecialchars($field['value'], ENT_QUOTES, $this->charset);
			// $content = '<input type="text" ' . ($field['width'] ? 'style="width: ' . $field['width'] . 'px" ' : '') . 'class="' . $read_only_class . '"'. $read_only .'name="item' . $id_item_key . '[' . $field['field_key'] . ']"
				// maxlength="' . $field['maxlength'] . '" ' . (isset($field['tag_extras']) ? $field['tag_extras'] : '') . ' value="' . $field['value'] . '">';
			break;

			case 'file_upload':
				$field['value'] = bcrm_htmlspecialchars($field['value'], ENT_QUOTES, $this->charset);
				$content = '<input type="file" ' . ($field['width'] ? 'style="width: ' . $field['width'] . 'px" ' : '') . 'class="' . $read_only_class . '"'. $read_only .' name="item' . $id_item_key . '[' . $field['field_key'] . ']"' .
					$field['tag_extras'] . ' value="' . $field['value'] . '">';
				break;

			// Added 2004-04-06, Phil - needs to be tested, particularly for validation etc
			case 'password':
				if (!isset($field['maxlength']) || !$field['maxlength']) {
					 $field['maxlength'] = (isset($defaults[$field['type']]['maxlength']) ? $defaults[$field['type']]['maxlength'] : '');
				}
				$field['value'] = bcrm_htmlspecialchars((isset($field['value']) ? $field['value'] : ''), ENT_QUOTES, $this->charset);
				$content = '<input type="password" ' . ($field['width'] ? 'style="width: ' . $field['width'] . 'px" ' : '') . 'class="' . $read_only_class . '" name="item' . $id_item_key . '[' . $field['field_key'] . ']"
					'. $read_only .'maxlength="' . $field['maxlength'] . '" ' . (isset($field['tag_extras']) ? $field['tag_extras'] : '') . ' value="' . $field['value'] . '"' . $this->getFormAutoCompleteOff('string') . '>';
				break;

			case 'medium_text':
			case 'long_text':
				// print '<div>' . get_class($this) . '::getHtmlField</div>'; print_r($field); print '<hr>';
				$field['value'] = str_replace('<br>', "\n", $field['value']); // Phil 20050407
				$field['value'] = str_replace('<br />', "\n", $field['value']); // Phil 20051101
				$field['value'] = bcrm_htmlspecialchars($field['value'], ENT_QUOTES, $this->charset);
					// if $field['rows'] is set make it the number of rows of the textarea otherwise it's 5
				
				//$content = '<textarea ' . ($field['width'] ? 'style="width: ' . $field['width'] . 'px" ' : '') . 'name="item' . $id_item_key . '[' . $field['field_key'] . ']"
				//	rows="' . (($field['rows']) ? ($field['rows']) : 5) . '"'. $read_only .'' . $field['tag_extras'] . '>' . $field['value'] . '</textarea>';
				
				// Phil 20060608 added long_text append only functionality
				if (isset($field['append_only']) && $field['append_only']) {
					$list_detail_length = false;
					$list_detail_length = (isset($field['list_detail_length']) ? $field['list_detail_length'] : false);
					bcrm_unset($field, 'list_detail_length');
					if (strlen($current_value = $this->getStringField($field))) {	
						$field['value'] = '';
					}
					if ($list_detail_length !== false) {
						$field['list_detail_length'] = $list_detail_length;
					}
					// print '<div>current_value:</div>'; print $current_value; print '<hr>';
				}
				
				if (!isset($field['no_resize']) || !$field['no_resize']) { // Phil 20070306
					$small_text_class = (($field['rows']==1) ? (' smalltextarea') : '');
				}
				// read_only_class set above
				$lt_classes = '';
				$lt_classes = ' class="' . $read_only_class . $small_text_class . '" ';
				
				//MCR 20060105 added id for long_text; should really also have module name for uniqueness
				$content = '<textarea style="width: 100%" id="fld_' . $field['field_key'] . '" name="item' . $id_item_key . '[' . $field['field_key'] . ']"
				rows="' . (($field['rows']) ? ($field['rows']) : 5) . '" ' . $lt_classes . '' . $read_only . 
					$field['tag_extras'] . '>' . $field['value'] . '</textarea>';
				
					// Phil 20060608 for long_text append only functionality
					if (isset($current_value) && strlen($current_value)) {
						$content = '<div>' . $current_value . '</div>' . $content;
					}
				break;

			case 'contact':
				//add stuff to allow array of ids to be passed, and returned
				// if (is_array($field['value'])) {
				//}
				//
				// print '<div>contact field (id_item_key=' . $id_item_key . ')</div>'; print_r($field); print '<hr>';
				$content = $this->getEditLinkContact($field, $id_item_key);
				break;

			case 'hidden':
				$field['value'] = bcrm_htmlspecialchars($field['value'], ENT_QUOTES, $this->charset);
				$content = '<input type="hidden" name="item' . $id_item_key . '[' . $field['field_key'] . ']" ' . $field['tag_extras'] . ' value="' . $field['value'] . '">';
				break;

			case 'integer':
				// if maxsize is set, it represents the highest number allowable. The field maxlength is set to the
				// number of characters that number takes or to 11 (the size of the field that can contain the value) by default
				if (!$field['min']) {
					 $field['min'] = $defaults[$field['type']]['min'];
				}
				if (!$field['max']) {
					 $field['max'] = $defaults[$field['type']]['max'];
				}

				// echo '*' . $field['max'];
				
				// Phil testing 20051111
				// if (isset($field['no_value']) && $field['no_value']) {
					// print '<div>no_value</div>'; print_r($field); print '<hr>';
					// // bcrm_unset($field, 'value');	
				// }
				
				// print '<div>value: ' . $field['value'] . '</div>';
				$content = '<input type="text" ' . ($field['width'] ? 'style="width: ' . $field['width'] . 'px" ' : '') . 'class="' . $read_only_class . '" name="item' . $id_item_key . '[' . $field['field_key'] . ']"
					maxlength="' . (($field['max']) ? strlen($field['max']) : 11) . '" ' . $field['tag_extras'] . $read_only .' value="' . $field['value'] . '" ' . $read_only . '>';
				break;

			case 'currency':
			case 'float':
				// set the number of decimal places to show.
				if($field['decimals']) {
					$field['value'] = number_format($field['value'], $field['decimals']);
				}
				$content = '<input type="text" ' . ($field['width'] ? 'style="width: ' . $field['width'] . 'px" ' : '') . 'class="' . $read_only_class . '"'. $read_only .'name="item' . $id_item_key . '[' . $field['field_key'] . ']" ' .
				$field['tag_extras'] . ' value="' . $field['value'] .'">';
				break;


			/*
			Added by Matthew Corich for percentage values 18/06/03
			*/
			case 'percentage':
				// set the number of decimal places to show.
				if($field['decimals']) {
					$field['value'] = number_format($field['value'], $field['decimals']);
				}
				if ($field['value'] != 0) {
					$field['value'] = ($field['value']*100) . '%';
				} else {
					$field['value'] = '';
				}
				$content = '<input type="text" ' . ($field['width'] ? 'style="width: ' . $field['width'] . 'px" ' : '') . 'class="' . $read_only_class . '"'. $read_only .'name="item' . $id_item_key . '[' . $field['field_key'] . ']" ' .
				$field['tag_extras'] . ' value="' . $field['value'] .'">';
				break;
			/*
			Add Ends
			*/

			case 'integer_date':
				$SERVICES->getService('DateLib');
				//convert integer date to MySql Fromat Y/m/d expected by the following "case 'date':"
				$field['value'] = $SERVICES->datelib->getUserPrefDateFromTimestamp($field['value'], 'Y/m/d', '-');

			case 'date':
				$SERVICES->getService('DateLib');
				// Phil 20060810 commented this - affects entire page
				// if ($field['read_only']) {
					// $this->has_calendar_button = false;
				// } else {
					// $this->has_calendar_button = true;
				// }
				// Phil 20060810
				if (!$field['read_only']) {
					$this->has_calendar_button = true;
				}
				
				// Allow for single digit day and month
				$tmp_mask = $SERVICES->preferences->getUserPref('cfgShortDateFormat');
				$tmp_mask = str_replace('j', 'd', $tmp_mask);
				$tmp_mask = str_replace('n', 'm', $tmp_mask);
				
				//for jscalendar
				$ifFormat = $tmp_mask;
				$ifFormat = str_replace('y', '%y', $ifFormat);
				$ifFormat = str_replace('Y', '%Y', $ifFormat);
				$ifFormat = str_replace('d', '%d', $ifFormat);
				$ifFormat = str_replace('m', '%m', $ifFormat);
				
				// show the data entry field with the user pref format
				// 20040517: override user pref with tmp_mask - single digit day and month format, length should allow for 2 digit day and month
				if ($field['hide_mask']) {
					$datemask = '';
				} else {
					 $datemask = '<td valign="middle">' . str_replace(array('/', '-'), array('&frasl;','&minus;'), $SERVICES->datelib->getDateInputMask()) . '&nbsp;</td>';
				}

				//MCR 20060202; previous cal: include_once('calendar_js.inc'); // loads javascript files to handle the calendar popup.
				//this version will load the calendar not-in-a-window but there is a problem in IE
				//<SCRIPT LANGUAGE="JavaScript">var cal' . $cal_name . ' = new CalendarPopup("CalendarStyle");
				
				// tidy up name so that any name that is an array item is turned into something the javascript can use
				// ie turn 'field[10]' into 'field_10'.
				$cal_name = str_replace('[', '_', $field['field_key']);
				$cal_name = str_replace(']', '', $cal_name);
				
				//echo 'fn - '.$field_name.': cn - '.$cal_name;
				// setup the calendar date selector.
				if ($id_item_key) {
					$cal_name = $cal_name . str_replace(array('s[',']'), '', $id_item_key);
				}
				
				// the date coming into this function is must by in mySQL db format ('2004-12-31') so now translate into userpref format
				$field['value'] = $SERVICES->datelib->getUserPrefDateFromMySqlDate($field['value']);
				// TODO: Phil 20060615
				// if (is_array($field['value'])) {
					// foreach ($field['value'] as $k => $v) {
						// $field['value'][$k] = $SERVICES->datelib->getMysqlDateFromTimestamp($v);
					// }
					// // $field['value'] = $SERVICES->datelib->getUserPrefDateFromMySqlDate(current($field['value']));
				// } else {
					// $field['value'] = $SERVICES->datelib->getUserPrefDateFromMySqlDate($field['value']);
				// }
				
				// lets be tidy and put it all in its own table
				$content = '<table' . $this->getTagContent('informTbl') . '><tr>';
				$key = $field['field_key'];
				// TODO: Phil 20060615
				// if ($field['search_multiple']) { // AFTER
					// $key = $field['field_key'] . '][a'; // ie: "item[field_key][a]"
				// }
				
				$content .= $datemask;
				
				$content .=  '<td valign="middle"><input id="fld_' . $cal_name . '" type="text" class="' . $read_only_class . '"  name="item' . $id_item_key . '[' . $key . ']" style="width: ' .
				$field['width'] . 'px" maxlength="'.strlen($SERVICES->datelib->getJsDateMaskFromUserPref($tmp_mask)).'"'. $read_only .' value="' . $field['value'] . '"' . $field['tag_extras'] . '>&nbsp;</td>';
				
				$content .='<td valign="middle"><img src="' . $this->getSkinWebPath() . 'img/calen.gif" border="0" id="trigger_' . $cal_name . '" alt="">';
				
				// Phil: TODO: if read_only, use a different img/calen.gif (greyed-out?)
				// Phil 20060810 added if
				if (!$field['read_only']) {
					$this->setCalendarSetup(array('inputField' => 'fld_' . $cal_name, 'ifFormat' => $ifFormat, 'button' => 'trigger_' . $cal_name));
				}
				$content .= '</td>';
				
				// TODO: Phil 20060615
				// if ($field['search_multiple']) { // BEFORE
					// $key = $field['field_key'] . '][b'; // ie: "item[field_key][b]"
					// $cal_name = $cal_name . '_after';
					// $content .= '</tr><tr>';
					// $content.= $datemask . '<td><input id="fld_' . $cal_name . '" type="text" class="' . $read_only_class . '"  name="item' . $id_item_key . '[' . $key . ']" style="width: ' .
					// $field['width'] . 'px" maxlength="'.strlen($SERVICES->datelib->getJsDateMaskFromUserPref($tmp_mask)).'"'. $read_only .' value="' . $field['value'] . '"' . $field['tag_extras'] . '>&nbsp;</td><td>
					// <img src="' . $this->getSkinWebPath() . 'img/calen.gif" border="0" id="trigger_' . $cal_name . '" alt="">
					// ';
					// $this->setCalendarSetup(array('inputField' => 'fld_' . $cal_name, 'ifFormat' => $ifFormat, 'button' => 'trigger_' . $cal_name));
					// $content .= '</td>';
				// }
				
				$content .= '</tr></table>';
				
				//TODO fix/delete mouseover js above

				break;

			case 'integer_time':
				// if ($field['value']) {	
				if ($field['value'] && ($field['value'] >= 0 )) { // Phil 20070308
					$field['value'] = date(("H:i"), $field['value']);
				}

			case 'time':
				$SERVICES->getService('DateLib');
				// likewise translate format for times

				$field['value'] = $SERVICES->datelib->getUserPrefTimeFromMySqlDate($field['value']);  //this means H:i 23:09; H is 24-hour format of an hour with leading zeros 00 through 23; i = Minutes with leading zeros

				$time_format = '^[0-9]{1,2}'.$SERVICES->preferences->getUserPref('cfgTimeSeparator').'[0-9]{2}$';
				$content = 'hh'.$SERVICES->preferences->getUserPref('cfgTimeSeparator').'mm&nbsp;<input
				type="text" class="' . $read_only_class . '"  name="item' . $id_item_key . '[' . $field['field_key'] . ']" style="width: ' . $field['width'] . 'px"' . $read_only .  'value="' . $field['value'] . '">';
				break;

			case 'multi_select':
			$multi_select = 'multiple ';
				if ($field['csv'] && isset($field['value'])) {
					if (!is_array($field['value'])) { //check for array :: malar 20050628
						// TODO: print '<div>** value: ' . $field['value'] . '</div>';
						$field['value'] = explode(",", $field['value']);
					}
				}

				if ($field['size']) {
					//
				} else {  //default value = 6
					$temp_limit = sizeof($field['dropdown_values']);
					if (!$field['notnull']) {
						$temp_limit++;
					}
					$field['size'] = max(min(6, $temp_limit), 4);
				}

				$multi_select_size = 'size="' . ($field['size'] ? $field['size'] : '6') . '" ';
				$multi_select_ar = '[]';

			case 'dropdown':
			case 'radio_v':
			case 'radio_h':

				if ($field['read_only']) {

					$text = '';
					if (!is_array($field['dropdown_values'])) {
						switch ($field['dropdown_values']) {
							case '__USERS__':
								$field['dropdown_values'] = $this->getUsersList();
							break;
							case '__USERSALL__': // Phil 20060814 includes deleted users
								$field['dropdown_values'] = $this->getUsersList(false, false, true, true); // Show deleted users and resources
								// print '<div>getHtmlField ' . $field['field_key'] . ' USERSALL: </div>'; print_r($field['dropdown_values']); print '<hr>';
							break;
							
							case '__GROUPS__':
								$field['dropdown_values'] = $this->getGroupList();
							break;
							case '__TEAMGROUPS__':
								$field['dropdown_values'] = $this->getGroupList('team');
							break;
							case '__PRIVSGROUPS__':
								$field['dropdown_values'] = $this->getGroupList('privs');
							break;
							case '__PRIVATEGROUPS__':
								$field['dropdown_values'] = $this->getGroupList('private');
							break;
						}
					}
					
					if ( ($field['type'] == 'multi_select') && isset($field['dropdown_values']) && is_array($field['dropdown_values'])) {
						if (is_string($field['value'])) {
							$_values = explode(',', $field['value']);
						} else {
							$_values = (array)$field['value'];
						}
						
						foreach ($_values as $k => $v) {
							// print '<div>_values: ' . $v . '</div>';
							$text .= $field['dropdown_values'][$v] . "\n";
						}
						$text = rtrim($text, ", ");
						
						// print '<div>key: ' . $field['field_key'] . ' field[value]: ' . $field['value'] . ' - ' . $text . '<hr>'; print_r($field['value']); print_r($field['dropdown_values']); 
						$content = '<input type="hidden" value="' . $field['value'] . '" ' .
						'name="item' . $id_item_key . '[' . $field['field_key'] . ']' . (isset($multi_select_ar) ? $multi_select_ar : '') . '">';
						$content .= '<textarea ' . ($field['width'] ? 'style="width: ' . $field['width'] . 'px" ' : '') . 'class="' . $read_only_class . '"'. $read_only .
							(isset($field['tag_extras']) ? $field['tag_extras'] : '') . '>' . $text . '</textarea>';
						
					} else {
						$text = $field['dropdown_values'][$field['value']];
					
					
						$content = '<input type="hidden" value="' . $field['value'] . '" ' .
						'name="item' . $id_item_key . '[' . $field['field_key'] . ']' . (isset($multi_select_ar) ? $multi_select_ar : '') . '">';
						$content .= '<input type="text" ' . ($field['width'] ? 'style="width: ' . $field['width'] . 'px" ' : '') . 'class="' . $read_only_class . '"'. $read_only .
							(isset($field['tag_extras']) ? $field['tag_extras'] : '') . ' value="' . $text . '">';
						
					}
				} else {
					// Bosco 20050822 | Adding new value Popup
					/// Phil 20060223 Checks whether this feature is enabled (disabled for multi edit pages)
					if ($this->getAllowDropdownConfigKey() && $SERVICES->authentication->adminAccess($USER['user_id'],'')) {
						if ($field['config_key'] && stristr($SERVICES->getGlobal('php_self'), 'edit')) {
							$element_name = 'item' . $id_item_key . '[' . $field['field_key'] . ']' . (isset($multi_select_ar) ? $multi_select_ar : '');
							$admin_config = $SERVICES->config->getModuleConfig('Admin');

							// Bosco 20050901 | Added validation for config_key
							if ((array_key_exists($field['config_key'], $admin_config['modulelink'])) && !empty($admin_config['modulelink'][$field['config_key']]['link'])) {

								$new_value_link = $admin_config['modulelink'][$field['config_key']]['link'] . '&id=-1&pop_up=1&element_name=' . $element_name;
								$new_value_options = array(
									'url' => $new_value_link, 
									'window_name' => 'new_value',
									'resizable' => true,
									'scrollbars' => true,
									'width' => 500,
									'height' => 400,
								);
								$this->addJSPopup($field['config_key'] . '_newValue', $new_value_options);
								if (!$this->script_added) {
							
									$js = '<script type="text/javascript">
											var last_id = 0;
                                							function reloadDropdown(new_id, name, element_name) {
												if (last_id != new_id) {
													var obj = eval(\'document.forms["mainform"].elements["\' + element_name + \'"]\');
													var tmp = obj[obj.length - 1];
                                									obj[obj.length - 1] = new Option(name,new_id);
													obj[obj.length] = new Option(tmp.text,tmp.value);
													last_id = new_id;
												}
											}
	        	               						</script>';
	        	        					$this->addToHead($js);	
									$this->script_added = true;
								}
								$tag_extras_popup = ' onChange="javascript:if(this.options[selectedIndex].value==\'popup\'){' . $field['config_key'] . '_newValue();}"';
							}
						} 	
					}
					
					//check for special case lists

					// quick check as to whether or not we need a blank entry.
					// used to be done separate for each case, but that was redundant
					// so doing it all in 1 section
					// set the flag
					$require_empty_value = false;

					switch ($field['dropdown_values']) {
						case '__USERS__':
							$field['dropdown_values'] = $this->getUsersList();
							if (!$field['notnull']) { // if not mandatory - allow empty option
								$require_empty_value = true;
							}
						break;
						case '__USERSALL__': // Phil 20060814 includes deleted users, and resources
							$field['dropdown_values'] = $this->getUsersList(false, false, true, true);
							if (!$field['notnull']) { // if not mandatory - allow empty option
								$require_empty_value = true;
							}
						break;


						case '__GROUPS__':
							$field['dropdown_values'] = $this->getGroupList();

							if (!$field['notnull']) { // if not mandatory - allow empty option
								if ($field['blank_label_for_null']) { // if not mandatory - allow empty option
									$field['dropdown_values'][0] = ''; // unrestricted will overwrite groups[0] (if it exists) groups[0] wont exist
								} else {
									$field['dropdown_values'][0] = $L['Unrestricted']; // unrestricted will overwrite groups[0] (if it exists) groups[0] wont exist
								}
							}

							if ($field['is_in_filter']) {
								$require_empty_value = true;
							}
						break;
						case '__TEAMGROUPS__':
							$field['dropdown_values'] = $this->getGroupList('team');
							if (!$field['notnull']) { // if not mandatory - allow empty option
								if ($field['blank_label_for_null']) { // if not mandatory - allow empty option
									$field['dropdown_values'][0] = ''; // unrestricted will overwrite groups[0] (if it exists) groups[0] wont exist
								} else {
									$field['dropdown_values'][0] = $L['Unrestricted']; // unrestricted will overwrite groups[0] (if it exists) groups[0] wont exist
								}
							}

							if ($field['is_in_filter']) {
								$require_empty_value = true;
							}
						break;
						case '__PRIVSGROUPS__':
							$field['dropdown_values'] = $this->getGroupList('privs');
							if (!$field['notnull']) { // if not mandatory - allow empty option
								if ($field['blank_label_for_null']) { // if not mandatory - allow empty option
									$field['dropdown_values'][0] = ''; // unrestricted will overwrite groups[0] (if it exists) groups[0] wont exist
								} else {
									$field['dropdown_values'][0] = $L['Unrestricted']; // unrestricted will overwrite groups[0] (if it exists) groups[0] wont exist
								}
							}

							if ($field['is_in_filter']) {
								$require_empty_value = true;
							}
						break;
						case '__PRIVATEGROUPS__':
							$field['dropdown_values'] = $this->getGroupList('private');
							if (!$field['notnull']) { // if not mandatory - allow empty option
								if ($field['blank_label_for_null']) { // if not mandatory - allow empty option
									$field['dropdown_values'][0] = ''; // unrestricted will overwrite groups[0] (if it exists) groups[0] wont exist
								} else {
									$field['dropdown_values'][0] = $L['Unrestricted']; // unrestricted will overwrite groups[0] (if it exists) groups[0] wont exist
								}
							}

							if ($field['is_in_filter']) {
								$require_empty_value = true;
							}
						break;

						default:
							if (!$field['notnull']) { // if not mandatory - allow empty option
								$require_empty_value = true;
							}
						break;
					}
					
					// lets allow the drop down values to be accepted as a comma separated list of values as well
					if (!is_array($field['dropdown_values'])) {
						$field['dropdown_values'] = explode(',', $field['dropdown_values']);
					}

					// to allow the values returned to be mapped against different text values allow $field['dropdown_text']
					// to contain the visible text. By default they will be the same.
					if ($field['dropdown_text']) {
						if (!is_array($field['dropdown_text']))	{
							$field['dropdown_text'] = explode(',', $field['dropdown_text']);
						}
					} else {
						$field['dropdown_text'] = $field['dropdown_values'];
					}
					//

					// now draw the list items

					if (($field['type'] == 'dropdown') || ($field['type'] == 'multi_select')) {
						$content = '<select ' . $multi_select . '' . ($field['width'] ? 'style="width: ' . $field['width'] . 'px" ' : '') . '' . $multi_select_size . (empty($read_only_class) ? '' : 'class="' . $read_only_class . '"' ) . ' name="item' . $id_item_key . '[' . $field['field_key'] . ']' . (isset($multi_select_ar) ? $multi_select_ar : '') . '" ' . $field['tag_extras'] . $tag_extras_popup . '>';
	
						// fixups - Brenton Bartel 20050411
						// very specific item: if we're dealing with filters,
						// we will need a blank entry.
						if ($require_empty_value){
							$content .= '<option value=""></option>';
						}
						
						// TODO: print '<div>** value: ' . $field['value'] . '</div>';
						$array_flag = is_array($field['value']);
						foreach ($field['dropdown_values'] as $k => $dropdown_value) {
							$content .= '<option ';
							if ($array_flag) {
								$content .= (in_array($k, $field['value']) ? 'selected ' : '');
							} else {
								// if (!($field['value'] == 'no_search')) { // Phil 20060530 Not necessary. Needs testing with search form (eg CONTAINS, etc)
								if ($field['value'] != '') { // Phil 20060530
									$content .= ($k == $field['value'] ? 'selected ' : '');
								}
								// print '<div>k=' . $k . ', field_key=' . $field['field_key'] . ', value=' . $field['value'] . '</div>';
								// } 
							}
							$content .= 'value="' . bcrm_htmlspecialchars($k, ENT_QUOTES, $this->charset) . '">' . bcrm_htmlspecialchars($dropdown_value, ENT_QUOTES, $this->charset) . '</option>';
						}
						if (!empty($tag_extras_popup)) {
							$content .= '<option value="popup">' . $L['LinkNewValue'] . '</option>';
						}
						$content .= '</select>';
						
					} else {   //then radio_X
						
						//($field['type'] == 'dropdown')
						/*
						<INPUT TYPE=RADIO NAME="pizzasize" VALUE="S">small<BR>
						<INPUT TYPE=RADIO NAME="pizzasize" VALUE="M">medium<BR>
						<INPUT TYPE=RADIO NAME="pizzasize" VALUE="L">large<P>
						*/
						//$content = '<select ' . $multi_select . '' . ($field['width'] ? 'style="width: ' . $field['width'] . 'px" ' : '') . '' . $multi_select_size . (empty($read_only_class) ? '' : 'class="' . $read_only_class . '"' ) . ' name="item' . $id_item_key . '[' . $field['field_key'] . ']' . (isset($multi_select_ar) ? $multi_select_ar : '') . '" ' . $field['tag_extras'] . $tag_extras_popup . '>';
						$content = '';
						
						// fixups - Brenton Bartel 20050411
						// very specific item: if we're dealing with filters,
						// we will need a blank entry.
						/*
						if ($require_empty_value){
							$content .= '<option value=""></option>';
						}
						*/
						$array_flag = is_array($field['value']);
						$content .= '<table border="0" cellpadding="0" cellspacing="0" class="informTbl">'; 
						$content .= ($field['type'] != 'radio_v') ? '<tr>' : ''; 
						foreach ($field['dropdown_values'] as $k => $dropdown_value) {
							$temp_button = '<input type="radio" name="item' . $id_item_key . '[' . $field['field_key'] . ']' . '" ' . $field['tag_extras'];  // . $tag_extras_popup . ''
							if ($array_flag) {
								$temp_button .= (in_array($k, $field['value']) ? 'checked ' : '');
							} else {
								$temp_button .= ($k == $field['value'] ? 'checked ' : '');
							}
							$temp_button .= ' value="' . bcrm_htmlspecialchars($k, ENT_QUOTES, $this->charset) . '">';
							
							$temp_label = bcrm_htmlspecialchars($dropdown_value, ENT_QUOTES, $this->charset);
							
							//$content .= ($field['type'] == 'radio_v') ? '<br>' : '';
							if ($field['type'] == 'radio_v') {
								$content .= '<tr><th>' . $temp_button . '</th><td>' . $temp_label . '</td></tr>'; 
							} else {
								$content .= '<th>' . $temp_button . '</th><td>' . $temp_label . '</td>'; 
							}
						}
						$content .= ($field['type'] != 'radio_v') ? '</tr>' : ''; 
						$content .= '</table>'; 
						
						
						/*
						if (!empty($tag_extras_popup)) {
							$content .= '<option value="popup">' . $L['LinkNewValue'] . '</option>';
						}
						$content .= '</select>';
						*/
					}
				}

				break;

			case 'checkbox':
				if (array_key_exists('checked', $field)) {
					$is_checked = $field['checked'];  //parameter to override usual rule when using an array of checkbooxes with ids; especially to cover the case when id = 1
					$html_value = $field['value'];
				} else {
					$is_checked = $field['value'];  //for normal usage with a boolean 0/1 field direct from the database
					$html_value = 1;
				}

				//TODO relook at ids
				//$field['html_id']  is set by webmatList  20060126
				
				$content = '<input type="checkbox" class="cb' . $read_only_class . '" name="item' . $id_item_key . '[' . $field['field_key'] . ']" ' . 
					(isset($field['tag_extras']) ? $field['tag_extras'] : '') . 
					(isset($field['html_id']) ? ' id="' . $field['html_id'] . '"' : '') . 
					($read_only ? ' disabled ' : '') . ' value="' . $html_value . '"' . ($is_checked ? ' checked' : '') .'>';
				break;

			default:
				// echo 'There was no field type in the definition: '; print_r($field);
				
				$this->logError('Output::getHtmlField There was no field type in the definition');
				$this->logError($field);
				// see the Service class for activating printDebug for the Output Service
				// $this->printDebug('Output::getHtmlField There was no field type in the definition');
				// $this->printDebug($field);
		}

		if (!isset($multi_select_arr)) {

			$multi_select_ar = '';

		}

		//set flag to draw javascript validation; done during buildPage() function
		$this->_setClientSideCheck($field, $multi_select_ar, $id, $extra_text);


		//check for plain text addition
		if (isset($field['text']) && $field['text']) {
			$content = $content . $field['text'];
		}

		return "\n" . $content . "\n";
	}
	
	/**
	 * Sets value and returns current value
	 *
	 * @author Phil Calder
	 */
	function setAllowDropdownConfigKey($value = true) {
		$current_value = $this->getAllowDropdownConfigKey();
		$this->allow_dropdown_config_key = ($value ? true : false);
		return $current_value;
	}
	
	/**
	 * Returns current value
	 *
	 * @author Phil Calder
	 */
	function getAllowDropdownConfigKey() {
		if (!isset($this->allow_dropdown_config_key)) {
			$this->allow_dropdown_config_key = true;
		}
		return $this->allow_dropdown_config_key;
	}
	
	/**
	 *
	 */
	function getStringField($field) {
		global $USER, $L;
		switch ($field['type']) {
			case 'free':   //allows free text or html to be submitted in the field loop // edited 15/1/05
				$content = $field['value'];//'';
				break;

			case 'email':
				if (isset($field['no_link']) && $field['no_link']) { // Phil - see WebMAT::getStringField, WebMATPublicView::getStringField
					$content = $field['value'];
				} else {
					$content = $this->getLinkEmail($field['value'], $field['text']);
					// print '<p>content: ' . $content .'</p>';
				}
				bcrm_unset($field, 'text'); // text value for email is not a text 'addition' (see below)
				break;
			case 'url': // Phil 20041110 - Note: should add this to getHtmlField
				$field['value'] = $this->getUrl(urldecode($field['value'])); //syed 20050622
				if (isset($field['no_link']) && $field['no_link']) { // Phil - see WebMAT::getStringField, WebMATPublicView::getStringField
					$content = $field['value'];
				} else {
					if ($field['value']) {
						$content = $this->getLinkHref(array('href'=>$field['value'], 'target'=>'_blank', 'text'=>$field['text']));
					} else {
						$content = '&nbsp;';
					}
					
				}
				bcrm_unset($field, 'text'); // text value for url may have been sent in - this is not a text 'addition' (see below)
				break;
			case 'text':
				$content = $field['value'];
				break;

			case 'iframe':
				$content = '<iframe width="' . $field['width'] . '" height="' . $field['rows'] . '" src="' . $field['value'] . '"></iframe>';
				//MCR 20051026 slightly experimental - pretty ugly to do it this way
				//to position iframe in a table row on its own below the label, colspan 2
				//MCR 20060221 $content = '</td></tr><tr><td colspan="2">' . $content;
				
				break;

			case 'password':
				$content = '*********';
				break;

			case 'medium_text':
			case 'long_text':
				
				// Phil 20050926 (readded 20051011) - Facility to preserve br tags in content
				if (isset($field['preserve_linebreaks']) && $field['preserve_linebreaks']) {
					$field['value'] = str_replace('<br>', "\n", $field['value']);
					$field['value'] = str_replace('<br />', "\n", $field['value']); // Phil 20051101
				}
				
				//MCR 20060504, introduced this for Mail:medium_text fields; currently just a code-based-parameter
				
				if ( $field['no_html_translate'] ) {
					$content = $field['value'];
				} else {
					// Gurdeep Singh 20050727 | Added get_html_translation_table php function to display HTML tags as text
					// $field['value'] = bcrm_htmlspecialchars($field['value'], ENT_QUOTES, $this->charset); // Removed, Phil 20050307
					
					// Gurdeep Singh 20050727 | Added the code to translate certain characters
					// $html_translation = get_html_translation_table(HTML_ENTITIES);
					// $content = strtr($field['value'], $html_translation);
					
					// Phil 20070605 OK to use this now
					$content =  bcrm_htmlspecialchars($field['value'], ENT_QUOTES, $this->charset);
					
					// Phil 20070412 Turn http and email addrsses into links
					if (strlen($content)) {
						$content = $this->linkify($content);
					}
					
				}
				
				$content = nl2br($content);
				// print_r($field); echo '******<br>' . $content;echo '******<br>';
				// $content = str_replace('\n', "<br>\n", $field['value']); // Phil 20050127
				break;

			case 'contact':
				if (isset($field['no_link']) && $field['no_link']) { // Phil - see WebMAT::getStringField, WebMATPublicView::getStringField
					$content = $this->getContactName($field['value']); // value could be an array or a contact_id
				} else {
					$content = $this->getLinkContact($field['value'], '', $field['popup_only']);
				}
				break;

			case 'hidden':
				$content = '-----';
				break;

			case 'integer':
			case 'float':
				if (isset($field['no_link']) && $field['no_link']) { // Phil 20051124 See Export
					$this->getFormatedNumber($field['value']);
				} else {
					//$content = '<div style="text-align: right">' . $this->getFormatedNumber($field['value']) . '</div>';
					$content = $this->getFormatedNumber($field['value']);
				}
				break;
			
			case 'percentage':
				if (isset($field['no_link']) && $field['no_link']) { // Phil 20051124 See Export
					$this->getFormatedNumber($field['value']*100);
				} else {
					//$content = '<div style="text-align: right">' . $this->getFormatedNumber($field['value']*100) . '%</div>';
					$content = $this->getFormatedNumber($field['value']*100) . '%';
				}
				break;

			case 'currency':
				if ($field['value'] && $field['value'] !== 0) {
					global $SERVICES;
					$currency = $SERVICES->preferences->getUserPref('frontCurrencyFormat', 0);
					switch ($currency) {
						case ('EUR'):
							 $currency_symbol = '&euro;';
							 $number_display_format = 'currency';
							break;

						case ('GBP'):
							 $currency_symbol = ' &pound;';
							 $number_display_format = 'currency';
							break;

						case ('PNL'):
							$currency_symbol = 'PL ';
							$number_display_format = 'currency';
							break;

						case ('RUR'):
							$currency_symbol = 'RU ';
							$number_display_format = 'currency';
							break;

						case ('USD'):
						case ('CAD'):
						case ('NZD'):
						default:
							$currency_symbol = '$';
							$number_display_format = 'currency';
							break;
					}
					if ($number_display_format) {
						$val = $this->getFormatedNumber($field['value'], $number_display_format);
					} else {
						$val = $this->getFormatedNumber($field['value']);
					}
					
					if ($currency_after) {
						$content = $val . $currency_symbol;
					} else {
						$content = $currency_symbol . $val;
					}
					
					/*MCR 20060221
					if (!isset($field['no_link']) || !$field['no_link']) { // Phil 20051124 See Export
						$content = '<div style="text-align: right;  white-space: nowrap;">' . $content . '</div>';
					}
					*/
					
				} else {
					$content = '&nbsp;';
				}
				break;

			case 'integer_date':
			case 'date':
				$content = $this->getDisplayDate($field['value']);
				break;

			case 'date_time':
				$content = $this->getDisplayDate($field['value'], 'date_time');
				break;

			case 'integer_time':
				if ($field['value']) {
					$field['value'] = date(("H:i"), $field['value']);
				}

			case 'time':
				$content = $this->getDisplayDate($field['value'], 'time');
				break;

			case 'multi_select':
				if ($field['csv']) {
					$field['value'] = explode(",", $field['value']);
				}

			case 'dropdown':
			case 'radio_v':
			case 'radio_h':
				// now draw the list items

				if ($field['dropdown_values'] == '__USERS__') {
					$field['dropdown_values'] = $this->getUsersList();
				} else if ($field['dropdown_values'] == '__USERSALL__') {
					$field['dropdown_values'] = $this->getUsersList(false, false, true, true);
					// print '<div>field: ' . $field['field_key'] . ' USERSALL = </div>'; print_r($field['dropdown_values']); print '<hr>';
				} else if ($field['dropdown_values'] == '__GROUPS__') {
					$field['dropdown_values'] = $this->getGroupList();
					if (!$field['notnull']) { // if not mandatory - allow empty option
						$field['dropdown_values'][0] = $L['Unrestricted']; // unrestricted will overwrite groups[0] (if it exists) groups[0] wont exist
					}
				} else if ($field['dropdown_values'] == '__TEAMGROUPS__') {
					$field['dropdown_values'] = $this->getGroupList('team');
					if (!$field['notnull']) { // if not mandatory - allow empty option
						$field['dropdown_values'][0] = $L['Unrestricted']; // unrestricted will overwrite groups[0] (if it exists) groups[0] wont exist
					}
				} else if ($field['dropdown_values'] == '__PRIVSGROUPS__') {
					$field['dropdown_values'] = $this->getGroupList('privs');
					if (!$field['notnull']) { // if not mandatory - allow empty option
						$field['dropdown_values'][0] = $L['Unrestricted']; // unrestricted will overwrite groups[0] (if it exists) groups[0] wont exist
					}
				} else if ($field['dropdown_values'] == '__PRIVATEGROUPS__') {
					$field['dropdown_values'] = $this->getGroupList('private');
					if (!$field['notnull']) { // if not mandatory - allow empty option
						$field['dropdown_values'][0] = $L['Unrestricted']; // unrestricted will overwrite groups[0] (if it exists) groups[0] wont exist
					}
				}  

				if (is_array($field['dropdown_values'])){
					if (is_array($field['value'])) {
						foreach ($field['value'] as $k => $v) {
							$content .= $field['dropdown_values'][$v] . '<br>';
						}
					} else {
						$content = $field['dropdown_values'][$field['value']];
					}

				}
				//$content .= 'value="' . bcrm_htmlspecialchars($k, ENT_QUOTES, $this->charset) . '">' . bcrm_htmlspecialchars($dropdown_value, ENT_QUOTES, $this->charset) . '</option>';
				break;

			case 'checkbox':
				if ($field['list_values']) {
					$false_value = $field['list_values'][0];
					$true_value = $field['list_values'][1];
				} else {
					$false_value = $L['False'];
					$true_value = $L['True'];
				}
				if ($field['value'] == 1) {
					$content = $true_value;
				} else {
					$content = $false_value;
				}
				break;

			case 'file_upload':
				$content = $field['value'];
				break;

			default:
				// echo 'There was no field type in the definition:.. '; print_r($field);
				$this->logError('Output::getStringField There was no field type in the definition');
				$this->logError($field);
				// see the Service class for activating printDebug for the Output Service
				// $this->printDebug('Output::getStringField There was no field type in the definition');
				// $this->printDebug($field);
		}

		//check for plain text addition
		if (isset($field['text']) && $field['text']) {
			$content = $content . $field['text'];
		}

		// Phil 20051031 Trims table text based on field value or user preference
		if (!isset($field['list_detail_length']) || !$field['list_detail_length']) {
			$field['list_detail_length'] = $this->getListDetailLength($field); // ['type']);
			// print '<div>field_key: ' . $field['field_key'] . ' length: ' . $field['list_detail_length'] . '</div>';
		}
		// print '<div>field_key: ' . $field['field_key'] . ' length: ' . $field['list_detail_length'] . '</div>';
		
		if ( ($field['list_detail_length'] !== false) && ($field['list_detail_length'] != 'All') && ($field['list_detail_length'] != '') ) {
			if (strlen($field['value']) > $field['list_detail_length']) {
				$content = substr($field['value'], 0, $field['list_detail_length']) . '...';
			}
		}
		
		//display_with_nbsp processing, needs to be after trim above
		if ($field['display_with_nbsp']) {
			if (isset($field['no_link']) && $field['no_link']) { // see eg Export, WebMAT::getStringField, WebMATPublicView::getStringField
				//
			} else {
				$content = str_replace(' ', '&nbsp;', $content);
			}
		}
		// print '<div>field: ' . $field['field_key'] . ', type: ' . $field['type'] . ',length: ' . $field['list_detail_length'] . '</div>';
		return $content;
	} // getStringField
	
	/**
	 * Returns the user preference for list detail length
	 *
	 * Can be overridden by setting {@link setNoTrim}
	 * contact, email, and url fields are not trimmed
	 *
	 * @author Phil Calder
	 */
	function getListDetailLength($field = false) { // $type = false) {
		if ($this->getNoTrim()) {
			// print '<div>returning false</div>';
			return false;	
		}
		if ($field['type'] && $this->isTrimtype($field)) { // is_array($no_trim_types = $this->getNoTrimTypes()) && !in_array($type, $no_trim_types)) {
			global $SERVICES;
			$prefs =& $SERVICES->getService('Preferences');
			// print '<div>pref: ' . $prefs->getUserPref('cfgListDetailLength', 'All') . '</div>';
			return $prefs->getUserPref('cfgListDetailLength', 'All');
		
		}
		return false;
	}
	
	/**
	 * Sets the no trim value for {@link getListDetailLength}
	 *
	 * @param boolean $value The no trim value to set (Optional, default true)
	 * @return boolean The current no trim value
	 * @author Phil Calder
	 */
	function setNoTrim($value = true) {
		$current = $this->no_trim;
		$this->no_trim = $value;
		return $current;
	}
	/**
	 * Gets the no trim value for {@link getListDetailLength}
	 *
	 * @return boolean The current no trim value
	 * @author Phil Calder
	 */
	function getNoTrim() {
		return $this->no_trim;
	}
	
	/**
	 * Sets an array of field types we dont trim
	 *
	 * Default functionality is to merge with existing types. Override this with parameter $override
	 *
	 * @param array $types An array of field types we dont trim (eg contact, url, email)
	 * @param boolean $override When true will replace existing no trim types, otherwise merges.
	 * @return array Field types not to trim
	 * @author Phil Calder
	 */
	function setNoTrimTypes($types = array(), $override = false) {
		if ($override) {
			$this->no_trim_types = (array)$types;
		} else {
			$this->no_trim_types = bcrm_array_merge((array)$this->no_trim_types, (array)$types);
		}
		// print_r($this->no_trim_types);
	}
	
	/**
	 * Gets an array of field types we dont trim
	 *
	 * @return array Field types not to trim
	 * @author Phil Calder
	 */
	function getNoTrimTypes() {
		return (array)$this->no_trim_types; // eg array('contact', 'url', 'email'));
	}
	
	function isTrimType($field = false) {
		
		if (isset($field['foreign']) && $field['foreign']) {
			return false;
		}
		if (isset($field['linked']) && $field['linked']) {
			return false;
		}
		if (isset($field['type']) && in_array($field['type'], $this->getNoTrimTypes()) ) {
			return false;
		}
		return true;
	}


	/**
	 *
	 */
	function getFormatedNumber($number, $format = '') {
		global $SERVICES;
		if (!$format) {

			$SERVICES->getService('Preferences');
			$format = $SERVICES->preferences->getUserPref('frontNumberFormat', 0);

		}
		switch ($format) {
			case 'francais':
				return number_format($number, '', ',', ' ');
				break;
			case 'currency':
				return number_format($number, 2);
				break;
			default:
				return number_format($number, '');
				break;
		}
	}

	// Paging functions

	/**
	 * Sets variables needed by paging functions and returns a MySQL limit clause
	 *
	 * @param string $pref_name The paging limits preference. Optional.
	 * @param integer $items_count The total number of items in the query we are paging.
	 * @param boolean $return_str Either true: string, false: array
	 * @return mixed Either string sql limit clause for module specific getItems function, or a limit array with keys start,count
	 * eg LIMIT 1, 100 (start at 1, return 100 results)
	 */
	function setPagingLimits($pref_name = '', $items_count = 0, $return_str = true) {
		if (empty($pref_name)) {
			$pref_name = 'frontDefaultListSize';
		}

		$this->paging_pref_name = $pref_name;

		if (isset($_GET['view_all']) && $_GET['view_all']) {
			$this->paging_list_size = $this->paging_max_list_size;
		} else {
			global $SERVICES;
			$SERVICES->getService('Preferences');
			// $this->paging_list_size = ( ($val = intVal($SERVICES->preferences->getUserPref($pref_name))) > 0 ? $val : $this->paging_list_size);
			// Phil - 20051115 Correcting for cases where pref value is 'All'
			$pref_value = $SERVICES->preferences->getUserPref($pref_name);
			
			if (strtolower($pref_value) == strtolower('All')) {
				$this->paging_list_size = $this->paging_max_list_size;
			} else {
				$this->paging_list_size = ( ($val = intVal($pref_value)) > 0 ? $val : $this->paging_list_size);
			}
		}

		$this->paging_items_count = ( ($val = intVal($items_count)) > 0 ? $val : $this->paging_items_count);
		$this->paging_start_num = $this->getPagingItemStartNumber($this->paging_list_size);

		if ($return_str) {
			$limit = 'LIMIT ' . $this->paging_start_num . ', ' . $this->paging_list_size . '';
		} else {
			$limit = array(
				'start' => $this->paging_start_num, 
				'count' => $this->paging_list_size, 
			);
		}

		return $limit;
	}


	/**
	 * given a number of items to display on each page and a page number ($_GET['page_num'])
	 * @return integer The item index number to display first, or 0
	 */
	function getPagingItemStartNumber($num_items_per_page = 20) {
		if ( (!isset($_GET['page_num'])) || ($_GET['page_num'] < 2) || (!intVal($num_items_per_page))) {
			return 0;
		}

		return (($_GET['page_num']-1) * $num_items_per_page);
	}

	/**
	 * return pagelinks based on parameters:
	 * eg: Page 5 of 6   prev | next  |<  2 3 4 5 6 ]
	 * The default number of items/page is hard coded in paging_max_list_size
	 * The number of page links is hard coded in function paging_num_page_links
	 * (could be a company / user pref in future)
	 */
	function getPagingLinks($item_count, $cur_page = 1, $items_per_page = 20) {
		global $L, $SERVICES;

		if(!$cur_page) {
			$cur_page = 1; // by default, show page 1
		}

		if ((!intVal($items_per_page))) {
			$items_per_page = $this->paging_max_list_size; // eg 1000 items per page
			$total_num_pages = 1;
			$num_page_links = 0;
		}

		$total_num_pages =  ceil($item_count/$items_per_page);

		// print '<div>item_count = ' . $item_count . '</div>';
		// Dont display any paging links if there is only one page
		if (($cur_page == 1) && ($total_num_pages == 1)) {
			// print '<div>not displaying paging links</div>';
			return '';
		}

		$num_page_links = $this->paging_num_page_links; // links to other pages, eg [1 2 3 4 5]

		$content = '';

		// start page_links from...
		$first_page_link = ($cur_page - floor($num_page_links / 2));

		// Cant be more than the total number of pages
		if (($first_page_link + $num_page_links) > $total_num_pages) {
			$first_page_link = $total_num_pages - $num_page_links +1;
		}
		// ..or less than 1
		if ($first_page_link < 1) {
			$first_page_link = 1;
		}

		// Build a list of page numbers / links
		$page_num = $first_page_link;
		for (
			;
			($page_num <= $total_num_pages) AND ($page_num < ($first_page_link + $num_page_links));
			$page_num++
		) {

			//MCR 20060130; keep current page as a link for a refresh option
			$new_query = $this->updateQueryString(array('page_num'=>$page_num));
			$content .= '<a href="' . $SERVICES->getGlobal('php_self') . $new_query . '" class="' . ($page_num == $cur_page ? 'c' : '') . 'pg">' . $page_num . '</a>';
		}

		// prev and next links
		$new_query = $this->updateQueryString(array('page_num'=>($cur_page-1)));
		$previous_link = ( (($cur_page)>=2) ? '<a href="' . $SERVICES->getGlobal('php_self') . $new_query . '" class="pg">' . $L['Previous'] . '</a>' : $L['Previous']);

		$new_query = $this->updateQueryString(array('page_num'=>($cur_page+1)));
		$next_link = ( (($cur_page) < $total_num_pages) ? '<a href="' . $SERVICES->getGlobal('php_self') . $new_query . '" class="pg">' . $L['Next'] . '</a>' : $L['Next']);

		// first link (|<) and  last link (>|)
		$new_query = $this->updateQueryString(array('page_num'=>1));
		$first_link = ( (($first_page_link)>1) ? '<a href="' . $SERVICES->getGlobal('php_self') . $new_query . '" class="pg">' . '|&lt;' . '</a>' : '');

		$new_query = $this->updateQueryString(array('page_num'=>($total_num_pages)));
		$last_link = ( ($page_num <= $total_num_pages) ? '<a href="' . $SERVICES->getGlobal('php_self') . $new_query . '" class="pg">' . '&gt;|' . '</a>' : '');

		// make total pages link
		$total_pages = $total_num_pages;
		//  put them all together
		$content = '' . $L['Page'] . ' ' . $cur_page . ' ' . $L['ListOf'] . ' ' . $total_pages . ' &nbsp;&nbsp; ' . $previous_link . ' ' . $next_link . ' &nbsp;&nbsp; ' . $first_link . $content . $last_link . '';
		return $content;
	}

	/**
	 * Joins these three parameters together and returns a string
	 * eg: Showing 1 - 5 of 20
	 */
	function getPagingListShowing($first, $last, $total) {
		global $L;
		$content = '' . $L['ListShowing'] . ' ' . $first . ' ' . $L['ListTo'] . ' ' . $last . '&nbsp;' . $L['ListOf'] . '&nbsp;' . $total . '';
		if ($last <= 1 || $total <= 1) {
			$content = '';
		}
		return $content;
	}

	/**
	 * Given the last displayed item, and the number of items, return link to 'View All' or 'Page View'
	 * updates existing query string (overwriting old values where necessary)
	 */
	function getPagingLinkViewAll($last_displayed, $items_count, $labels= array()) {
		global $L, $SERVICES;

		if (!isset($labels['all'])) {
			if ($items_count > $this->paging_max_list_size) {
				$labels['more'] = $L['ListViewMore'];
			} else {
				$labels['more'] = $L['ListViewAll'];
			}
		}
		if (!isset($labels['less'])) {
			$labels['less'] = $L['ListPageView'];
		}

		// print '<div>last_displayed: ' . $last_displayed . ', items_count: ' . $items_count. '</div>';
		//removed from if statement below: ($last_displayed <= $items_count)  &&
		if (!isset($_GET['view_all']) || !$_GET['view_all']) {
			//if ($last_displayed >= $items_count) {
			if ($items_count <= $this->paging_list_size) {   //Syed 20050624
				$content = '';
			} else {
				$new_query = $this->updateQueryString(array('view_all'=>1, 'page_num'=>1));
				// $content = '<a href="' . $SERVICES->getGlobal('php_self') . '' . $new_query . '" class="menu _ links">' . $L['ListViewAll'] . '</a>';
				$content = '<a href="' . $SERVICES->getGlobal('php_self') . '' . $new_query . '" class="pg">' . $labels['more'] . '</a>';
			}

		} else {
			$new_query = $this->updateQueryString(array('view_all'=>0));
			// $content = '<a href="' . $SERVICES->getGlobal('php_self') . '' . $new_query . '" class="menu _ links">' . $L['ListPageView'] . '</a>';
			$content = '<a href="' . $SERVICES->getGlobal('php_self') . '' . $new_query . '" class="pg">' . $labels['less'] . '</a>';
		}

		return $content;
	}
	// --

	/**
	 * accepts an array of new query fields as key=>val
	 * overwrites existing query string values and returns new query string
	 */
	function updateQueryString($new_values, $include_questionmark = true) {
		global $SERVICES;
		$query_arr = array();
		if (is_array($new_values)) {
			parse_str($SERVICES->getGlobal('query_string'), $query_arr);
			$query_arr =  $new_values + $query_arr;
		}
		$arr = array();
		foreach ($query_arr as $k => $val) {
			if ($val != 'Array' && !is_array($val)) {
				$arr[] = $k . '=' . $val;
			}
		}
		if ($include_questionmark ) {
			$begin = '?';
		} else {
			$begin = '';
		}
		$query_str = '';
		if (sizeof($arr)) {
			$query_str = $begin . implode('&', $arr);
		}
		return $query_str;
	}

	/**
	 *
	 */
	function _setClientSideCheck($field, $multi_select = false, $id = false, $extra_text = false) {
		// Phil 20060810 Added option to disable client side check for a field
		// even if the item has an id
		// print '<div>_setClientSideCheck</div>'; print_r($field); print '<hr>';
		if (isset($field['no_client_validation'])&& $field['no_client_validation']) {
			// print '<div>*** No Client Validation for ' . $field['field_key'] . '</div>';
			return;
		}
		
		if ($id) {
			$field['multi_id'] = $id;
		}

		if ($extra_text) {
			$field['extra_text'] = $extra_text;
		}

		if ($multi_select) {
			$field['field_key'] = $field['field_key'] . '][';
		}

		$this->client_side_checks[] = $field;
	}

	/**
	 *
	 */
	function setFieldDefaults() {
		global $SERVICES, $L;
		$SERVICES->getservice('Preferences');
		$SERVICES->getservice('DateLib');

		//date regex
		$separator = $SERVICES->preferences->getUserPref('cfgDateSeparator');
		$mask = $SERVICES->datelib->getJsDateMaskFromUserPref();

		// $list = explode($SERVICES->preferences->getUserPref('cfgDateSeparator'), $SERVICES->datelib->getJsDateMaskFromUserPref());

		// Phil, This is already set above
		// $separator = $SERVICES->preferences->getUserPref('cfgDateSeparator');
		if (empty($separator)) {
			$separator = '/'; // Ensures we have a default value
		}

		$list = explode($separator, $SERVICES->datelib->getJsDateMaskFromUserPref());
		$date_regex = '^';

		if (sizeof($list) < 1) {
			$date_regex .= "[0-9]{2}/[0-9]{2}/[0-9]{4}";
		} else {
			foreach($list as $k => $form){
				if ($form == 'yyyy'){
					$date_regex .= "[0-9]{4}$separator";
				} elseif ($form == 'yy'){
					$date_regex .= "[0-9]{2}$separator";
				} else {
					$date_regex .= "[0-9]{1,2}$separator";
				}

			}
			$date_regex = trim($date_regex, "$separator");
		}

		$date_regex .= '$';
		
		//time regex
		// $separator = $SERVICES->preferences->getUserPref('cfgTimeSeparator');
		// $mask = $SERVICES->getUserTimeFormat();
		// $list = explode($SERVICES->preferences->getUserPref('cfgTimeSeparator'), $SERVICES->getUserTimeFormat());
		
		$time_regex = '^';
		$time_regex .= "[0-9]{1,2}" . $SERVICES->preferences->getUserPref('cfgTimeSeparator') . "[0-9]{2}";
		//$time_regex .= "[0-9]{2}:[0-9]{2}";
		$time_regex .= '$';
		
		$email_regex = '^';
		$email_regex .='[_a-zA-Z0-9-]+(\.[_a-zA-Z0-9-]+)*@[a-zA-Z0-9-]+(\.[a-zA-Z0-9-]+)*(\.[a-zA-Z]{2,3})';
		$email_regex .='$';
		
		// Phil 20070416 Added url regex
		$url_regex = '^';
		// $url_regex .='(([a-z0-9\.]+):\/\/)?([a-z0-9-]+(\.[a-z0-9-]+)+)?';
		$url_regex .='(([a-z0-9\.]+):\/\/)?([a-z0-9-]+(\.[a-z0-9-]+(.*))+)?';
		$url_regex .='$';
		
		$this->field_defaults = array(
			'text' => array('maxlength'=> 128),
			'integer' => array('min'=> 0, 'max'=>999),
			'integer_date' => array('regex' => $date_regex),
			'date' => array('regex' => $date_regex),
			'integer_time' => array('regex' => $time_regex),
			'time' => array('regex' => $time_regex),
			'email' => array('regex' => $email_regex), 
			'url' => array('regex' => $url_regex) // Phil 20070416 Added url default and regex
		);
	}
	
	/**
	 * @return field default constraints for when none are submitted with the field definition
	 */
	function getFieldDefaults() {
		//print_r($this->field_defaults);
		return $this->field_defaults;
	}
	
	/**
	 * @return the params to check with Server Side validation
	 */
	function getServerSideFieldChecks($field_type) {

		switch ($field_type) {
			case 'medium_text': 
			case 'long_text': 
				return array('notnull', 'regex', 'maxlength');
			case 'text': return array('notnull', 'regex', 'maxlength');  //see this is different to the JS checks
			case 'email': return array('notnull', 'regex', 'maxlength');  //see this is different to the JS checks  // 20050622 S.Senthil Validate email field
			case 'url':  return array('notnull', 'regex'); // Phil 20070416 Added URL
			case 'integer': return array('notnull', 'min', 'max');
			case 'float': return array('notnull', 'min', 'max');
			case 'currency': return array('notnull', 'min', 'max');
			case 'dropdown': return array('notnull');
			case 'multi_select': return array('notnull');
			case 'radio': return array('notnull');
			case 'integer_date':
			case 'date':
				return array('notnull', 'regex');
			case 'integer_time':
			case 'time':
				return array('notnull', 'regex');
		}

		return null;
	}

	/**
	 * @return the parameters to check with JavaScript for each field type
	 */
	function getJsFieldChecks($field_type) {

		switch ($field_type) {
			case 'medium_text': 
			case 'long_text': 
				return array('notnull', 'regex', 'maxlength');
			case 'text': return array('notnull', 'regex');
			case 'email': return array('notnull', 'regex', 'maxlength');  //// 20050622 S.Senthil Validate email field change the attribute email to regex
			case 'url':  return array('notnull', 'regex'); // Phil 20070416 Added regex check
			case 'integer': return array('notnull', 'min', 'max');
			case 'float': return array('notnull', 'min', 'max');
			case 'currency': return array('notnull', 'min', 'max');
			case 'dropdown': return array('notnull');
			case 'multi_select': return array('notnull');
			case 'radio': return array('notnull');
			case 'integer_date':
			case 'date':
				return array('notnull', 'regex');
			case 'integer_time':
			case 'time':
				return array('notnull', 'regex');
		}

		return null;
	}

	/**
	 *
	 */
	function _getClientValidationScript() {
		// returns the javascript necessary for the page to perform client validation
		// based on the contents of the array $client_side_checks. The function validateForm
		// checks that the field value matches the regexp $client_side_checks['field_key']['format']
		// if $client_side_checks['field_key']['non_empty'] that the field is non empty.
		// If either error exists an alert box is displayed and the page is not submitted
		// To use - the form must be called 'mainform' and the 'field_key' must match a
		// field name in the form.
       	global $L;
		$client_side_checks = $this->client_side_checks;
		// print '<div>_getClientValidationScript, client_side_checks:</div>'; print_r($client_side_checks); print '<hr>';

		if ($this->js_validate) {

			//document.forms[\'mainform\'].elements[\'submit\'].disabled = true;
			if (sizeof($client_side_checks) > 0) {
				$content = '
						<script type="text/javascript">
						function validateForm(){
							var message= \'\';
							';
				$field_defaults = $this->getFieldDefaults();

				foreach ($client_side_checks as $k => $client_side_check) {

					$notnull_js = '';
					$other_checks_js = '';

					if (isset($client_side_check['multi_id'])) {
						$name = 'items[' . $client_side_check['multi_id'] . '][' . $client_side_check['field_key'] . ']';
						if ($client_side_check['multi_id'] < 1) {
							$ignore_check = '(!document.forms["mainform"].elements["items[' . $client_side_check['multi_id'] . '][_ignore_row]"].checked) && ';
						} else {
							$ignore_check = '';
						}
						bcrm_unset($client_side_check, 'multi_id');
					} else {
						$name = 'item[' . $client_side_check['field_key'] . ']';
						$ignore_check = '';
					}
					
					//MSC 20050121 to bug fix for multi select
					if ($client_side_check['type'] == 'multi_select') {
						$name .= '[]';
					}
					if (isset($client_side_check['extra_text'])) {
						$extra_text = $client_side_check['extra_text'];
						bcrm_unset($client_side_check, 'extra_text');
					}

					//new MCR
					$checks = $this->getJsFieldChecks($client_side_check['type']);  	//get checks that may be needed, based on field type
					// print '<div>type: ' . $client_side_check['type'] . '</div>'; print_r($checks); print '<hr>'; print_r($field_defaults); print '<hr>';
					if (is_array($checks)) {
						$checks = array_flip($checks);

						foreach ($checks as $k => $v) {

							if (array_key_exists($k, $client_side_check) && $client_side_check[$k]) {   //if check param has been passed with field def
								$checks[$k] = $client_side_check[$k];
							} elseif (is_array($field_defaults[$client_side_check['type']])) {		//if there is a default list for this type
								if(array_key_exists($k, $field_defaults[$client_side_check['type']])) {	//if check param is in defaults list for this field type
									$checks[$k] = $field_defaults[$client_side_check['type']][$k];
								} else {
									bcrm_unset($checks, $k);
								}
							} else {
								bcrm_unset($checks, $k);
							}
						}
						
						if (is_array($checks)) {
							foreach ($checks as $check => $setting) {
							//if (in_array($check,$checks)) {
								//if the check is in our js_checks list

								switch ($check) {
									case 'notnull':
										if (!isset($this->ignore_notnull_clientside_check) || !$this->ignore_notnull_clientside_check) { // Phil 20060201 Added to prevent not null checks (mainly for list page search filter)
											$notnull_js = '
											if ('. $ignore_check .' (document.forms["mainform"].elements["'.$name.'"].value == \'\')) {
											message = message + \''. (isset($extra_text) ? $extra_text : '') .$client_side_check['label'].' '.$L['errEmptyFieldNamed'] . '\' + \'\n\';
											}';
										}
										break;
									case 'regex':
										// print '<p>Setting: ' . $setting . '</p>'; exit();
										$other_checks_js .= '
										re = new RegExp(\'' . $setting . '\');
										if ('. $ignore_check .' (!document.forms["mainform"].elements["'.$name.'"].value.match(re) && document.forms["mainform"].elements["'.$name.'"].value != \'\')){
											message = message + \''. $extra_text .$client_side_check['label'].' '. $L['errFormat'] .'\' + \'\n\';
										}';
										break;
									case 'email':
										// TODO
										$other_checks_js .= 'alert("email");';
										break;
									case 'max':
										$other_checks_js .= '
										if (document.forms["mainform"].elements["'.$name.'"].value > ' . $setting .' && document.forms["mainform"].elements["'.$name.'"].value != \'\'){
											message = message + \'' . $client_side_check['label'] . ' ' . $L['errExceedMaxFieldName'] . ' ' . $setting . '\' + \'\n\';
										}';
										break;
									case 'notnull':
										$other_checks_js .= '
										if (document.forms["mainform"].elements["'.$name.'"].value < ' . $setting .' && document.forms["mainform"].elements["'.$name.'"].value != \'\'){
											message = message + \'' . $client_side_check['label'] . ' ' . $L['errExceedMinFieldName'] . ' ' . $setting . '\' + \'\n\';
										}';
										break;
									case 'min':
										$other_checks_js .= '
										if (document.forms["mainform"].elements["'.$name.'"].value < ' . $setting .' && document.forms["mainform"].elements["'.$name.'"].value != \'\'){
											message = message + \'' . $client_side_check['label'] . ' ' . $L['errExceedMinFieldName'] . ' ' . $setting . '\' + \'\n\';
										}';
										break;
								}
							}
						}
						
						if ($notnull_js != '') {
							if ($other_checks_js != ''){
							$notnull_js .= ' else {
												' . $other_checks_js . '
										}';
							}
							$content .= $notnull_js;
						} else {
							$content .= $other_checks_js;
						}
					}
				}
				$content .= '
						if (message == \'\'){
							return true;
						} else {
							alert(message);
							return false;
						}

					}
					</script>';
			} else {
				$content = '
				<script type="text/javascript">
					function validateForm(){
						return true;
					}
				</script>';
			}
			return $content;
		}
	}

	/**
	 *
	 */
	function validateFields(&$item, &$fields, &$error_msg) {
		
		reset($fields);
		$has_error = false;
		foreach ($fields as $k => $v) {
			//print_r($v);
			$validated = $this->_validateField($v);
			if ($validated['error']) {
				$error_msg[$v['field_key']] = $validated['error'];
				$has_error = true;
			}
			$item[$v['field_key']] = $validated['value'];
			//print_r($v); echo '<hr>';
		}
		// print_r($item); print_r($fields); print '<hr>'; exit();
		// Phil 20050307
		// Need to remove slashes so we can display these fields when the page is loaded.
		// This is consistant with getting data from the database
		if ($has_error) {
			foreach($fields as $k => $v) {
				// Brenton 20050316
				// Only strip slashes if it's not an array - otherwise it messes up the array
				//if (!is_array($item[$v['field_key']])) {
					// Brenton 20050316
					// and only strip if it's not numeric, otherwise it returns a string
					// which could break something later on (ex: timestamp, then date stuff breaks)
				//	if (!is_numeric($item[$v['field_key']])) {
						$item[$v['field_key']] = stripslashes($item[$v['field_key']]);
				//	}
				//}
				//else{
				//	print_r($item[$v['field_key']]); echo "<hr />";
				//}
			}
		}
	}

	/**
	 *
	 */
	function _validateField($field) {
		// print '<p>_validateField</p>'; print_r($field); print '<hr>';
		// validates a field with any constraints passed with $field and
		//any other default constraints based on the data type - $field['type']

		//the function returns an error msg if there is one and the updated value

		// The constraints tested for are as follows: 'format' - a regex for
		// comparison, 'min', 'max', 'maxlength', 'notnull'.
		global $L, $SERVICES, $USER;
		if (is_array($field['value'])) {
			//date_time combo
			if ($field['type'] == 'date_time') {  //special block to split date_time into two fields

				$temp_error_msg = '';
				$temp_val = '';
				$temp_date = $field['value']['date'];
				$temp_time = $field['value']['time'];

				if (trim($temp_date) == '' || trim($temp_time) == '') { // Either they are both set, or neither of them should be
					$temp_date = $temp_time = '';
					return array('error'=>'', 'value'=>'');
				}

				//prepare to pass to function (recursively) to get date error msg
				$field['type'] = 'integer_date';
				$field['value'] = $temp_date;
				$validated = $this->_validateField($field);
				if ($validated['error']) {
					$temp_error_msg = $validated['error'];
				}
				
				//prepare to pass to function (recursively) to get time error msg
				$field['type'] = 'time';
				$field['value'] = $temp_time;
				$validated = $this->_validateField($field);
				if ($validated['error']) {
					$temp_error_msg .= $validated['error'];
				}
				
				$value = $SERVICES->datelib->getTimestampFromUserPrefDateAndTime($temp_date, $temp_time);
				//offset by timezone for write
				$value = $SERVICES->offsetTimeZoneForWrite($value);
				return array('error'=>$temp_error_msg, 'value'=>$value);
			}

			//add processing if necessary (multi-selects)
			if ($field['type'] == 'multi_select') {
				if ($field['csv']) {
					if (is_array($field['value'])) {
						$field['value'] = implode(",", $field['value']);
					}
				}
			}
			//checkbox arrays also process here; this is good; we don't process them like normal checkboxes below
			//
			return array('error'=>$msg, 'value'=>$field['value']);
		}
		
		$msg = '';
		
		// Added by Matt Corich to fix the file upload problem with not null
		if ($field['type'] == 'upload_file') {
			$field['value'] = $_FILES['item']['name'][$field['field_key']];
		}
		//

		// notnull test first
		if (isset($field['notnull']) && $field['notnull'] && !strlen($field['value'])) {
			$msg .= $field['label'] . ' ' . $L['errEmptyField'] . '<br>';
		} else {
			//Added By Matthew Corich for file_upload
			if ($field['type'] == 'file_upload') {
				global $SERVICES;
				$SERVICES->getService('FileSystem');
				if ($field['get_content']) {
					$file_info = $SERVICES->filesystem->processUpload($field['field_key'], false);
				} else {
					$file_info = $SERVICES->filesystem->processUpload($field['field_key'], true);
				}
				if ($SERVICES->filesystem->hasError()) {
					$msg .= $field['label'] . ' ' . $SERVICES->filesystem->printError();
				} else {
					$field['value'] = $file_info;
				}
			}

			// Added By Matthew Corich for percentage value
			if ($field['type'] == 'percentage') {
				if (!preg_match('/^-{0,1}[0-9]*\.{0,1}[0-9]*%{0,1}$/', $field['value'])) {
					$msg .= $field['label'] . ' ' . $L['errNotPercentageFormat'] . '<br>';
				} else {
					// Else Statement
				}
				$field['value'] =  rtrim ($field['value'] , '%');
				$field['value'] = $field['value']/100;
				$value = $field['value'];
			}
			
			if ($field['type'] == 'email') {
				if ( ($field['value'] === '0') || ($field['value'] && !$this->isValidEmail($field['value']))) {
					$msg .= $field['label'] . ' ' . $L['errEmail'] . '<br>';
				}
			}
			
			// print '<p>field:</p>'; print_r($field); print '<hr>'; exit();
			if ($field['type'] == 'url') {
				// print '<p>url:</p>'; print_r($field); print '<hr>';
				if ( ($field['value'] == $this->getDefaultUrlPrefix()) || ($field['value'] == "") ) { // Phil 20050810
					if (!isset($field['notnull']) || !$field['notnull']) {
						// $value = $field['value'] = ' '; // Phil 20050810 - Lets not reject a form because of the default url prefix, unless it is required.
						$value = $field['value'] = ''; // Phil 20070416 - Allow it to be empty
					} else {
						$msg .= $field['label'] . ' ' . $L['errEmptyField'] . '<br>';
						$msg .= $L['errUrl'] . '<br>';
					}
					// if ($field['notnull'] && !$this->isValidURL($field['value'])) { // Phil 20050810
					// if (!$this->isValidURL($field['value'])) { // Phil 20050810
						// $msg .= $L['errUrl'] . '<br>';
					// }
				} else {
					// print '<div>***** Check for valid URL [' . $field['value'] . '] **************************</div>';
					if (!$this->isValidURL($field['value'])) { // Phil 20050810
						$msg .= $L['errUrl'] . '<br>';
						// print '<div>******************************* ' . $L['errUrl'] . '</div>';
					}
				}
			}
			if (strlen($field['value'])) {  //if submit value is not null, perform checks
				//$field['value'] = trim($field['value']);
				// maxlength
				if (isset($field['maxlength']) && $field['maxlength'] && (strlen($db_val) > $field['maxlength'])) {
					$msg .= $field['label'] . ' ' . $L['errTooLong'] . '(' . $field['maxlength'].')' . '<br>';
				}
				// check regex for format
				if (isset($field['format']) && $field['format'] && strlen($field['value']) && !ereg($field['format'], $field['value'])) {
					$msg .= $field['label'] . ' ' . $L['errFormat'] . '<br>';
				}

				// Phil - Commented Out 20040617
				// numbers - using max and min
				// if ($field['type'] == 'float' || $field['type'] == 'integer' && $field['value']) {
					// // if there isn't a specific formular defined check that it's a number
					// if (!$fields['format']) {
						// if (($field['value']) && !is_numeric($field['value'])) {
							// $msg .= $field['label'] . ' ' . $L['errNotNumeric'] . '<br>';
						// }
						// // not an integer
						// else if ($field['type'] == 'integer' && (intval($field['value']) != $field['value'])) {
							// $msg .= $field['label'] . ' ' . $L['errNotInteger'] . '<br>';
						// }
					// }
					// if ($field['value'] < $fields['min']) {
						// $msg .= $field['label'] . ' ' . $L['errTooSmall'] . $fields['min'] . '<br>';
					// }
					// if ($field['value'] > $fields['max']) {
						// $msg .= $field['label'] . ' ' . $L['errTooLarge'] . $fields['max'] . '<br>';
					// }
				// }

				if ($field['type'] == 'float' || $field['type'] == 'currency' || $field['type'] == 'integer' || $field['type'] == 'percentage' && $field['value']) {
					// if there isn't a specific formular defined check that it's a number
					//Pecentage dose no check for is_numeric
					if ($field['type'] == 'currency') {
						$field['value'] = str_replace(array('&euro;', ' &pound;', 'PL ', 'RU ', '$'), '',$field['value']);
						$field['value'] = str_replace(',', '', $field['value']);
					}
					if (!$fields['format'] && $field['type'] != 'percentage') {
						if (($field['value']) && !is_numeric($field['value'])) {
							$msg .= $field['label'] . ' ' . $L['errNotNumeric'] . '<br>';
						}
						// not an integer
						else if ($field['type'] == 'integer' && (intval($field['value']) != $field['value'])) {
							$msg .= $field['label'] . ' ' . $L['errNotInteger'] . '<br>';
						}
					}
					// Matt C - Changed because not working 20040621
					if ($field['min'] || $field['max']) {
						if ($field['value'] < $field['min']) {
							 $msg .= $field['label'] . ' ' . $L['errTooSmall'] . $field['min'] . '<br>';
						}
						if ($field['value'] > $field['max']) {
							$msg .= $field['label'] . ' ' . $L['errTooLarge'] . $field['max'] . '<br>';
							//$msg .= $field['label'] . ' ' . $L['errTooLarge'] . $field['max'] . ' ( value: ' . $field['value'] . ')<br>';
						}
					}
				}
				//regex checks
				
				// convert date
				if (($field['type'] == 'date') || ($field['type'] == 'integer_date')) {
					if (($field['type'] == 'integer_date') && ($field['hidden'])) { // a hidden integer_date is already a timestamp
						// do nothing
					} else {
						$converted_value = $SERVICES->datelib->getMySqlDateFromUserPrefDate($field['value']);
						// TODO: Phil 20060615
						// if (is_int($field['value'])) {
							// $converted_value = $SERVICES->datelib->getMySqlDateFromUserPrefDate($field['value']);
						// } else if (is_array($field['value'])) {
							// foreach ($field['value'] as $k => $v) {
								// $field['value'][$k] =$SERVICES->datelib->getMySqlDateFromUserPrefDate($v);
								// // print '<div>converting v: ' . $v . ' converted: ' . $field['value'][$k] . '</div>';
							// }
						// }
						
						if (!$converted_value) {
							$msg .= $field['label'] . ' ' . $L['errNotDateFormat'] . '<br>';
						} else {
							$value = $converted_value;
						}

						if ($field['type'] == 'integer_date') {
							$value = $SERVICES->datelib->getTimestampFromUserPrefDate($value, 'Y/m/d', '-');  //from MySql format to timestamp
						}
					}
				}

				if ($field['type'] == 'time') {
					$value = $SERVICES->datelib->getTimestampFromUserPrefTime($field['value']);
				}
			} //end checks block where value is notnull
		}

		//reinstate 0 value for checkbox definitions with no item element; ie, unchecked boxes on submit
		if ($field['type'] == 'checkbox') {
			if ($field['value'] == '') {
				$value = 0;
			}
		}

		//if $value hasn't been updated by this function, keep the old value
		if (!$value && (!($value===0))) {
			$value = $field['value'];
		}
		return array('error'=>$msg, 'value'=>$value);
	}
	
	/**
	 * returns display date for any date or date time field
	 *
	 * @param integer or mysql date $date  the date variable
	 * @param $display format  can be 'date', 'date_time', or 'time' 
	 * @return string representation of date or date/time in UserPref format
	 * 20051006 updated by MCR to fix handling for dates prior to 1970 (mysql date format of course)
	 */
	function getDisplayDate($date, $display_format = 'date', $adjust_for_timezone = 1) {  //$date can be integer or mySQL format only
		global $SERVICES;
		$SERVICES->getService('DateLib');
		if (($date == '') || ($date == 0)) {
			return '';
		}
		
		//if mysql format
		if (eregi("[^0-9]",$date)) {  //if (!is_numeric($date))
			//$date = $SERVICES->datelib->getTimestampFromUserPrefDate($date, 'Y/m/d', '-');  //get integer for MySql format
			$content = $SERVICES->datelib->getUserPrefDateFromMySqlDate($date);  //display in UserPref format
		} else {
			if ($adjust_for_timezone) {			//it only makes sense to offset values where there is more precision than just the day
				$date = $SERVICES->offsetTimeZoneForRead($date);  //shift for timezone
			}
			switch ($display_format) {
				case 'date':
					$content = $SERVICES->datelib->getUserPrefDateFromTimestamp($date);  //display in UserPref format
				break;
				case 'date_time':
					$content = $SERVICES->datelib->getUserPrefDateFromTimestamp($date) . '&nbsp;' . $SERVICES->datelib->getUserPrefTimeFromTimestamp($date);  //display in UserPref format
				break;
				case 'time':
					$content = $SERVICES->datelib->getUserPrefTimeFromTimestamp($date);  //display in UserPref format
				break;
			}
		}
		return $content;
	}
	
	/**
	 * creates a tree view table from an array
	 *
	 * @see TreeView
	 * @param array $tree the tree array see $this->setTree() for more details
	 * @param array $icons the icons array see $this->setIcons() for more details (optional)
	 * @param string $sort_by string $sort_by the field key you want to sort on (optional)
	 * @return string the tree view table
	 * 20050524 - TreeView class in its own file now
	 */
	function getTreeView($tree, $icons = false, $sort_by = false) {
		if ($this->tree_view) {
			$this->tree_view->setTree($tree);
			$this->tree_view->setIcons($icons);
			$this->tree_view->setSortBy($sort_by);
		} else {
			includeService('TreeView');
			$this->tree_view = new TreeView($tree, $icons, $sort_by);
		}
		return $this->tree_view->getTreeTable();
	}
	
 	// Function set for optimising css class, style, id, and older html tag parameters eg width, cellpadding
	
	/**
	 * Adds an html tag element eg 'class' => 'new'
	 * adds to any existing content for that element
	 *
	 * Function set for optimising css class, style, id, and older html tag parameters eg width, cellpadding
	 * @author Matthew Ross
	 */
	function addTagContent($tag, $element, $content) {
		if ($this->tag_content[$tag][$element]) {
			$this->tag_content[$tag][$element] .= ($content ? ' ' . $content : '');
		} else {
			$this->tag_content[$tag][$element] = $content;
		}
	}
	
	/**
	 * Sets an html tag element eg 'class' => 'new'
	 * overwrites any existing content for that element
	 *
	 * Function set for optimising css class, style, id, and older html tag parameters eg width, cellpadding
	 * @author Matthew Ross
	 */
	function setTagContent($tag, $element, $content) {
		$this->tag_content[$tag][$element] = $content;
	}
	
	/**
	 * Empties an html tag
	 *
	 * Function set for optimising css class, style, id, and older html tag parameters eg width, cellpadding
	 * @author Matthew Ross
	 */
	function emptyTagContent($tag, $element) {
		bcrm_unset($this->tag_content, $tag, $element);
	}
	
	/**
	 * Sets default tag content for specified tag
	 *
	 * Function set for optimising css class, style, id, and older html tag parameters eg width, cellpadding
	 * @author Matthew Ross
	 */
	function resetTagContent($tag) {  //set to default
		$this->tag_content[$tag] = $this->getDefaultTagContent($tag);
	}
	
	/**
	 * Replace a tag with default setting from another
	 *
	 * Function set for optimising css class, style, id, and older html tag parameters eg width, cellpadding
	 * @author Matthew Ross
	 */
	function substituteTagContent($master_tag, $replace_with) {
		$this->tag_content[$master_tag] = $this->getDefaultTagContent($replace_with);
	}
	
	/**
	 * Gets default tag content for specified tag
	 *
	 * Function set for optimising css class, style, id, and older html tag parameters eg width, cellpadding
	 * @author Matthew Ross
	 * @return string
	 */
	function getDefaultTagContent($tag) {
		return $this->skin_config['tags'][$tag];
	}
	
	/**
	 * Gets formatted tag content for the specified tag
	 *
	 * Function set for optimising css class, style, id, and older html tag parameters eg width, cellpadding
	 * @author Matthew Ross
	 * @return string formatted for within the html tag
	 */
	function getTagContent($tag, $element = '') {
		if ($element && $this->tag_content[$tag][$element]) {
			return $this->tag_content[$tag][$element];
		}
		$tag_content = '';
		if (!is_array($tag)) {
			$tag = (isset($this->tag_content[$tag]) ? $this->tag_content[$tag] : false);
		}
		if (is_array($tag)) {
			foreach ($tag as $k => $v) {
				if ($k != 'text') {
					$tag_content .=  $k . '="' . $v . '" '; //eg 'class="r0" ';
				} else {
					$tag_content .=  ($v ? $v . ' ' : ''); //eg 'nowrap '
				}
			}
			$tag_content = trim($tag_content);
			if ($tag_content) {
				$tag_content = ' ' . $tag_content;
			}
		}
		return $tag_content;
	}
	
	/**
	 * Sets tag content to default for all tags
	 *
	 * Function set for optimising css class, style, id, and older html tag parameters eg width, cellpadding
	 * @author Matthew Ross
	 */
	function initTagContents() {
		//needs to be duplicated to enable reset
		$this->tag_content = $this->skin_config['tags'];
	}
	
	/**
	 * Sets default tag content for all tags
	 *
	 * Function set for optimising css class, style, id, and older html tag parameters eg width, cellpadding
	 * @author Matthew Ross
	 */
	function setSkinConfig() {
		global $SERVICES;
		$SERVICES->getService('Config');
		
		$config_base = $SERVICES->config->readConfig($this->getBaseSkinFilePath() . 'skin.config');
		$config = $SERVICES->config->readConfig($this->getSkinFilePath() . 'skin.config');
		//$this->skin_config = $config['skin'];
		$this->skin_config = $SERVICES->config->arrayMergeReplace($config_base['skin'], $config['skin']);

		if (!$this->skin_config['padded_table_switch']) {
			$this->setPaddedTableSwitch(false);
		}
	}
	
	/**
	 *
	 */
	function getSkinFilePath() {
		return $this->skin_file_path;
	}
	function getBaseSkinFilePath() {
		return $this->base_skin_file_path;
	}
	
	/**
	 * @author Phil Calder
	 */
	function setPrintView() {
		$this->is_print_view = ((isset($_REQUEST['print']) ? $_REQUEST['print'] == 'true' : false));
	}
	
	/**
	 * @author Phil Calder
	 */
	function isPrintView() {
		return (boolean)$this->is_print_view;
	}
	
	/**
	 *
	 */
	function setSkinFilePath() {
		global $USER, $SERVICES;
		if ($USER['skin_file_path']) {
			$this->base_skin_file_path = $USER['base_skin_file_path'];  //as seen by the file system
			$this->skin_file_path = $USER['skin_file_path'];  //as seen by the file system
			$this->ext_theme = $USER['ext_theme'];
		} else {
			//where a user is not logged in
			$this->base_skin_file_path = $SERVICES->config->lookup('site.root') . '/skins/_base/';  //as seen by the file system
			$this->skin_file_path = $SERVICES->config->lookup('site.root') . '/skins/' . $SERVICES->config->lookup('login.skin') . '/';  //as seen by the file system
		}
	}
	
	/**
	 *
	 */
	function getBaseSkinWebPath() {
		return $this->base_skin_web_path;
	}
	function getSkinWebPath() {
		return $this->skin_web_path;
	}
	
	/**
	 *
	 */
	function getJsWebPath() {
		return $this->js_web_path;
	}
	
	/**
	 *
	 */
	function setJsWebPath() {
		global $USER;
		if (isset($USER) && is_array($USER)) {
			$this->js_web_path = $USER['js_web_path'];  //as seen by the browser
		} else {
			// Assumes we are not in a sub directory
			// TODO Fix
			// leading slash is wrong for subdir installs - don't think we ever get here now
			$this->js_web_path = '/js/';  //default, as seen by the browser
		}
	}
	
	/**
	 * Sets the skin web path based on either a user setting or default value
	 */
	function setSkinWebPath() {
		global $USER;
		if (isset($USER) && is_array($USER) && (!$this->isPrintView()) ) {
			$this->base_skin_web_path = $USER['base_skin_web_path'];  //as seen by the browser
			$this->skin_web_path = $USER['skin_web_path'];  //as seen by the browser
		} else {
			// Phil 20060228 This is a 'temporary' websubdir setting.
			global $SERVICES;
			if ( is_object($SERVICES) && is_object($SERVICES->config) && ($websubdir = $SERVICES->config->lookup('site.websubdir')) ) {
				$websubdir = '/' . $websubdir; // prefix to make path absolute
			} else {
				$websubdir = ''; // Assumes we are not in a sub directory
			}
			if ($this->isPrintView()) { // Phil 20070510
				$this->base_skin_web_path = $websubdir . '/skins/_base/';  //dont change for print; css change handled elsewhere
				//$this->skin_web_path = $websubdir . '/skins/print/';
				$this->skin_web_path = $USER['skin_web_path'];  //dont change for print; css change handled elsewhere
			} else {
				$this->base_skin_web_path = $websubdir . '/skins/_base/';
				$this->skin_web_path = $websubdir . '/skins/_base/';  //default, as seen by the browser
			}
		}
	}
	
	/**
	 * Sets the display row flag (0 or 1)
	 */
	function setRowFlag($value = 0) {
		if ($value) {
			$value = 1;
		}
		$this->row_flag = $value;
	}
	
	/**
	 * Determines if we'll be using the template.
	 *
	 * @return boolean True or False depending on the value of $this->has_template
	 * @author Brenton Bartel
	 */
	function getHasTemplate(){
		return $this->has_template;
	}
	
	/**
	 * turns email addresses or URLs in $text into html links
	 *
	 * @param $text the text to linkify
	 * @returns copy of $text with URL translated to &lt;a href=URL, etc..
	 *
	 * @todo Currently links emails to BrowserCRM mail compose page. Check user has access to this module first.
	 * @todo Currently links URLs only if a scheme is provided (eg http://)
	 */
	function linkify($text) {
		// $ret = eregi_replace( "([[:alnum:]]+)://([^[:space:]]*)([[:alnum:]#?/&=])",  "<a href=\"\\1://\\2\\3\" target=\"_blank\">\\1://\\2\\3</a>", $text);
		$ret = eregi_replace( "([[:alnum:]]+)://([^[:space:]]*)([[:alnum:]#?/&=])",  '<a href="\\1://\\2\\3" target="_blank">\\1://\\2\\3</a>', $text);
		// $ret = eregi_replace( "(([a-z0-9_]|\\-|\\.)+@([^[:space:]]*)([[:alnum:]-]))",  "<a href=\"mailto:\\1\" target=\"_new\">\\1</a>", $ret);
		// $ret = eregi_replace( "(([a-z0-9_]|\\-|\\.)+@([^[:space:]]*)([[:alnum:]-]))",  "<a href=\"../../modules/Mail/edit.php?address=\\1\" target=\"_blank\">\\1</a>", $ret);
		$ret = eregi_replace( "(([a-z0-9_]|\\-|\\.)+@([^[:space:]]*)([[:alnum:]-]))",  '<a href="../../modules/Mail/edit.php?address=\\1" target="_blank">\\1</a>', $ret);
		return ($ret);
	}
	
} //end Output class

?>